import { Component, Input, Output, EventEmitter, TemplateRef, Pipe, ViewEncapsulation, HostListener, ElementRef, Renderer2, ViewChild, forwardRef, ContentChild, NgModule, Directive, ViewContainerRef, ContentChildren, Injectable, NgZone, HostBinding, Optional, Inject, Renderer, ChangeDetectorRef, ComponentFactoryResolver, Host } from '@angular/core';
import { animate, state, style, transition, trigger } from '@angular/animations';
import { DOWN_ARROW, ENTER, TAB } from '@angular/cdk/keycodes';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';
import { OverlayModule } from '@angular/cdk/overlay';
import { Subject } from 'rxjs/Subject';
import { Directionality } from '@angular/cdk/bidi';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { merge } from 'rxjs/observable/merge';
import { of } from 'rxjs/observable/of';
import { auditTime } from 'rxjs/operators/auditTime';
import { startWith } from 'rxjs/operators/startWith';
import { Observable } from 'rxjs/Observable';
import { map } from 'rxjs/operators/map';
import { ObserversModule } from '@angular/cdk/observers';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Subject as Subject$1 } from 'rxjs/Rx';
import { RouterModule, Router } from '@angular/router';
import { MatRippleModule } from '@angular/material';
import { DomSanitizer } from '@angular/platform-browser';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class PaginationComponent {
    constructor() {
        this.pageLinkSize = 5;
        this.onPageChange = new EventEmitter();
        this.alwaysShow = true;
        this._totalRecords = 0;
        this._first = 0;
        this._rows = 0;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.updatePaginatorState();
    }
    /**
     * @return {?}
     */
    get totalRecords() {
        return this._totalRecords;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set totalRecords(val) {
        this._totalRecords = val;
        this.updatePageLinks();
    }
    /**
     * @return {?}
     */
    get first() {
        return this._first;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set first(val) {
        this._first = val;
        this.updatePageLinks();
    }
    /**
     * @return {?}
     */
    get rows() {
        return this._rows;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set rows(val) {
        this._rows = val;
        this.updatePageLinks();
    }
    /**
     * @return {?}
     */
    get rowsPerPageOptions() {
        return this._rowsPerPageOptions;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set rowsPerPageOptions(val) {
        this._rowsPerPageOptions = val;
        if (this._rowsPerPageOptions) {
            this.rowsPerPageItems = [];
            for (const /** @type {?} */ opt of this._rowsPerPageOptions) {
                this.rowsPerPageItems.push({ label: String(opt), value: opt });
            }
        }
    }
    /**
     * @return {?}
     */
    isFirstPage() {
        return this.getPage() === 0;
    }
    /**
     * @return {?}
     */
    isLastPage() {
        return this.getPage() === this.getPageCount() - 1;
    }
    /**
     * @return {?}
     */
    getPageCount() {
        return Math.ceil(this.totalRecords / this.rows) || 1;
    }
    /**
     * @return {?}
     */
    calculatePageLinkBoundaries() {
        const /** @type {?} */ numberOfPages = this.getPageCount(), /** @type {?} */
        visiblePages = Math.min(this.pageLinkSize, numberOfPages);
        let /** @type {?} */ start = Math.max(0, Math.ceil(this.getPage() - ((visiblePages) / 2)));
        const /** @type {?} */ end = Math.min(numberOfPages - 1, start + visiblePages - 1);
        const /** @type {?} */ delta = this.pageLinkSize - (end - start + 1);
        start = Math.max(0, start - delta);
        return [start, end];
    }
    /**
     * @return {?}
     */
    updatePageLinks() {
        this.pageLinks = [];
        const /** @type {?} */ boundaries = this.calculatePageLinkBoundaries(), /** @type {?} */
        start = boundaries[0], /** @type {?} */
        end = boundaries[1];
        for (let /** @type {?} */ i = start; i <= end; i++) {
            this.pageLinks.push(i + 1);
        }
    }
    /**
     * @param {?} p
     * @return {?}
     */
    changePage(p) {
        const /** @type {?} */ pc = this.getPageCount();
        if (p >= 0 && p < pc) {
            this.first = this.rows * p;
            const /** @type {?} */ state$$1 = {
                page: p,
                first: this.first,
                rows: this.rows,
                pageCount: pc
            };
            this.updatePageLinks();
            this.onPageChange.emit(state$$1);
            this.updatePaginatorState();
        }
    }
    /**
     * @return {?}
     */
    getPage() {
        return Math.floor(this.first / this.rows);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    changePageToFirst(event) {
        if (!this.isFirstPage()) {
            this.changePage(0);
        }
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    changePageToPrev(event) {
        this.changePage(this.getPage() - 1);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    changePageToNext(event) {
        this.changePage(this.getPage() + 1);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    changePageToLast(event) {
        if (!this.isLastPage()) {
            this.changePage(this.getPageCount() - 1);
        }
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @param {?} page
     * @return {?}
     */
    onPageLinkClick(event, page) {
        this.changePage(page);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onRppChange(event) {
        this.changePage(this.getPage());
    }
    /**
     * @return {?}
     */
    updatePaginatorState() {
        this.paginatorState = {
            page: this.getPage(),
            rows: this.rows,
            first: this.first,
            totalRecords: this.totalRecords
        };
    }
}
PaginationComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-paginator',
                template: `<div [class]="styleClass" [ngStyle]="style" class="grf-ui-paginator  grf-ui-unselectable-text grf-ui-helper-clearfix"
    *ngIf="alwaysShow ? true : (pageLinks && pageLinks.length > 1)" [ngClass]="{'grf-ui-paginator-left': displayDirection == 'left','grf-ui-paginator-right': displayDirection == 'right'}">
    <div class="grf-ui-paginator-left-content" *ngIf="templateLeft">
        <ng-container *ngTemplateOutlet="templateLeft; context: {$implicit: paginatorState}"></ng-container>
    </div>
    <a href="#" class="grf-ui-paginator-first grf-ui-paginator-element grf-ui-state-default grf-ui-corner-all" (click)="changePageToFirst($event)"
        [ngClass]="{'grf-ui-state-disabled':isFirstPage()}" [tabindex]="isFirstPage() ? -1 : null">
        <span class="fa fa-angle-double-left"></span>
    </a>
    <a href="#" class="grf-ui-paginator-prev grf-ui-paginator-element grf-ui-state-default grf-ui-corner-all" (click)="changePageToPrev($event)"
        [ngClass]="{'grf-ui-state-disabled':isFirstPage()}" [tabindex]="isFirstPage() ? -1 : null">
        <span class="fa fa-angle-left"></span>
    </a>
    <span class="grf-ui-paginator-pages">
        <a href="#" *ngFor="let pageLink of pageLinks" class="grf-ui-paginator-page grf-ui-paginator-element grf-ui-state-default grf-ui-corner-all"
            (click)="onPageLinkClick($event, pageLink - 1)" [ngClass]="{'grf-ui-state-active': (pageLink-1 == getPage())}">{{pageLink}}</a>
    </span>
    <a href="#" class="grf-ui-paginator-next grf-ui-paginator-element grf-ui-state-default grf-ui-corner-all" (click)="changePageToNext($event)"
        [ngClass]="{'grf-ui-state-disabled':isLastPage()}" [tabindex]="isLastPage() ? -1 : null">
        <span class="fa fa-angle-right"></span>
    </a>
    <a href="#" class="grf-ui-paginator-last grf-ui-paginator-element grf-ui-state-default grf-ui-corner-all" (click)="changePageToLast($event)"
        [ngClass]="{'grf-ui-state-disabled':isLastPage()}" [tabindex]="isLastPage() ? -1 : null">
        <span class="fa fa-angle-double-right"></span>
    </a>
<!--     <p-dropdown [options]="rowsPerPageItems" [(ngModel)]="rows" *ngIf="rowsPerPageOptions" (onChange)="onRppChange($event)" [lazy]="false"
        [autoWidth]="false" [appendTo]="dropdownAppendTo"></p-dropdown> -->
    <grf-select *ngIf="rowsPerPageOptions"  class="grf-ui-paginator-select" [(ngModel)]="rows"  (uiOpenChange)="onRppChange($event)">
        <grf-option *ngFor="let option of rowsPerPageItems"  [uiLabel]="option.label"
        [uiValue]="option.value">
        </grf-option>
    </grf-select>
    <div class="grf-ui-paginator-right-content" *ngIf="templateRight">
        <ng-container *ngTemplateOutlet="templateRight; context: {$implicit: paginatorState}"></ng-container>
    </div>
</div>
`,
                styles: [`@charset "UTF-8";.grf-ui-paginator{margin:0;text-align:center;padding:0;background-color:#fff;-webkit-box-sizing:border-box;box-sizing:border-box;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.grf-ui-paginator-top{border-bottom:0}.grf-ui-paginator .grf-ui-paginator-select{min-width:50px}.grf-ui-paginator-bottom{border-top:0}.grf-ui-paginator-left-content{float:left}.grf-ui-paginator-right-content{float:right}.grf-ui-paginator-current,.grf-ui-paginator-first,.grf-ui-paginator-last,.grf-ui-paginator-next,.grf-ui-paginator-pages,.grf-ui-paginator-prev{display:inline-block;zoom:1;text-decoration:none}.grf-ui-paginator-first,.grf-ui-paginator-last,.grf-ui-paginator-next,.grf-ui-paginator-page,.grf-ui-paginator-prev{cursor:pointer}.grf-ui-paginator-current,.grf-ui-paginator-rpp-options{margin-left:1em;margin-right:1em;background-image:none}.grf-ui-paginator-jtp-select option,.grf-ui-paginator-options option{background-image:none;border:0;box-shadow:none;-moz-box-shadow:none;-webkit-box-shadow:none}.grf-ui-paginator a.grf-ui-state-disabled{outline:0}.grf-ui-paginator-first,.grf-ui-paginator-last,.grf-ui-paginator-next,.grf-ui-paginator-prev{-webkit-transition:background-color .3s;transition:background-color .3s;min-height:.875em;min-width:2em;vertical-align:top;border-radius:4px;font-size:16px;text-align:center;color:rgba(0,0,0,.65);outline:0}.grf-ui-paginator-first:hover,.grf-ui-paginator-last:hover,.grf-ui-paginator-next:hover,.grf-ui-paginator-prev:hover{background-color:#f4f3f4}.grf-ui-paginator-first::before,.grf-ui-paginator-last::before,.grf-ui-paginator-next::before,.grf-ui-paginator-prev::before{position:relative;top:.063em}.grf-ui-paginator-pages{padding:0}.grf-ui-paginator-pages .grf-ui-paginator-page{padding:6px 12px;border-radius:4px;font-size:14px;min-width:2em;color:#495057;text-decoration:none;-webkit-transition:background-color .3s;transition:background-color .3s}.grf-ui-paginator-pages .grf-ui-paginator-page:hover{background-color:#f4f3f4}.grf-ui-paginator-pages .grf-ui-paginator-page:hover.grf-ui-state-active{background-color:#0275d8;color:#fff}.grf-ui-paginator-pages .grf-ui-paginator-page.grf-ui-state-active{color:#fff;background-color:#0275d8}.grf-ui-paginator-left{text-align:left}.grf-ui-paginator-right{text-align:right}`]
            },] },
];
/** @nocollapse */
PaginationComponent.ctorParameters = () => [];
PaginationComponent.propDecorators = {
    "pageLinkSize": [{ type: Input },],
    "onPageChange": [{ type: Output },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "alwaysShow": [{ type: Input },],
    "templateLeft": [{ type: Input },],
    "templateRight": [{ type: Input },],
    "dropdownAppendTo": [{ type: Input },],
    "displayDirection": [{ type: Input },],
    "totalRecords": [{ type: Input },],
    "first": [{ type: Input },],
    "rows": [{ type: Input },],
    "rowsPerPageOptions": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class OptionPipe {
    /**
     * @param {?} options
     * @param {?} value
     * @return {?}
     */
    transform(options, value) {
        if (value.searchText) {
            let /** @type {?} */ _options = options.filter(option => option.uiLabel && (option.uiLabel.toLowerCase().indexOf(value.searchText.toLowerCase()) !== -1));
            if (value.tags) {
                _options = options.filter(option => option.uiLabel && (option.uiLabel.toLowerCase() === value.searchText.toLowerCase()));
            }
            if (_options.length) {
                return _options;
            }
            else {
                return /** @type {?} */ ([{
                        uiValue: value.value,
                        _value: value.value,
                        uiDisabled: value.disabled,
                        _disabled: value.disabled,
                        uiLabel: value.notFoundContent,
                        _label: value.notFoundContent,
                    }]);
            }
        }
        else {
            return options;
        }
    }
}
OptionPipe.decorators = [
    { type: Pipe, args: [{ name: 'uiOptionPipe' },] },
];
/** @nocollapse */
OptionPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._prefixCls = 'grf-ui-select';
        this._classList = [];
        this._dropDownPrefixCls = `${this._prefixCls}-dropdown`;
        this._selectionPrefixCls = `${this._prefixCls}-selection`;
        this._placeholder = 'Placeholder';
        this._notFoundContent = 'notFound';
        this._isOpen = false;
        this._disabled = false;
        this._showSearch = false;
        this._isTags = false;
        this._searchText = '';
        this._triggerWidth = 0;
        this._selectedOptions = new Set();
        this._options = [];
        this._cacheOptions = [];
        this._filterOptions = [];
        this._tagsOptions = [];
        this._isMultiInit = false;
        this._dropDownPosition = 'bottom';
        this._isMultiple = false;
        this._composing = false;
        this._keepUnListOptions = false;
        this._allowClear = false;
        // ngModel Access
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
        this.uiSearchChange = new EventEmitter();
        this.uiOpenChange = new EventEmitter();
        this.uiScrollToBottom = new EventEmitter();
        this.uiFilter = true;
        this.uiMaxMultiple = Infinity;
        /**
         * new ui-option insert or new tags insert
         */
        this.addOption = (option) => {
            this._options.push(option);
            if (!this._isTags) {
                if (option.uiValue) {
                    this.updateSelectedOption(this._value);
                }
                else {
                    this.forceUpdateSelectedOption(this._value);
                }
            }
        };
        /**
         * cancel select multiple option
         */
        this.unSelectMultipleOption = (option, $event, emitChange = true) => {
            this._operatingMultipleOption = option;
            this._selectedOptions.delete(option);
            if (emitChange) {
                this.emitMultipleOptions();
            }
            // 对Tag进行特殊处理
            if (this._isTags && (this._options.indexOf(option) !== -1) && (this._tagsOptions.indexOf(option) !== -1)) {
                this.removeOption(option);
                this._tagsOptions.splice(this._tagsOptions.indexOf(option), 1);
            }
            if ($event) {
                $event.preventDefault();
                $event.stopPropagation();
            }
        };
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAllowClear(value) {
        if (value === '') {
            this._allowClear = true;
        }
        else {
            this._allowClear = /** @type {?} */ (value);
        }
    }
    /**
     * @return {?}
     */
    get uiAllowClear() {
        return this._allowClear;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiKeepUnListOptions(value) {
        if (value === '') {
            this._keepUnListOptions = true;
        }
        else {
            this._keepUnListOptions = /** @type {?} */ (value);
        }
    }
    /**
     * @return {?}
     */
    get uiKeepUnListOptions() {
        return this._keepUnListOptions;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiMode(value) {
        this._mode = value;
        if (this._mode === 'multiple') {
            this.uiMultiple = true;
        }
        else if (this._mode === 'tags') {
            this.uiTags = true;
        }
        else if (this._mode === 'combobox') {
            this.uiShowSearch = true;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiMultiple(value) {
        this._isMultiple = value;
        if (this._isMultiple) {
            this.uiShowSearch = true;
        }
    }
    /**
     * @return {?}
     */
    get uiMultiple() {
        return this._isMultiple;
    }
    /**
     * @return {?}
     */
    get uiPlaceHolder() {
        return this._placeholder;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiPlaceHolder(value) {
        this._placeholder = value;
    }
    /**
     * @return {?}
     */
    get uiNotFoundContent() {
        return this._notFoundContent;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiNotFoundContent(value) {
        this._notFoundContent = value;
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this._size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this._size = value;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiShowSearch() {
        return this._showSearch;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiShowSearch(value) {
        this._showSearch = value;
    }
    /**
     * @return {?}
     */
    get uiTags() {
        return this._isTags;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiTags(value) {
        this._isTags = value;
        this.uiMultiple = value;
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = value;
        this.closeDropDown();
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiOpen() {
        return this._isOpen;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiOpen(value) {
        if (this._isOpen === value) {
            return;
        }
        if (value === true) {
            this.scrollToActive();
            if (!this._triggerWidth) {
                this._setTriggerWidth();
            }
        }
        this._isOpen = value;
        this.uiOpenChange.emit(this._isOpen);
        this.setClassMap();
    }
    /**
     * ui-option remove or tags remove
     * @param {?} option
     * @return {?}
     */
    removeOption(option) {
        this._options.splice(this._options.indexOf(option), 1);
        if (!this._isTags) {
            this.forceUpdateSelectedOption(this._value);
        }
    }
    /**
     * dropdown position changed
     * @param {?} position
     * @return {?}
     */
    onPositionChange(position) {
        this._dropDownPosition = position.connectionPair.originY;
    }
    /**
     * @return {?}
     */
    compositionStart() {
        this._composing = true;
    }
    /**
     * @return {?}
     */
    compositionEnd() {
        this._composing = false;
    }
    /**
     * clear single selected option
     * @param {?=} $event
     * @return {?}
     */
    clearSelect($event) {
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
        this._selectedOption = null;
        this.uiValue = null;
        this.onChange(null);
    }
    /**
     * click dropdown option by user
     * @param {?} option
     * @param {?=} $event
     * @return {?}
     */
    clickOption(option, $event) {
        if (!option) {
            return;
        }
        this.chooseOption(option, true, $event);
        this.clearSearchText();
        if (!this._isMultiple) {
            this.uiOpen = false;
        }
    }
    /**
     * choose option
     * @param {?} option
     * @param {?=} isUserClick
     * @param {?=} $event
     * @return {?}
     */
    chooseOption(option, isUserClick = false, $event) {
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
        this._activeFilterOption = option;
        if (option && !option.uiDisabled) {
            if (!this.uiMultiple) {
                this._selectedOption = option;
                this._value = option.uiValue;
                if (isUserClick) {
                    this.onChange(option.uiValue);
                }
            }
            else {
                if (isUserClick) {
                    this.isInSet(this._selectedOptions, option) ? this.unSelectMultipleOption(option) : this.selectMultipleOption(option);
                }
            }
        }
    }
    /**
     * @param {?} element
     * @param {?} text
     * @return {?}
     */
    updateWidth(element, text) {
        if (text) {
            /** wait for scroll width change */
            setTimeout(() => {
                this._renderer.setStyle(element, 'width', `${element.scrollWidth}px`);
            });
        }
        else {
            this._renderer.removeStyle(element, 'width');
        }
    }
    /**
     * @param {?} ul
     * @return {?}
     */
    dropDownScroll(ul) {
        if (ul && (ul.scrollHeight - ul.scrollTop === ul.clientHeight)) {
            this.uiScrollToBottom.emit(true);
        }
    }
    /**
     * @return {?}
     */
    checkDropDownScroll() {
        if (this.dropdownUl && (this.dropdownUl.nativeElement.scrollHeight === this.dropdownUl.nativeElement.clientHeight)) {
            this.uiScrollToBottom.emit(true);
        }
    }
    /**
     * determine if option in set
     * @param {?} set
     * @param {?} option
     * @return {?}
     */
    isInSet(set, option) {
        return ((/** @type {?} */ (Array.from(set))).find((data) => data.uiValue === option.uiValue));
    }
    /**
     * select multiple option
     * @param {?} option
     * @param {?=} $event
     * @return {?}
     */
    selectMultipleOption(option, $event) {
        /** if tags do push to tag option */
        if (this._isTags && (this._options.indexOf(option) === -1) && (this._tagsOptions.indexOf(option) === -1)) {
            this.addOption(option);
            this._tagsOptions.push(option);
        }
        this._operatingMultipleOption = option;
        if (this._selectedOptions.size < this.uiMaxMultiple) {
            this._selectedOptions.add(option);
        }
        this.emitMultipleOptions();
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    }
    /**
     * emit multiple options
     * @return {?}
     */
    emitMultipleOptions() {
        if (this._isMultiInit) {
            return;
        }
        const /** @type {?} */ arrayOptions = /** @type {?} */ (Array.from(this._selectedOptions));
        this.onChange(arrayOptions.map((item) => item.uiValue));
    }
    /**
     * update selected option when add remove option etc
     * @param {?} currentModelValue
     * @param {?=} triggerByNgModel
     * @return {?}
     */
    updateSelectedOption(currentModelValue, triggerByNgModel = false) {
        if (currentModelValue === null) {
            return;
        }
        if (this.uiMultiple) {
            const /** @type {?} */ selectedOptions = this._options.filter((item) => {
                return (item != null) && (JSON.stringify(currentModelValue).indexOf(item.uiValue) !== -1);
            });
            if ((this.uiKeepUnListOptions || this.uiTags) && (!triggerByNgModel)) {
                const /** @type {?} */ _selectedOptions = Array.from(this._selectedOptions);
                selectedOptions.forEach(option => {
                    const /** @type {?} */ _exist = _selectedOptions.some(item => item._value === option._value);
                    if (!_exist) {
                        this._selectedOptions.add(option);
                    }
                });
            }
            else {
                this._selectedOptions = new Set();
                selectedOptions.forEach(option => {
                    this._selectedOptions.add(option);
                });
            }
        }
        else {
            const /** @type {?} */ selectedOption = this._options.filter((item) => {
                return (item != null) && (item.uiValue === currentModelValue);
            });
            this.chooseOption(selectedOption[0]);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    forceUpdateSelectedOption(value) {
        /** trigger dirty check */
        setTimeout(() => {
            this.updateSelectedOption(value);
        });
    }
    /**
     * @return {?}
     */
    get uiValue() {
        return this._value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiValue(value) {
        this._updateValue(value);
    }
    /**
     * @param {?=} emitChange
     * @return {?}
     */
    clearAllSelectedOption(emitChange = true) {
        this._selectedOptions.forEach(item => {
            this.unSelectMultipleOption(item, null, emitChange);
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleKeyEnterEvent(event) {
        /** when composing end */
        if (!this._composing && this._isOpen) {
            event.preventDefault();
            event.stopPropagation();
            this.updateFilterOption(false);
            this.clickOption(this._activeFilterOption);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleKeyBackspaceEvent(event) {
        if ((!this._searchText) && (!this._composing) && (this._isMultiple)) {
            event.preventDefault();
            const /** @type {?} */ lastOption = Array.from(this._selectedOptions).pop();
            this.unSelectMultipleOption(lastOption);
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    handleKeyDownEvent($event) {
        if (this._isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this._activeFilterOption = this.nextOption(this._activeFilterOption, this._filterOptions);
            this.scrollToActive();
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    handleKeyUpEvent($event) {
        if (this._isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this._activeFilterOption = this.preOption(this._activeFilterOption, this._filterOptions);
            this.scrollToActive();
        }
    }
    /**
     * @param {?} option
     * @param {?} options
     * @return {?}
     */
    preOption(option, options) {
        return options[options.indexOf(option) - 1] || options[options.length - 1];
    }
    /**
     * @param {?} option
     * @param {?} options
     * @return {?}
     */
    nextOption(option, options) {
        return options[options.indexOf(option) + 1] || options[0];
    }
    /**
     * @return {?}
     */
    clearSearchText() {
        this._searchText = '';
        this.updateFilterOption();
    }
    /**
     * @param {?=} updateActiveFilter
     * @return {?}
     */
    updateFilterOption(updateActiveFilter = true) {
        if (this.uiFilter) {
            this._filterOptions = new OptionPipe().transform(this._options, {
                'searchText': this._searchText,
                'tags': this._isTags,
                'notFoundContent': this._isTags ? this._searchText : this._notFoundContent,
                'disabled': !this._isTags,
                'value': this._isTags ? this._searchText : 'disabled'
            });
        }
        else {
            this._filterOptions = this._options;
        }
        /** TODO: cause pre & next key selection not work */
        if (updateActiveFilter && !this._selectedOption) {
            this._activeFilterOption = this._filterOptions[0];
        }
    }
    /**
     * @param {?} searchValue
     * @return {?}
     */
    onSearchChange(searchValue) {
        this.uiSearchChange.emit(searchValue);
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        e.preventDefault();
        if (!this._disabled) {
            this.uiOpen = !this.uiOpen;
            if (this.uiShowSearch) {
                /** wait for search display */
                setTimeout(() => {
                    this.searchInputElementRef.nativeElement.focus();
                });
            }
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onKeyDown(e) {
        const /** @type {?} */ keyCode = e.keyCode;
        if (keyCode === TAB && this.uiOpen) {
            this.uiOpen = false;
            return;
        }
        if ((keyCode !== DOWN_ARROW && keyCode !== ENTER) || this.uiOpen) {
            return;
        }
        e.preventDefault();
        if (!this._disabled) {
            this.uiOpen = true;
            if (this.uiShowSearch) {
                /** wait for search display */
                setTimeout(() => {
                    this.searchInputElementRef.nativeElement.focus();
                });
            }
        }
    }
    /**
     * @return {?}
     */
    closeDropDown() {
        if (!this.uiOpen) {
            return;
        }
        this.onTouched();
        if (this.uiMultiple) {
            this._renderer.removeStyle(this.searchInputElementRef.nativeElement, 'width');
        }
        this.clearSearchText();
        this.uiOpen = false;
    }
    /**
     * @return {?}
     */
    setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.removeClass(this._el, _className);
        });
        this._classList = [
            this._prefixCls,
            (this._mode === 'combobox') && `${this._prefixCls}-combobox`,
            (!this._disabled) && `${this._prefixCls}-enabled`,
            (this._disabled) && `${this._prefixCls}-disabled`,
            this._isOpen && `${this._prefixCls}-open`,
            this._showSearch && `${this._prefixCls}-show-search`,
            this._size && `${this._prefixCls}-${this._size}`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this._el, _className);
        });
        this._selectionClassMap = {
            [this._selectionPrefixCls]: true,
            [`${this._selectionPrefixCls}--single`]: !this.uiMultiple,
            [`${this._selectionPrefixCls}-multiple`]: this.uiMultiple
        };
    }
    /**
     * @return {?}
     */
    setDropDownClassMap() {
        this._dropDownClassMap = {
            [this._dropDownPrefixCls]: true,
            ['component-select']: this._mode === 'combobox',
            [`${this._dropDownPrefixCls}--single`]: !this.uiMultiple,
            [`${this._dropDownPrefixCls}-multiple`]: this.uiMultiple,
            [`${this._dropDownPrefixCls}-placement-bottomLeft`]: this._dropDownPosition === 'bottom',
            [`${this._dropDownPrefixCls}-placement-topLeft`]: this._dropDownPosition === 'top'
        };
    }
    /**
     * @return {?}
     */
    scrollToActive() {
        /** wait for dropdown display */
        setTimeout(() => {
            if (this._activeFilterOption && this._activeFilterOption.uiValue) {
                const /** @type {?} */ index = this._filterOptions.findIndex(option => option.uiValue === this._activeFilterOption.uiValue);
                try {
                    const /** @type {?} */ scrollPane = this.dropdownUl.nativeElement.children[index];
                    scrollPane.scrollIntoViewIfNeeded(false);
                }
                catch (/** @type {?} */ e) {
                }
            }
        });
    }
    /**
     * @return {?}
     */
    flushComponentState() {
        this.updateFilterOption();
        if (!this.uiMultiple) {
            this.updateSelectedOption(this._value);
        }
        else {
            if (this._value) {
                this.updateSelectedOption(this._value);
            }
        }
    }
    /**
     * @return {?}
     */
    _setTriggerWidth() {
        this._triggerWidth = this._getTriggerRect().width;
    }
    /**
     * @return {?}
     */
    _getTriggerRect() {
        return this.trigger.nativeElement.getBoundingClientRect();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._updateValue(value, false);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.uiDisabled = isDisabled;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this._value != null) {
            this.flushComponentState();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.updateFilterOption();
        this.setClassMap();
        this.setDropDownClassMap();
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this._cacheOptions !== this._options) {
            /** update filter option after every content check cycle */
            this.updateFilterOption();
            this._cacheOptions = this._options;
        }
        else {
            this.updateFilterOption(false);
        }
    }
    /**
     * @param {?} value
     * @param {?=} emitChange
     * @return {?}
     */
    _updateValue(value, emitChange = true) {
        if (this._value === value) {
            return;
        }
        if ((value == null) && this.uiMultiple) {
            this._value = [];
        }
        else {
            this._value = value;
        }
        if (!this.uiMultiple) {
            if (value == null) {
                this._selectedOption = null;
            }
            else {
                this.updateSelectedOption(value);
            }
        }
        else {
            if (value) {
                if (value.length === 0) {
                    this.clearAllSelectedOption(emitChange);
                }
                else {
                    this.updateSelectedOption(value, true);
                }
            }
            else if (value == null) {
                this.clearAllSelectedOption(emitChange);
            }
        }
    }
}
SelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-select',
                template: `<div tabindex="0" #trigger cdkOverlayOrigin #origin="cdkOverlayOrigin" [ngClass]="_selectionClassMap" (keydown.Enter)="handleKeyEnterEvent($event)"
    (keydown.Backspace)="handleKeyBackspaceEvent($event)" (keydown.ArrowUp)="handleKeyUpEvent($event)" (keydown.ArrowDown)="handleKeyDownEvent($event)">
    <div class="grf-ui-select-selection__rendered" *ngIf="!uiShowSearch">
        <div class="grf-ui-select-selection-selected-value">
            {{_selectedOption?.uiLabel}}
        </div>
    </div>
    <div class="grf-ui-select-selection__rendered" *ngIf="uiShowSearch">
        <div [hidden]="_searchText||(!uiOpen&&_selectedOption)||_selectedOptions.size" class="grf-ui-select-selection__placeholder">
            <ng-template [ngIf]="(!_composing)&&_selectedOption">
                {{_selectedOption.uiLabel}}
            </ng-template>
            <ng-template [ngIf]="(!_composing)&&(!_selectedOption)">
                {{uiPlaceHolder}}
            </ng-template>
        </div>
        <ul *ngIf="uiMultiple">
            <li *ngFor="let option of _selectedOptions" [@tagAnimation] [attr.title]="option?.uiValue" class="grf-ui-select-selection__choice"
                style="user-select: none;">
                <div class="grf-ui-select-selection__choice__content">{{option?.uiLabel}}</div>
                <!---->
                <span class="grf-ui-select-selection__choice__remove" (click)="unSelectMultipleOption(option,$event)">
                    <i class="fa fa-times-circle grf-ui-input-suffix-clear"></i>
                </span>
            </li>
            <li class="grf-ui-select-search grf-ui-select-search--inline">
                <div class="grf-ui-select-search__field__wrap">
                    <input class="grf-ui-select-search__field" (compositionstart)="compositionStart()" (compositionend)="compositionEnd();updateWidth(searchInput,_searchText);"
                        [(ngModel)]="_searchText" (ngModelChange)="updateFilterOption();onSearchChange($event);" (keydown)="updateWidth(searchInput,_searchText)"
                        (input)="updateWidth(searchInput,_searchText)" (blur)="onTouched()" #searchInput>
                    <span class="grf-ui-select-search__field__mirror"></span>
                </div>
            </li>
        </ul>
        <div *ngIf="!uiMultiple" class="grf-ui-select-selection-selected-value" [hidden]="!(_selectedOption?.uiLabel)||uiOpen">
            {{_selectedOption?.uiLabel}}
        </div>
        <div *ngIf="!uiMultiple" [hidden]="!uiOpen" class="grf-ui-select-search grf-ui-select-search--inline">
            <div class="grf-ui-select-search__field__wrap">
                <input class="grf-ui-select-search__field" (blur)="onTouched()" (compositionstart)="compositionStart()" (compositionend)="compositionEnd()"
                    [(ngModel)]="_searchText" (ngModelChange)="updateFilterOption();onSearchChange($event);" #searchInput>
                <span class="grf-ui-select-search__field__mirror"></span>
            </div>
        </div>
    </div>
    <span (click)="onTouched();clearSelect($event)" class="grf-ui-select-selection__clear" style="user-select: none;" *ngIf="_selectedOption?.uiLabel&&uiAllowClear&&!uiMultiple">
        <i class="fa fa-times-circle grf-ui-input-suffix-clear"></i>
    </span>
    <span class="grf-ui-select-arrow fa fa-chevron-down">
        <b></b>
    </span>
</div>
<ng-template cdkConnectedOverlay cdkConnectedOverlayHasBackdrop [cdkConnectedOverlayOrigin]="origin" (backdropClick)="closeDropDown()"
    (detach)="closeDropDown();" (positionChange)="onPositionChange($event)" [cdkConnectedOverlayWidth]="_triggerWidth" [cdkConnectedOverlayOpen]="_isOpen">
    <div [ngClass]="_dropDownClassMap" [@dropDownAnimation]="_dropDownPosition">
        <div style="overflow: auto;">
            <ul class="grf-ui-select-dropdown-menu grf-ui-select-dropdown-menu-vertical grf-ui-select-dropdown-menu-root" #dropdownUl (scroll)="dropDownScroll(dropdownUl)">
                <li *ngFor="let option of _filterOptions" [class.grf-ui-select-dropdown-menu-item-disabled]="option.uiDisabled" [class.grf-ui-select-dropdown-menu-item-active]="option.uiValue == _activeFilterOption?.uiValue"
                    [class.grf-ui-select-dropdown-menu-item-selected]="(option.uiValue==(_selectedOption?.uiValue))||(isInSet(_selectedOptions,option))"
                    class="grf-ui-select-dropdown-menu-item" (click)="clickOption(option,$event)">
                    <ng-template *ngIf="option.uiOptionTemplate" [ngTemplateOutlet]="option.uiOptionTemplate">
                    </ng-template>
                    <ng-template [ngIf]="!option.uiOptionTemplate">
                        {{ option.uiLabel }}
                    </ng-template>
                    <i class="fa fa-check-circle grf-ui-item-selected"></i>
                </li>
            </ul>
        </div>
    </div>
</ng-template>
`,
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";@-webkit-keyframes antSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@keyframes antSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@-webkit-keyframes antSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@keyframes antSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@-webkit-keyframes antSlideDownIn{0%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}}@keyframes antSlideDownIn{0%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}}@-webkit-keyframes antSlideDownOut{0%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@keyframes antSlideDownOut{0%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@-webkit-keyframes antSlideLeftIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@keyframes antSlideLeftIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@-webkit-keyframes antSlideLeftOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@keyframes antSlideLeftOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@-webkit-keyframes antSlideRightIn{0%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@keyframes antSlideRightIn{0%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@-webkit-keyframes antSlideRightOut{0%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@keyframes antSlideRightOut{0%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}.grf-ui-select{-webkit-box-sizing:border-box;box-sizing:border-box;display:inline-block;position:relative;color:rgba(0,0,0,.65);font-size:14px}.grf-ui-select>ul>li>a{padding:0;background-color:#fff}.grf-ui-select-arrow{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;position:absolute;top:50%;right:8px;line-height:1;margin-top:-6px}.grf-ui-select-arrow:before{display:block;font-family:FontAwesome!important;color:#cac7c7;-webkit-transition:-webkit-transform .2s ease;transition:-webkit-transform .2s ease;transition:transform .2s ease;transition:transform .2s ease,-webkit-transform .2s ease}.grf-ui-select-arrow *{display:none}.grf-ui-select-selection{outline:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-box-sizing:border-box;box-sizing:border-box;display:block;background-color:#fff;border-radius:4px;border:1px solid #d9d9d9;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-select-selection:hover{border-left:2px solid #0275d8}.grf-ui-select-selection-focused .grf-ui-select-selection,.grf-ui-select-selection:active,.grf-ui-select-selection:focus{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-select-selection__clear{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;text-rendering:auto;opacity:0;position:absolute;right:8px;z-index:1;background:#fff;top:50%;font-size:14px;color:rgba(0,0,0,.25);width:12px;height:12px;margin-top:-6px;line-height:12px;cursor:pointer;-webkit-transition:color .3s ease,opacity .15s ease;transition:color .3s ease,opacity .15s ease}.grf-ui-select-selection__clear:before{display:block}.grf-ui-select-selection__clear:hover{color:rgba(0,0,0,.43)}.grf-ui-select-selection:hover .grf-ui-select-selection__clear{opacity:1}.grf-ui-select-selection-selected-value{float:left;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:100%;padding-right:14px}.grf-ui-select-disabled{color:rgba(0,0,0,.25)}.grf-ui-select-disabled .grf-ui-select-selection{background:#f7f7f7;cursor:not-allowed}.grf-ui-select-disabled .grf-ui-select-selection:active,.grf-ui-select-disabled .grf-ui-select-selection:focus,.grf-ui-select-disabled .grf-ui-select-selection:hover{border-color:#d9d9d9;-webkit-box-shadow:none;box-shadow:none}.grf-ui-select-disabled .grf-ui-select-selection__clear{display:none;visibility:hidden;pointer-events:none}.grf-ui-select-disabled .grf-ui-select-selection--multiple .grf-ui-select-selection__choice{background:#eee;color:#aaa;padding-right:10px}.grf-ui-select-disabled .grf-ui-select-selection--multiple .grf-ui-select-selection__choice__remove{display:none}.grf-ui-select-selection--single{height:30px;position:relative;cursor:pointer}.grf-ui-select-selection__rendered{display:block;margin-left:7px;margin-right:7px;position:relative;line-height:28px}.grf-ui-select-selection__rendered:after{content:'.';visibility:hidden;pointer-events:none;display:inline-block;width:0}.grf-ui-select-lg .grf-ui-select-selection--single{height:34px}.grf-ui-select-lg .grf-ui-select-selection__rendered{line-height:32px}.grf-ui-select-lg .grf-ui-select-selection--multiple{min-height:34px}.grf-ui-select-lg .grf-ui-select-selection--multiple .grf-ui-select-selection__rendered li{height:26px;line-height:26px}.grf-ui-select-lg .grf-ui-select-selection--multiple .grf-ui-select-selection__clear{top:17px}.grf-ui-select-sm .grf-ui-select-selection--single{height:24px}.grf-ui-select-sm .grf-ui-select-selection__rendered{line-height:22px}.grf-ui-select-sm .grf-ui-select-selection--multiple{min-height:24px}.grf-ui-select-sm .grf-ui-select-selection--multiple .grf-ui-select-selection__rendered li{height:16px;line-height:16px}.grf-ui-select-sm .grf-ui-select-selection--multiple .grf-ui-select-selection__clear{top:12px}.grf-ui-select-disabled .grf-ui-select-selection__choice__remove{color:rgba(0,0,0,.25);cursor:default}.grf-ui-select-disabled .grf-ui-select-selection__choice__remove:hover{color:rgba(0,0,0,.25)}.grf-ui-select-search__field__wrap{display:inline-block;position:relative}.grf-ui-select-search__field__placeholder,.grf-ui-select-selection__placeholder{position:absolute;top:50%;left:0;right:9px;color:rgba(0,0,0,.25);line-height:20px;height:20px;max-width:100%;margin-top:-10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;text-align:left}.grf-ui-select-search__field__placeholder{left:8px}.grf-ui-select-search--inline{position:absolute;height:100%;width:100%}.grf-ui-select-selection-multiple .grf-ui-select-search--inline{float:left;position:static}.grf-ui-select-search--inline .grf-ui-select-search__field__wrap{width:100%;height:100%}.grf-ui-select-search--inline .grf-ui-select-search__field{border-width:0;font-size:100%;height:100%;width:100%;background:0 0;outline:0;border-radius:4px}.grf-ui-select-search--inline .grf-ui-select-search__field__mirror{position:absolute;top:0;left:-9999px;white-space:pre;pointer-events:none}.grf-ui-select-search--inline>i{float:right}.grf-ui-select-selection-multiple{min-height:30px;cursor:text;padding-bottom:3px;zoom:1}.grf-ui-select-selection-multiple:after,.grf-ui-select-selection-multiple:before{content:" ";display:table}.grf-ui-select-selection-multiple:after{clear:both;visibility:hidden;font-size:0;height:0}.grf-ui-select-selection-multiple .grf-ui-select-search--inline{width:auto;padding:0;max-width:100%}.grf-ui-select-selection-multiple .grf-ui-select-search--inline .grf-ui-select-search__field{max-width:100%;width:.75em}.grf-ui-select-selection-multiple .grf-ui-select-selection__rendered{margin-left:5px;margin-bottom:-3px;height:auto}.grf-ui-select-selection-multiple ul{list-style:none;margin:0;padding:0}.grf-ui-select-selection-multiple .grf-ui-select-selection__rendered>ul>li,.grf-ui-select-selection-multiple>ul>li{margin-top:3px;height:26px;line-height:26px;display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-select-selection-multiple .grf-ui-select-selection__choice{color:rgba(0,0,0,.65);background-color:#f3f3f3;border-radius:4px;cursor:default;float:left;margin-right:4px;max-width:99%;position:relative;overflow:hidden;-webkit-transition:padding .3s cubic-bezier(.645,.045,.355,1);transition:padding .3s cubic-bezier(.645,.045,.355,1);padding:0 10px;border:1px solid #ccc}.grf-ui-select-selection-multiple .grf-ui-select-selection__choice__disabled{padding:0 10px}.grf-ui-select-selection-multiple .grf-ui-select-selection__choice__content{display:inline-block;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;max-width:100%;-webkit-transition:margin .3s cubic-bezier(.645,.045,.355,1);transition:margin .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-select-selection-multiple .grf-ui-select-selection__choice__remove{color:rgba(0,0,0,.43);line-height:inherit;cursor:pointer;display:inline-block;font-weight:700;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1);padding:0 0 0 5px}.grf-ui-select-selection-multiple .grf-ui-select-selection__choice__remove:hover{color:#404040}.grf-ui-select-selection-multiple .grf-ui-select-selection__clear{top:15px}.grf-ui-select-allow-clear .grf-ui-select-selection--multiple .grf-ui-select-selection__rendered{margin-right:20px}.grf-ui-select-open .grf-ui-select-arrow{-ms-transform:rotate(180deg)}.grf-ui-select-open .grf-ui-select-arrow:before{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.grf-ui-select-open .grf-ui-select-selection{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-select-combobox .grf-ui-select-arrow{display:none}.grf-ui-select-combobox .grf-ui-select-search--inline{height:100%;width:100%;float:none}.grf-ui-select-combobox .grf-ui-select-search__field__wrap{width:100%;height:100%}.grf-ui-select-combobox .grf-ui-select-search__field{width:100%;height:100%;position:relative;z-index:1;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1);-webkit-box-shadow:none;box-shadow:none}.grf-ui-select-combobox .grf-ui-select-allow-clear .grf-ui-select-selection:hover .grf-ui-select-selection__rendered{margin-right:20px}.grf-ui-select-dropdown{top:100%;left:0;width:100%;margin-top:2px;margin-bottom:2px;background-color:#fff;-webkit-box-shadow:0 1px 6px rgba(0,0,0,.2);box-shadow:0 1px 6px rgba(0,0,0,.2);border-radius:4px;-webkit-box-sizing:border-box;box-sizing:border-box;z-index:5;position:absolute;outline:0;overflow:hidden;font-size:14px}.grf-ui-select-dropdown .grf-ui-item-selected{display:none}.grf-ui-select-dropdown .grf-ui-select-dropdown-placement-topLeft{top:-100%;-webkit-transform:translateY(-100%)!important;transform:translateY(-100%)!important}.grf-ui-select-dropdown-menu-item:hover{border-left:2px solid #0275d8}.grf-ui-select-dropdown.slide-up-appear .slide-up-appear-active .grf-ui-select-dropdown-placement-bottomLeft,.grf-ui-select-dropdown.slide-up-enter .slide-up-enter-active .grf-ui-select-dropdown-placement-bottomLeft{-webkit-animation-name:antSlideUpIn;animation-name:antSlideUpIn}.grf-ui-select-dropdown.slide-up-appear .slide-up-appear-active .grf-ui-select-dropdown-placement-topLeft,.grf-ui-select-dropdown.slide-up-enter .slide-up-enter-active .grf-ui-select-dropdown-placement-topLeft{-webkit-animation-name:antSlideDownIn;animation-name:antSlideDownIn}.grf-ui-select-dropdown.slide-up-leave .slide-up-leave-active .grf-ui-select-dropdown-placement-bottomLeft{-webkit-animation-name:antSlideUpOut;animation-name:antSlideUpOut}.grf-ui-select-dropdown.slide-up-leave .slide-up-leave-active .grf-ui-select-dropdown-placement-topLeft{-webkit-animation-name:antSlideDownOut;animation-name:antSlideDownOut}.grf-ui-select-dropdown-hidden{display:none}.grf-ui-select-dropdown-menu{outline:0;margin:0;padding-left:0;list-style:none;max-height:250px;overflow:auto}.grf-ui-select-dropdown-menu-item-group-list{margin:0;padding:0}.grf-ui-select-dropdown-menu-item-group-list>.grf-ui-select-dropdown-menu-item{padding-left:16px}.grf-ui-select-dropdown-menu-item-group-title{color:rgba(0,0,0,.43);line-height:1.5;padding:8px}.grf-ui-select-dropdown-menu-item{line-height:1.5;position:relative;display:block;padding:7px 8px;font-weight:400;color:rgba(0,0,0,.65);white-space:nowrap;cursor:pointer;overflow:hidden;-webkit-transition:background .3s ease;transition:background .3s ease}.grf-ui-select-dropdown-menu-item-disabled{color:rgba(0,0,0,.25);cursor:not-allowed}.grf-ui-select-dropdown-menu-item-disabled:hover{color:rgba(0,0,0,.25);background-color:#fff;cursor:not-allowed}.grf-ui-select-dropdown-menu-item-selected,.grf-ui-select-dropdown-menu-item-selected:hover{font-weight:700;color:rgba(0,0,0,.65);border-left:2px solid #0275d8}.grf-ui-select-dropdown-menu-item-divider{height:1px;margin:1px 0;overflow:hidden;background-color:#e5e5e5;line-height:0}.grf-ui-select-dropdown-multiple .grf-ui-select-dropdown-menu-item .grf-ui-item-selected{color:transparent;-webkit-transition:all .2s ease;transition:all .2s ease;position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);right:8px;font-weight:700;text-shadow:0 .1px 0,.1px 0 0,0 -.1px 0,-.1px 0}.grf-ui-select-dropdown-multiple .grf-ui-select-dropdown-menu-item:hover .grf-ui-item-selected{color:#ddd}.grf-ui-select-dropdown-multiple .grf-ui-select-dropdown-menu-item-disabled .grf-ui-item-selected{display:none}.grf-ui-select-dropdown-multiple .grf-ui-select-dropdown-menu-item-selected .grf-ui-item-selected,.grf-ui-select-dropdown-multiple .grf-ui-select-dropdown-menu-item-selected:hover .grf-ui-item-selected{color:#0275d8;display:inline-block}.grf-ui-select-dropdown-container-open .grf-ui-select-dropdown,.grf-ui-select-dropdown-open .grf-ui-select-dropdown{display:block}.cdk-overlay-dark-backdrop{background:0 0}`],
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => SelectComponent),
                        multi: true
                    }
                ],
                animations: [
                    trigger('dropDownAnimation', [
                        state('bottom', style({
                            opacity: 1,
                            transform: 'scaleY(1)',
                            transformOrigin: '0% 0%'
                        })),
                        transition('void => bottom', [
                            style({
                                opacity: 0,
                                transform: 'scaleY(0)',
                                transformOrigin: '0% 0%'
                            }),
                            animate('150ms cubic-bezier(0.25, 0.8, 0.25, 1)')
                        ]),
                        state('top', style({
                            opacity: 1,
                            transform: 'scaleY(1)',
                            transformOrigin: '0% 100%'
                        })),
                        transition('void => top', [
                            style({
                                opacity: 0,
                                transform: 'scaleY(0)',
                                transformOrigin: '0% 100%'
                            }),
                            animate('150ms cubic-bezier(0.25, 0.8, 0.25, 1)')
                        ]),
                        transition('* => void', [
                            animate('250ms 100ms linear', style({ opacity: 0 }))
                        ])
                    ]),
                    trigger('tagAnimation', [
                        state('*', style({ opacity: 1, transform: 'scale(1)' })),
                        transition('void => *', [
                            style({ opacity: 0, transform: 'scale(0)' }),
                            animate('150ms linear')
                        ]),
                        state('void', style({ opacity: 0, transform: 'scale(0)' })),
                        transition('* => void', [
                            style({ opacity: 1, transform: 'scale(1)' }),
                            animate('150ms linear')
                        ])
                    ])
                ]
            },] },
];
/** @nocollapse */
SelectComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
SelectComponent.propDecorators = {
    "searchInputElementRef": [{ type: ViewChild, args: ['searchInput',] },],
    "trigger": [{ type: ViewChild, args: ['trigger',] },],
    "dropdownUl": [{ type: ViewChild, args: ['dropdownUl',] },],
    "uiSearchChange": [{ type: Output },],
    "uiOpenChange": [{ type: Output },],
    "uiScrollToBottom": [{ type: Output },],
    "uiFilter": [{ type: Input },],
    "uiMaxMultiple": [{ type: Input },],
    "uiAllowClear": [{ type: Input },],
    "uiKeepUnListOptions": [{ type: Input },],
    "uiMode": [{ type: Input },],
    "uiMultiple": [{ type: Input },],
    "uiPlaceHolder": [{ type: Input },],
    "uiNotFoundContent": [{ type: Input },],
    "uiSize": [{ type: Input },],
    "uiShowSearch": [{ type: Input },],
    "uiTags": [{ type: Input },],
    "uiDisabled": [{ type: Input },],
    "uiOpen": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
    "onKeyDown": [{ type: HostListener, args: ['keydown', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class OptionComponent {
    /**
     * @param {?} _Select
     */
    constructor(_Select) {
        this._Select = _Select;
        this._disabled = false;
    }
    /**
     * @return {?}
     */
    get uiValue() {
        return this._value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiValue(value) {
        if (this._value === value) {
            return;
        }
        this._value = value;
    }
    /**
     * @return {?}
     */
    get uiLabel() {
        return this._label;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiLabel(value) {
        if (this._label === value) {
            return;
        }
        this._label = value;
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = value;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._Select.addOption(this);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._Select.removeOption(this);
    }
}
OptionComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-option',
                encapsulation: ViewEncapsulation.None,
                template: `
      <ng-content></ng-content>
    `,
                styles: []
            },] },
];
/** @nocollapse */
OptionComponent.ctorParameters = () => [
    { type: SelectComponent, },
];
OptionComponent.propDecorators = {
    "uiOptionTemplate": [{ type: ContentChild, args: ['uiOptionTemplate',] },],
    "uiValue": [{ type: Input },],
    "uiLabel": [{ type: Input },],
    "uiDisabled": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectModule {
}
SelectModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule, FormsModule, OverlayModule
                ],
                declarations: [OptionPipe, OptionComponent, SelectComponent],
                exports: [OptionPipe, OptionComponent, SelectComponent]
            },] },
];
/** @nocollapse */
SelectModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class HeaderComponent {
}
HeaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-header',
                template: '<ng-content></ng-content>'
            },] },
];
/** @nocollapse */
HeaderComponent.ctorParameters = () => [];
class FooterComponent {
}
FooterComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-footer',
                template: '<ng-content></ng-content>'
            },] },
];
/** @nocollapse */
FooterComponent.ctorParameters = () => [];
class TemplateDirective {
    /**
     * @param {?} template
     */
    constructor(template) {
        this.template = template;
    }
    /**
     * @return {?}
     */
    getType() {
        return this.name;
    }
}
TemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTemplate]',
            },] },
];
/** @nocollapse */
TemplateDirective.ctorParameters = () => [
    { type: TemplateRef, },
];
TemplateDirective.propDecorators = {
    "type": [{ type: Input },],
    "name": [{ type: Input },],
};
class TemplateWrapperDirective {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.templateRef, {
            '\$implicit': this.item,
            'index': this.index
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
TemplateWrapperDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTemplateWrapper]'
            },] },
];
/** @nocollapse */
TemplateWrapperDirective.ctorParameters = () => [
    { type: ViewContainerRef, },
];
TemplateWrapperDirective.propDecorators = {
    "item": [{ type: Input },],
    "index": [{ type: Input },],
    "templateRef": [{ type: Input },],
};
class ColumnComponent {
    constructor() {
        this.sortFunction = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.templates.forEach((item) => {
            switch (item.getType()) {
                case 'header':
                    this.headerTemplate = item.template;
                    break;
                case 'body':
                    this.bodyTemplate = item.template;
                    break;
                case 'footer':
                    this.footerTemplate = item.template;
                    break;
                case 'filter':
                    this.filterTemplate = item.template;
                    break;
                case 'editor':
                    this.editorTemplate = item.template;
                    break;
                default:
                    this.bodyTemplate = item.template;
                    break;
            }
        });
    }
}
ColumnComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-column',
                template: ``
            },] },
];
/** @nocollapse */
ColumnComponent.ctorParameters = () => [];
ColumnComponent.propDecorators = {
    "field": [{ type: Input },],
    "colId": [{ type: Input },],
    "sortField": [{ type: Input },],
    "filterField": [{ type: Input },],
    "header": [{ type: Input },],
    "footer": [{ type: Input },],
    "sortable": [{ type: Input },],
    "editable": [{ type: Input },],
    "filter": [{ type: Input },],
    "filterMatchMode": [{ type: Input },],
    "filterType": [{ type: Input },],
    "rowspan": [{ type: Input },],
    "colspan": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "hidden": [{ type: Input },],
    "expander": [{ type: Input },],
    "selectionMode": [{ type: Input },],
    "filterPlaceholder": [{ type: Input },],
    "filterMaxlength": [{ type: Input },],
    "frozen": [{ type: Input },],
    "sortFunction": [{ type: Output },],
    "templates": [{ type: ContentChildren, args: [TemplateDirective,] },],
    "template": [{ type: ContentChild, args: [TemplateRef,] },],
};
class RowComponent {
}
RowComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-row',
                template: ``
            },] },
];
/** @nocollapse */
RowComponent.ctorParameters = () => [];
RowComponent.propDecorators = {
    "columns": [{ type: ContentChildren, args: [ColumnComponent,] },],
};
class HeaderColumnGroupComponent {
}
HeaderColumnGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-headercolumngroup',
                template: ``
            },] },
];
/** @nocollapse */
HeaderColumnGroupComponent.ctorParameters = () => [];
HeaderColumnGroupComponent.propDecorators = {
    "rows": [{ type: ContentChildren, args: [RowComponent,] },],
};
class FooterColumnGroupComponent {
}
FooterColumnGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-footercolumngroup',
                template: ``
            },] },
];
/** @nocollapse */
FooterColumnGroupComponent.ctorParameters = () => [];
FooterColumnGroupComponent.propDecorators = {
    "rows": [{ type: ContentChildren, args: [RowComponent,] },],
};
class ColumnBodyTemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.column.bodyTemplate, {
            '\$implicit': this.column,
            'rowData': this.rowData,
            'rowIndex': this.rowIndex
        });
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (!this.view) {
            return;
        }
        if ('rowIndex' in changes) {
            this.view.context.rowIndex = changes['rowIndex'].currentValue;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
ColumnBodyTemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-columnbodytemplateloader',
                template: ``
            },] },
];
/** @nocollapse */
ColumnBodyTemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
ColumnBodyTemplateLoaderComponent.propDecorators = {
    "column": [{ type: Input },],
    "rowData": [{ type: Input },],
    "rowIndex": [{ type: Input },],
};
class ColumnHeaderTemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.column.headerTemplate, {
            '\$implicit': this.column
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
ColumnHeaderTemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-columnheadertemplateloader',
                template: ``
            },] },
];
/** @nocollapse */
ColumnHeaderTemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
ColumnHeaderTemplateLoaderComponent.propDecorators = {
    "column": [{ type: Input },],
};
class ColumnFooterTemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.column.footerTemplate, {
            '\$implicit': this.column
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
ColumnFooterTemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-columnfootertemplateloader',
                template: ``
            },] },
];
/** @nocollapse */
ColumnFooterTemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
ColumnFooterTemplateLoaderComponent.propDecorators = {
    "column": [{ type: Input },],
};
class ColumnFilterTemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.column.filterTemplate, {
            '\$implicit': this.column
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
ColumnFilterTemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-columnfiltertemplateloader',
                template: ``
            },] },
];
/** @nocollapse */
ColumnFilterTemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
ColumnFilterTemplateLoaderComponent.propDecorators = {
    "column": [{ type: Input },],
};
class ColumnEditorTemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.view = this.viewContainer.createEmbeddedView(this.column.editorTemplate, {
            '\$implicit': this.column,
            'rowData': this.rowData,
            'rowIndex': this.rowIndex
        });
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.view.destroy();
    }
}
ColumnEditorTemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-columneditortemplateloader',
                template: ``
            },] },
];
/** @nocollapse */
ColumnEditorTemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
ColumnEditorTemplateLoaderComponent.propDecorators = {
    "column": [{ type: Input },],
    "rowData": [{ type: Input },],
    "rowIndex": [{ type: Input },],
};
class TemplateLoaderComponent {
    /**
     * @param {?} viewContainer
     */
    constructor(viewContainer) {
        this.viewContainer = viewContainer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.template) {
            this.view = this.viewContainer.createEmbeddedView(this.template, {
                '\$implicit': this.data
            });
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.view) {
            this.view.destroy();
        }
    }
}
TemplateLoaderComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-templateloader',
                template: ``
            },] },
];
/** @nocollapse */
TemplateLoaderComponent.ctorParameters = () => [
    { type: ViewContainerRef, },
];
TemplateLoaderComponent.propDecorators = {
    "template": [{ type: Input },],
    "data": [{ type: Input },],
};
class SharedModule {
}
SharedModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [HeaderComponent, FooterComponent, ColumnComponent, TemplateWrapperDirective, ColumnHeaderTemplateLoaderComponent,
                    ColumnBodyTemplateLoaderComponent, ColumnFooterTemplateLoaderComponent, ColumnFilterTemplateLoaderComponent, TemplateDirective,
                    TemplateLoaderComponent, RowComponent, HeaderColumnGroupComponent, FooterColumnGroupComponent, ColumnEditorTemplateLoaderComponent],
                declarations: [HeaderComponent, FooterComponent, ColumnComponent, TemplateWrapperDirective, ColumnHeaderTemplateLoaderComponent,
                    ColumnBodyTemplateLoaderComponent, ColumnFooterTemplateLoaderComponent, ColumnFilterTemplateLoaderComponent, TemplateDirective,
                    TemplateLoaderComponent, RowComponent, HeaderColumnGroupComponent, FooterColumnGroupComponent, ColumnEditorTemplateLoaderComponent]
            },] },
];
/** @nocollapse */
SharedModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class PaginationModule {
}
PaginationModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, SelectModule, FormsModule, SharedModule],
                exports: [PaginationComponent, SelectModule, FormsModule, SharedModule],
                declarations: [PaginationComponent]
            },] },
];
/** @nocollapse */
PaginationModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DomHandler {
    constructor() {
        this.calculatedScrollbarWidth = null;
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    addClass(element, className) {
        if (element.classList) {
            element.classList.add(className);
        }
        else {
            element.className += ' ' + className;
        }
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    addMultipleClasses(element, className) {
        if (element.classList) {
            const /** @type {?} */ styles = className.split(' ');
            for (let /** @type {?} */ i = 0; i < styles.length; i++) {
                element.classList.add(styles[i]);
            }
        }
        else {
            const /** @type {?} */ styles = className.split(' ');
            for (let /** @type {?} */ i = 0; i < styles.length; i++) {
                element.className += ' ' + styles[i];
            }
        }
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    removeClass(element, className) {
        if (element.classList) {
            element.classList.remove(className);
        }
        else {
            element.className = element.className.replace(new RegExp('(^|\\b)' + className.split(' ').join('|') + '(\\b|$)', 'gi'), ' ');
        }
    }
    /**
     * @param {?} element
     * @param {?} className
     * @return {?}
     */
    hasClass(element, className) {
        if (element.classList) {
            return element.classList.contains(className);
        }
        else {
            return new RegExp('(^| )' + className + '( |$)', 'gi').test(element.className);
        }
    }
    /**
     * @param {?} element
     * @return {?}
     */
    siblings(element) {
        return Array.prototype.filter.call(element.parentNode.children, function (child) {
            return child !== element;
        });
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    find(element, selector) {
        return element.querySelectorAll(selector);
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    findSingle(element, selector) {
        return element.querySelector(selector);
    }
    /**
     * @param {?} element
     * @return {?}
     */
    index(element) {
        const /** @type {?} */ children = element.parentNode.childNodes;
        let /** @type {?} */ num = 0;
        for (let /** @type {?} */ i = 0; i < children.length; i++) {
            if (children[i] === element) {
                return num;
            }
            if (children[i].nodeType === 1) {
                num++;
            }
        }
        return -1;
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    relativePosition(element, target) {
        const /** @type {?} */ elementDimensions = element.offsetParent ? { width: element.offsetWidth, height: element.offsetHeight } : this.getHiddenElementDimensions(element);
        const /** @type {?} */ targetHeight = target.offsetHeight;
        const /** @type {?} */ targetWidth = target.offsetWidth;
        const /** @type {?} */ targetOffset = target.getBoundingClientRect();
        const /** @type {?} */ windowScrollTop = this.getWindowScrollTop();
        const /** @type {?} */ viewport = this.getViewport();
        let /** @type {?} */ top, /** @type {?} */ left;
        if ((targetOffset.top + targetHeight + elementDimensions.height) > viewport.height) {
            top = -1 * (elementDimensions.height);
            if (targetOffset.top + top < 0) {
                top = 0;
            }
        }
        else {
            top = targetHeight;
        }
        if ((targetOffset.left + elementDimensions.width) > viewport.width) {
            left = targetWidth - elementDimensions.width;
        }
        else {
            left = 0;
        }
        element.style.top = top + 'px';
        element.style.left = left + 'px';
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    absolutePosition(element, target) {
        const /** @type {?} */ elementDimensions = element.offsetParent ? { width: element.offsetWidth, height: element.offsetHeight } : this.getHiddenElementDimensions(element);
        const /** @type {?} */ elementOuterHeight = elementDimensions.height;
        const /** @type {?} */ elementOuterWidth = elementDimensions.width;
        const /** @type {?} */ targetOuterHeight = target.offsetHeight;
        const /** @type {?} */ targetOuterWidth = target.offsetWidth;
        const /** @type {?} */ targetOffset = target.getBoundingClientRect();
        const /** @type {?} */ windowScrollTop = this.getWindowScrollTop();
        const /** @type {?} */ windowScrollLeft = this.getWindowScrollLeft();
        const /** @type {?} */ viewport = this.getViewport();
        let /** @type {?} */ top, /** @type {?} */ left;
        if (targetOffset.top + targetOuterHeight + elementOuterHeight > viewport.height) {
            top = targetOffset.top + windowScrollTop - elementOuterHeight;
            if (top < 0) {
                top = 0 + windowScrollTop;
            }
        }
        else {
            top = targetOuterHeight + targetOffset.top + windowScrollTop;
        }
        if (targetOffset.left + targetOuterWidth + elementOuterWidth > viewport.width) {
            left = targetOffset.left + windowScrollLeft + targetOuterWidth - elementOuterWidth;
        }
        else {
            left = targetOffset.left + windowScrollLeft;
        }
        element.style.top = top + 'px';
        element.style.left = left + 'px';
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementOuterHeight(element) {
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        const /** @type {?} */ elementHeight = element.offsetHeight;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return elementHeight;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementOuterWidth(element) {
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        const /** @type {?} */ elementWidth = element.offsetWidth;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return elementWidth;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    getHiddenElementDimensions(element) {
        const /** @type {?} */ dimensions = {};
        element.style.visibility = 'hidden';
        element.style.display = 'block';
        dimensions.width = element.offsetWidth;
        dimensions.height = element.offsetHeight;
        element.style.display = 'none';
        element.style.visibility = 'visible';
        return dimensions;
    }
    /**
     * @param {?} container
     * @param {?} item
     * @return {?}
     */
    scrollInView(container, item) {
        const /** @type {?} */ borderTopValue = getComputedStyle(container).getPropertyValue('borderTopWidth');
        const /** @type {?} */ borderTop = borderTopValue ? parseFloat(borderTopValue) : 0;
        const /** @type {?} */ paddingTopValue = getComputedStyle(container).getPropertyValue('paddingTop');
        const /** @type {?} */ paddingTop = paddingTopValue ? parseFloat(paddingTopValue) : 0;
        const /** @type {?} */ containerRect = container.getBoundingClientRect();
        const /** @type {?} */ itemRect = item.getBoundingClientRect();
        const /** @type {?} */ offset = (itemRect.top + document.body.scrollTop) - (containerRect.top + document.body.scrollTop) - borderTop - paddingTop;
        const /** @type {?} */ scroll = container.scrollTop;
        const /** @type {?} */ elementHeight = container.clientHeight;
        const /** @type {?} */ itemHeight = this.getOuterHeight(item);
        if (offset < 0) {
            container.scrollTop = scroll + offset;
        }
        else if ((offset + itemHeight) > elementHeight) {
            container.scrollTop = scroll + offset - elementHeight + itemHeight;
        }
    }
    /**
     * @param {?} element
     * @param {?} duration
     * @return {?}
     */
    fadeIn(element, duration) {
        element.style.opacity = 0;
        let /** @type {?} */ last = +new Date();
        let /** @type {?} */ opacity = 0;
        const /** @type {?} */ tick = function () {
            opacity = +element.style.opacity + (new Date().getTime() - last) / duration;
            element.style.opacity = opacity;
            last = +new Date();
            if (+opacity < 1) {
                return (window.requestAnimationFrame && requestAnimationFrame(tick)) || setTimeout(tick, 16);
            }
        };
        tick();
    }
    /**
     * @param {?} element
     * @param {?} ms
     * @return {?}
     */
    fadeOut(element, ms) {
        let /** @type {?} */ opacity = 1;
        const /** @type {?} */ interval = 50;
        const /** @type {?} */ duration = ms;
        const /** @type {?} */ gap = interval / duration;
        const /** @type {?} */ fading = setInterval(() => {
            opacity = opacity - gap;
            if (opacity <= 0) {
                opacity = 0;
                clearInterval(fading);
            }
            element.style.opacity = opacity;
        }, interval);
    }
    /**
     * @return {?}
     */
    getWindowScrollTop() {
        const /** @type {?} */ doc = document.documentElement;
        return (window.pageYOffset || doc.scrollTop) - (doc.clientTop || 0);
    }
    /**
     * @return {?}
     */
    getWindowScrollLeft() {
        const /** @type {?} */ doc = document.documentElement;
        return (window.pageXOffset || doc.scrollLeft) - (doc.clientLeft || 0);
    }
    /**
     * @param {?} element
     * @param {?} selector
     * @return {?}
     */
    matches(element, selector) {
        const /** @type {?} */ p = Element.prototype;
        const /** @type {?} */ f = p['matches'] || p.webkitMatchesSelector || p.msMatchesSelector || function (s) {
            return [].indexOf.call(document.querySelectorAll(s), this) !== -1;
        };
        return f.call(element, selector);
    }
    /**
     * @param {?} el
     * @param {?=} margin
     * @return {?}
     */
    getOuterWidth(el, margin) {
        let /** @type {?} */ width = el.offsetWidth;
        if (margin) {
            const /** @type {?} */ style$$1 = getComputedStyle(el);
            width += parseFloat(style$$1.marginLeft) + parseFloat(style$$1.marginRight);
        }
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHorizontalPadding(el) {
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        return parseFloat(style$$1.paddingLeft) + parseFloat(style$$1.paddingRight);
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHorizontalMargin(el) {
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        return parseFloat(style$$1.marginLeft) + parseFloat(style$$1.marginRight);
    }
    /**
     * @param {?} el
     * @return {?}
     */
    innerWidth(el) {
        let /** @type {?} */ width = el.offsetWidth;
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        width += parseFloat(style$$1.paddingLeft) + parseFloat(style$$1.paddingRight);
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    width(el) {
        let /** @type {?} */ width = el.offsetWidth;
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        width -= parseFloat(style$$1.paddingLeft) + parseFloat(style$$1.paddingRight);
        return width;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getInnerHeight(el) {
        let /** @type {?} */ height = el.offsetHeight;
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        height += parseFloat(style$$1.paddingTop) + parseFloat(style$$1.paddingBottom);
        return height;
    }
    /**
     * @param {?} el
     * @param {?=} margin
     * @return {?}
     */
    getOuterHeight(el, margin) {
        let /** @type {?} */ height = el.offsetHeight;
        if (margin) {
            const /** @type {?} */ style$$1 = getComputedStyle(el);
            height += parseFloat(style$$1.marginTop) + parseFloat(style$$1.marginBottom);
        }
        return height;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getHeight(el) {
        let /** @type {?} */ height = el.offsetHeight;
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        height -= parseFloat(style$$1.paddingTop) + parseFloat(style$$1.paddingBottom) + parseFloat(style$$1.borderTopWidth) + parseFloat(style$$1.borderBottomWidth);
        return height;
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getWidth(el) {
        let /** @type {?} */ width = el.offsetWidth;
        const /** @type {?} */ style$$1 = getComputedStyle(el);
        width -= parseFloat(style$$1.paddingLeft) + parseFloat(style$$1.paddingRight) + parseFloat(style$$1.borderLeftWidth) + parseFloat(style$$1.borderRightWidth);
        return width;
    }
    /**
     * @return {?}
     */
    getViewport() {
        const /** @type {?} */ win = window, /** @type {?} */
        d = document, /** @type {?} */
        e = d.documentElement, /** @type {?} */
        g = d.getElementsByTagName('body')[0], /** @type {?} */
        w = win.innerWidth || e.clientWidth || g.clientWidth, /** @type {?} */
        h = win.innerHeight || e.clientHeight || g.clientHeight;
        return { width: w, height: h };
    }
    /**
     * @param {?} el
     * @return {?}
     */
    getOffset(el) {
        const /** @type {?} */ rect = el.getBoundingClientRect();
        return {
            top: rect.top + document.body.scrollTop,
            left: rect.left + document.body.scrollLeft
        };
    }
    /**
     * @return {?}
     */
    getUserAgent() {
        return navigator.userAgent;
    }
    /**
     * @return {?}
     */
    isIE() {
        const /** @type {?} */ ua = window.navigator.userAgent;
        const /** @type {?} */ msie = ua.indexOf('MSIE ');
        if (msie > 0) {
            // IE 10 or older => return version number
            return true;
        }
        const /** @type {?} */ trident = ua.indexOf('Trident/');
        if (trident > 0) {
            // IE 11 => return version number
            const /** @type {?} */ rv = ua.indexOf('rv:');
            return true;
        }
        const /** @type {?} */ edge = ua.indexOf('Edge/');
        if (edge > 0) {
            // Edge (IE 12+) => return version number
            return true;
        }
        // other browser
        return false;
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    appendChild(element, target) {
        if (this.isElement(target)) {
            target.appendChild(element);
        }
        else if (target.el && target.el.nativeElement) {
            target.el.nativeElement.appendChild(element);
        }
        else {
            throw new Error('Cannot append ' + target + ' to ' + element);
        }
    }
    /**
     * @param {?} element
     * @param {?} target
     * @return {?}
     */
    removeChild(element, target) {
        if (this.isElement(target)) {
            target.removeChild(element);
        }
        else if (target.el && target.el.nativeElement) {
            target.el.nativeElement.removeChild(element);
        }
        else {
            throw new Error('Cannot remove ' + element + ' from ' + target);
        }
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    isElement(obj) {
        return (typeof HTMLElement === 'object' ? obj instanceof HTMLElement :
            obj && typeof obj === 'object' && obj !== null && obj.nodeType === 1 && typeof obj.nodeName === 'string');
    }
    /**
     * @return {?}
     */
    calculateScrollbarWidth() {
        if (this.calculatedScrollbarWidth !== null) {
            return this.calculatedScrollbarWidth;
        }
        const /** @type {?} */ scrollDiv = document.createElement('div');
        scrollDiv.className = 'grf-ui-scrollbar-measure';
        document.body.appendChild(scrollDiv);
        const /** @type {?} */ scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth;
        document.body.removeChild(scrollDiv);
        this.calculatedScrollbarWidth = scrollbarWidth;
        return scrollbarWidth;
    }
    /**
     * @param {?} element
     * @param {?} methodName
     * @param {?=} args
     * @return {?}
     */
    invokeElementMethod(element, methodName, args) {
        (/** @type {?} */ (element))[methodName].apply(element, args);
    }
    /**
     * @return {?}
     */
    clearSelection() {
        if (window.getSelection) {
            if (window.getSelection().empty) {
                window.getSelection().empty();
            }
            else if (window.getSelection().removeAllRanges && window.getSelection().rangeCount > 0 && window.getSelection().getRangeAt(0).getClientRects().length > 0) {
                window.getSelection().removeAllRanges();
            }
        }
        /**
                else if (document[selection] && document['selection'].empty) {
                    try {
                        document['selection'].empty();
                    } catch (error) {
                        // ignore IE bug
                    }
                }
                 */
    }
    /**
     * @param {?} element
     * @param {?} attributeName
     * @return {?}
     */
    indexWithinGroup(element, attributeName) {
        const /** @type {?} */ children = element.parentNode.childNodes;
        let /** @type {?} */ num = 0;
        for (let /** @type {?} */ i = 0; i < children.length; i++) {
            if (children[i] === element) {
                return num;
            }
            if (children[i].attributes && children[i].attributes[attributeName] && children[i].nodeType === 1) {
                num++;
            }
        }
        return -1;
    }
}
DomHandler.zindex = 1000;
DomHandler.decorators = [
    { type: Injectable },
];
/** @nocollapse */
DomHandler.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ObjectUtils {
    /**
     * @param {?} obj1
     * @param {?} obj2
     * @param {?=} field
     * @return {?}
     */
    equals(obj1, obj2, field) {
        if (field) {
            return (this.resolveFieldData(obj1, field) === this.resolveFieldData(obj2, field));
        }
        else {
            return this.equalsByValue(obj1, obj2);
        }
    }
    /**
     * @param {?} obj1
     * @param {?} obj2
     * @param {?=} visited
     * @return {?}
     */
    equalsByValue(obj1, obj2, visited) {
        if (obj1 == null && obj2 == null) {
            return true;
        }
        if (obj1 == null || obj2 == null) {
            return false;
        }
        if (obj1 === obj2) {
            return true;
        }
        if (typeof obj1 === 'object' && typeof obj2 === 'object') {
            if (visited) {
                if (visited.indexOf(obj1) !== -1) {
                    return false;
                }
            }
            else {
                visited = [];
            }
            visited.push(obj1);
            for (const /** @type {?} */ p in obj1) {
                if (p) {
                    if (obj1.hasOwnProperty(p) !== obj2.hasOwnProperty(p)) {
                        return false;
                    }
                    switch (typeof (obj1[p])) {
                        case 'object':
                            if (!this.equalsByValue(obj1[p], obj2[p], visited)) {
                                return false;
                            }
                            break;
                        case 'function':
                            if (typeof (obj2[p]) === 'undefined' || (p !== 'compare' && obj1[p].toString() !== obj2[p].toString())) {
                                return false;
                            }
                            break;
                        default:
                            if (obj1[p] !== obj2[p]) {
                                return false;
                            }
                            break;
                    }
                }
            }
            for (const /** @type {?} */ p in obj2) {
                if (p) {
                    if (typeof (obj1[p]) === 'undefined') {
                        return false;
                    }
                }
            }
            delete obj1._$visited;
            return true;
        }
        return false;
    }
    /**
     * @param {?} data
     * @param {?} field
     * @return {?}
     */
    resolveFieldData(data, field) {
        if (data && field) {
            if (field.indexOf('.') === -1) {
                return data[field];
            }
            else {
                const /** @type {?} */ fields = field.split('.');
                let /** @type {?} */ value = data;
                for (let /** @type {?} */ i = 0, /** @type {?} */ len = fields.length; i < len; ++i) {
                    if (value == null) {
                        return null;
                    }
                    value = value[fields[i]];
                }
                return value;
            }
        }
        else {
            return null;
        }
    }
    /**
     * @param {?} value
     * @param {?} fields
     * @param {?} filterValue
     * @return {?}
     */
    filter(value, fields, filterValue) {
        const /** @type {?} */ filteredItems = [];
        if (value) {
            for (const /** @type {?} */ item of value) {
                for (const /** @type {?} */ field of fields) {
                    if (String(this.resolveFieldData(item, field)).toLowerCase().indexOf(filterValue.toLowerCase()) > -1) {
                        filteredItems.push(item);
                        break;
                    }
                }
            }
        }
        return filteredItems;
    }
    /**
     * @param {?} value
     * @param {?} from
     * @param {?} to
     * @return {?}
     */
    reorderArray(value, from, to) {
        let /** @type {?} */ target;
        if (value && (from !== to)) {
            if (to >= value.length) {
                target = to - value.length;
                while ((target--) + 1) {
                    value.push(undefined);
                }
            }
            value.splice(to, 0, value.splice(from, 1)[0]);
        }
    }
    /**
     * @param {?} val
     * @param {?} field
     * @return {?}
     */
    generateSelectItems(val, field) {
        let /** @type {?} */ selectItems;
        if (val && val.length) {
            selectItems = [];
            for (const /** @type {?} */ item of val) {
                selectItems.push({ label: this.resolveFieldData(item, field), value: item });
            }
        }
        return selectItems;
    }
    /**
     * @param {?} item
     * @param {?} index
     * @param {?} arr
     * @param {?} sourceArr
     * @return {?}
     */
    insertIntoOrderedArray(item, index, arr, sourceArr) {
        if (arr.length > 0) {
            let /** @type {?} */ injected = false;
            for (let /** @type {?} */ i = 0; i < arr.length; i++) {
                const /** @type {?} */ currentItemIndex = this.findIndexInList(arr[i], sourceArr);
                if (currentItemIndex > index) {
                    arr.splice(i, 0, item);
                    injected = true;
                    break;
                }
            }
            if (!injected) {
                arr.push(item);
            }
        }
        else {
            arr.push(item);
        }
    }
    /**
     * @param {?} item
     * @param {?} list
     * @return {?}
     */
    findIndexInList(item, list) {
        let /** @type {?} */ index = -1;
        if (list) {
            for (let /** @type {?} */ i = 0; i < list.length; i++) {
                if (list[i] === item) {
                    index = i;
                    break;
                }
            }
        }
        return index;
    }
}
ObjectUtils.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ObjectUtils.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableService {
    constructor() {
        this.sortSource = new Subject();
        this.selectionSource = new Subject();
        this.contextMenuSource = new Subject();
        this.valueSource = new Subject();
        this.totalRecordsSource = new Subject();
        this.sortSource$ = this.sortSource.asObservable();
        this.selectionSource$ = this.selectionSource.asObservable();
        this.contextMenuSource$ = this.contextMenuSource.asObservable();
        this.valueSource$ = this.valueSource.asObservable();
        this.totalRecordsSource$ = this.totalRecordsSource.asObservable();
    }
    /**
     * @param {?} sortMeta
     * @return {?}
     */
    onSort(sortMeta) {
        this.sortSource.next(sortMeta);
    }
    /**
     * @return {?}
     */
    onSelectionChange() {
        this.selectionSource.next();
    }
    /**
     * @param {?} data
     * @return {?}
     */
    onContextMenu(data) {
        this.contextMenuSource.next(data);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onValueChange(value) {
        this.valueSource.next(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onTotalRecordsChange(value) {
        this.totalRecordsSource.next(value);
    }
}
TableService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
TableService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableTemplateDirective {
    /**
     * @param {?} template
     */
    constructor(template) {
        this.template = template;
    }
    /**
     * @return {?}
     */
    getType() {
        return this.name;
    }
}
TableTemplateDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTableTemplate]',
            },] },
];
/** @nocollapse */
TableTemplateDirective.ctorParameters = () => [
    { type: TemplateRef, },
];
TableTemplateDirective.propDecorators = {
    "type": [{ type: Input },],
    "name": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} objectUtils
     * @param {?} zone
     * @param {?} tableService
     */
    constructor(el, domHandler, objectUtils, zone, tableService) {
        this.el = el;
        this.domHandler = domHandler;
        this.objectUtils = objectUtils;
        this.zone = zone;
        this.tableService = tableService;
        this.first = 0;
        this.pageLinks = 5;
        this.alwaysShowPaginator = true;
        this.paginatorPosition = 'bottom';
        this.defaultSortOrder = 1;
        this.sortMode = 'single';
        this.resetPageOnSort = true;
        this.selectionChange = new EventEmitter();
        this.contextMenuSelectionChange = new EventEmitter();
        this.lazy = false;
        this.compareSelectionBy = 'deepEquals';
        this.csvSeparator = ',';
        this.exportFilename = 'download';
        this.filters = {};
        this.filterDelay = 300;
        this.expandedRowKeys = {};
        this.rowExpandMode = 'multiple';
        this.virtualScrollDelay = 500;
        this.virtualRowHeight = 27;
        this.columnResizeMode = 'fit';
        this.loadingIcon = 'fa fa-spin fa-2x fa-spinner';
        this.onRowSelect = new EventEmitter();
        this.onRowUnselect = new EventEmitter();
        this.onPage = new EventEmitter();
        this.onSort = new EventEmitter();
        this.onFilter = new EventEmitter();
        this.onLazyLoad = new EventEmitter();
        this.onRowExpand = new EventEmitter();
        this.onRowCollapse = new EventEmitter();
        this.onContextMenuSelect = new EventEmitter();
        this.onColResize = new EventEmitter();
        this.onColReorder = new EventEmitter();
        this.onRowReorder = new EventEmitter();
        this.onEditInit = new EventEmitter();
        this.onEditComplete = new EventEmitter();
        this.onEditCancel = new EventEmitter();
        this.onHeaderCheckboxToggle = new EventEmitter();
        this.sortFunction = new EventEmitter();
        this._value = [];
        this._totalRecords = 0;
        this.selectionKeys = {};
        this._sortOrder = 1;
        this.filterConstraints = {
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            startsWith(value, filter) {
                if (filter === undefined || filter === null || filter.trim() === '') {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                const /** @type {?} */ filterValue = filter.toLowerCase();
                return value.toString().toLowerCase().slice(0, filterValue.length) === filterValue;
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            contains(value, filter) {
                if (filter === undefined || filter === null || (typeof filter === 'string' && filter.trim() === '')) {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                return value.toString().toLowerCase().indexOf(filter.toLowerCase()) !== -1;
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            endsWith(value, filter) {
                if (filter === undefined || filter === null || filter.trim() === '') {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                const /** @type {?} */ filterValue = filter.toString().toLowerCase();
                return value.toString().toLowerCase().indexOf(filterValue, value.toString().length - filterValue.length) !== -1;
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            equals(value, filter) {
                if (filter === undefined || filter === null || (typeof filter === 'string' && filter.trim() === '')) {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                return value.toString().toLowerCase() === filter.toString().toLowerCase();
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            notEquals(value, filter) {
                if (filter === undefined || filter === null || (typeof filter === 'string' && filter.trim() === '')) {
                    return false;
                }
                if (value === undefined || value === null) {
                    return true;
                }
                return value.toString().toLowerCase() !== filter.toString().toLowerCase();
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            in(value, filter) {
                if (filter === undefined || filter === null || filter.length === 0) {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                for (let /** @type {?} */ i = 0; i < filter.length; i++) {
                    if (filter[i] === value) {
                        return true;
                    }
                }
                return false;
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            lt(value, filter) {
                if (filter === undefined || filter === null) {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                return value < filter;
            },
            /**
             * @param {?} value
             * @param {?} filter
             * @return {?}
             */
            gt(value, filter) {
                if (filter === undefined || filter === null) {
                    return true;
                }
                if (value === undefined || value === null) {
                    return false;
                }
                return value > filter;
            }
        };
        this.rowTrackBy = (index, item) => item;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.lazy) {
            this.onLazyLoad.emit(this.createLazyLoadMetadata());
        }
        this.initialized = true;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.templates.forEach((item) => {
            switch (item.getType()) {
                case 'caption':
                    this.captionTemplate = item.template;
                    break;
                case 'header':
                    this.headerTemplate = item.template;
                    break;
                case 'body':
                    this.bodyTemplate = item.template;
                    break;
                case 'footer':
                    this.footerTemplate = item.template;
                    break;
                case 'summary':
                    this.summaryTemplate = item.template;
                    break;
                case 'colgroup':
                    this.colGroupTemplate = item.template;
                    break;
                case 'rowexpansion':
                    this.expandedRowTemplate = item.template;
                    break;
                case 'frozenrows':
                    this.frozenRowsTemplate = item.template;
                    break;
                case 'frozenheader':
                    this.frozenHeaderTemplate = item.template;
                    break;
                case 'frozenbody':
                    this.frozenBodyTemplate = item.template;
                    break;
                case 'frozenfooter':
                    this.frozenFooterTemplate = item.template;
                    break;
                case 'frozencolgroup':
                    this.frozenColGroupTemplate = item.template;
                    break;
                case 'emptymessage':
                    this.emptyMessageTemplate = item.template;
                    break;
                case 'paginatorleft':
                    this.paginatorLeftTemplate = item.template;
                    break;
                case 'paginatorright':
                    this.paginatorRightTemplate = item.template;
                    break;
            }
        });
    }
    /**
     * @return {?}
     */
    get value() {
        return this._value;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set value(val) {
        this._value = val;
        if (!this.lazy) {
            this.totalRecords = (this._value ? this._value.length : 0);
            if (this.sortMode === 'single') {
                this.sortSingle();
            }
            else if (this.sortMode === 'multiple') {
                this.sortMultiple();
            }
        }
        if (this.virtualScroll && this.virtualScrollCallback) {
            this.virtualScrollCallback();
        }
        this.tableService.onValueChange(val);
    }
    /**
     * @return {?}
     */
    get totalRecords() {
        return this._totalRecords;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set totalRecords(val) {
        this._totalRecords = val;
        this.tableService.onTotalRecordsChange(this._totalRecords);
    }
    /**
     * @return {?}
     */
    get sortField() {
        return this._sortField;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set sortField(val) {
        this._sortField = val;
        if (!this.lazy || this.initialized) {
            if (this.sortMode === 'single') {
                this.sortSingle();
            }
        }
    }
    /**
     * @return {?}
     */
    get sortOrder() {
        return this._sortOrder;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set sortOrder(val) {
        this._sortOrder = val;
        if (!this.lazy || this.initialized) {
            if (this.sortMode === 'single') {
                this.sortSingle();
            }
        }
    }
    /**
     * @return {?}
     */
    get multiSortMeta() {
        return this._multiSortMeta;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set multiSortMeta(val) {
        this._multiSortMeta = val;
        if (this.sortMode === 'multiple') {
            this.sortMultiple();
        }
    }
    /**
     * @return {?}
     */
    get selection() {
        return this._selection;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set selection(val) {
        this._selection = val;
        if (!this.preventSelectionSetterPropagation) {
            this.updateSelectionKeys();
            this.tableService.onSelectionChange();
        }
        this.preventSelectionSetterPropagation = false;
    }
    /**
     * @return {?}
     */
    updateSelectionKeys() {
        if (this.dataKey && this._selection) {
            this.selectionKeys = {};
            if (Array.isArray(this._selection)) {
                for (const /** @type {?} */ data of this._selection) {
                    this.selectionKeys[String(this.objectUtils.resolveFieldData(data, this.dataKey))] = 1;
                }
            }
            else {
                this.selectionKeys[String(this.objectUtils.resolveFieldData(this._selection, this.dataKey))] = 1;
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onPageChange(event) {
        this.first = event.first;
        this.rows = event.rows;
        if (this.lazy) {
            this.onLazyLoad.emit(this.createLazyLoadMetadata());
        }
        this.onPage.emit({
            first: this.first,
            rows: this.rows
        });
        this.tableService.onValueChange(this.value);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    sort(event) {
        const /** @type {?} */ originalEvent = event.originalEvent;
        if (this.sortMode === 'single') {
            this._sortOrder = (this.sortField === event.field) ? this.sortOrder * -1 : this.defaultSortOrder;
            this._sortField = event.field;
            this.sortSingle();
        }
        if (this.sortMode === 'multiple') {
            const /** @type {?} */ metaKey = originalEvent.metaKey || originalEvent.ctrlKey;
            const /** @type {?} */ sortMeta = this.getSortMeta(event.field);
            if (sortMeta) {
                if (!metaKey) {
                    this._multiSortMeta = [{ field: event.field, order: sortMeta.order * -1 }];
                }
                else {
                    sortMeta.order = sortMeta.order * -1;
                }
            }
            else {
                if (!metaKey || !this.multiSortMeta) {
                    this._multiSortMeta = [];
                }
                this.multiSortMeta.push({ field: event.field, order: this.defaultSortOrder });
            }
            this.sortMultiple();
        }
    }
    /**
     * @return {?}
     */
    sortSingle() {
        if (this.sortField && this.sortOrder) {
            this.first = this.resetPageOnSort ? 0 : this.first;
            if (this.lazy) {
                this.onLazyLoad.emit(this.createLazyLoadMetadata());
            }
            else if (this.value) {
                if (this.customSort) {
                    this.sortFunction.emit({
                        data: this.value,
                        mode: this.sortMode,
                        field: this.sortField,
                        order: this.sortOrder
                    });
                }
                else {
                    this.value.sort((data1, data2) => {
                        const /** @type {?} */ value1 = this.objectUtils.resolveFieldData(data1, this.sortField);
                        const /** @type {?} */ value2 = this.objectUtils.resolveFieldData(data2, this.sortField);
                        let /** @type {?} */ result = null;
                        if (value1 == null && value2 != null) {
                            result = -1;
                        }
                        else if (value1 != null && value2 == null) {
                            result = 1;
                        }
                        else if (value1 == null && value2 == null) {
                            result = 0;
                        }
                        else if (typeof value1 === 'string' && typeof value2 === 'string') {
                            result = value1.localeCompare(value2);
                        }
                        else {
                            result = (value1 < value2) ? -1 : (value1 > value2) ? 1 : 0;
                        }
                        return (this.sortOrder * result);
                    });
                }
                if (this.hasFilter()) {
                    this._filter();
                }
            }
            const /** @type {?} */ sortMeta = {
                field: this.sortField,
                order: this.sortOrder
            };
            this.onSort.emit(sortMeta);
            this.tableService.onSort(sortMeta);
        }
    }
    /**
     * @return {?}
     */
    sortMultiple() {
        if (this.multiSortMeta) {
            if (this.lazy) {
                this.onLazyLoad.emit(this.createLazyLoadMetadata());
            }
            else if (this.value) {
                if (this.customSort) {
                    this.sortFunction.emit({
                        data: this.value,
                        mode: this.sortMode,
                        multiSortMeta: this.multiSortMeta
                    });
                }
                else {
                    this.value.sort((data1, data2) => {
                        return this.multisortField(data1, data2, this.multiSortMeta, 0);
                    });
                }
                if (this.hasFilter()) {
                    this._filter();
                }
            }
            this.onSort.emit({
                multisortmeta: this.multiSortMeta
            });
            this.tableService.onSort(this.multiSortMeta);
        }
    }
    /**
     * @param {?} data1
     * @param {?} data2
     * @param {?} multiSortMeta
     * @param {?} index
     * @return {?}
     */
    multisortField(data1, data2, multiSortMeta, index) {
        const /** @type {?} */ value1 = this.objectUtils.resolveFieldData(data1, multiSortMeta[index].field);
        const /** @type {?} */ value2 = this.objectUtils.resolveFieldData(data2, multiSortMeta[index].field);
        let /** @type {?} */ result = null;
        if (typeof value1 === 'string' || value1 instanceof String) {
            if (value1.localeCompare && (value1 !== value2)) {
                return (multiSortMeta[index].order * value1.localeCompare(value2));
            }
        }
        else {
            result = (value1 < value2) ? -1 : 1;
        }
        if (value1 === value2) {
            return (multiSortMeta.length - 1) > (index) ? (this.multisortField(data1, data2, multiSortMeta, index + 1)) : 0;
        }
        return (multiSortMeta[index].order * result);
    }
    /**
     * @param {?} field
     * @return {?}
     */
    getSortMeta(field) {
        if (this.multiSortMeta && this.multiSortMeta.length) {
            for (let /** @type {?} */ i = 0; i < this.multiSortMeta.length; i++) {
                if (this.multiSortMeta[i].field === field) {
                    return this.multiSortMeta[i];
                }
            }
        }
        return null;
    }
    /**
     * @param {?} field
     * @return {?}
     */
    isSorted(field) {
        if (this.sortMode === 'single') {
            return (this.sortField && this.sortField === field);
        }
        else if (this.sortMode === 'multiple') {
            let /** @type {?} */ sorted = false;
            if (this.multiSortMeta) {
                for (let /** @type {?} */ i = 0; i < this.multiSortMeta.length; i++) {
                    if (this.multiSortMeta[i].field === field) {
                        sorted = true;
                        break;
                    }
                }
            }
            return sorted;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleRowClick(event) {
        const /** @type {?} */ targetNode = (/** @type {?} */ (event.originalEvent.target)).nodeName;
        if (targetNode === 'INPUT' || targetNode === 'BUTTON' || targetNode === 'A' || (this.domHandler.hasClass(event.originalEvent.target, 'grf-ui-clickable'))) {
            return;
        }
        if (this.selectionMode) {
            this.preventSelectionSetterPropagation = true;
            if (this.isMultipleSelectionMode() && event.originalEvent.shiftKey && this.anchorRowIndex != null) {
                this.domHandler.clearSelection();
                if (this.rangeRowIndex != null) {
                    this.clearSelectionRange(event.originalEvent);
                }
                this.rangeRowIndex = event.rowIndex;
                this.selectRange(event.originalEvent, event.rowIndex);
            }
            else {
                const /** @type {?} */ rowData = event.rowData;
                const /** @type {?} */ selected = this.isSelected(rowData);
                const /** @type {?} */ metaSelection = this.rowTouched ? false : this.metaKeySelection;
                const /** @type {?} */ dataKeyValue = this.dataKey ? String(this.objectUtils.resolveFieldData(rowData, this.dataKey)) : null;
                this.anchorRowIndex = event.rowIndex;
                this.rangeRowIndex = event.rowIndex;
                if (metaSelection) {
                    const /** @type {?} */ metaKey = event.originalEvent.metaKey || event.originalEvent.ctrlKey;
                    if (selected && metaKey) {
                        if (this.isSingleSelectionMode()) {
                            this._selection = null;
                            this.selectionKeys = {};
                            this.selectionChange.emit(null);
                        }
                        else {
                            const /** @type {?} */ selectionIndex = this.findIndexInSelection(rowData);
                            this._selection = this.selection.filter((val, i) => i !== selectionIndex);
                            this.selectionChange.emit(this.selection);
                            if (dataKeyValue) {
                                delete this.selectionKeys[dataKeyValue];
                            }
                        }
                        this.onRowUnselect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row' });
                    }
                    else {
                        if (this.isSingleSelectionMode()) {
                            this._selection = rowData;
                            this.selectionChange.emit(rowData);
                            if (dataKeyValue) {
                                this.selectionKeys = {};
                                this.selectionKeys[dataKeyValue] = 1;
                            }
                        }
                        else if (this.isMultipleSelectionMode()) {
                            if (metaKey) {
                                this._selection = this.selection || [];
                            }
                            else {
                                this._selection = [];
                                this.selectionKeys = {};
                            }
                            this._selection = [...this.selection, rowData];
                            this.selectionChange.emit(this.selection);
                            if (dataKeyValue) {
                                this.selectionKeys[dataKeyValue] = 1;
                            }
                        }
                        this.onRowSelect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row', index: event.rowIndex });
                    }
                }
                else {
                    if (this.selectionMode === 'single') {
                        if (selected) {
                            this._selection = null;
                            this.selectionKeys = {};
                            this.selectionChange.emit(this.selection);
                            this.onRowUnselect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row' });
                        }
                        else {
                            this._selection = rowData;
                            this.selectionChange.emit(this.selection);
                            this.onRowSelect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row', index: event.rowIndex });
                            if (dataKeyValue) {
                                this.selectionKeys = {};
                                this.selectionKeys[dataKeyValue] = 1;
                            }
                        }
                    }
                    else if (this.selectionMode === 'multiple') {
                        if (selected) {
                            const /** @type {?} */ selectionIndex = this.findIndexInSelection(rowData);
                            this._selection = this.selection.filter((val, i) => i !== selectionIndex);
                            this.selectionChange.emit(this.selection);
                            this.onRowUnselect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row' });
                            if (dataKeyValue) {
                                delete this.selectionKeys[dataKeyValue];
                            }
                        }
                        else {
                            this._selection = this.selection ? [...this.selection, rowData] : [rowData];
                            this.selectionChange.emit(this.selection);
                            this.onRowSelect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'row', index: event.rowIndex });
                            if (dataKeyValue) {
                                this.selectionKeys[dataKeyValue] = 1;
                            }
                        }
                    }
                }
            }
            this.tableService.onSelectionChange();
        }
        this.rowTouched = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleRowTouchEnd(event) {
        this.rowTouched = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleRowRightClick(event) {
        if (this.contextMenu) {
            this.contextMenuSelection = event.rowData;
            this.contextMenuSelectionChange.emit(event.rowData);
            this.onContextMenuSelect.emit({ originalEvent: event.originalEvent, data: event.rowData });
            this.contextMenu.show(event.originalEvent);
            this.tableService.onContextMenu(event.rowData);
        }
    }
    /**
     * @param {?} event
     * @param {?} rowIndex
     * @return {?}
     */
    selectRange(event, rowIndex) {
        let /** @type {?} */ rangeStart, /** @type {?} */ rangeEnd;
        if (this.anchorRowIndex > rowIndex) {
            rangeStart = rowIndex;
            rangeEnd = this.anchorRowIndex;
        }
        else if (this.anchorRowIndex < rowIndex) {
            rangeStart = this.anchorRowIndex;
            rangeEnd = rowIndex;
        }
        else {
            rangeStart = rowIndex;
            rangeEnd = rowIndex;
        }
        for (let /** @type {?} */ i = rangeStart; i <= rangeEnd; i++) {
            const /** @type {?} */ rangeRowData = this.value[i];
            if (!this.isSelected(rangeRowData)) {
                this._selection = [...this.selection, rangeRowData];
                const /** @type {?} */ dataKeyValue = this.dataKey ? String(this.objectUtils.resolveFieldData(rangeRowData, this.dataKey)) : null;
                if (dataKeyValue) {
                    this.selectionKeys[dataKeyValue] = 1;
                }
                this.onRowSelect.emit({ originalEvent: event, data: rangeRowData, type: 'row' });
            }
        }
        this.selectionChange.emit(this.selection);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    clearSelectionRange(event) {
        let /** @type {?} */ rangeStart, /** @type {?} */ rangeEnd;
        if (this.rangeRowIndex > this.anchorRowIndex) {
            rangeStart = this.anchorRowIndex;
            rangeEnd = this.rangeRowIndex;
        }
        else if (this.rangeRowIndex < this.anchorRowIndex) {
            rangeStart = this.rangeRowIndex;
            rangeEnd = this.anchorRowIndex;
        }
        else {
            rangeStart = this.rangeRowIndex;
            rangeEnd = this.rangeRowIndex;
        }
        for (let /** @type {?} */ i = rangeStart; i <= rangeEnd; i++) {
            const /** @type {?} */ rangeRowData = this.value[i];
            const /** @type {?} */ selectionIndex = this.findIndexInSelection(rangeRowData);
            this._selection = this.selection.filter((val, index = i) => index !== selectionIndex);
            const /** @type {?} */ dataKeyValue = this.dataKey ? String(this.objectUtils.resolveFieldData(rangeRowData, this.dataKey)) : null;
            if (dataKeyValue) {
                delete this.selectionKeys[dataKeyValue];
            }
            this.onRowUnselect.emit({ originalEvent: event, data: rangeRowData, type: 'row' });
        }
    }
    /**
     * @param {?} rowData
     * @return {?}
     */
    isSelected(rowData) {
        if (rowData && this.selection) {
            if (this.dataKey) {
                return this.selectionKeys[this.objectUtils.resolveFieldData(rowData, this.dataKey)] !== undefined;
            }
            else {
                if (this.selection instanceof Array) {
                    return this.findIndexInSelection(rowData) > -1;
                }
                else {
                    return this.equals(rowData, this.selection);
                }
            }
        }
        return false;
    }
    /**
     * @param {?} rowData
     * @return {?}
     */
    findIndexInSelection(rowData) {
        let /** @type {?} */ index = -1;
        if (this.selection && this.selection.length) {
            for (let /** @type {?} */ i = 0; i < this.selection.length; i++) {
                if (this.equals(rowData, this.selection[i])) {
                    index = i;
                    break;
                }
            }
        }
        return index;
    }
    /**
     * @param {?} event
     * @param {?} rowData
     * @return {?}
     */
    toggleRowWithRadio(event, rowData) {
        this.preventSelectionSetterPropagation = true;
        if (this.selection !== rowData) {
            this._selection = rowData;
            this.selectionChange.emit(this.selection);
            this.onRowSelect.emit({ originalEvent: event, data: rowData, type: 'radiobutton' });
            if (this.dataKey) {
                this.selectionKeys = {};
                this.selectionKeys[String(this.objectUtils.resolveFieldData(rowData, this.dataKey))] = 1;
            }
        }
        else {
            this._selection = null;
            this.selectionChange.emit(this.selection);
            this.onRowUnselect.emit({ originalEvent: event, data: rowData, type: 'radiobutton' });
        }
        this.tableService.onSelectionChange();
    }
    /**
     * @param {?} event
     * @param {?} rowData
     * @return {?}
     */
    toggleRowWithCheckbox(event, rowData) {
        this.selection = this.selection || [];
        const /** @type {?} */ selected = this.isSelected(rowData);
        const /** @type {?} */ dataKeyValue = this.dataKey ? String(this.objectUtils.resolveFieldData(rowData, this.dataKey)) : null;
        this.preventSelectionSetterPropagation = true;
        if (selected) {
            const /** @type {?} */ selectionIndex = this.findIndexInSelection(rowData);
            this._selection = this.selection.filter((val, i) => i !== selectionIndex);
            this.selectionChange.emit(this.selection);
            this.onRowUnselect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'checkbox' });
            if (dataKeyValue) {
                delete this.selectionKeys[dataKeyValue];
            }
        }
        else {
            this._selection = this.selection ? [...this.selection, rowData] : [rowData];
            this.selectionChange.emit(this.selection);
            this.onRowSelect.emit({ originalEvent: event.originalEvent, data: rowData, type: 'checkbox' });
            if (dataKeyValue) {
                this.selectionKeys[dataKeyValue] = 1;
            }
        }
        this.tableService.onSelectionChange();
    }
    /**
     * @param {?} event
     * @param {?} check
     * @return {?}
     */
    toggleRowsWithCheckbox(event, check) {
        this._selection = check ? this.filteredValue ? this.filteredValue.slice() : this.value.slice() : [];
        this.preventSelectionSetterPropagation = true;
        this.updateSelectionKeys();
        this.selectionChange.emit(this._selection);
        this.tableService.onSelectionChange();
        this.onHeaderCheckboxToggle.emit({ originalEvent: event, checked: check });
    }
    /**
     * @param {?} data1
     * @param {?} data2
     * @return {?}
     */
    equals(data1, data2) {
        return this.compareSelectionBy === 'equals' ? (data1 === data2) : this.objectUtils.equals(data1, data2, this.dataKey);
    }
    /**
     * @param {?} value
     * @param {?} field
     * @param {?} matchMode
     * @return {?}
     */
    filter(value, field, matchMode) {
        if (this.filterTimeout) {
            clearTimeout(this.filterTimeout);
        }
        this.filterTimeout = setTimeout(() => {
            if (!this.isFilterBlank(value)) {
                this.filters[field] = { value: value, matchMode: matchMode };
            }
            else if (this.filters[field]) {
                delete this.filters[field];
            }
            this._filter();
            this.filterTimeout = null;
        }, this.filterDelay);
    }
    /**
     * @param {?} value
     * @param {?} matchMode
     * @return {?}
     */
    filterGlobal(value, matchMode) {
        this.filter(value, 'global', matchMode);
    }
    /**
     * @param {?} filter
     * @return {?}
     */
    isFilterBlank(filter) {
        if (filter !== null && filter !== undefined) {
            if ((typeof filter === 'string' && filter.trim().length === 0) || (filter instanceof Array && filter.length === 0)) {
                return true;
            }
            else {
                return false;
            }
        }
        return true;
    }
    /**
     * @return {?}
     */
    _filter() {
        this.first = 0;
        if (this.lazy) {
            this.onLazyLoad.emit(this.createLazyLoadMetadata());
        }
        else {
            if (!this.value) {
                return;
            }
            if (!this.hasFilter()) {
                this.filteredValue = null;
                if (this.paginator) {
                    this.totalRecords = this.value ? this.value.length : 0;
                }
            }
            else {
                let /** @type {?} */ globalFilterFieldsArray;
                if (this.filters['global']) {
                    if (!this.columns && !this.globalFilterFields) {
                        throw new Error('Global filtering requires dynamic columns or globalFilterFields to be defined.');
                    }
                    else {
                        globalFilterFieldsArray = this.globalFilterFields || this.columns;
                    }
                }
                this.filteredValue = [];
                for (let /** @type {?} */ i = 0; i < this.value.length; i++) {
                    let /** @type {?} */ localMatch = true;
                    let /** @type {?} */ globalMatch = false;
                    let /** @type {?} */ localFiltered = false;
                    for (const /** @type {?} */ prop in this.filters) {
                        if (this.filters.hasOwnProperty(prop) && prop !== 'global') {
                            localFiltered = true;
                            const /** @type {?} */ filterMeta = this.filters[prop];
                            const /** @type {?} */ filterField = prop;
                            const /** @type {?} */ filterValue = filterMeta.value;
                            const /** @type {?} */ filterMatchMode = filterMeta.matchMode || 'startsWith';
                            const /** @type {?} */ dataFieldValue = this.objectUtils.resolveFieldData(this.value[i], filterField);
                            const /** @type {?} */ filterConstraint = this.filterConstraints[filterMatchMode];
                            if (!filterConstraint(dataFieldValue, filterValue)) {
                                localMatch = false;
                            }
                            if (!localMatch) {
                                break;
                            }
                        }
                    }
                    if (this.filters['global'] && !globalMatch && globalFilterFieldsArray) {
                        for (let /** @type {?} */ j = 0; j < globalFilterFieldsArray.length; j++) {
                            const /** @type {?} */ globalFilterField = globalFilterFieldsArray[j].field || globalFilterFieldsArray[j];
                            globalMatch = this.filterConstraints[this.filters['global'].matchMode](this.objectUtils.resolveFieldData(this.value[i], globalFilterField), this.filters['global'].value);
                            if (globalMatch) {
                                break;
                            }
                        }
                    }
                    let /** @type {?} */ matches;
                    if (this.filters['global']) {
                        matches = localFiltered ? (localFiltered && localMatch && globalMatch) : globalMatch;
                    }
                    else {
                        matches = localFiltered && localMatch;
                    }
                    if (matches) {
                        this.filteredValue.push(this.value[i]);
                    }
                }
                if (this.filteredValue.length === this.value.length) {
                    this.filteredValue = null;
                }
                if (this.paginator) {
                    this.totalRecords = this.filteredValue ? this.filteredValue.length : this.value ? this.value.length : 0;
                }
            }
        }
        this.onFilter.emit({
            filters: this.filters,
            filteredValue: this.filteredValue || this.value
        });
        this.tableService.onValueChange(this.value);
    }
    /**
     * @return {?}
     */
    hasFilter() {
        let /** @type {?} */ empty = true;
        for (const /** @type {?} */ prop in this.filters) {
            if (this.filters.hasOwnProperty(prop)) {
                empty = false;
                break;
            }
        }
        return !empty;
    }
    /**
     * @return {?}
     */
    createLazyLoadMetadata() {
        return {
            first: this.first,
            rows: this.virtualScroll ? this.rows * 2 : this.rows,
            sortField: this.sortField,
            sortOrder: this.sortOrder,
            filters: this.filters,
            globalFilter: this.filters && this.filters['global'] ? this.filters['global'].value : null,
            multiSortMeta: this.multiSortMeta
        };
    }
    /**
     * @return {?}
     */
    reset() {
        this._sortField = null;
        this._sortOrder = 1;
        this._multiSortMeta = null;
        this.filteredValue = null;
        this.filters = {};
        this.first = 0;
        if (this.lazy) {
            this.onLazyLoad.emit(this.createLazyLoadMetadata());
        }
        else {
            this.totalRecords = (this._value ? this._value.length : 0);
        }
    }
    /**
     * @param {?=} options
     * @return {?}
     */
    exportCSV(options) {
        let /** @type {?} */ data = this.filteredValue || this.value;
        let /** @type {?} */ csv = '\ufeff';
        if (options && options.selectionOnly) {
            data = this.selection || [];
        }
        for (let /** @type {?} */ i = 0; i < this.columns.length; i++) {
            const /** @type {?} */ column = this.columns[i];
            if (column.exportable !== false && column.field) {
                csv += '"' + (column.header || column.field) + '"';
                if (i < (this.columns.length - 1)) {
                    csv += this.csvSeparator;
                }
            }
        }
        data.forEach((record, i) => {
            csv += '\n';
            for (i = 0; i < this.columns.length; i++) {
                const /** @type {?} */ column = this.columns[i];
                if (column.exportable !== false && column.field) {
                    let /** @type {?} */ cellData = this.objectUtils.resolveFieldData(record, column.field);
                    if (cellData != null) {
                        if (this.exportFunction) {
                            cellData = this.exportFunction({
                                data: cellData,
                                field: column.field
                            });
                        }
                        else {
                            cellData = String(cellData).replace(/"/g, '""');
                        }
                    }
                    else {
                        cellData = '';
                    }
                    csv += '"' + cellData + '"';
                    if (i < (this.columns.length - 1)) {
                        csv += this.csvSeparator;
                    }
                }
            }
        });
        const /** @type {?} */ blob = new Blob([csv], {
            type: 'text/csv;charset=utf-8;'
        });
        if (window.navigator.msSaveOrOpenBlob) {
            navigator.msSaveOrOpenBlob(blob, this.exportFilename + '.csv');
        }
        else {
            const /** @type {?} */ link = document.createElement('a');
            link.style.display = 'none';
            document.body.appendChild(link);
            if (link.download !== undefined) {
                link.setAttribute('href', URL.createObjectURL(blob));
                link.setAttribute('download', this.exportFilename + '.csv');
                link.click();
            }
            else {
                csv = 'data:text/csv;charset=utf-8,' + csv;
                window.open(encodeURI(csv));
            }
            document.body.removeChild(link);
        }
    }
    /**
     * @param {?} rowData
     * @param {?=} event
     * @return {?}
     */
    toggleRow(rowData, event) {
        if (!this.dataKey) {
            throw new Error('dataKey must be defined to use row expansion');
        }
        const /** @type {?} */ dataKeyValue = String(this.objectUtils.resolveFieldData(rowData, this.dataKey));
        if (this.expandedRowKeys[dataKeyValue] != null) {
            delete this.expandedRowKeys[dataKeyValue];
            this.onRowCollapse.emit({
                originalEvent: event,
                data: rowData
            });
        }
        else {
            if (this.rowExpandMode === 'single') {
                this.expandedRowKeys = {};
            }
            this.expandedRowKeys[dataKeyValue] = 1;
            this.onRowExpand.emit({
                originalEvent: event,
                data: rowData
            });
        }
        if (event) {
            event.preventDefault();
        }
    }
    /**
     * @param {?} rowData
     * @return {?}
     */
    isRowExpanded(rowData) {
        return this.expandedRowKeys[String(this.objectUtils.resolveFieldData(rowData, this.dataKey))] === 1;
    }
    /**
     * @return {?}
     */
    isSingleSelectionMode() {
        return this.selectionMode === 'single';
    }
    /**
     * @return {?}
     */
    isMultipleSelectionMode() {
        return this.selectionMode === 'multiple';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onColumnResizeBegin(event) {
        const /** @type {?} */ containerLeft = this.domHandler.getOffset(this.containerViewChild.nativeElement).left;
        this.lastResizerHelperX = (event.pageX - containerLeft + this.containerViewChild.nativeElement.scrollLeft);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onColumnResize(event) {
        const /** @type {?} */ containerLeft = this.domHandler.getOffset(this.containerViewChild.nativeElement).left;
        this.domHandler.addClass(this.containerViewChild.nativeElement, 'grf-ui-unselectable-text');
        this.resizeHelperViewChild.nativeElement.style.height = this.containerViewChild.nativeElement.offsetHeight + 'px';
        this.resizeHelperViewChild.nativeElement.style.top = 0 + 'px';
        this.resizeHelperViewChild.nativeElement.style.left = (event.pageX - containerLeft + this.containerViewChild.nativeElement.scrollLeft) + 'px';
        this.resizeHelperViewChild.nativeElement.style.display = 'block';
    }
    /**
     * @param {?} event
     * @param {?} column
     * @return {?}
     */
    onColumnResizeEnd(event, column) {
        const /** @type {?} */ delta = this.resizeHelperViewChild.nativeElement.offsetLeft - this.lastResizerHelperX;
        const /** @type {?} */ columnWidth = column.offsetWidth;
        const /** @type {?} */ newColumnWidth = columnWidth + delta;
        const /** @type {?} */ minWidth = column.style.minWidth || 15;
        if (columnWidth + delta > parseInt(minWidth, 0)) {
            if (this.columnResizeMode === 'fit') {
                let /** @type {?} */ nextColumn = column.nextElementSibling;
                while (!nextColumn.offsetParent) {
                    nextColumn = nextColumn.nextElementSibling;
                }
                if (nextColumn) {
                    const /** @type {?} */ nextColumnWidth = nextColumn.offsetWidth - delta;
                    const /** @type {?} */ nextColumnMinWidth = nextColumn.style.minWidth || 15;
                    if (newColumnWidth > 15 && nextColumnWidth > parseInt(nextColumnMinWidth, 0)) {
                        if (this.scrollable) {
                            const /** @type {?} */ scrollableBodyTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-body-table');
                            const /** @type {?} */ scrollableHeaderTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-header-table');
                            const /** @type {?} */ scrollableFooterTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-footer-table');
                            const /** @type {?} */ resizeColumnIndex = this.domHandler.index(column);
                            this.resizeColGroup(scrollableHeaderTable, resizeColumnIndex, newColumnWidth, nextColumnWidth);
                            this.resizeColGroup(scrollableBodyTable, resizeColumnIndex, newColumnWidth, nextColumnWidth);
                            this.resizeColGroup(scrollableFooterTable, resizeColumnIndex, newColumnWidth, nextColumnWidth);
                        }
                        else {
                            column.style.width = newColumnWidth + 'px';
                            if (nextColumn) {
                                nextColumn.style.width = nextColumnWidth + 'px';
                            }
                        }
                    }
                }
            }
            else if (this.columnResizeMode === 'expand') {
                if (this.scrollable) {
                    const /** @type {?} */ scrollableBodyTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-body-table');
                    const /** @type {?} */ scrollableHeaderTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-header-table');
                    const /** @type {?} */ scrollableFooterTable = this.domHandler.findSingle(this.el.nativeElement, 'table.grf-ui-table-scrollable-footer-table');
                    scrollableBodyTable.style.width = scrollableBodyTable.offsetWidth + delta + 'px';
                    scrollableHeaderTable.style.width = scrollableHeaderTable.offsetWidth + delta + 'px';
                    if (scrollableFooterTable) {
                        scrollableFooterTable.style.width = scrollableHeaderTable.offsetWidth + delta + 'px';
                    }
                    const /** @type {?} */ resizeColumnIndex = this.domHandler.index(column);
                    this.resizeColGroup(scrollableHeaderTable, resizeColumnIndex, newColumnWidth, null);
                    this.resizeColGroup(scrollableBodyTable, resizeColumnIndex, newColumnWidth, null);
                    this.resizeColGroup(scrollableFooterTable, resizeColumnIndex, newColumnWidth, null);
                }
                else {
                    this.tableViewChild.nativeElement.style.width = this.tableViewChild.nativeElement.offsetWidth + delta + 'px';
                    column.style.width = newColumnWidth + 'px';
                    const /** @type {?} */ containerWidth = this.tableViewChild.nativeElement.style.width;
                    this.containerViewChild.nativeElement.style.width = containerWidth + 'px';
                }
            }
            this.onColResize.emit({
                element: column,
                delta: delta
            });
        }
        this.resizeHelperViewChild.nativeElement.style.display = 'none';
        this.domHandler.removeClass(this.containerViewChild.nativeElement, 'grf-ui-unselectable-text');
    }
    /**
     * @param {?} table
     * @param {?} resizeColumnIndex
     * @param {?} newColumnWidth
     * @param {?} nextColumnWidth
     * @return {?}
     */
    resizeColGroup(table, resizeColumnIndex, newColumnWidth, nextColumnWidth) {
        if (table) {
            const /** @type {?} */ colGroup = table.children[0].nodeName === 'COLGROUP' ? table.children[0] : null;
            if (colGroup) {
                const /** @type {?} */ col = colGroup.children[resizeColumnIndex];
                const /** @type {?} */ nextCol = col.nextElementSibling;
                col.style.width = newColumnWidth + 'px';
                if (nextCol && nextColumnWidth) {
                    nextCol.style.width = nextColumnWidth + 'px';
                }
            }
            else {
                return;
            }
        }
    }
    /**
     * @param {?} event
     * @param {?} columnElement
     * @return {?}
     */
    onColumnDragStart(event, columnElement) {
        this.reorderIconWidth = this.domHandler.getHiddenElementOuterWidth(this.reorderIndicatorUpViewChild.nativeElement);
        this.reorderIconHeight = this.domHandler.getHiddenElementOuterHeight(this.reorderIndicatorDownViewChild.nativeElement);
        this.draggedColumn = columnElement;
        event.dataTransfer.setData('text', 'b'); // For firefox
    }
    /**
     * @param {?} event
     * @param {?} dropHeader
     * @return {?}
     */
    onColumnDragEnter(event, dropHeader) {
        if (this.reorderableColumns && this.draggedColumn && dropHeader) {
            event.preventDefault();
            const /** @type {?} */ containerOffset = this.domHandler.getOffset(this.containerViewChild.nativeElement);
            const /** @type {?} */ dropHeaderOffset = this.domHandler.getOffset(dropHeader);
            if (this.draggedColumn !== dropHeader) {
                const /** @type {?} */ targetLeft = dropHeaderOffset.left - containerOffset.left;
                const /** @type {?} */ targetTop = containerOffset.top - dropHeaderOffset.top;
                const /** @type {?} */ columnCenter = dropHeaderOffset.left + dropHeader.offsetWidth / 2;
                this.reorderIndicatorUpViewChild.nativeElement.style.top = dropHeaderOffset.top - containerOffset.top - (this.reorderIconHeight - 1) + 'px';
                this.reorderIndicatorDownViewChild.nativeElement.style.top = dropHeaderOffset.top - containerOffset.top + dropHeader.offsetHeight + 'px';
                if (event.pageX > columnCenter) {
                    this.reorderIndicatorUpViewChild.nativeElement.style.left = (targetLeft + dropHeader.offsetWidth - Math.ceil(this.reorderIconWidth / 2)) + 'px';
                    this.reorderIndicatorDownViewChild.nativeElement.style.left = (targetLeft + dropHeader.offsetWidth - Math.ceil(this.reorderIconWidth / 2)) + 'px';
                    this.dropPosition = 1;
                }
                else {
                    this.reorderIndicatorUpViewChild.nativeElement.style.left = (targetLeft - Math.ceil(this.reorderIconWidth / 2)) + 'px';
                    this.reorderIndicatorDownViewChild.nativeElement.style.left = (targetLeft - Math.ceil(this.reorderIconWidth / 2)) + 'px';
                    this.dropPosition = -1;
                }
                this.reorderIndicatorUpViewChild.nativeElement.style.display = 'block';
                this.reorderIndicatorDownViewChild.nativeElement.style.display = 'block';
            }
            else {
                event.dataTransfer.dropEffect = 'none';
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onColumnDragLeave(event) {
        if (this.reorderableColumns && this.draggedColumn) {
            event.preventDefault();
            this.reorderIndicatorUpViewChild.nativeElement.style.display = 'none';
            this.reorderIndicatorDownViewChild.nativeElement.style.display = 'none';
        }
    }
    /**
     * @param {?} event
     * @param {?} dropColumn
     * @return {?}
     */
    onColumnDrop(event, dropColumn) {
        event.preventDefault();
        if (this.draggedColumn) {
            const /** @type {?} */ dragIndex = this.domHandler.indexWithinGroup(this.draggedColumn, 'grfReorderableColumn');
            const /** @type {?} */ dropIndex = this.domHandler.indexWithinGroup(dropColumn, 'grfReorderableColumn');
            let /** @type {?} */ allowDrop = (dragIndex !== dropIndex);
            if (allowDrop && ((dropIndex - dragIndex === 1 && this.dropPosition === -1) || (dragIndex - dropIndex === 1 && this.dropPosition === 1))) {
                allowDrop = false;
            }
            if (allowDrop) {
                this.objectUtils.reorderArray(this.columns, dragIndex, dropIndex);
                this.onColReorder.emit({
                    dragIndex: dragIndex,
                    dropIndex: dropIndex,
                    columns: this.columns
                });
            }
            this.reorderIndicatorUpViewChild.nativeElement.style.display = 'none';
            this.reorderIndicatorDownViewChild.nativeElement.style.display = 'none';
            this.draggedColumn.draggable = false;
            this.draggedColumn = null;
            this.dropPosition = null;
        }
    }
    /**
     * @param {?} event
     * @param {?} index
     * @return {?}
     */
    onRowDragStart(event, index) {
        this.rowDragging = true;
        this.draggedRowIndex = index;
        event.dataTransfer.setData('text', 'b'); // For firefox
    }
    /**
     * @param {?} event
     * @param {?} index
     * @param {?} rowElement
     * @return {?}
     */
    onRowDragOver(event, index, rowElement) {
        if (this.rowDragging && this.draggedRowIndex !== index) {
            const /** @type {?} */ rowY = this.domHandler.getOffset(rowElement).top + this.domHandler.getWindowScrollTop();
            const /** @type {?} */ pageY = event.pageY;
            const /** @type {?} */ rowMidY = rowY + this.domHandler.getOuterHeight(rowElement) / 2;
            const /** @type {?} */ prevRowElement = rowElement.previousElementSibling;
            if (pageY < rowMidY) {
                this.domHandler.removeClass(rowElement, 'grf-ui-table-dragpoint-bottom');
                this.droppedRowIndex = index;
                if (prevRowElement) {
                    this.domHandler.addClass(prevRowElement, 'grf-ui-table-dragpoint-bottom');
                }
                else {
                    this.domHandler.addClass(rowElement, 'grf-ui-table-dragpoint-top');
                }
            }
            else {
                if (prevRowElement) {
                    this.domHandler.removeClass(prevRowElement, 'grf-ui-table-dragpoint-bottom');
                }
                else {
                    this.domHandler.addClass(rowElement, 'grf-ui-table-dragpoint-top');
                }
                this.droppedRowIndex = index + 1;
                this.domHandler.addClass(rowElement, 'grf-ui-table-dragpoint-bottom');
            }
        }
    }
    /**
     * @param {?} event
     * @param {?} rowElement
     * @return {?}
     */
    onRowDragLeave(event, rowElement) {
        const /** @type {?} */ prevRowElement = rowElement.previousElementSibling;
        if (prevRowElement) {
            this.domHandler.removeClass(prevRowElement, 'grf-ui-table-dragpoint-bottom');
        }
        this.domHandler.removeClass(rowElement, 'grf-ui-table-dragpoint-bottom');
        this.domHandler.removeClass(rowElement, 'grf-ui-table-dragpoint-top');
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onRowDragEnd(event) {
        this.rowDragging = false;
        this.draggedRowIndex = null;
        this.droppedRowIndex = null;
    }
    /**
     * @param {?} event
     * @param {?} rowElement
     * @return {?}
     */
    onRowDrop(event, rowElement) {
        if (this.droppedRowIndex != null) {
            const /** @type {?} */ dropIndex = (this.draggedRowIndex > this.droppedRowIndex) ? this.droppedRowIndex : (this.droppedRowIndex === 0) ? 0 : this.droppedRowIndex - 1;
            this.objectUtils.reorderArray(this.value, this.draggedRowIndex, dropIndex);
            this.onRowReorder.emit({
                dragIndex: this.draggedRowIndex,
                dropIndex: this.droppedRowIndex
            });
        }
        // cleanup
        this.onRowDragLeave(event, rowElement);
        this.onRowDragEnd(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleVirtualScroll(event) {
        this.first = (event.page - 1) * this.rows;
        this.virtualScrollCallback = event.callback;
        this.zone.run(() => {
            if (this.virtualScrollTimer) {
                clearTimeout(this.virtualScrollTimer);
            }
            this.virtualScrollTimer = setTimeout(() => {
                this.onLazyLoad.emit(this.createLazyLoadMetadata());
            }, this.virtualScrollDelay);
        });
    }
    /**
     * @return {?}
     */
    isEmpty() {
        const /** @type {?} */ data = this.filteredValue || this.value;
        return data == null || data.length === 0;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.editingCell = null;
        this.initialized = null;
    }
}
TableComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-table',
                template: `<div #container [ngStyle]="style" [class]="styleClass" class="grf-ui-table grf-ui-widget" [ngClass]="{'grf-ui-table-responsive': responsive, 'grf-ui-table-resizable': resizableColumns,
                'grf-ui-table-resizable-fit': (resizableColumns && columnResizeMode === 'fit'),
                'grf-ui-table-hoverable-rows': (rowHover||selectionMode), 'grf-ui-table-auto-layout': autoLayout}">
    <div class="grf-ui-table-loading grf-ui-widget-overlay" *ngIf="loading"></div>
    <div class="grf-ui-table-loading-content" *ngIf="loading">
        <i [class]="'fa fa-spin fa-2x ' + loadingIcon"></i>
    </div>
    <div *ngIf="captionTemplate" class="grf-ui-table-caption grf-ui-widget-header">
        <ng-container *ngTemplateOutlet="captionTemplate"></ng-container>
    </div>
    <grf-paginator [rows]="rows" [first]="first" [totalRecords]="totalRecords" [pageLinkSize]="pageLinks" styleClass="grf-ui-paginator-top" [alwaysShow]="alwaysShowPaginator" (onPageChange)="onPageChange($event)" [rowsPerPageOptions]="rowsPerPageOptions" *ngIf="paginator && (paginatorPosition === 'top' || paginatorPosition =='both')"
        [templateLeft]="paginatorLeftTemplate" [templateRight]="paginatorRightTemplate" [dropdownAppendTo]="paginatorDropdownAppendTo"></grf-paginator>
        <div class="grf-ui-table-wrapper" *ngIf="!scrollable">
        <table #table>
            <ng-container *ngTemplateOutlet="colGroupTemplate; context {$implicit: columns}"></ng-container>
            <thead class="grf-ui-table-thead">
                <ng-container *ngTemplateOutlet="headerTemplate; context: {$implicit: columns}"></ng-container>
            </thead>
            <tfoot class="grf-ui-table-tfoot">
                <div class="grf-ui-table-foot">
                    <ng-container *ngTemplateOutlet="footerTemplate; context {$implicit: columns}"></ng-container>
                </div>
            </tfoot>
            <tbody class="grf-ui-table-tbody" grfTableBody [columns]="columns" [template]="bodyTemplate">
            </tbody>
        </table>
    </div>

    <div class="grf-ui-table-scrollable-wrapper" *ngIf="scrollable">
        <div class="grf-ui-table-frozen-view" *ngIf="frozenColumns||frozenBodyTemplate" grfScrollableView [columns]="frozenColumns" [frozen]="true" [ngStyle]="{width: frozenWidth}" [scrollHeight]="scrollHeight"></div>
        <div grfScrollableView [columns]="columns" [frozen]="false" [scrollHeight]="scrollHeight"></div>
    </div>
    <grf-paginator [rows]="rows" [first]="first" [totalRecords]="totalRecords" [pageLinkSize]="pageLinks" styleClass="grf-ui-paginator-bottom" [alwaysShow]="alwaysShowPaginator" (onPageChange)="onPageChange($event)" [rowsPerPageOptions]="rowsPerPageOptions" *ngIf="paginator && (paginatorPosition === 'bottom' || paginatorPosition =='both')"
        [templateLeft]="paginatorLeftTemplate" [templateRight]="paginatorRightTemplate" [dropdownAppendTo]="paginatorDropdownAppendTo" [displayDirection]="pagiDisplayDirection"></grf-paginator>
    <div *ngIf="summaryTemplate" class="grf-ui-table-summary grf-ui-widget-header">
        <ng-container *ngTemplateOutlet="summaryTemplate"></ng-container>
    </div>

    <div #resizeHelper class="grf-ui-column-resizer-helper grf-ui-state-highlight" style="display:none" *ngIf="resizableColumns"></div>

    <span #reorderIndicatorUp class="fa fa-arrow-down grf-ui-table-reorder-indicator-up" *ngIf="reorderableColumns"></span>
    <span #reorderIndicatorDown class="fa fa-arrow-up grf-ui-table-reorder-indicator-down" *ngIf="reorderableColumns"></span>
</div>
`,
                providers: [DomHandler, ObjectUtils, TableService],
            },] },
];
/** @nocollapse */
TableComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: ObjectUtils, },
    { type: NgZone, },
    { type: TableService, },
];
TableComponent.propDecorators = {
    "columns": [{ type: Input },],
    "frozenColumns": [{ type: Input },],
    "frozenValue": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "paginator": [{ type: Input },],
    "rows": [{ type: Input },],
    "first": [{ type: Input },],
    "pageLinks": [{ type: Input },],
    "rowsPerPageOptions": [{ type: Input },],
    "alwaysShowPaginator": [{ type: Input },],
    "paginatorPosition": [{ type: Input },],
    "paginatorDropdownAppendTo": [{ type: Input },],
    "defaultSortOrder": [{ type: Input },],
    "sortMode": [{ type: Input },],
    "resetPageOnSort": [{ type: Input },],
    "selectionMode": [{ type: Input },],
    "selectionChange": [{ type: Output },],
    "contextMenuSelection": [{ type: Input },],
    "contextMenuSelectionChange": [{ type: Output },],
    "dataKey": [{ type: Input },],
    "metaKeySelection": [{ type: Input },],
    "lazy": [{ type: Input },],
    "compareSelectionBy": [{ type: Input },],
    "csvSeparator": [{ type: Input },],
    "exportFilename": [{ type: Input },],
    "filters": [{ type: Input },],
    "globalFilterFields": [{ type: Input },],
    "filterDelay": [{ type: Input },],
    "expandedRowKeys": [{ type: Input },],
    "rowExpandMode": [{ type: Input },],
    "scrollable": [{ type: Input },],
    "scrollHeight": [{ type: Input },],
    "virtualScroll": [{ type: Input },],
    "virtualScrollDelay": [{ type: Input },],
    "virtualRowHeight": [{ type: Input },],
    "frozenWidth": [{ type: Input },],
    "responsive": [{ type: Input },],
    "contextMenu": [{ type: Input },],
    "resizableColumns": [{ type: Input },],
    "columnResizeMode": [{ type: Input },],
    "reorderableColumns": [{ type: Input },],
    "loading": [{ type: Input },],
    "loadingIcon": [{ type: Input },],
    "rowHover": [{ type: Input },],
    "customSort": [{ type: Input },],
    "autoLayout": [{ type: Input },],
    "exportFunction": [{ type: Input },],
    "pagiDisplayDirection": [{ type: Input },],
    "onRowSelect": [{ type: Output },],
    "onRowUnselect": [{ type: Output },],
    "onPage": [{ type: Output },],
    "onSort": [{ type: Output },],
    "onFilter": [{ type: Output },],
    "onLazyLoad": [{ type: Output },],
    "onRowExpand": [{ type: Output },],
    "onRowCollapse": [{ type: Output },],
    "onContextMenuSelect": [{ type: Output },],
    "onColResize": [{ type: Output },],
    "onColReorder": [{ type: Output },],
    "onRowReorder": [{ type: Output },],
    "onEditInit": [{ type: Output },],
    "onEditComplete": [{ type: Output },],
    "onEditCancel": [{ type: Output },],
    "onHeaderCheckboxToggle": [{ type: Output },],
    "sortFunction": [{ type: Output },],
    "containerViewChild": [{ type: ViewChild, args: ['container',] },],
    "resizeHelperViewChild": [{ type: ViewChild, args: ['resizeHelper',] },],
    "reorderIndicatorUpViewChild": [{ type: ViewChild, args: ['reorderIndicatorUp',] },],
    "reorderIndicatorDownViewChild": [{ type: ViewChild, args: ['reorderIndicatorDown',] },],
    "tableViewChild": [{ type: ViewChild, args: ['table',] },],
    "templates": [{ type: ContentChildren, args: [TableTemplateDirective,] },],
    "headerTemplate": [{ type: ContentChild, args: ['headerTemplate',] },],
    "bodyTemplate": [{ type: ContentChild, args: ['bodyTemplate',] },],
    "captionTemplate": [{ type: ContentChild, args: ['captionTemplate',] },],
    "frozenRowsTemplate": [{ type: ContentChild, args: ['frozenRowsTemplate',] },],
    "footerTemplate": [{ type: ContentChild, args: ['footerTemplate',] },],
    "summaryTemplate": [{ type: ContentChild, args: ['summaryTemplate',] },],
    "colGroupTemplate": [{ type: ContentChild, args: ['colGroupTemplate',] },],
    "expandedRowTemplate": [{ type: ContentChild, args: ['expandedRowTemplate',] },],
    "frozenHeaderTemplate": [{ type: ContentChild, args: ['frozenHeaderTemplate',] },],
    "frozenBodyTemplate": [{ type: ContentChild, args: ['frozenBodyTemplate',] },],
    "frozenFooterTemplate": [{ type: ContentChild, args: ['frozenFooterTemplate',] },],
    "frozenColGroupTemplate": [{ type: ContentChild, args: ['frozenColGroupTemplate',] },],
    "emptyMessageTemplate": [{ type: ContentChild, args: ['emptyMessageTemplate',] },],
    "paginatorLeftTemplate": [{ type: ContentChild, args: ['paginatorLeftTemplate',] },],
    "paginatorRightTemplate": [{ type: ContentChild, args: ['paginatorRightTemplate',] },],
    "rowTrackBy": [{ type: Input },],
    "value": [{ type: Input },],
    "totalRecords": [{ type: Input },],
    "sortField": [{ type: Input },],
    "sortOrder": [{ type: Input },],
    "multiSortMeta": [{ type: Input },],
    "selection": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SortableColumnDirective {
    /**
     * @param {?} dt
     * @param {?} domHandler
     */
    constructor(dt, domHandler) {
        this.dt = dt;
        this.domHandler = domHandler;
        this._uiColumn = true;
        if (this.isEnabled()) {
            this.subscription = this.dt.tableService.sortSource$.subscribe(sortMeta => {
                this.updateSortState();
            });
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.isEnabled()) {
            this.updateSortState();
        }
    }
    /**
     * @return {?}
     */
    updateSortState() {
        this.sorted = this.dt.isSorted(this.field);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        if (this.isEnabled()) {
            this.updateSortState();
            this.dt.sort({
                originalEvent: event,
                field: this.field
            });
            this.domHandler.clearSelection();
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfSortableColumnDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
SortableColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfSortableColumn]',
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
SortableColumnDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
];
SortableColumnDirective.propDecorators = {
    "_uiColumn": [{ type: HostBinding, args: ['class.grf-ui-sortable-column',] },],
    "sorted": [{ type: HostBinding, args: ['class.grf-ui-state-highlight',] },],
    "field": [{ type: Input },],
    "grfSortableColumnDisabled": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectableRowDirective {
    /**
     * @param {?} dt
     * @param {?} domHandler
     * @param {?} tableService
     */
    constructor(dt, domHandler, tableService) {
        this.dt = dt;
        this.domHandler = domHandler;
        this.tableService = tableService;
        if (this.isEnabled()) {
            this.subscription = this.dt.tableService.selectionSource$.subscribe(() => {
                this.selected = this.dt.isSelected(this.data);
            });
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.isEnabled()) {
            this.selected = this.dt.isSelected(this.data);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        if (this.isEnabled()) {
            this.dt.handleRowClick({
                originalEvent: event,
                rowData: this.data,
                rowIndex: this.index
            });
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchEnd(event) {
        if (this.isEnabled()) {
            this.dt.handleRowTouchEnd(event);
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfSelectableRowDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
SelectableRowDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfSelectableRow]',
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
SelectableRowDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
    { type: TableService, },
];
SelectableRowDirective.propDecorators = {
    "data": [{ type: Input },],
    "index": [{ type: Input },],
    "grfSelectableRowDisabled": [{ type: Input },],
    "selected": [{ type: HostBinding, args: ['class.grf-ui-state-highlight',] },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
    "onTouchEnd": [{ type: HostListener, args: ['touchend', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RowTogglerDirective {
    /**
     * @param {?} dt
     */
    constructor(dt) {
        this.dt = dt;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        if (this.isEnabled()) {
            this.dt.toggleRow(this.data, event);
            event.preventDefault();
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfRowTogglerDisabled !== true;
    }
}
RowTogglerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfRowToggler]'
            },] },
];
/** @nocollapse */
RowTogglerDirective.ctorParameters = () => [
    { type: TableComponent, },
];
RowTogglerDirective.propDecorators = {
    "data": [{ type: Input },],
    "grfRowTogglerDisabled": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ContextMenuRowDirective {
    /**
     * @param {?} dt
     * @param {?} tableService
     */
    constructor(dt, tableService) {
        this.dt = dt;
        this.tableService = tableService;
        if (this.isEnabled()) {
            this.subscription = this.dt.tableService.contextMenuSource$.subscribe((data) => {
                this.selected = this.dt.equals(this.data, data);
            });
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onContextMenu(event) {
        if (this.isEnabled()) {
            this.dt.handleRowRightClick({
                originalEvent: event,
                rowData: this.data
            });
            event.preventDefault();
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfContextMenuRowDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
ContextMenuRowDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfContextMenuRow]'
            },] },
];
/** @nocollapse */
ContextMenuRowDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: TableService, },
];
ContextMenuRowDirective.propDecorators = {
    "data": [{ type: Input },],
    "grfContextMenuRowDisabled": [{ type: Input },],
    "selected": [{ type: HostBinding, args: ['class.grf-ui-contextmenu-selected',] },],
    "onContextMenu": [{ type: HostListener, args: ['contextmenu', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ResizableColumnDirective {
    /**
     * @param {?} dt
     * @param {?} el
     * @param {?} domHandler
     * @param {?} zone
     */
    constructor(dt, el, domHandler, zone) {
        this.dt = dt;
        this.el = el;
        this.domHandler = domHandler;
        this.zone = zone;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.isEnabled()) {
            this.domHandler.addClass(this.el.nativeElement, 'grf-ui-resizable-column');
            this.resizer = document.createElement('span');
            this.resizer.className = 'grf-ui-column-resizer grf-ui-clickable';
            this.el.nativeElement.appendChild(this.resizer);
            this.zone.runOutsideAngular(() => {
                this.resizerMouseDownListener = this.onMouseDown.bind(this);
                this.resizer.addEventListener('mousedown', this.resizerMouseDownListener);
            });
        }
    }
    /**
     * @return {?}
     */
    bindDocumentEvents() {
        this.zone.runOutsideAngular(() => {
            this.documentMouseMoveListener = this.onDocumentMouseMove.bind(this);
            document.addEventListener('mousemove', this.documentMouseMoveListener);
            this.documentMouseUpListener = this.onDocumentMouseUp.bind(this);
            document.addEventListener('mouseup', this.documentMouseUpListener);
        });
    }
    /**
     * @return {?}
     */
    unbindDocumentEvents() {
        if (this.documentMouseMoveListener) {
            document.removeEventListener('mousemove', this.documentMouseMoveListener);
            this.documentMouseMoveListener = null;
        }
        if (this.documentMouseUpListener) {
            document.removeEventListener('mouseup', this.documentMouseUpListener);
            this.documentMouseUpListener = null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        this.dt.onColumnResizeBegin(event);
        this.bindDocumentEvents();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDocumentMouseMove(event) {
        this.dt.onColumnResize(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDocumentMouseUp(event) {
        this.dt.onColumnResizeEnd(event, this.el.nativeElement);
        this.unbindDocumentEvents();
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfResizableColumnDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.resizerMouseDownListener) {
            this.resizer.removeEventListener('mousedown', this.resizerMouseDownListener);
        }
        this.unbindDocumentEvents();
    }
}
ResizableColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfResizableColumn]'
            },] },
];
/** @nocollapse */
ResizableColumnDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: ElementRef, },
    { type: DomHandler, },
    { type: NgZone, },
];
ResizableColumnDirective.propDecorators = {
    "grfResizableColumnDisabled": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReorderableColumnDirective {
    /**
     * @param {?} dt
     * @param {?} el
     * @param {?} domHandler
     * @param {?} zone
     */
    constructor(dt, el, domHandler, zone) {
        this.dt = dt;
        this.el = el;
        this.domHandler = domHandler;
        this.zone = zone;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.isEnabled()) {
            this.bindEvents();
        }
    }
    /**
     * @return {?}
     */
    bindEvents() {
        this.zone.runOutsideAngular(() => {
            this.mouseDownListener = this.onMouseDown.bind(this);
            this.el.nativeElement.addEventListener('mousedown', this.mouseDownListener);
            this.dragStartListener = this.onDragStart.bind(this);
            this.el.nativeElement.addEventListener('dragstart', this.dragStartListener);
            this.dragOverListener = this.onDragEnter.bind(this);
            this.el.nativeElement.addEventListener('dragover', this.dragOverListener);
            this.dragEnterListener = this.onDragEnter.bind(this);
            this.el.nativeElement.addEventListener('dragenter', this.dragEnterListener);
            this.dragLeaveListener = this.onDragLeave.bind(this);
            this.el.nativeElement.addEventListener('dragleave', this.dragLeaveListener);
        });
    }
    /**
     * @return {?}
     */
    unbindEvents() {
        if (this.mouseDownListener) {
            document.removeEventListener('mousedown', this.mouseDownListener);
            this.mouseDownListener = null;
        }
        if (this.dragOverListener) {
            document.removeEventListener('dragover', this.dragOverListener);
            this.dragOverListener = null;
        }
        if (this.dragEnterListener) {
            document.removeEventListener('dragenter', this.dragEnterListener);
            this.dragEnterListener = null;
        }
        if (this.dragEnterListener) {
            document.removeEventListener('dragenter', this.dragEnterListener);
            this.dragEnterListener = null;
        }
        if (this.dragLeaveListener) {
            document.removeEventListener('dragleave', this.dragLeaveListener);
            this.dragLeaveListener = null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        if (event.target.nodeName === 'INPUT' || this.domHandler.hasClass(event.target, 'grf-ui-column-resizer')) {
            this.el.nativeElement.draggable = false;
        }
        else {
            this.el.nativeElement.draggable = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStart(event) {
        this.dt.onColumnDragStart(event, this.el.nativeElement);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragOver(event) {
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragEnter(event) {
        this.dt.onColumnDragEnter(event, this.el.nativeElement);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragLeave(event) {
        this.dt.onColumnDragLeave(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDrop(event) {
        if (this.isEnabled()) {
            this.dt.onColumnDrop(event, this.el.nativeElement);
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfReorderableColumnDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unbindEvents();
    }
}
ReorderableColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfReorderableColumn]'
            },] },
];
/** @nocollapse */
ReorderableColumnDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: ElementRef, },
    { type: DomHandler, },
    { type: NgZone, },
];
ReorderableColumnDirective.propDecorators = {
    "grfReorderableColumnDisabled": [{ type: Input },],
    "onDrop": [{ type: HostListener, args: ['drop', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class EditableColumnDirective {
    /**
     * @param {?} dt
     * @param {?} el
     * @param {?} domHandler
     * @param {?} zone
     */
    constructor(dt, el, domHandler, zone) {
        this.dt = dt;
        this.el = el;
        this.domHandler = domHandler;
        this.zone = zone;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.isEnabled()) {
            this.domHandler.addClass(this.el.nativeElement, 'grf-ui-editable-column');
        }
    }
    /**
     * @return {?}
     */
    isValid() {
        return (this.dt.editingCell && this.domHandler.find(this.dt.editingCell, '.ng-invalid.ng-dirty').length === 0);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        if (this.isEnabled()) {
            if (this.dt.editingCell) {
                if (this.dt.editingCell !== this.el.nativeElement) {
                    if (!this.isValid()) {
                        return;
                    }
                    this.domHandler.removeClass(this.dt.editingCell, 'grf-ui-editing-cell');
                    this.openCell();
                }
            }
            else {
                this.openCell();
            }
        }
    }
    /**
     * @return {?}
     */
    openCell() {
        this.dt.editingCell = this.el.nativeElement;
        this.domHandler.addClass(this.el.nativeElement, 'grf-ui-editing-cell');
        this.dt.onEditInit.emit({ field: this.field, data: this.data });
        this.zone.runOutsideAngular(() => {
            setTimeout(() => {
                const /** @type {?} */ focusable = this.domHandler.findSingle(this.el.nativeElement, 'input, textarea');
                if (focusable) {
                    focusable.focus();
                }
            }, 50);
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyDown(event) {
        if (this.isEnabled()) {
            if (event.keyCode === 13) {
                if (this.isValid()) {
                    this.domHandler.removeClass(this.dt.editingCell, 'grf-ui-editing-cell');
                    this.dt.editingCell = null;
                    this.dt.onEditComplete.emit({ field: this.field, data: this.data });
                }
                event.preventDefault();
            }
            else if (event.keyCode === 27) {
                if (this.isValid()) {
                    this.domHandler.removeClass(this.dt.editingCell, 'grf-ui-editing-cell');
                    this.dt.editingCell = null;
                    this.dt.onEditCancel.emit({ field: this.field, data: this.data });
                }
                event.preventDefault();
            }
            else if (event.keyCode === 9) {
                if (event.shiftKey) {
                    this.moveToPreviousCell(event);
                }
                else {
                    this.moveToNextCell(event);
                }
            }
        }
    }
    /**
     * @param {?} element
     * @return {?}
     */
    findCell(element) {
        if (element) {
            let /** @type {?} */ cell = element;
            while (cell && !this.domHandler.hasClass(cell, 'grf-ui-editing-cell')) {
                cell = cell.parentElement;
            }
            return cell;
        }
        else {
            return null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    moveToPreviousCell(event) {
        const /** @type {?} */ currentCell = this.findCell(event.target);
        const /** @type {?} */ row = currentCell.parentElement;
        const /** @type {?} */ targetCell = this.findPreviousEditableColumn(currentCell);
        if (targetCell) {
            this.domHandler.invokeElementMethod(targetCell, 'click');
            event.preventDefault();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    moveToNextCell(event) {
        const /** @type {?} */ currentCell = this.findCell(event.target);
        const /** @type {?} */ row = currentCell.parentElement;
        const /** @type {?} */ targetCell = this.findNextEditableColumn(currentCell);
        if (targetCell) {
            this.domHandler.invokeElementMethod(targetCell, 'click');
            event.preventDefault();
        }
    }
    /**
     * @param {?} cell
     * @return {?}
     */
    findPreviousEditableColumn(cell) {
        let /** @type {?} */ prevCell = cell.previousElementSibling;
        if (!prevCell) {
            const /** @type {?} */ previousRow = cell.parentElement.previousElementSibling;
            if (previousRow) {
                prevCell = previousRow.lastElementChild;
            }
        }
        if (prevCell) {
            if (this.domHandler.hasClass(prevCell, 'grf-ui-editable-column')) {
                return prevCell;
            }
            else {
                return this.findPreviousEditableColumn(prevCell);
            }
        }
        else {
            return null;
        }
    }
    /**
     * @param {?} cell
     * @return {?}
     */
    findNextEditableColumn(cell) {
        let /** @type {?} */ nextCell = cell.nextElementSibling;
        if (!nextCell) {
            const /** @type {?} */ nextRow = cell.parentElement.nextElementSibling;
            if (nextRow) {
                nextCell = nextRow.firstElementChild;
            }
        }
        if (nextCell) {
            if (this.domHandler.hasClass(nextCell, 'grf-ui-editable-column')) {
                return nextCell;
            }
            else {
                return this.findNextEditableColumn(nextCell);
            }
        }
        else {
            return null;
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfEditableColumnDisabled !== true;
    }
}
EditableColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfEditableColumn]'
            },] },
];
/** @nocollapse */
EditableColumnDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: ElementRef, },
    { type: DomHandler, },
    { type: NgZone, },
];
EditableColumnDirective.propDecorators = {
    "data": [{ type: Input },],
    "field": [{ type: Input },],
    "grfEditableColumnDisabled": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
    "onKeyDown": [{ type: HostListener, args: ['keydown', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CellEditorComponent {
    /**
     * @param {?} dt
     * @param {?} editableColumn
     */
    constructor(dt, editableColumn) {
        this.dt = dt;
        this.editableColumn = editableColumn;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.templates.forEach((item) => {
            switch (item.getType()) {
                case 'input':
                    this.inputTemplate = item.template;
                    break;
                case 'output':
                    this.outputTemplate = item.template;
                    break;
            }
        });
    }
}
CellEditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-celleditor',
                template: `
        <ng-container *ngIf="dt.editingCell === editableColumn.el.nativeElement">
            <ng-container *ngTemplateOutlet="inputTemplate"></ng-container>
        </ng-container>
        <ng-container *ngIf="!dt.editingCell || dt.editingCell !== editableColumn.el.nativeElement">
            <ng-container *ngTemplateOutlet="outputTemplate"></ng-container>
        </ng-container>
    `,
            },] },
];
/** @nocollapse */
CellEditorComponent.ctorParameters = () => [
    { type: TableComponent, },
    { type: EditableColumnDirective, },
];
CellEditorComponent.propDecorators = {
    "templates": [{ type: ContentChildren, args: [TableTemplateDirective,] },],
    "inputTemplate": [{ type: ContentChild, args: ['inputTemplate',] },],
    "outputTemplate": [{ type: ContentChild, args: ['outputTemplate',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SortIconComponent {
    /**
     * @param {?} dt
     */
    constructor(dt) {
        this.dt = dt;
        this.subscription = this.dt.tableService.sortSource$.subscribe(sortMeta => {
            this.updateSortState();
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.updateSortState();
    }
    /**
     * @return {?}
     */
    updateSortState() {
        if (this.dt.sortMode === 'single') {
            this.sortOrder = this.dt.isSorted(this.field) ? this.dt.sortOrder : 0;
        }
        else if (this.dt.sortMode === 'multiple') {
            const /** @type {?} */ sortMeta = this.dt.getSortMeta(this.field);
            this.sortOrder = sortMeta ? sortMeta.order : 0;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
SortIconComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-sorticon',
                template: `
        <span class="grf-ui-sortable-column-icon fa fa-fw fa-sort" [ngClass]="{'fa-sort-asc': sortOrder === 1, 'fa-sort-desc': sortOrder === -1}"></span>
    `,
            },] },
];
/** @nocollapse */
SortIconComponent.ctorParameters = () => [
    { type: TableComponent, },
];
SortIconComponent.propDecorators = {
    "field": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableRadioButtonComponent {
    /**
     * @param {?} dt
     * @param {?} domHandler
     * @param {?} tableService
     */
    constructor(dt, domHandler, tableService) {
        this.dt = dt;
        this.domHandler = domHandler;
        this.tableService = tableService;
        this.subscription = this.dt.tableService.selectionSource$.subscribe(() => {
            this.checked = this.dt.isSelected(this.value);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.checked = this.dt.isSelected(this.value);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        this.dt.toggleRowWithRadio(event, this.value);
        this.domHandler.clearSelection();
    }
    /**
     * @return {?}
     */
    onFocus() {
        this.domHandler.addClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.domHandler.removeClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
TableRadioButtonComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tableradiobutton',
                template: `
        <div class="grf-ui-radiobutton grf-ui-widget" (click)="onClick($event)">
            <div class="grf-ui-helper-hidden-accessible">
                <input type="radio" [checked]="checked" (focus)="onFocus()" (blur)="onBlur()">
            </div>
            <div #box [ngClass]="{'grf-ui-radiobutton-box grf-ui-widget grf-ui-state-default':true,
                'grf-ui-state-active':checked, 'grf-ui-state-disabled':disabled}">
                <span class="grf-ui-radiobutton-icon grf-ui-clickable" [ngClass]="{'fa fa-circle':checked}"></span>
            </div>
        </div>
    `,
            },] },
];
/** @nocollapse */
TableRadioButtonComponent.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
    { type: TableService, },
];
TableRadioButtonComponent.propDecorators = {
    "disabled": [{ type: Input },],
    "value": [{ type: Input },],
    "boxViewChild": [{ type: ViewChild, args: ['box',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableCheckboxComponent {
    /**
     * @param {?} dt
     * @param {?} domHandler
     * @param {?} tableService
     */
    constructor(dt, domHandler, tableService) {
        this.dt = dt;
        this.domHandler = domHandler;
        this.tableService = tableService;
        this.subscription = this.dt.tableService.selectionSource$.subscribe(() => {
            this.checked = this.dt.isSelected(this.value);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.checked = this.dt.isSelected(this.value);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        this.dt.toggleRowWithCheckbox(event, this.value);
        this.domHandler.clearSelection();
    }
    /**
     * @return {?}
     */
    onFocus() {
        this.domHandler.addClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.domHandler.removeClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
TableCheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tablecheckbox',
                template: `
        <div class="grf-ui-chkbox grf-ui-widget" (click)="onClick($event)">
            <div class="grf-ui-helper-hidden-accessible">
                <input type="checkbox" [checked]="checked" (focus)="onFocus()" (blur)="onBlur()">
            </div>
            <div #box [ngClass]="{'grf-ui-chkbox-box grf-ui-widget grf-ui-state-default':true,
                'grf-ui-state-active':checked, 'grf-ui-state-disabled':disabled}">
                <span class="grf-ui-chkbox-icon grf-ui-clickable" [ngClass]="{'fa fa-check':checked}"></span>
            </div>
        </div>
    `,
            },] },
];
/** @nocollapse */
TableCheckboxComponent.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
    { type: TableService, },
];
TableCheckboxComponent.propDecorators = {
    "disabled": [{ type: Input },],
    "value": [{ type: Input },],
    "boxViewChild": [{ type: ViewChild, args: ['box',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableHeaderCheckboxComponent {
    /**
     * @param {?} dt
     * @param {?} domHandler
     * @param {?} tableService
     */
    constructor(dt, domHandler, tableService) {
        this.dt = dt;
        this.domHandler = domHandler;
        this.tableService = tableService;
        this.subscription = this.dt.tableService.selectionSource$.subscribe(() => {
            this.checked = this.updateCheckedState();
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.checked = this.updateCheckedState();
    }
    /**
     * @param {?} event
     * @param {?} checked
     * @return {?}
     */
    onClick(event, checked) {
        if (this.dt.value && this.dt.value.length > 0) {
            this.dt.toggleRowsWithCheckbox(event, !checked);
        }
        this.domHandler.clearSelection();
    }
    /**
     * @return {?}
     */
    onFocus() {
        this.domHandler.addClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    onBlur() {
        this.domHandler.removeClass(this.boxViewChild.nativeElement, 'grf-ui-state-focus');
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
    /**
     * @return {?}
     */
    updateCheckedState() {
        return (this.dt.value && this.dt.value.length > 0 && this.dt.selection && this.dt.selection.length > 0 && this.dt.selection.length === this.dt.value.length);
    }
}
TableHeaderCheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tableheadercheckbox',
                template: `
        <div class="grf-ui-chkbox grf-ui-widget" (click)="onClick($event, cb.checked)">
            <div class="grf-ui-helper-hidden-accessible">
                <input #cb type="checkbox" [checked]="checked" (focus)="onFocus()" (blur)="onBlur()" [disabled]="!dt.value || dt.value.length === 0">
            </div>
            <div #box [ngClass]="{'grf-ui-chkbox-box grf-ui-widget grf-ui-state-default':true,
                'grf-ui-state-active':checked, 'grf-ui-state-disabled': (!dt.value || dt.value.length === 0)}">
                <span class="grf-ui-chkbox-icon grf-ui-clickable" [ngClass]="{'fa fa-check':checked}"></span>
            </div>
        </div>
    `,
            },] },
];
/** @nocollapse */
TableHeaderCheckboxComponent.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
    { type: TableService, },
];
TableHeaderCheckboxComponent.propDecorators = {
    "boxViewChild": [{ type: ViewChild, args: ['box',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReorderableRowHandleDirective {
    /**
     * @param {?} el
     * @param {?} domHandler
     */
    constructor(el, domHandler) {
        this.el = el;
        this.domHandler = domHandler;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.domHandler.addClass(this.el.nativeElement, 'grf-ui-table-reorderablerow-handle');
    }
}
ReorderableRowHandleDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfReorderableRowHandle]'
            },] },
];
/** @nocollapse */
ReorderableRowHandleDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
];
ReorderableRowHandleDirective.propDecorators = {
    "index": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReorderableRowDirective {
    /**
     * @param {?} dt
     * @param {?} el
     * @param {?} domHandler
     * @param {?} zone
     */
    constructor(dt, el, domHandler, zone) {
        this.dt = dt;
        this.el = el;
        this.domHandler = domHandler;
        this.zone = zone;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.isEnabled()) {
            this.el.nativeElement.droppable = true;
            this.bindEvents();
        }
    }
    /**
     * @return {?}
     */
    bindEvents() {
        this.zone.runOutsideAngular(() => {
            this.mouseDownListener = this.onMouseDown.bind(this);
            this.el.nativeElement.addEventListener('mousedown', this.mouseDownListener);
            this.dragStartListener = this.onDragStart.bind(this);
            this.el.nativeElement.addEventListener('dragstart', this.dragStartListener);
            this.dragEndListener = this.onDragEnd.bind(this);
            this.el.nativeElement.addEventListener('dragend', this.dragEndListener);
            this.dragOverListener = this.onDragOver.bind(this);
            this.el.nativeElement.addEventListener('dragover', this.dragOverListener);
            this.dragLeaveListener = this.onDragLeave.bind(this);
            this.el.nativeElement.addEventListener('dragleave', this.dragLeaveListener);
        });
    }
    /**
     * @return {?}
     */
    unbindEvents() {
        if (this.mouseDownListener) {
            document.removeEventListener('mousedown', this.mouseDownListener);
            this.mouseDownListener = null;
        }
        if (this.dragStartListener) {
            document.removeEventListener('dragstart', this.dragStartListener);
            this.dragStartListener = null;
        }
        if (this.dragEndListener) {
            document.removeEventListener('dragend', this.dragEndListener);
            this.dragEndListener = null;
        }
        if (this.dragOverListener) {
            document.removeEventListener('dragover', this.dragOverListener);
            this.dragOverListener = null;
        }
        if (this.dragLeaveListener) {
            document.removeEventListener('dragleave', this.dragLeaveListener);
            this.dragLeaveListener = null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        if (this.domHandler.hasClass(event.target, 'grf-ui-table-reorderablerow-handle')) {
            this.el.nativeElement.draggable = true;
        }
        else {
            this.el.nativeElement.draggable = false;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStart(event) {
        this.dt.onRowDragStart(event, this.index);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragEnd(event) {
        this.dt.onRowDragEnd(event);
        this.el.nativeElement.draggable = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragOver(event) {
        this.dt.onRowDragOver(event, this.index, this.el.nativeElement);
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragLeave(event) {
        this.dt.onRowDragLeave(event, this.el.nativeElement);
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfReorderableRowDisabled !== true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDrop(event) {
        if (this.isEnabled() && this.dt.rowDragging) {
            this.dt.onRowDrop(event, this.el.nativeElement);
        }
    }
}
ReorderableRowDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfReorderableRow]'
            },] },
];
/** @nocollapse */
ReorderableRowDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: ElementRef, },
    { type: DomHandler, },
    { type: NgZone, },
];
ReorderableRowDirective.propDecorators = {
    "index": [{ type: Input },],
    "grfReorderableRowDisabled": [{ type: Input },],
    "onDrop": [{ type: HostListener, args: ['drop', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SelectableRowDblClickDirective {
    /**
     * @param {?} dt
     * @param {?} domHandler
     * @param {?} tableService
     */
    constructor(dt, domHandler, tableService) {
        this.dt = dt;
        this.domHandler = domHandler;
        this.tableService = tableService;
        if (this.isEnabled()) {
            this.subscription = this.dt.tableService.selectionSource$.subscribe(() => {
                this.selected = this.dt.isSelected(this.data);
            });
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.isEnabled()) {
            this.selected = this.dt.isSelected(this.data);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onClick(event) {
        if (this.isEnabled()) {
            this.dt.handleRowClick({
                originalEvent: event,
                rowData: this.data,
                rowIndex: this.index
            });
        }
    }
    /**
     * @return {?}
     */
    isEnabled() {
        return this.grfSelectableRowDisabled !== true;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
    }
}
SelectableRowDblClickDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfSelectableRowDblClick]',
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
SelectableRowDblClickDirective.ctorParameters = () => [
    { type: TableComponent, },
    { type: DomHandler, },
    { type: TableService, },
];
SelectableRowDblClickDirective.propDecorators = {
    "data": [{ type: Input },],
    "index": [{ type: Input },],
    "grfSelectableRowDisabled": [{ type: Input },],
    "selected": [{ type: HostBinding, args: ['class.grf-ui-state-highlight',] },],
    "onClick": [{ type: HostListener, args: ['dblclick', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ScrollableViewComponent {
    /**
     * @param {?} dt
     * @param {?} el
     * @param {?} domHandler
     * @param {?} zone
     */
    constructor(dt, el, domHandler, zone) {
        this.dt = dt;
        this.el = el;
        this.domHandler = domHandler;
        this.zone = zone;
        this.subscription = this.dt.tableService.valueSource$.subscribe(() => {
            this.zone.runOutsideAngular(() => {
                setTimeout(() => {
                    this.alignScrollBar();
                }, 50);
            });
        });
        if (this.dt.virtualScroll) {
            this.totalRecordsSubscription = this.dt.tableService.totalRecordsSource$.subscribe(() => {
                this.zone.runOutsideAngular(() => {
                    setTimeout(() => {
                        this.setVirtualScrollerHeight();
                    }, 50);
                });
            });
        }
        this.initialized = false;
    }
    /**
     * @return {?}
     */
    get scrollHeight() {
        return this._scrollHeight;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set scrollHeight(val) {
        this._scrollHeight = val;
        this.setScrollHeight();
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        if (!this.initialized && this.el.nativeElement.offsetParent) {
            this.alignScrollBar();
            this.initialized = true;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.bindEvents();
        this.setScrollHeight();
        this.alignScrollBar();
        if (!this.frozen) {
            if (this.dt.frozenColumns || this.dt.frozenBodyTemplate) {
                this.domHandler.addClass(this.el.nativeElement, 'grf-ui-table-unfrozen-view');
            }
            if (this.dt.frozenWidth) {
                this.el.nativeElement.style.left = this.dt.frozenWidth;
                this.el.nativeElement.style.width = 'calc(100% - ' + this.dt.frozenWidth + ')';
            }
            const /** @type {?} */ frozenView = this.el.nativeElement.previousElementSibling;
            if (frozenView) {
                this.frozenSiblingBody = this.domHandler.findSingle(frozenView, '.grf-ui-table-scrollable-body');
            }
        }
        else {
            this.scrollBodyViewChild.nativeElement.style.paddingBottom = this.domHandler.calculateScrollbarWidth() + 'px';
        }
        if (this.dt.virtualScroll) {
            this.setVirtualScrollerHeight();
        }
    }
    /**
     * @return {?}
     */
    bindEvents() {
        this.zone.runOutsideAngular(() => {
            const /** @type {?} */ scrollBarWidth = this.domHandler.calculateScrollbarWidth();
            if (this.scrollHeaderViewChild && this.scrollHeaderViewChild.nativeElement) {
                this.headerScrollListener = this.onHeaderScroll.bind(this);
                this.scrollHeaderBoxViewChild.nativeElement.addEventListener('scroll', this.headerScrollListener);
            }
            if (this.scrollFooterViewChild && this.scrollFooterViewChild.nativeElement) {
                this.footerScrollListener = this.onFooterScroll.bind(this);
                this.scrollFooterViewChild.nativeElement.addEventListener('scroll', this.footerScrollListener);
            }
            if (!this.frozen) {
                this.bodyScrollListener = this.onBodyScroll.bind(this);
                this.scrollBodyViewChild.nativeElement.addEventListener('scroll', this.bodyScrollListener);
            }
        });
    }
    /**
     * @return {?}
     */
    unbindEvents() {
        if (this.scrollHeaderViewChild && this.scrollHeaderViewChild.nativeElement) {
            this.scrollHeaderBoxViewChild.nativeElement.removeEventListener('scroll', this.headerScrollListener);
        }
        if (this.scrollFooterViewChild && this.scrollFooterViewChild.nativeElement) {
            this.scrollFooterViewChild.nativeElement.removeEventListener('scroll', this.footerScrollListener);
        }
        this.scrollBodyViewChild.nativeElement.addEventListener('scroll', this.bodyScrollListener);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onHeaderScroll(event) {
        this.scrollHeaderViewChild.nativeElement.scrollLeft = 0;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onFooterScroll(event) {
        this.scrollFooterViewChild.nativeElement.scrollLeft = 0;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onBodyScroll(event) {
        if (this.scrollHeaderViewChild && this.scrollHeaderViewChild.nativeElement) {
            this.scrollHeaderBoxViewChild.nativeElement.style.marginLeft = -1 * this.scrollBodyViewChild.nativeElement.scrollLeft + 'px';
        }
        if (this.scrollFooterViewChild && this.scrollFooterViewChild.nativeElement) {
            this.scrollFooterBoxViewChild.nativeElement.style.marginLeft = -1 * this.scrollBodyViewChild.nativeElement.scrollLeft + 'px';
        }
        if (this.frozenSiblingBody) {
            this.frozenSiblingBody.scrollTop = this.scrollBodyViewChild.nativeElement.scrollTop;
        }
        if (this.dt.virtualScroll) {
            const /** @type {?} */ viewport = this.domHandler.getOuterHeight(this.scrollBodyViewChild.nativeElement);
            const /** @type {?} */ tableHeight = this.domHandler.getOuterHeight(this.scrollTableViewChild.nativeElement);
            const /** @type {?} */ pageHeight = 28 * this.dt.rows;
            const /** @type {?} */ virtualTableHeight = this.domHandler.getOuterHeight(this.virtualScrollerViewChild.nativeElement);
            const /** @type {?} */ pageCount = (virtualTableHeight / pageHeight) || 1;
            const /** @type {?} */ scrollBodyTop = this.scrollTableViewChild.nativeElement.style.top || '0';
            if ((this.scrollBodyViewChild.nativeElement.scrollTop + viewport > parseFloat(scrollBodyTop) + tableHeight) || (this.scrollBodyViewChild.nativeElement.scrollTop < parseFloat(scrollBodyTop))) {
                const /** @type {?} */ page = Math.floor((this.scrollBodyViewChild.nativeElement.scrollTop * pageCount) / (this.scrollBodyViewChild.nativeElement.scrollHeight)) + 1;
                this.dt.handleVirtualScroll({
                    page: page,
                    callback: () => {
                        this.scrollTableViewChild.nativeElement.style.top = ((page - 1) * pageHeight) + 'px';
                        if (this.frozenSiblingBody) {
                            (/** @type {?} */ (this.frozenSiblingBody.children[0])).style.top = this.scrollTableViewChild.nativeElement.style.top;
                        }
                    }
                });
            }
        }
    }
    /**
     * @return {?}
     */
    setScrollHeight() {
        if (this.scrollHeight && this.scrollBodyViewChild && this.scrollBodyViewChild.nativeElement) {
            if (this.scrollHeight.indexOf('%') !== -1) {
                this.scrollBodyViewChild.nativeElement.style.visibility = 'hidden';
                this.scrollBodyViewChild.nativeElement.style.height = '100px';
                const /** @type {?} */ containerHeight = this.domHandler.getOuterHeight(this.dt.el.nativeElement.children[0]);
                const /** @type {?} */ relativeHeight = this.domHandler.getOuterHeight(this.dt.el.nativeElement.parentElement) * parseInt(this.scrollHeight, 0) / 100;
                const /** @type {?} */ staticHeight = containerHeight - 100;
                let /** @type {?} */ scrollBodyHeight = (relativeHeight - staticHeight);
                if (this.frozen) {
                    scrollBodyHeight -= this.domHandler.calculateScrollbarWidth();
                }
                this.scrollBodyViewChild.nativeElement.style.height = 'auto';
                this.scrollBodyViewChild.nativeElement.style.maxHeight = scrollBodyHeight + 'px';
                this.scrollBodyViewChild.nativeElement.style.visibility = 'visible';
            }
            else {
                if (this.frozen) {
                    this.scrollBodyViewChild.nativeElement.style.maxHeight = (parseInt(this.scrollHeight, 0) - this.domHandler.calculateScrollbarWidth()) + 'px';
                }
                else {
                    this.scrollBodyViewChild.nativeElement.style.maxHeight = this.scrollHeight;
                }
            }
        }
    }
    /**
     * @return {?}
     */
    setVirtualScrollerHeight() {
        if (this.virtualScrollerViewChild.nativeElement) {
            this.virtualScrollerViewChild.nativeElement.style.height = this.dt.totalRecords * this.dt.virtualRowHeight + 'px';
        }
    }
    /**
     * @return {?}
     */
    hasVerticalOverflow() {
        return this.domHandler.getOuterHeight(this.scrollTableViewChild.nativeElement) > this.domHandler.getOuterHeight(this.scrollBodyViewChild.nativeElement);
    }
    /**
     * @return {?}
     */
    alignScrollBar() {
        if (!this.frozen) {
            const /** @type {?} */ scrollBarWidth = this.hasVerticalOverflow() ? this.domHandler.calculateScrollbarWidth() : 0;
            this.scrollHeaderBoxViewChild.nativeElement.style.marginRight = scrollBarWidth + 'px';
            if (this.scrollFooterBoxViewChild && this.scrollFooterBoxViewChild.nativeElement) {
                this.scrollFooterBoxViewChild.nativeElement.style.marginRight = scrollBarWidth + 'px';
            }
        }
        this.initialized = false;
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.unbindEvents();
        this.frozenSiblingBody = null;
        if (this.subscription) {
            this.subscription.unsubscribe();
        }
        if (this.totalRecordsSubscription) {
            this.totalRecordsSubscription.unsubscribe();
        }
        this.initialized = false;
    }
}
ScrollableViewComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[grfScrollableView]',
                template: `
        <div #scrollHeader class="grf-ui-table-scrollable-header grf-ui-widget-header">
            <div #scrollHeaderBox class="grf-ui-table-scrollable-header-box">
                <table class="grf-ui-table-scrollable-header-table">
                    <ng-container *ngTemplateOutlet="frozen ? dt.frozenColGroupTemplate||dt.colGroupTemplate : dt.colGroupTemplate; context {$implicit: columns}"></ng-container>
                    <thead class="grf-ui-table-thead">
                        <ng-container *ngTemplateOutlet="frozen ? dt.frozenHeaderTemplate||dt.headerTemplate : dt.headerTemplate; context {$implicit: columns}"></ng-container>
                    </thead>
                    <tbody class="grf-ui-table-tbody">
                        <ng-template ngFor let-rowData let-rowIndex="index" [ngForOf]="dt.frozenValue" [ngForTrackBy]="dt.rowTrackBy">
                            <ng-container *ngTemplateOutlet="dt.frozenRowsTemplate; context: {$implicit: rowData, rowIndex: rowIndex, columns: columns}"></ng-container>
                        </ng-template>
                    </tbody>
                </table>
            </div>
        </div>
        <div #scrollBody class="grf-ui-table-scrollable-body">
            <table #scrollTable [ngClass]="{'grf-ui-table-virtual-table': dt.virtualScroll}" class="grf-ui-table-scrollable-body-table">
                <ng-container *ngTemplateOutlet="frozen ? dt.frozenColGroupTemplate||dt.colGroupTemplate : dt.colGroupTemplate; context {$implicit: columns}"></ng-container>
                <tbody class="grf-ui-table-tbody" grfTableBody [columns]="columns" [template]="frozen ? dt.frozenBodyTemplate||dt.bodyTemplate : dt.bodyTemplate"></tbody>
            </table>
            <div #virtualScroller class="grf-ui-table-virtual-scroller"></div>
        </div>
        <div #scrollFooter *ngIf="dt.footerTemplate" class="grf-ui-table-scrollable-footer grf-ui-widget-header">
            <div #scrollFooterBox class="grf-ui-table-scrollable-footer-box">
                <table class="grf-ui-table-scrollable-footer-table">
                    <ng-container *ngTemplateOutlet="frozen ? dt.frozenColGroupTemplate||dt.colGroupTemplate : dt.colGroupTemplate; context {$implicit: columns}"></ng-container>
                    <tfoot class="grf-ui-table-tfoot">
                        <ng-container *ngTemplateOutlet="frozen ? dt.frozenFooterTemplate||dt.footerTemplate : dt.footerTemplate; context {$implicit: columns}"></ng-container>
                    </tfoot>
                </table>
            </div>
        </div>
    `,
            },] },
];
/** @nocollapse */
ScrollableViewComponent.ctorParameters = () => [
    { type: TableComponent, },
    { type: ElementRef, },
    { type: DomHandler, },
    { type: NgZone, },
];
ScrollableViewComponent.propDecorators = {
    "columns": [{ type: Input },],
    "frozen": [{ type: Input },],
    "scrollHeaderViewChild": [{ type: ViewChild, args: ['scrollHeader',] },],
    "scrollHeaderBoxViewChild": [{ type: ViewChild, args: ['scrollHeaderBox',] },],
    "scrollBodyViewChild": [{ type: ViewChild, args: ['scrollBody',] },],
    "scrollTableViewChild": [{ type: ViewChild, args: ['scrollTable',] },],
    "scrollFooterViewChild": [{ type: ViewChild, args: ['scrollFooter',] },],
    "scrollFooterBoxViewChild": [{ type: ViewChild, args: ['scrollFooterBox',] },],
    "virtualScrollerViewChild": [{ type: ViewChild, args: ['virtualScroller',] },],
    "scrollHeight": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableBodyComponent {
    /**
     * @param {?} dt
     */
    constructor(dt) {
        this.dt = dt;
    }
}
TableBodyComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[grfTableBody]',
                template: `
        <ng-container *ngIf="!dt.expandedRowTemplate">
            <ng-template ngFor let-rowData let-rowIndex="index" [ngForOf]="dt.paginator ?
            ((dt.filteredValue||dt.value) | slice:(dt.lazy ? 0 : dt.first):((dt.lazy ? 0 : dt.first) + dt.rows)) : (dt.filteredValue||dt.value)" [ngForTrackBy]="dt.rowTrackBy">
                <ng-container *ngTemplateOutlet="template; context: {$implicit: rowData, rowIndex: dt.paginator ? (dt.first + rowIndex) : rowIndex, columns: columns}"></ng-container>
            </ng-template>
        </ng-container>
        <ng-container *ngIf="dt.expandedRowTemplate">
            <ng-template ngFor let-rowData let-rowIndex="index" [ngForOf]="dt.paginator ?
            ((dt.filteredValue||dt.value) | slice:(dt.lazy ? 0 : dt.first):((dt.lazy ? 0 : dt.first) + dt.rows)) : (dt.filteredValue||dt.value)" [ngForTrackBy]="dt.rowTrackBy">
                <ng-container *ngIf="dt.isRowExpanded(rowData); else collapsedrow">
                    <ng-container *ngTemplateOutlet="template; context: {$implicit: rowData, rowIndex: dt.paginator ? (dt.first + rowIndex) : rowIndex, columns: columns, expanded: true}"></ng-container>
                    <ng-container *ngTemplateOutlet="dt.expandedRowTemplate; context: {$implicit: rowData, rowIndex: dt.paginator ? (dt.first + rowIndex) : rowIndex, columns: columns}"></ng-container>
                </ng-container>
                <ng-template #collapsedrow>
                    <ng-container *ngTemplateOutlet="template; context: {$implicit: rowData, rowIndex: dt.paginator ? (dt.first + rowIndex) : rowIndex, expanded: false, columns: columns}"></ng-container>
                </ng-template>
            </ng-template>
        </ng-container>
        <ng-container *ngIf="dt.isEmpty()">
            <ng-container *ngTemplateOutlet="dt.emptyMessageTemplate; context: {$implicit: columns}"></ng-container>
        </ng-container>
    `,
            },] },
];
/** @nocollapse */
TableBodyComponent.ctorParameters = () => [
    { type: TableComponent, },
];
TableBodyComponent.propDecorators = {
    "columns": [{ type: Input },],
    "template": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TableModule {
}
TableModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, PaginationModule],
                exports: [TableComponent, SharedModule, SortableColumnDirective, SelectableRowDirective, RowTogglerDirective, ContextMenuRowDirective, ResizableColumnDirective,
                    ReorderableColumnDirective, EditableColumnDirective, CellEditorComponent, SortIconComponent, TableRadioButtonComponent, ScrollableViewComponent, TableBodyComponent,
                    TableCheckboxComponent, TableHeaderCheckboxComponent, ReorderableRowHandleDirective, ReorderableRowDirective, SelectableRowDblClickDirective, TableTemplateDirective],
                declarations: [TableComponent, SortableColumnDirective, SelectableRowDirective, RowTogglerDirective, ContextMenuRowDirective, ResizableColumnDirective,
                    ReorderableColumnDirective, EditableColumnDirective, CellEditorComponent, SortIconComponent, TableRadioButtonComponent, ScrollableViewComponent, TableBodyComponent,
                    TableCheckboxComponent, TableHeaderCheckboxComponent, ReorderableRowHandleDirective, ReorderableRowDirective, SelectableRowDblClickDirective, TableTemplateDirective]
            },] },
];
/** @nocollapse */
TableModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TabBodyComponent {
}
TabBodyComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tab-body',
                encapsulation: ViewEncapsulation.None,
                template: `
    <ng-template [ngTemplateOutlet]="content"></ng-template>
  `,
            },] },
];
/** @nocollapse */
TabBodyComponent.ctorParameters = () => [];
TabBodyComponent.propDecorators = {
    "content": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TabLabelDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        this._disabled = false;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set disabled(value) {
        this._disabled = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get disabled() {
        return this._disabled;
    }
    /**
     * @return {?}
     */
    getOffsetLeft() {
        return this.elementRef.nativeElement.offsetLeft;
    }
    /**
     * @return {?}
     */
    getOffsetWidth() {
        return this.elementRef.nativeElement.offsetWidth;
    }
    /**
     * @return {?}
     */
    getOffsetTop() {
        return this.elementRef.nativeElement.offsetTop;
    }
    /**
     * @return {?}
     */
    getOffsetHeight() {
        return this.elementRef.nativeElement.offsetHeight;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
}
TabLabelDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTabLabel]'
            },] },
];
/** @nocollapse */
TabLabelDirective.ctorParameters = () => [
    { type: ElementRef, },
];
TabLabelDirective.propDecorators = {
    "true": [{ type: HostBinding, args: ['class.grf-ui-tabs-tab',] },],
    "disabled": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-tabs-tab-disabled',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const availablePrefixs = ['moz', 'ms', 'webkit'];
/**
 * @return {?}
 */
function requestAnimationFramePolyfill() {
    let /** @type {?} */ lastTime = 0;
    return function (callback) {
        const /** @type {?} */ currTime = new Date().getTime();
        const /** @type {?} */ timeToCall = Math.max(0, 16 - (currTime - lastTime));
        const /** @type {?} */ id = window.setTimeout(() => { callback(currTime + timeToCall); }, timeToCall);
        lastTime = currTime + timeToCall;
        return id;
    };
}
/**
 * @return {?}
 */
function getRequestAnimationFrame() {
    if (typeof window === 'undefined') {
        return () => null;
    }
    if (window.requestAnimationFrame) {
        // https://github.com/vuejs/vue/issues/4465
        return window.requestAnimationFrame.bind(window);
    }
    const /** @type {?} */ prefix = availablePrefixs.filter(key => `${key}RequestAnimationFrame` in window)[0];
    return prefix
        ? window[`${prefix}RequestAnimationFrame`]
        : requestAnimationFramePolyfill();
}
const reqAnimFrame = getRequestAnimationFrame();

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TabsInkBarDirective {
    /**
     * @param {?} _renderer
     * @param {?} _elementRef
     * @param {?} _ngZone
     */
    constructor(_renderer, _elementRef, _ngZone) {
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._ngZone = _ngZone;
        this._animated = false;
        this.uiPositionMode = 'horizontal';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAnimated(value) {
        this._animated = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiAnimated() {
        return this._animated;
    }
    /**
     * @param {?} element
     * @return {?}
     */
    alignToElement(element) {
        this.show();
        this._ngZone.runOutsideAngular(() => {
            reqAnimFrame(() => {
                /** when horizontal remove height style and add transfrom left **/
                if (this.uiPositionMode === 'horizontal') {
                    this._renderer.removeStyle(this._elementRef.nativeElement, 'height');
                    this._renderer.setStyle(this._elementRef.nativeElement, 'transform', `translate3d(${this._getLeftPosition(element)}, 0px, 0px)`);
                    this._renderer.setStyle(this._elementRef.nativeElement, 'width', this._getElementWidth(element));
                }
                else {
                    /** when vertical remove width style and add transfrom top **/
                    this._renderer.removeStyle(this._elementRef.nativeElement, 'width');
                    this._renderer.setStyle(this._elementRef.nativeElement, 'transform', `translate3d(0px, ${this._getTopPosition(element)}, 0px)`);
                    this._renderer.setStyle(this._elementRef.nativeElement, 'height', this._getElementHeight(element));
                }
            });
        });
    }
    /**
     * @return {?}
     */
    show() {
        this._renderer.setStyle(this._elementRef.nativeElement, 'visibility', 'visible');
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setDisplay(value) {
        this._renderer.setStyle(this._elementRef.nativeElement, 'display', value);
    }
    /**
     * @return {?}
     */
    hide() {
        this._renderer.setStyle(this._elementRef.nativeElement, 'visibility', 'hidden');
    }
    /**
     * @param {?} element
     * @return {?}
     */
    _getLeftPosition(element) {
        return element ? element.offsetLeft + 'px' : '0';
    }
    /**
     * @param {?} element
     * @return {?}
     */
    _getElementWidth(element) {
        return element ? element.offsetWidth + 'px' : '0';
    }
    /**
     * @param {?} element
     * @return {?}
     */
    _getTopPosition(element) {
        return element ? element.offsetTop + 'px' : '0';
    }
    /**
     * @param {?} element
     * @return {?}
     */
    _getElementHeight(element) {
        return element ? element.offsetHeight + 'px' : '0';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
}
TabsInkBarDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTabsInkBar]',
            },] },
];
/** @nocollapse */
TabsInkBarDirective.ctorParameters = () => [
    { type: Renderer2, },
    { type: ElementRef, },
    { type: NgZone, },
];
TabsInkBarDirective.propDecorators = {
    "true": [{ type: HostBinding, args: ['class.grf-ui-tabs-ink-bar',] },],
    "uiAnimated": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-tabs-ink-bar-animated',] },],
    "uiPositionMode": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const EXAGGERATED_OVERSCROLL = 64;
class TabsNavComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _ngZone
     * @param {?} _renderer
     * @param {?} _dir
     */
    constructor(_elementRef, _ngZone, _renderer, _dir) {
        this._elementRef = _elementRef;
        this._ngZone = _ngZone;
        this._renderer = _renderer;
        this._dir = _dir;
        this._animated = true;
        this._hideBar = false;
        this._showPagination = true;
        this._showPaginationControls = false;
        this._disableScrollAfter = true;
        this._disableScrollBefore = true;
        this._scrollDistance = 0;
        this._selectedIndexChanged = false;
        this._realignInkBar = null;
        this._selectedIndex = 0;
        this._tabPositionMode = 'horizontal';
        this.uiSize = 'default';
        this._type = 'line';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAnimated(value) {
        this._animated = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiAnimated() {
        return this._animated;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiHideBar(value) {
        this._hideBar = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiHideBar() {
        return this._hideBar;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        this._type = value;
        if (this._type !== 'line') {
            this._inkBar.setDisplay('none');
        }
        else {
            this._inkBar.setDisplay('block');
        }
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiShowPagination(value) {
        this._showPagination = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiShowPagination() {
        return this._showPagination;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiPositionMode(value) {
        this._tabPositionMode = value;
        this._alignInkBarToSelectedTab();
        if (this.uiShowPagination) {
            this._updatePagination();
        }
    }
    /**
     * @return {?}
     */
    get uiPositionMode() {
        return this._tabPositionMode;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set selectedIndex(value) {
        this._selectedIndexChanged = this._selectedIndex !== value;
        this._selectedIndex = value;
    }
    /**
     * @return {?}
     */
    get selectedIndex() {
        return this._selectedIndex;
    }
    /**
     * @return {?}
     */
    _onContentChanges() {
        if (this.uiShowPagination) {
            this._updatePagination();
        }
        this._alignInkBarToSelectedTab();
    }
    /**
     * @param {?} scrollDir
     * @return {?}
     */
    _scrollHeader(scrollDir) {
        // Move the scroll distance one-third the length of the tab list's viewport.
        this.scrollDistance += (scrollDir === 'before' ? -1 : 1) * this.viewWidthHeightPix / 3;
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this._tabLabelCount !== this._labelWrappers.length) {
            if (this.uiShowPagination) {
                this._updatePagination();
            }
            this._tabLabelCount = this._labelWrappers.length;
        }
        if (this._selectedIndexChanged) {
            this._scrollToLabel(this._selectedIndex);
            if (this.uiShowPagination) {
                this._checkScrollingControls();
            }
            this._alignInkBarToSelectedTab();
            this._selectedIndexChanged = false;
        }
        if (this._scrollDistanceChanged) {
            if (this.uiShowPagination) {
                this._updateTabScrollPosition();
            }
            this._scrollDistanceChanged = false;
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._realignInkBar = this._ngZone.runOutsideAngular(() => {
            const /** @type {?} */ dirChange = this._dir ? this._dir.change : of(null);
            const /** @type {?} */ resize = typeof window !== 'undefined' ?
                fromEvent(window, 'resize').pipe(auditTime(10)) :
                of(null);
            return merge(dirChange, resize).pipe(startWith(null)).subscribe(() => {
                if (this.uiShowPagination) {
                    this._updatePagination();
                }
                this._alignInkBarToSelectedTab();
            });
        });
    }
    /**
     * @return {?}
     */
    _updateTabScrollPosition() {
        const /** @type {?} */ scrollDistance = this.scrollDistance;
        if (this.uiPositionMode === 'horizontal') {
            const /** @type {?} */ translateX = this._getLayoutDirection() === 'ltr' ? -scrollDistance : scrollDistance;
            this._renderer.setStyle(this._tabList.nativeElement, 'transform', `translate3d(${translateX}px, 0, 0)`);
        }
        else {
            this._renderer.setStyle(this._tabList.nativeElement, 'transform', `translate3d(0,${-scrollDistance}px, 0)`);
        }
    }
    /**
     * @return {?}
     */
    _updatePagination() {
        this._checkPaginationEnabled();
        this._checkScrollingControls();
        this._updateTabScrollPosition();
    }
    /**
     * @return {?}
     */
    _checkPaginationEnabled() {
        this._showPaginationControls =
            this.tabListScrollWidthHeightPix > this.elementRefOffSetWidthHeight;
        if (!this._showPaginationControls) {
            this.scrollDistance = 0;
        }
    }
    /**
     * @param {?} labelIndex
     * @return {?}
     */
    _scrollToLabel(labelIndex) {
        const /** @type {?} */ selectedLabel = this._labelWrappers
            ? this._labelWrappers.toArray()[labelIndex]
            : null;
        if (!selectedLabel) {
            return;
        }
        // The view length is the visible width of the tab labels.
        let /** @type {?} */ labelBeforePos;
        let /** @type {?} */ labelAfterPos;
        if (this.uiPositionMode === 'horizontal') {
            if (this._getLayoutDirection() === 'ltr') {
                labelBeforePos = selectedLabel.getOffsetLeft();
                labelAfterPos = labelBeforePos + selectedLabel.getOffsetWidth();
            }
            else {
                labelAfterPos = this._tabList.nativeElement.offsetWidth - selectedLabel.getOffsetLeft();
                labelBeforePos = labelAfterPos - selectedLabel.getOffsetWidth();
            }
        }
        else {
            labelBeforePos = selectedLabel.getOffsetTop();
            labelAfterPos = labelBeforePos + selectedLabel.getOffsetHeight();
        }
        const /** @type {?} */ beforeVisiblePos = this.scrollDistance;
        const /** @type {?} */ afterVisiblePos = this.scrollDistance + this.viewWidthHeightPix;
        if (labelBeforePos < beforeVisiblePos) {
            // Scroll header to move label to the before direction
            this.scrollDistance -= beforeVisiblePos - labelBeforePos + EXAGGERATED_OVERSCROLL;
        }
        else if (labelAfterPos > afterVisiblePos) {
            // Scroll header to move label to the after direction
            this.scrollDistance += labelAfterPos - afterVisiblePos + EXAGGERATED_OVERSCROLL;
        }
    }
    /**
     * @return {?}
     */
    _checkScrollingControls() {
        // Check if the pagination arrows should be activated.
        this._disableScrollBefore = this.scrollDistance === 0;
        this._disableScrollAfter = this.scrollDistance === this._getMaxScrollDistance();
    }
    /**
     * Determines what is the maximum length in pixels that can be set for the scroll distance. This
     * is equal to the difference in width between the tab list container and tab header container.
     *
     * This is an expensive call that forces a layout reflow to compute box and scroll metrics and
     * should be called sparingly.
     * @return {?}
     */
    _getMaxScrollDistance() {
        return (this.tabListScrollWidthHeightPix - this.viewWidthHeightPix) || 0;
    }
    /**
     * Sets the distance in pixels that the tab header should be transformed in the X-axis.
     * @param {?} v
     * @return {?}
     */
    set scrollDistance(v) {
        this._scrollDistance = Math.max(0, Math.min(this._getMaxScrollDistance(), v));
        // Mark that the scroll distance has changed so that after the view is checked, the CSS
        // transformation can move the header.
        this._scrollDistanceChanged = true;
        this._checkScrollingControls();
    }
    /**
     * @return {?}
     */
    get scrollDistance() {
        return this._scrollDistance;
    }
    /**
     * @return {?}
     */
    get viewWidthHeightPix() {
        let /** @type {?} */ PAGINATION_PIX = 0;
        if (this._showPaginationControls) {
            PAGINATION_PIX = 64;
        }
        if (this.uiPositionMode === 'horizontal') {
            return this._tabListContainer.nativeElement.offsetWidth - PAGINATION_PIX;
        }
        else {
            return this._tabListContainer.nativeElement.offsetHeight - PAGINATION_PIX;
        }
    }
    /**
     * @return {?}
     */
    get tabListScrollWidthHeightPix() {
        if (this.uiPositionMode === 'horizontal') {
            return this._tabList.nativeElement.scrollWidth;
        }
        else {
            return this._tabList.nativeElement.scrollHeight;
        }
    }
    /**
     * @return {?}
     */
    get elementRefOffSetWidthHeight() {
        if (this.uiPositionMode === 'horizontal') {
            return this._elementRef.nativeElement.offsetWidth;
        }
        else {
            return this._elementRef.nativeElement.offsetHeight;
        }
    }
    /**
     * @return {?}
     */
    _getLayoutDirection() {
        return this._dir && this._dir.value === 'rtl' ? 'rtl' : 'ltr';
    }
    /**
     * @return {?}
     */
    _alignInkBarToSelectedTab() {
        if (this.uiType === 'line') {
            const /** @type {?} */ selectedLabelWrapper = this._labelWrappers && this._labelWrappers.length
                ? this._labelWrappers.toArray()[this.selectedIndex].elementRef.nativeElement
                : null;
            if (this._inkBar) {
                this._inkBar.alignToElement(selectedLabelWrapper);
            }
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
}
TabsNavComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tabs-nav',
                encapsulation: ViewEncapsulation.None,
                template: `<div style="float:right;" *ngIf="_tabBarExtraContent">
    <div class="grf-ui-tabs-extra-content">
        <ng-template [ngTemplateOutlet]="_tabBarExtraContent">
        </ng-template>
    </div>
</div>
<div class="grf-ui-tabs-nav-container" [class.grf-ui-tabs-nav-container-scrolling]="_showPagination" #tabListContainer>
    <span class="grf-ui-tabs-tab-prev grf-ui-tabs-tab-arrow-show" (click)="_scrollHeader('before')" [class.grf-ui-tabs-tab-btn-disabled]="_disableScrollBefore"
        *ngIf="_showPagination">
        <span class="grf-ui-tabs-tab-prev-icon">
            <i class="fa" [class.fa-chevron-left]="uiPositionMode == 'horizontal'" [class.fa-chevron-up]="uiPositionMode !== 'horizontal'"></i>
        </span>
    </span>
    <span class="grf-ui-tabs-tab-next grf-ui-tabs-tab-arrow-show" (click)="_scrollHeader('after')" [class.grf-ui-tabs-tab-btn-disabled]="_disableScrollAfter"
        *ngIf="_showPagination">
        <span class="grf-ui-tabs-tab-next-icon">
            <i class="fa" [class.fa-chevron-right]="uiPositionMode == 'horizontal'" [class.fa-chevron-down]="uiPositionMode !== 'horizontal'"></i>
        </span>
    </span>
    <div class="grf-ui-tabs-nav-wrap">
        <div class="grf-ui-tabs-nav-scroll">
            <div class="grf-ui-tabs-nav" [class.grf-ui-tabs-nav-animated]="uiAnimated" #tabList (cdkObserveContent)="_onContentChanges()">
                <div grfTabsInkBar [hidden]="uiHideBar" [uiAnimated]="uiAnimated" [uiPositionMode]="uiPositionMode" style="display: block;"></div>
                <ng-content></ng-content>
            </div>
        </div>
    </div>
</div>
`
            },] },
];
/** @nocollapse */
TabsNavComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: NgZone, },
    { type: Renderer2, },
    { type: Directionality, decorators: [{ type: Optional },] },
];
TabsNavComponent.propDecorators = {
    "uiSize": [{ type: Input },],
    "true": [{ type: HostBinding, args: ['class.grf-ui-tabs-bar',] },],
    "uiAnimated": [{ type: Input },],
    "uiHideBar": [{ type: Input },],
    "uiType": [{ type: Input },],
    "_tabBarExtraContent": [{ type: ContentChild, args: ['tabBarExtraContent',] },],
    "_labelWrappers": [{ type: ContentChildren, args: [TabLabelDirective,] },],
    "_inkBar": [{ type: ViewChild, args: [TabsInkBarDirective,] },],
    "_tabListContainer": [{ type: ViewChild, args: ['tabListContainer',] },],
    "_tabList": [{ type: ViewChild, args: ['tabList',] },],
    "uiShowPagination": [{ type: Input },],
    "uiPositionMode": [{ type: Input },],
    "selectedIndex": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** code from https://github.com/angular/material2 */
/**
 * @record
 */

class TabChangeEvent {
}
class TabSetComponent {
    /**
     * @param {?} _renderer
     */
    constructor(_renderer) {
        this._renderer = _renderer;
        this._prefixCls = 'grf-ui-tabs';
        this._tabPosition = 'top';
        this._tabPositionMode = 'horizontal';
        this._indexToSelect = 0;
        this._selectedIndex = null;
        this._isViewInit = false;
        this._tabs = [];
        this.uiAnimated = true;
        this.uiShowPagination = false;
        this.uiHide = false;
        this.uiSelectChange = new EventEmitter(true);
        this.uiSize = 'default';
        this._type = 'line';
        this.tabs = [];
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSelectedIndex(value) {
        this._indexToSelect = value;
    }
    /**
     * @return {?}
     */
    get uiSelectedIndex() {
        return this._selectedIndex;
    }
    /**
     * @return {?}
     */
    get uiSelectedIndexChange() {
        return this.uiSelectChange.pipe(map(event => event.index));
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiTabPosition(value) {
        if (this._tabPosition === value) {
            return;
        }
        this._tabPosition = value;
        if ((this._tabPosition === 'top') || (this._tabPosition === 'bottom')) {
            this._tabPositionMode = 'horizontal';
        }
        else {
            this._tabPositionMode = 'vertical';
        }
        this._setPosition(value);
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiTabPosition() {
        return this._tabPosition;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        if (this._type === value) {
            return;
        }
        this._type = value;
        if (this._type === 'card') {
            this.uiAnimated = false;
        }
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    _setPosition(value) {
        if (this._isViewInit) {
            if (value === 'bottom') {
                this._renderer.insertBefore(this._hostContent.nativeElement, this._tabContent.nativeElement, this._tabNav._elementRef.nativeElement);
            }
            else {
                this._renderer.insertBefore(this._hostContent.nativeElement, this._tabNav._elementRef.nativeElement, this._tabContent.nativeElement);
            }
        }
    }
    /**
     * @return {?}
     */
    _setClassMap() {
        this._classMap = {
            [this._prefixCls]: true,
            [`${this._prefixCls}-vertical`]: (this._tabPosition === 'left') || (this._tabPosition === 'right'),
            [`${this._prefixCls}-${this._tabPosition}`]: this._tabPosition,
            [`${this._prefixCls}-no-animation`]: (this.uiAnimated === false) || ((/** @type {?} */ (this.uiAnimated)).tabPane === false),
            [`${this._prefixCls}-${this._type}`]: this._type,
            [`${this._prefixCls}-mini`]: (this.uiSize === 'small')
        };
    }
    /**
     * @param {?} index
     * @return {?}
     */
    clickLabel(index) {
        this.uiSelectedIndex = index;
        this._tabs[index].uiClick.emit();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        // Clamp the next selected index to the bounds of 0 and the tabs length. Note the `|| 0`, which
        // ensures that values like NaN can't get through and which would otherwise throw the
        // component into an infinite loop (since Math.max(NaN, 0) === NaN).
        const /** @type {?} */ indexToSelect = this._indexToSelect =
            Math.min(this._tabs.length - 1, Math.max(this._indexToSelect || 0, 0));
        // If there is a change in selected index, emit a change event. Should not trigger if
        // the selected index has not yet been initialized.
        if (this._selectedIndex !== indexToSelect && this._selectedIndex != null) {
            this.uiSelectChange.emit(this._createChangeEvent(indexToSelect));
        }
        // Setup the position for each tab and optionally setup an origin on the next selected tab.
        this._tabs.forEach((tab, index) => {
            tab.position = index - indexToSelect;
            // If there is already a selected tab, then set up an origin for the next selected tab
            // if it doesn't have one already.
            if (this._selectedIndex != null && tab.position === 0 && !tab.origin) {
                tab.origin = indexToSelect - this._selectedIndex;
            }
        });
        this._selectedIndex = indexToSelect;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this._isViewInit = true;
        this._setPosition(this._tabPosition);
    }
    /**
     * @param {?} index
     * @return {?}
     */
    _createChangeEvent(index) {
        const /** @type {?} */ event = new TabChangeEvent();
        event.index = index;
        if (this._tabs && this._tabs.length) {
            event.tab = this._tabs[index];
            this._tabs.forEach((item, i) => {
                if (i !== index) {
                    item.uiDeselect.emit();
                }
            });
            event.tab.uiSelect.emit();
        }
        return event;
    }
    /**
     * @return {?}
     */
    get inkBarAnimated() {
        return (this.uiAnimated === true) || ((/** @type {?} */ (this.uiAnimated)).inkBar === true);
    }
    /**
     * @return {?}
     */
    get tabPaneAnimated() {
        return (this.uiAnimated === true) || ((/** @type {?} */ (this.uiAnimated)).tabPane === true);
    }
}
TabSetComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tabset',
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";.grf-ui-tabs{-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;overflow:hidden;zoom:1;color:rgba(0,0,0,.85)}.grf-ui-tabs:after,.grf-ui-tabs:before{content:" ";display:table}.grf-ui-tabs:after{clear:both;visibility:hidden;font-size:0;height:0}.grf-ui-tabs-ink-bar{z-index:1;position:absolute;left:0;bottom:1px;-webkit-box-sizing:border-box;box-sizing:border-box;height:2px;background-color:#0275d8;-webkit-transform-origin:0 0;transform-origin:0 0}.grf-ui-tabs-bar{border-bottom:1px solid #ced4da;margin-bottom:16px;outline:0;-webkit-transition:padding .3s cubic-bezier(.645,.045,.355,1);transition:padding .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-nav-container{overflow:hidden;font-size:14px;line-height:1.5;-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;white-space:nowrap;margin-bottom:-1px;-webkit-transition:padding .3s cubic-bezier(.645,.045,.355,1);transition:padding .3s cubic-bezier(.645,.045,.355,1);zoom:1}.grf-ui-tabs-nav-container:after,.grf-ui-tabs-nav-container:before{content:" ";display:table}.grf-ui-tabs-nav-container:after{clear:both;visibility:hidden;font-size:0;height:0}.grf-ui-tabs-nav-container-scrolling{padding-left:32px;padding-right:32px}.grf-ui-tabs-tab-next,.grf-ui-tabs-tab-prev{-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;z-index:2;width:0;height:100%;line-height:32px;cursor:pointer;border:0;background-color:transparent;position:absolute;text-align:center;color:rgba(0,0,0,.55);-webkit-transition:width .3s cubic-bezier(.645,.045,.355,1),opacity .3s cubic-bezier(.645,.045,.355,1),color .3s cubic-bezier(.645,.045,.355,1);transition:width .3s cubic-bezier(.645,.045,.355,1),opacity .3s cubic-bezier(.645,.045,.355,1),color .3s cubic-bezier(.645,.045,.355,1);opacity:0;pointer-events:none}.grf-ui-tabs-tab-next.grf-ui-tabs-tab-arrow-show,.grf-ui-tabs-tab-prev.grf-ui-tabs-tab-arrow-show{opacity:1;width:32px;height:100%;pointer-events:auto}.grf-ui-tabs-tab-next:hover,.grf-ui-tabs-tab-prev:hover{color:rgba(0,0,0,.85)}.grf-ui-tabs-tab-next-icon,.grf-ui-tabs-tab-prev-icon{font-style:normal;font-weight:700;font-variant:normal;line-height:inherit;vertical-align:baseline;position:absolute;top:50%;left:50%;-webkit-transform:translate(-50%,-50%);transform:translate(-50%,-50%);text-align:center;text-transform:none}.grf-ui-tabs-tab-next-icon:before,.grf-ui-tabs-tab-prev-icon:before{display:block}.grf-ui-tabs-tab-btn-disabled{cursor:not-allowed}.grf-ui-tabs-tab-btn-disabled,.grf-ui-tabs-tab-btn-disabled:hover{color:rgba(0,0,0,.1)}.grf-ui-tabs-tab-next{right:2px}.grf-ui-tabs-tab-prev{left:0}:root .grf-ui-tabs-tab-prev{-webkit-filter:none;filter:none}.grf-ui-tabs-nav-wrap{overflow:hidden;margin-bottom:-1px}.grf-ui-tabs-nav-scroll{overflow:hidden;white-space:nowrap}.grf-ui-tabs-nav{-webkit-box-sizing:border-box;box-sizing:border-box;padding-left:0;-webkit-transition:-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1);position:relative;margin:0;list-style:none;display:inline-block}.grf-ui-tabs-nav:after,.grf-ui-tabs-nav:before{display:table;content:" "}.grf-ui-tabs-nav:after{clear:both}.grf-ui-tabs-nav .grf-ui-tabs-tab-disabled{pointer-events:none;cursor:default;color:rgba(0,0,0,.1)}.grf-ui-tabs-nav .grf-ui-tabs-tab{display:inline-block;height:100%;margin-right:24px;-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;padding:8px 20px;-webkit-transition:color .3s cubic-bezier(.645,.045,.355,1);transition:color .3s cubic-bezier(.645,.045,.355,1);cursor:pointer;text-decoration:none}.grf-ui-tabs-nav .grf-ui-tabs-tab:last-child{margin-right:0}.grf-ui-tabs-nav .grf-ui-tabs-tab-active,.grf-ui-tabs-nav .grf-ui-tabs-tab:active,.grf-ui-tabs-nav .grf-ui-tabs-tab:hover{color:#0275d8}.grf-ui-tabs-mini .grf-ui-tabs-nav-container{font-size:14px}.grf-ui-tabs-mini .grf-ui-tabs-tab{margin-right:0;padding:8px 16px}.grf-ui-tabs:not(.grf-ui-tabs-vertical)>.grf-ui-tabs-content{width:100%}.grf-ui-tabs:not(.grf-ui-tabs-vertical)>.grf-ui-tabs-content>.grf-ui-tabs-tabpane{-ms-flex-negative:0;flex-shrink:0;width:100%;-webkit-transition:opacity .45s;transition:opacity .45s;opacity:1}.grf-ui-tabs:not(.grf-ui-tabs-vertical)>.grf-ui-tabs-content>.grf-ui-tabs-tabpane-inactive{opacity:0;height:0;padding:0!important;pointer-events:none}.grf-ui-tabs:not(.grf-ui-tabs-vertical)>.grf-ui-tabs-content-animated{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row;will-change:margin-left;-webkit-transition:margin-left .3s cubic-bezier(.645,.045,.355,1);transition:margin-left .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-vertical>.grf-ui-tabs-bar{border-bottom:0;height:100%}.grf-ui-tabs-vertical>.grf-ui-tabs-bar-tab-next,.grf-ui-tabs-vertical>.grf-ui-tabs-bar-tab-prev{width:32px;height:0;-webkit-transition:height .3s cubic-bezier(.645,.045,.355,1),opacity .3s cubic-bezier(.645,.045,.355,1),color .3s cubic-bezier(.645,.045,.355,1);transition:height .3s cubic-bezier(.645,.045,.355,1),opacity .3s cubic-bezier(.645,.045,.355,1),color .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-vertical>.grf-ui-tabs-bar-tab-next.grf-ui-tabs-tab-arrow-show,.grf-ui-tabs-vertical>.grf-ui-tabs-bar-tab-prev.grf-ui-tabs-tab-arrow-show{width:100%;height:32px}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-tab{float:none;margin-right:0;margin-bottom:16px;display:block;padding:8px 24px}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-tab:last-child{margin-bottom:0}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-extra-content{text-align:center}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-scroll{width:auto}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-container,.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{height:100%}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-container{margin-bottom:0}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-container.grf-ui-tabs-nav-container-scrolling{padding:32px 0}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-bottom:0}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-nav{width:100%}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-ink-bar{width:2px;left:auto;height:auto;top:0}.grf-ui-tabs-vertical>.grf-ui-tabs-content{overflow:hidden;width:auto;margin-top:0!important}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-tab-next{width:100%;bottom:0;height:32px}.grf-ui-tabs-vertical>.grf-ui-tabs-bar .grf-ui-tabs-tab-prev{top:0;width:100%;height:32px}.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-bar{float:left;border-right:1px solid #e9e9e9;margin-right:-1px;margin-bottom:0}.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-tab{text-align:right}.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-nav-container,.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-right:-1px}.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-ink-bar{right:1px}.grf-ui-tabs-vertical.grf-ui-tabs-left>.grf-ui-tabs-content{padding-left:24px;border-left:1px solid #e9e9e9}.grf-ui-tabs-vertical.grf-ui-tabs-right>.grf-ui-tabs-bar{float:right;border-left:1px solid #e9e9e9;margin-left:-1px;margin-bottom:0}.grf-ui-tabs-vertical.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-nav-container,.grf-ui-tabs-vertical.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-left:-1px}.grf-ui-tabs-vertical.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-ink-bar{left:1px}.grf-ui-tabs-vertical.grf-ui-tabs-right>.grf-ui-tabs-content{padding-right:24px;border-right:1px solid #e9e9e9}.grf-ui-tabs-bottom>.grf-ui-tabs-bar{margin-bottom:0;margin-top:16px}.grf-ui-tabs.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-nav-container{height:32px}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-ink-bar{visibility:hidden}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab{margin:0 2px 0 0;border:1px solid #ced4da;border-bottom:0;border-radius:4px 4px 0 0;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1);background:#f9f9f9;padding:5px 16px 4px;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab-active{background:#fff;border-color:#ced4da;color:#0275d8;padding-bottom:5px}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab-inactive{padding:0}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-bottom:0}.grf-ui-tabs-card>-bar -tab{color:rgba(0,0,0,.55);-webkit-transition:all .3s;transition:all .3s;margin-right:0;opacity:0;position:absolute;right:2px;top:50%;margin-top:-5px;overflow:hidden;text-align:center;border-radius:2px;width:14px;height:14px;line-height:1}.grf-ui-tabs-card>-bar -tab:hover{color:rgba(0,0,0,.85)}.grf-ui-tabs-card.grf-ui-tabs-content>-tabpane,.grf-ui-tabs-editable-card .grf-ui-tabs-content>-tabpane{-webkit-transition:none!important;transition:none!important}.grf-ui-tabs-card.grf-ui-tabs-content>-tabpane-inactive,.grf-ui-tabs-editable-card .grf-ui-tabs-content>-tabpane-inactive{overflow:hidden}.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab:hover{opacity:1}.grf-ui-tabs-extra-content{line-height:32px}.grf-ui-tabs-extra-content .grf-ui-tabs-new-tab{width:20px;height:20px;line-height:20px;text-align:center;cursor:pointer;border-radius:4px;border:1px solid #ced4da;font-size:14px;color:rgba(0,0,0,.55);-webkit-transition:all .3s;transition:all .3s}.grf-ui-tabs-extra-content .grf-ui-tabs-new-tab:hover{color:#0275d8;border-color:#0275d8}.grf-ui-tabs-vertical.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-nav-container{height:auto}.grf-ui-tabs-vertical.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab{border-bottom:1px solid #ced4da;margin-bottom:8px}.grf-ui-tabs-vertical.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab-active{padding-bottom:4px}.grf-ui-tabs-vertical.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-tab:last-child{margin-bottom:8px}.grf-ui-tabs-vertical.grf-ui-tabs-card>.grf-ui-tabs-bar .grf-ui-tabs-new-tab{width:90%}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-right:0}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-tab{border-right:0;border-radius:4px 0 0 4px;margin-right:1px}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-left>.grf-ui-tabs-bar .grf-ui-tabs-tab-active{margin-right:-1px;padding-right:18px}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-nav-wrap{margin-left:0}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-tab{border-left:0;border-radius:0 4px 4px 0;margin-left:1px}.grf-ui-tabs-vertical.grf-ui-tabs-card.grf-ui-tabs-right>.grf-ui-tabs-bar .grf-ui-tabs-tab-active{margin-left:-1px;padding-left:18px}.grf-ui-tabs-bottom .grf-ui-tabs-ink-bar-animated,.grf-ui-tabs-top .grf-ui-tabs-ink-bar-animated{-webkit-transition:width .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:width .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1),width .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1),width .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-left .grf-ui-tabs-ink-bar-animated,.grf-ui-tabs-right .grf-ui-tabs-ink-bar-animated{-webkit-transition:height .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:height .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1),height .3s cubic-bezier(.645,.045,.355,1);transition:transform .3s cubic-bezier(.645,.045,.355,1),height .3s cubic-bezier(.645,.045,.355,1),-webkit-transform .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tabs-no-animation>.grf-ui-tabs-content-animated,.grf-ui-tabs-vertical>.grf-ui-tabs-content-animated,.no-flex>.grf-ui-tabs-content-animated{-webkit-transform:none!important;transform:none!important;margin-left:0!important}.grf-ui-tabs-no-animation>.grf-ui-tabs-content>.grf-ui-tabs-tabpane-inactive,.grf-ui-tabs-vertical>.grf-ui-tabs-content>.grf-ui-tabs-tabpane-inactive,.no-flex>.grf-ui-tabs-content>.grf-ui-tabs-tabpane-inactive{display:none}grf-tab,grf-tabs-nav{display:block}grf-tabset{display:block;position:relative;overflow:hidden}grf-tabset .grf-ui-tabs{height:100%}grf-tab-body{display:block}`],
                template: `<div [ngClass]="_classMap" #hostContent>
    <grf-tabs-nav #tabNav [uiSize]="uiSize" [uiType]="uiType" [uiShowPagination]="uiShowPagination" [uiPositionMode]="_tabPositionMode"
        [uiAnimated]="inkBarAnimated" [uiHideBar]="uiHide" [selectedIndex]="uiSelectedIndex">
        <ng-template #tabBarExtraContent>
            <ng-template [ngTemplateOutlet]="uiTabBarExtraTemplate || uiTabBarExtraContent"></ng-template>
        </ng-template>
        <div grfTabLabel [class.grf-ui-tabs-tab-active]="(uiSelectedIndex == i)&&!uiHide" [disabled]="tab.disabled" (click)="clickLabel(i)"
            *ngFor="let tab of _tabs; let i = index">
            <ng-template [ngTemplateOutlet]="tab._tabHeading"></ng-template>
        </div>
    </grf-tabs-nav>
    <div class="grf-ui-tabs-content" #tabContent [class.grf-ui-tabs-content-animated]="tabPaneAnimated" [class.grf-ui-tabs-content-no-animated]="!tabPaneAnimated"
        [style.margin-left.%]="tabPaneAnimated&&(-uiSelectedIndex*100)">
        <grf-tab-body class="grf-ui-tabs-tabpane" [class.grf-ui-tabs-tabpane-active]="(uiSelectedIndex == i)&&!uiHide" [class.grf-ui-tabs-tabpane-inactive]="(uiSelectedIndex != i)||uiHide"
            [content]="tab.content" *ngFor="let tab of _tabs; let i = index">
        </grf-tab-body>
    </div>
</div>
`
            },] },
];
/** @nocollapse */
TabSetComponent.ctorParameters = () => [
    { type: Renderer2, },
];
TabSetComponent.propDecorators = {
    "uiTabBarExtraTemplate": [{ type: Input },],
    "uiTabBarExtraContent": [{ type: ContentChild, args: ['uiTabBarExtraContent',] },],
    "_tabNav": [{ type: ViewChild, args: ['tabNav',] },],
    "_tabContent": [{ type: ViewChild, args: ['tabContent',] },],
    "_hostContent": [{ type: ViewChild, args: ['hostContent',] },],
    "uiAnimated": [{ type: Input },],
    "uiShowPagination": [{ type: Input },],
    "uiHide": [{ type: Input },],
    "uiSelectedIndex": [{ type: Input },],
    "uiSelectedIndexChange": [{ type: Output },],
    "uiSelectChange": [{ type: Output },],
    "uiSize": [{ type: Input },],
    "uiTabPosition": [{ type: Input },],
    "uiType": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TabComponent {
    /**
     * @param {?} _tabSetComponent
     */
    constructor(_tabSetComponent) {
        this._tabSetComponent = _tabSetComponent;
        this.disabled = false;
        this.position = null;
        this.origin = null;
        this.uiSelect = new EventEmitter();
        this.uiClick = new EventEmitter();
        this.uiDeselect = new EventEmitter();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this.disabled = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this.disabled;
    }
    /**
     * @return {?}
     */
    get content() {
        return this._content;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._tabSetComponent._tabs.push(this);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this._tabSetComponent._tabs.splice(this._tabSetComponent._tabs.indexOf(this), 1);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
}
TabComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tab',
                template: `
    <ng-template>
      <ng-content></ng-content>
    </ng-template>
  `,
            },] },
];
/** @nocollapse */
TabComponent.ctorParameters = () => [
    { type: TabSetComponent, },
];
TabComponent.propDecorators = {
    "true": [{ type: HostBinding, args: ['class.grf-ui-tabs-tabpane',] },],
    "uiDisabled": [{ type: Input },],
    "uiSelect": [{ type: Output },],
    "uiClick": [{ type: Output },],
    "uiDeselect": [{ type: Output },],
    "_tabHeading": [{ type: ContentChild, args: ['uiTabHeading',] },],
    "_content": [{ type: ViewChild, args: [TemplateRef,] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TabModule {
}
TabModule.decorators = [
    { type: NgModule, args: [{
                declarations: [TabComponent, TabSetComponent, TabsNavComponent, TabLabelDirective, TabsInkBarDirective, TabBodyComponent],
                exports: [TabComponent, TabSetComponent, TabsNavComponent, TabLabelDirective, TabsInkBarDirective, TabBodyComponent],
                imports: [CommonModule, ObserversModule]
            },] },
];
/** @nocollapse */
TabModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeSelectComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._prefixCls = 'grf-ui-tree-select';
        this._classList = [];
        this._dropDownPrefixCls = `${this._prefixCls}-dropdown`;
        this._selectionPrefixCls = `${this._prefixCls}-selection`;
        this._placeholder = 'Placeholder';
        this._notFoundContent = 'notFound';
        this._isOpen = false;
        this._disabled = false;
        this._showSearch = false;
        this._isTags = false;
        this._searchText = '';
        this._triggerWidth = 0;
        this._options = [];
        this._cacheOptions = [];
        this._filterOptions = [];
        this._tagsOptions = [];
        this._isMultiInit = false;
        this._dropDownPosition = 'bottom';
        this._isMultiple = false;
        this._composing = false;
        this._keepUnListOptions = false;
        this._allowClear = false;
        // ngModel Access
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
        this.uiSearchChange = new EventEmitter();
        this.uiOpenChange = new EventEmitter();
        this.uiScrollToBottom = new EventEmitter();
        this.uiFilter = true;
        this.uiMaxMultiple = Infinity;
        /**
         * new ui-option insert or new tags insert
         */
        this.addOption = (option) => {
            this._options.push(option);
            if (!this._isTags) {
                if (option.uiValue) {
                    this.updateSelectedOption(this._value);
                }
                else {
                    this.forceUpdateSelectedOption(this._value);
                }
            }
        };
        /**
         * cancel select multiple option
         */
        this.unSelectMultipleOption = (option, $event, emitChange = true) => {
            this._operatingMultipleOption = option;
            for (let /** @type {?} */ i = 0; i < this._selectedOptions.length; i++) {
                if (this._selectedOptions[i] === option) {
                    this._selectedOptions.splice(i, 1);
                    this.unSelectMutipleTree(this._selectedOptions);
                }
            }
            this._tags = this.setTags(this._nodeSlected);
            if (emitChange) {
                this.emitMultipleOptions();
            }
            if ($event) {
                $event.preventDefault();
                $event.stopPropagation();
            }
        };
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiMaxTags(value) {
        if (value > 0) {
            this._maxTags = value;
        }
        else {
            this._maxTags = 10;
        }
    }
    /**
     * @return {?}
     */
    get uiMaxTags() {
        return this._maxTags;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAllowClear(value) {
        if (value === '') {
            this._allowClear = true;
        }
        else {
            this._allowClear = /** @type {?} */ (value);
        }
    }
    /**
     * @return {?}
     */
    get uiAllowClear() {
        return this._allowClear;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiKeepUnListOptions(value) {
        if (value === '') {
            this._keepUnListOptions = true;
        }
        else {
            this._keepUnListOptions = /** @type {?} */ (value);
        }
    }
    /**
     * @return {?}
     */
    get uiKeepUnListOptions() {
        return this._keepUnListOptions;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiMode(value) {
        this._mode = value;
        if (this._mode === 'multiple') {
            this.uiMultiple = true;
        }
        else if (this._mode === 'tags') {
            this.uiTags = true;
        }
        else if (this._mode === 'combobox') {
            this.uiShowSearch = true;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiMultiple(value) {
        this._isMultiple = value;
        if (this._isMultiple) {
            this.uiShowSearch = true;
        }
    }
    /**
     * @return {?}
     */
    get uiMultiple() {
        return this._isMultiple;
    }
    /**
     * @return {?}
     */
    get uiPlaceHolder() {
        return this._placeholder;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiPlaceHolder(value) {
        this._placeholder = value;
    }
    /**
     * @return {?}
     */
    get uiNotFoundContent() {
        return this._notFoundContent;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiNotFoundContent(value) {
        this._notFoundContent = value;
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this._size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this._size = value;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiShowSearch() {
        return this._showSearch;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiShowSearch(value) {
        this._showSearch = value;
    }
    /**
     * @return {?}
     */
    get uiTags() {
        return this._isTags;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiTags(value) {
        this._isTags = value;
        this.uiMultiple = value;
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = value;
        this.closeDropDown();
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiOpen() {
        return this._isOpen;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiOpen(value) {
        if (this._isOpen === value) {
            return;
        }
        if (value === true) {
            this.scrollToActive();
            if (!this._triggerWidth) {
                this._setTriggerWidth();
            }
        }
        this._isOpen = value;
        this.uiOpenChange.emit(this._isOpen);
        this.setClassMap();
    }
    /**
     * ui-option remove or tags remove
     * @param {?} option
     * @return {?}
     */
    removeOption(option) {
        this._options.splice(this._options.indexOf(option), 1);
        if (!this._isTags) {
            this.forceUpdateSelectedOption(this._value);
        }
    }
    /**
     * dropdown position changed
     * @param {?} position
     * @return {?}
     */
    onPositionChange(position) {
        this._dropDownPosition = position.connectionPair.originY;
    }
    /**
     * @return {?}
     */
    compositionStart() {
        this._composing = true;
    }
    /**
     * @return {?}
     */
    compositionEnd() {
        this._composing = false;
    }
    /**
     * clear single selected option
     * @param {?=} $event
     * @return {?}
     */
    clearSelect($event) {
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    }
    /**
     * click dropdown option by user
     * @param {?} node
     * @param {?=} $event
     * @return {?}
     */
    clickOption(node, $event) {
        if (!node) {
            return;
        }
        this.chooseOption(node, true, $event);
        this.clearSearchText();
    }
    /**
     * choose option
     * @param {?} node
     * @param {?=} isUserClick
     * @param {?=} $event
     * @return {?}
     */
    chooseOption(node, isUserClick = false, $event) {
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
        this._activeFilterOption = node;
        if (node) {
            if (!this.uiMultiple) {
                this._selectedOption = node;
                this._nodeSlected = node;
            }
            else {
                if (isUserClick) {
                    this._selectedOptions = this.cancelReaptChild(node);
                    this._tags = this.setTags(node);
                }
            }
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    setTags(node) {
        const /** @type {?} */ options = this.cancelReaptChild(node);
        const /** @type {?} */ amount = this._maxTags;
        if (options.length > amount) {
            const /** @type {?} */ setOptions = [];
            for (let /** @type {?} */ i = 0; i < amount; i++) {
                setOptions.push(options[i]);
            }
            return setOptions;
        }
        else {
            return options;
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    cancelReaptChild(node) {
        const /** @type {?} */ allnode = this.allItems(node);
        const /** @type {?} */ res = [];
        for (let /** @type {?} */ i = 0; i < allnode.length; i++) {
            let /** @type {?} */ repeat = false;
            for (let /** @type {?} */ j = 0; j < allnode.length; j++) {
                if (allnode[i] === allnode[j] && i !== j) {
                    repeat = true;
                    break;
                }
            }
            if (!repeat) {
                res.push(allnode[i]);
            }
        }
        return res;
    }
    /**
     * @param {?} node
     * @return {?}
     */
    allItems(node) {
        if (node) {
            const /** @type {?} */ items = [];
            node.forEach(item => {
                items.push(item);
                this.findChild(item, items);
            });
            return items;
        }
    }
    /**
     * @param {?} node
     * @param {?} array
     * @return {?}
     */
    findChild(node, array) {
        if (node.children) {
            node.children.forEach(child => {
                array.push(child);
                this.findChild(child, array);
            });
        }
    }
    /**
     * @param {?} element
     * @param {?} text
     * @return {?}
     */
    updateWidth(element, text) {
        if (text) {
            /** wait for scroll width change */
            setTimeout(() => {
                this._renderer.setStyle(element, 'width', `${element.scrollWidth}px`);
            });
        }
        else {
            this._renderer.removeStyle(element, 'width');
        }
    }
    /**
     * @param {?} ul
     * @return {?}
     */
    dropDownScroll(ul) {
        if (ul && (ul.scrollHeight - ul.scrollTop === ul.clientHeight)) {
            this.uiScrollToBottom.emit(true);
        }
    }
    /**
     * @return {?}
     */
    checkDropDownScroll() {
        if (this.dropdownUl && (this.dropdownUl.nativeElement.scrollHeight === this.dropdownUl.nativeElement.clientHeight)) {
            this.uiScrollToBottom.emit(true);
        }
    }
    /**
     * determine if option in set
     * @param {?} set
     * @param {?} node
     * @return {?}
     */
    isInSet(set, node) {
        return ((Array.from(set)).find((data) => data.label === node.label));
    }
    /**
     * @param {?} node
     * @return {?}
     */
    unSelectMutipleTree(node) {
        const /** @type {?} */ allnode = this.allItems(node);
        this._nodeSlected = allnode;
    }
    /**
     * select multiple option
     * @param {?} option
     * @param {?=} $event
     * @return {?}
     */
    selectMultipleOption(option, $event) {
        /** if tags do push to tag option */
        if (this._isTags && (this._options.indexOf(option) === -1) && (this._tagsOptions.indexOf(option) === -1)) {
            this.addOption(option);
            this._tagsOptions.push(option);
        }
        this._operatingMultipleOption = option;
        if (this._selectedOptions.size < this.uiMaxMultiple) {
            this._selectedOptions.add(option);
        }
        if ($event) {
            $event.preventDefault();
            $event.stopPropagation();
        }
    }
    /**
     * emit multiple options
     * @return {?}
     */
    emitMultipleOptions() {
        if (this._isMultiInit) {
            return;
        }
        const /** @type {?} */ arrayOptions = /** @type {?} */ (Array.from(this._selectedOptions));
        this.onChange(arrayOptions.map((item) => item.uiValue));
    }
    /**
     * update selected option when add remove option etc
     * @param {?} currentModelValue
     * @param {?=} triggerByNgModel
     * @return {?}
     */
    updateSelectedOption(currentModelValue, triggerByNgModel = false) {
        if (currentModelValue === null) {
            return;
        }
        if (this.uiMultiple && currentModelValue) {
            if (Array.isArray(currentModelValue)) {
                this._selectedOptions = currentModelValue;
            }
            else {
                this._selectedOptions = [currentModelValue];
            }
            this._nodeSlected = this.allItems(this._selectedOptions);
            if (this.uiMaxTags) {
                const /** @type {?} */ amount = this._maxTags;
                if (this._selectedOptions.length > amount) {
                    const /** @type {?} */ setOptions = [];
                    for (let /** @type {?} */ i = 0; i < amount; i++) {
                        setOptions.push(this._selectedOptions[i]);
                    }
                    this._tags = setOptions;
                }
                else {
                    this._tags = this._selectedOptions;
                }
            }
        }
        else {
            this.chooseOption(currentModelValue);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    forceUpdateSelectedOption(value) {
        /** trigger dirty check */
        setTimeout(() => {
            this.updateSelectedOption(value);
        });
    }
    /**
     * @return {?}
     */
    get uiValue() {
        return this._value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiValue(value) {
        this._updateValue(value);
    }
    /**
     * @param {?=} emitChange
     * @return {?}
     */
    clearAllSelectedOption(emitChange = true) {
        if (this._selectedOptions) {
            this._selectedOptions.forEach(item => {
                this.unSelectMultipleOption(item, null, emitChange);
            });
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleKeyEnterEvent(event) {
        /** when composing end */
        if (!this._composing && this._isOpen) {
            event.preventDefault();
            event.stopPropagation();
            this.clickOption(this._activeFilterOption);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    handleKeyBackspaceEvent(event) {
        if ((!this._searchText) && (!this._composing) && (this._isMultiple)) {
            event.preventDefault();
            const /** @type {?} */ lastOption = Array.from(this._selectedOptions).pop();
            this.unSelectMultipleOption(lastOption);
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    handleKeyDownEvent($event) {
        if (this._isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this._activeFilterOption = this.nextOption(this._activeFilterOption, this._filterOptions);
            this.scrollToActive();
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    handleKeyUpEvent($event) {
        if (this._isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this._activeFilterOption = this.preOption(this._activeFilterOption, this._filterOptions);
            this.scrollToActive();
        }
    }
    /**
     * @param {?} option
     * @param {?} options
     * @return {?}
     */
    preOption(option, options) {
        return options[options.indexOf(option) - 1] || options[options.length - 1];
    }
    /**
     * @param {?} option
     * @param {?} options
     * @return {?}
     */
    nextOption(option, options) {
        return options[options.indexOf(option) + 1] || options[0];
    }
    /**
     * @return {?}
     */
    clearSearchText() {
        this._searchText = '';
    }
    /**
     * @param {?} searchValue
     * @return {?}
     */
    onSearchChange(searchValue) {
        this.uiSearchChange.emit(searchValue);
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        e.preventDefault();
        if (!this._disabled) {
            this.uiOpen = !this.uiOpen;
            if (this.uiShowSearch) {
                /** wait for search display */
                setTimeout(() => {
                    this.searchInputElementRef.nativeElement.focus();
                });
            }
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onKeyDown(e) {
        const /** @type {?} */ keyCode = e.keyCode;
        if (keyCode === TAB && this.uiOpen) {
            this.uiOpen = false;
            return;
        }
        if ((keyCode !== DOWN_ARROW && keyCode !== ENTER) || this.uiOpen) {
            return;
        }
        e.preventDefault();
        if (!this._disabled) {
            this.uiOpen = true;
            if (this.uiShowSearch) {
                /** wait for search display */
                setTimeout(() => {
                    this.searchInputElementRef.nativeElement.focus();
                });
            }
        }
    }
    /**
     * @return {?}
     */
    closeDropDown() {
        if (!this.uiOpen) {
            return;
        }
        this.onTouched();
        if (this.uiMultiple) {
            this._renderer.removeStyle(this.searchInputElementRef.nativeElement, 'width');
        }
        this.clearSearchText();
        this.uiOpen = false;
    }
    /**
     * @return {?}
     */
    setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.removeClass(this._el, _className);
        });
        this._classList = [
            this._prefixCls,
            (this._mode === 'combobox') && `${this._prefixCls}-combobox`,
            (!this._disabled) && `${this._prefixCls}-enabled`,
            (this._disabled) && `${this._prefixCls}-disabled`,
            this._isOpen && `${this._prefixCls}-open`,
            this._showSearch && `${this._prefixCls}-show-search`,
            this._size && `${this._prefixCls}-${this._size}`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this._el, _className);
        });
        this._selectionClassMap = {
            [this._selectionPrefixCls]: true,
            [`${this._selectionPrefixCls}--single`]: !this.uiMultiple,
            [`${this._selectionPrefixCls}-multiple`]: this.uiMultiple
        };
    }
    /**
     * @return {?}
     */
    setDropDownClassMap() {
        this._dropDownClassMap = {
            [this._dropDownPrefixCls]: true,
            ['component-select']: this._mode === 'combobox',
            [`${this._dropDownPrefixCls}--single`]: !this.uiMultiple,
            [`${this._dropDownPrefixCls}-multiple`]: this.uiMultiple,
            [`${this._dropDownPrefixCls}-placement-bottomLeft`]: this._dropDownPosition === 'bottom',
            [`${this._dropDownPrefixCls}-placement-topLeft`]: this._dropDownPosition === 'top'
        };
    }
    /**
     * @return {?}
     */
    scrollToActive() {
        /** wait for dropdown display */
        setTimeout(() => {
            if (this._activeFilterOption && this._activeFilterOption.uiValue) {
                const /** @type {?} */ index = this._filterOptions.findIndex(option => option.uiValue === this._activeFilterOption.uiValue);
                try {
                    const /** @type {?} */ scrollPane = this.dropdownUl.nativeElement.children[index];
                    scrollPane.scrollIntoViewIfNeeded(false);
                }
                catch (/** @type {?} */ e) {
                }
            }
        });
    }
    /**
     * @return {?}
     */
    flushComponentState() {
        if (!this.uiMultiple) {
            this.updateSelectedOption(this._value);
        }
        else {
            if (this._value) {
                this.updateSelectedOption(this._value);
            }
        }
    }
    /**
     * @return {?}
     */
    _setTriggerWidth() {
        this._triggerWidth = this._getTriggerRect().width;
    }
    /**
     * @return {?}
     */
    _getTriggerRect() {
        return this.trigger.nativeElement.getBoundingClientRect();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._updateValue(value, false);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.uiDisabled = isDisabled;
    }
    /**
     * @return {?}
     */
    setTreeMode() {
        if (!this.uiMultiple) {
            this._selectionMode = 'single';
        }
        else {
            this._selectionMode = 'checkbox';
        }
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this._value != null) {
            this.flushComponentState();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.setClassMap();
        this.setDropDownClassMap();
        this.setTreeMode();
    }
    /**
     * @return {?}
     */
    ngAfterContentChecked() {
        if (this._cacheOptions !== this._options) {
            /** update filter option after every content check cycle */
            this._cacheOptions = this._options;
        }
    }
    /**
     * @param {?} value
     * @param {?=} emitChange
     * @return {?}
     */
    _updateValue(value, emitChange = true) {
        if (this._value === value) {
            return;
        }
        if ((value == null) && this.uiMultiple) {
            this._value = [];
        }
        else {
            this._value = value;
        }
        if (!this.uiMultiple) {
            if (value == null) {
                this._selectedOption = null;
            }
            else {
                this.updateSelectedOption(value);
            }
        }
        else {
            if (value) {
                if (value.length === 0) {
                    this.clearAllSelectedOption(emitChange);
                }
                else {
                    this.updateSelectedOption(value, true);
                }
            }
            else if (value == null) {
                this.clearAllSelectedOption(emitChange);
            }
        }
    }
}
TreeSelectComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tree-select',
                template: `<div  tabindex="0" #trigger cdkOverlayOrigin #origin="cdkOverlayOrigin" [ngClass]="_selectionClassMap" (keydown.Enter)="handleKeyEnterEvent($event)"
    (keydown.Backspace)="handleKeyBackspaceEvent($event)" (keydown.ArrowUp)="handleKeyUpEvent($event)" (keydown.ArrowDown)="handleKeyDownEvent($event)">
    <div class="grf-ui-tree-select-selection__rendered" *ngIf="!uiShowSearch">
        <div class="grf-ui-tree-select-selection-selected-value">
            {{_selectedOption?.label}}
        </div>
        <div class="grf-ui-tree-select-selection__placeholder" *ngIf="!_selectedOption">
                {{uiPlaceHolder}}
            </div>
    </div>
    <div class="grf-ui-tree-select-selection__rendered" *ngIf="uiMultiple">
        <div [hidden]="_searchText||(!uiOpen&&_selectedOption)" class="grf-ui-tree-select-selection__placeholder">
            <ng-template [ngIf]="(!_composing)&&(!_selectedOption)&&(!_nodeSlected)">
                {{uiPlaceHolder}}
            </ng-template>
        </div>
        <ul *ngIf="uiMultiple && !uiMaxTags">
            <li *ngFor="let option of _selectedOptions" [@tagAnimation] [attr.title]="option?.label" class="grf-ui-tree-select-selection__choice"
                style="user-select: none;" >
                <div class="grf-ui-tree-select-selection__choice__content" >{{option?.label}}</div>
                <!---->
                <span class="grf-ui-tree-select-selection__choice__remove" (click)="unSelectMultipleOption(option,$event)">
                    <i class="fa fa-times-circle grf-ui-input-suffix-clear"></i>
                </span>
            </li>
            <li class="grf-ui-tree-select-search grf-ui-tree-select-search--inline">
                <div class="grf-ui-tree-select-search__field__wrap">
                    <input class="grf-ui-tree-select-search__field" (compositionstart)="compositionStart()" (compositionend)="compositionEnd();updateWidth(searchInput,_searchText);"
                        [(ngModel)]="_searchText" (ngModelChange)="updateFilterOption();onSearchChange($event);" (keydown)="updateWidth(searchInput,_searchText)"
                        (input)="updateWidth(searchInput,_searchText)" (blur)="onTouched()" #searchInput>
                    <span class="grf-ui-tree-select-search__field__mirror"></span>
                </div>
            </li>
        </ul>
        <ul *ngIf="uiMultiple && uiMaxTags" >
                <li *ngFor="let option of _tags" [@tagAnimation] [attr.title]="option?.label" class="grf-ui-tree-select-selection__choice"
                    style="user-select: none;" >
                    <div class="grf-ui-tree-select-selection__choice__content" >{{option?.label}}</div>
                    <!---->
                    <span class="grf-ui-tree-select-selection__choice__remove" (click)="unSelectMultipleOption(option,$event)">
                        <i class="fa fa-times-circle grf-ui-input-suffix-clear"></i>
                    </span>
                </li>
                <li class="grf-ui-tree-select-search grf-ui-tree-select-search--inline">
                    <div class="grf-ui-tree-select-search__field__wrap">
                        <input class="grf-ui-tree-select-search__field" (compositionstart)="compositionStart()" (compositionend)="compositionEnd();updateWidth(searchInput,_searchText);"
                            [(ngModel)]="_searchText" (ngModelChange)="updateFilterOption();onSearchChange($event);" (keydown)="updateWidth(searchInput,_searchText)"
                            (input)="updateWidth(searchInput,_searchText)" (blur)="onTouched()" #searchInput>
                        <span class="grf-ui-tree-select-search__field__mirror"></span>
                    </div>
                </li>
            </ul>
        <div *ngIf="!uiMultiple" class="grf-ui-tree-select-selection-selected-value" [hidden]="!(_selectedOption?.uiLabel)||uiOpen">
            {{_selectedOption?.label}}
        </div>
        <div *ngIf="!uiMultiple" [hidden]="!uiOpen" class="grf-ui-tree-select-search grf-ui-tree-select-search--inline">
            <div class="grf-ui-tree-select-search__field__wrap">
                <input class="grf-ui-tree-select-search__field" (blur)="onTouched()" (compositionstart)="compositionStart()" (compositionend)="compositionEnd()"
                    [(ngModel)]="_searchText" (ngModelChange)="updateFilterOption();onSearchChange($event);" #searchInput>
                <span class="grf-ui-tree-select-search__field__mirror"></span>
            </div>
        </div>
    </div>
    <span (click)="onTouched();clearSelect($event)" class="grf-ui-tree-select-selection__clear" style="user-select: none;" *ngIf="_selectedOption?.uiLabel&&uiAllowClear&&!uiMultiple">
        <i class="fa fa-times-circle grf-ui-input-suffix-clear"></i>
    </span>
    <span class="grf-ui-tree-select-arrow fa fa-chevron-down">
        <b></b>
    </span>
</div>
<ng-template cdkConnectedOverlay cdkConnectedOverlayHasBackdrop [cdkConnectedOverlayOrigin]="origin" (backdropClick)="closeDropDown()"
    (detach)="closeDropDown();" (positionChange)="onPositionChange($event)" [cdkConnectedOverlayWidth]="_triggerWidth" [cdkConnectedOverlayOpen]="_isOpen">
    <div [ngClass]="_dropDownClassMap" [@dropDownAnimation]="_dropDownPosition">
        <div style="overflow: auto;">
            <ul class="grf-ui-tree-select-dropdown-menu grf-ui-tree-select-dropdown-menu-vertical grf-ui-tree-select-dropdown-menu-root" #dropdownUl (scroll)="dropDownScroll(dropdownUl)">
                <li class="grf-ui-tree-select-dropdown-menu-item" (click)="clickOption(_nodeSlected,$event)">
                    <grf-tree [value]="value" [selectionMode]="_selectionMode"  [(selection)]="_nodeSlected"></grf-tree>
                </li>
            </ul>
        </div>
    </div>
</ng-template>
`,
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";@-webkit-keyframes antSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@keyframes antSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@-webkit-keyframes antSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@keyframes antSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@-webkit-keyframes antSlideDownIn{0%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}}@keyframes antSlideDownIn{0%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}100%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}}@-webkit-keyframes antSlideDownOut{0%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@keyframes antSlideDownOut{0%{opacity:1;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:100% 100%;transform-origin:100% 100%;-webkit-transform:scaleY(.8);transform:scaleY(.8)}}@-webkit-keyframes antSlideLeftIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@keyframes antSlideLeftIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@-webkit-keyframes antSlideLeftOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@keyframes antSlideLeftOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@-webkit-keyframes antSlideRightIn{0%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@keyframes antSlideRightIn{0%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}100%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}}@-webkit-keyframes antSlideRightOut{0%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}@keyframes antSlideRightOut{0%{opacity:1;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(1);transform:scaleX(1)}100%{opacity:0;-webkit-transform-origin:100% 0;transform-origin:100% 0;-webkit-transform:scaleX(.8);transform:scaleX(.8)}}.grf-ui-tree-select{-webkit-box-sizing:border-box;box-sizing:border-box;display:inline-block;position:relative;color:rgba(0,0,0,.65);font-size:14px}.grf-ui-tree-select>ul>li>a{padding:0;background-color:#fff}.grf-ui-tree-select-arrow{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;position:absolute;top:50%;right:8px;line-height:1;margin-top:-6px}.grf-ui-tree-select-arrow:before{display:block;font-family:FontAwesome!important;color:#cac7c7;-webkit-transition:-webkit-transform .2s ease;transition:-webkit-transform .2s ease;transition:transform .2s ease;transition:transform .2s ease,-webkit-transform .2s ease}.grf-ui-tree-select-arrow *{display:none}.grf-ui-tree-select-selection{outline:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;-webkit-box-sizing:border-box;box-sizing:border-box;display:block;background-color:#fff;border-radius:4px;border:1px solid #d9d9d9;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tree-select-selection-focused .grf-ui-tree-select-selection,.grf-ui-tree-select-selection:active,.grf-ui-tree-select-selection:focus{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-tree-select-selection__clear{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;text-rendering:auto;opacity:0;position:absolute;right:8px;z-index:1;background:#fff;top:50%;font-size:14px;color:rgba(0,0,0,.25);width:12px;height:12px;margin-top:-6px;line-height:12px;cursor:pointer;-webkit-transition:color .3s ease,opacity .15s ease;transition:color .3s ease,opacity .15s ease}.grf-ui-tree-select-selection__clear:before{display:block}.grf-ui-tree-select-selection__clear:hover{color:rgba(0,0,0,.43)}.grf-ui-tree-select-selection:hover .grf-ui-tree-select-selection__clear{opacity:1}.grf-ui-tree-select-selection-selected-value{float:left;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;max-width:100%;padding-right:14px;font-size:14px}.grf-ui-tree-select-disabled{color:rgba(0,0,0,.25)}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection{background:#f7f7f7;cursor:not-allowed}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection:active,.grf-ui-tree-select-disabled .grf-ui-tree-select-selection:focus,.grf-ui-tree-select-disabled .grf-ui-tree-select-selection:hover{border-color:#d9d9d9;-webkit-box-shadow:none;box-shadow:none}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection__clear{display:none;visibility:hidden;pointer-events:none}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__choice{background:#eee;color:#aaa;padding-right:10px}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__choice__remove{display:none}.grf-ui-tree-select-selection--single{height:28px;position:relative;cursor:pointer}.grf-ui-tree-select-selection__rendered{display:block;margin-left:7px;margin-right:7px;position:relative;line-height:26px}.grf-ui-tree-select-selection__rendered:after{content:'.';visibility:hidden;pointer-events:none;display:inline-block;width:0}.grf-ui-tree-select-lg .grf-ui-tree-select-selection--single{height:32px}.grf-ui-tree-select-lg .grf-ui-tree-select-selection__rendered{line-height:30px}.grf-ui-tree-select-lg .grf-ui-tree-select-selection--multiple{min-height:32px}.grf-ui-tree-select-lg .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__rendered li{height:24px;line-height:24px}.grf-ui-tree-select-lg .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__clear{top:16px}.grf-ui-tree-select-sm .grf-ui-tree-select-selection--single{height:22px}.grf-ui-tree-select-sm .grf-ui-tree-select-selection__rendered{line-height:20px}.grf-ui-tree-select-sm .grf-ui-tree-select-selection--multiple{min-height:22px}.grf-ui-tree-select-sm .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__rendered li{height:14px;line-height:14px}.grf-ui-tree-select-sm .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__clear{top:11px}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection__choice__remove{color:rgba(0,0,0,.25);cursor:default}.grf-ui-tree-select-disabled .grf-ui-tree-select-selection__choice__remove:hover{color:rgba(0,0,0,.25)}.grf-ui-tree-select-search__field__wrap{display:inline-block;position:relative}.grf-ui-tree-select-search__field__placeholder,.grf-ui-tree-select-selection__placeholder{position:absolute;top:50%;left:0;right:9px;color:rgba(0,0,0,.25);line-height:20px;height:20px;max-width:100%;margin-top:-10px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;text-align:left}.grf-ui-tree-select-search__field__placeholder{left:8px}.grf-ui-tree-select-search--inline{position:absolute;height:100%;width:100%}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-search--inline{float:left;position:static}.grf-ui-tree-select-search--inline .grf-ui-tree-select-search__field__wrap{width:100%;height:100%}.grf-ui-tree-select-search--inline .grf-ui-tree-select-search__field{border-width:0;font-size:100%;height:100%;width:100%;background:0 0;outline:0;border-radius:4px}.grf-ui-tree-select-search--inline .grf-ui-tree-select-search__field__mirror{position:absolute;top:0;left:-9999px;white-space:pre;pointer-events:none}.grf-ui-tree-select-search--inline>i{float:right}.grf-ui-tree-select-selection-multiple{min-height:28px;cursor:text;padding-bottom:3px;zoom:1}.grf-ui-tree-select-selection-multiple:after,.grf-ui-tree-select-selection-multiple:before{content:" ";display:table}.grf-ui-tree-select-selection-multiple:after{clear:both;visibility:hidden;font-size:0;height:0}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-search--inline{width:auto;padding:0;max-width:100%}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-search--inline .grf-ui-tree-select-search__field{max-width:100%;width:.75em}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__rendered{margin-left:5px;margin-bottom:-3px;height:auto}.grf-ui-tree-select-selection-multiple ul{list-style:none;margin:0;padding:0}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__rendered>ul>li,.grf-ui-tree-select-selection-multiple>ul>li{margin-top:3px;height:24px;line-height:24px;display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__choice{color:rgba(0,0,0,.65);background-color:#f3f3f3;border-radius:4px;cursor:default;float:left;margin-right:4px;max-width:99%;position:relative;overflow:hidden;-webkit-transition:padding .3s cubic-bezier(.645,.045,.355,1);transition:padding .3s cubic-bezier(.645,.045,.355,1);padding:0 10px;border:1px solid #ccc}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__choice__disabled{padding:0 10px}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__choice__content{display:inline-block;white-space:nowrap;overflow:hidden;text-overflow:ellipsis;max-width:100%;-webkit-transition:margin .3s cubic-bezier(.645,.045,.355,1);transition:margin .3s cubic-bezier(.645,.045,.355,1)}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__choice__remove{color:rgba(0,0,0,.43);line-height:inherit;cursor:pointer;display:inline-block;font-weight:700;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1);padding:0 0 0 5px}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__choice__remove:hover{color:#404040}.grf-ui-tree-select-selection-multiple .grf-ui-tree-select-selection__clear{top:14px}.grf-ui-tree-select-allow-clear .grf-ui-tree-select-selection--multiple .grf-ui-tree-select-selection__rendered{margin-right:20px}.grf-ui-tree-select-open .grf-ui-tree-select-arrow{-ms-transform:rotate(180deg)}.grf-ui-tree-select-open .grf-ui-tree-select-arrow:before{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.grf-ui-tree-select-open .grf-ui-tree-select-selection{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-tree-select-combobox .grf-ui-tree-select-arrow{display:none}.grf-ui-tree-select-combobox .grf-ui-tree-select-search--inline{height:100%;width:100%;float:none}.grf-ui-tree-select-combobox .grf-ui-tree-select-search__field__wrap{width:100%;height:100%}.grf-ui-tree-select-combobox .grf-ui-tree-select-search__field{width:100%;height:100%;position:relative;z-index:1;-webkit-transition:all .3s cubic-bezier(.645,.045,.355,1);transition:all .3s cubic-bezier(.645,.045,.355,1);-webkit-box-shadow:none;box-shadow:none}.grf-ui-tree-select-combobox .grf-ui-tree-select-allow-clear .grf-ui-tree-select-selection:hover .grf-ui-tree-select-selection__rendered{margin-right:20px}.grf-ui-tree-select-dropdown{top:100%;left:0;width:100%;margin-top:2px;margin-bottom:2px;background-color:#fff;-webkit-box-shadow:0 1px 6px rgba(0,0,0,.2);box-shadow:0 1px 6px rgba(0,0,0,.2);border-radius:4px;-webkit-box-sizing:border-box;box-sizing:border-box;z-index:5;position:absolute;outline:0;overflow:hidden;font-size:14px}.grf-ui-tree-select-dropdown .grf-ui-item-selected{display:none}.grf-ui-tree-select-dropdown .grf-ui-tree-select-dropdown-placement-topLeft{top:-100%;-webkit-transform:translateY(-100%)!important;transform:translateY(-100%)!important}.grf-ui-tree-select-dropdown.slide-up-appear .slide-up-appear-active .grf-ui-tree-select-dropdown-placement-bottomLeft,.grf-ui-tree-select-dropdown.slide-up-enter .slide-up-enter-active .grf-ui-tree-select-dropdown-placement-bottomLeft{-webkit-animation-name:antSlideUpIn;animation-name:antSlideUpIn}.grf-ui-tree-select-dropdown.slide-up-appear .slide-up-appear-active .grf-ui-tree-select-dropdown-placement-topLeft,.grf-ui-tree-select-dropdown.slide-up-enter .slide-up-enter-active .grf-ui-tree-select-dropdown-placement-topLeft{-webkit-animation-name:antSlideDownIn;animation-name:antSlideDownIn}.grf-ui-tree-select-dropdown.slide-up-leave .slide-up-leave-active .grf-ui-tree-select-dropdown-placement-bottomLeft{-webkit-animation-name:antSlideUpOut;animation-name:antSlideUpOut}.grf-ui-tree-select-dropdown.slide-up-leave .slide-up-leave-active .grf-ui-tree-select-dropdown-placement-topLeft{-webkit-animation-name:antSlideDownOut;animation-name:antSlideDownOut}.grf-ui-tree-select-dropdown-hidden{display:none}.grf-ui-tree-select-dropdown-menu{outline:0;margin:0;padding-left:0;list-style:none;max-height:250px;overflow:auto}.grf-ui-tree-select-dropdown-menu-item-group-list{margin:0;padding:0}.grf-ui-tree-select-dropdown-menu-item-group-list>.grf-ui-tree-select-dropdown-menu-item{padding-left:16px}.grf-ui-tree-select-dropdown-menu-item-group-title{color:rgba(0,0,0,.43);line-height:1.5;padding:8px}.grf-ui-tree-select-dropdown-menu-item{line-height:1.5;position:relative;display:block;padding:7px 8px;font-weight:400;color:rgba(0,0,0,.65);white-space:nowrap;cursor:pointer;overflow:hidden;-webkit-transition:background .3s ease;transition:background .3s ease}.grf-ui-tree-select-dropdown-menu-item-disabled{color:rgba(0,0,0,.25);cursor:not-allowed}.grf-ui-tree-select-dropdown-menu-item-disabled:hover{color:rgba(0,0,0,.25);background-color:#fff;cursor:not-allowed}.grf-ui-tree-select-dropdown-menu-item-selected,.grf-ui-tree-select-dropdown-menu-item-selected:hover{font-weight:700;color:rgba(0,0,0,.65)}.grf-ui-tree-select-dropdown-menu-item-divider{height:1px;margin:1px 0;overflow:hidden;background-color:#e5e5e5;line-height:0}.grf-ui-tree-select-dropdown-multiple .grf-ui-tree-select-dropdown-menu-item .grf-ui-item-selected{color:transparent;-webkit-transition:all .2s ease;transition:all .2s ease;position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);right:8px;font-weight:700;text-shadow:0 .1px 0,.1px 0 0,0 -.1px 0,-.1px 0}.grf-ui-tree-select-dropdown-multiple .grf-ui-tree-select-dropdown-menu-item:hover .grf-ui-item-selected{color:#ddd}.grf-ui-tree-select-dropdown-multiple .grf-ui-tree-select-dropdown-menu-item-disabled .grf-ui-item-selected{display:none}.grf-ui-tree-select-dropdown-multiple .grf-ui-tree-select-dropdown-menu-item-selected .grf-ui-item-selected,.grf-ui-tree-select-dropdown-multiple .grf-ui-tree-select-dropdown-menu-item-selected:hover .grf-ui-item-selected{color:#0275d8;display:inline-block}.grf-ui-tree-select-dropdown-container-open .grf-ui-tree-select-dropdown,.grf-ui-tree-select-dropdown-open .grf-ui-tree-select-dropdown{display:block}.cdk-overlay-dark-backdrop{background:0 0}`],
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => TreeSelectComponent),
                        multi: true
                    }
                ],
                animations: [
                    trigger('dropDownAnimation', [
                        state('bottom', style({
                            opacity: 1,
                            transform: 'scaleY(1)',
                            transformOrigin: '0% 0%'
                        })),
                        transition('void => bottom', [
                            style({
                                opacity: 0,
                                transform: 'scaleY(0)',
                                transformOrigin: '0% 0%'
                            }),
                            animate('150ms cubic-bezier(0.25, 0.8, 0.25, 1)')
                        ]),
                        state('top', style({
                            opacity: 1,
                            transform: 'scaleY(1)',
                            transformOrigin: '0% 100%'
                        })),
                        transition('void => top', [
                            style({
                                opacity: 0,
                                transform: 'scaleY(0)',
                                transformOrigin: '0% 100%'
                            }),
                            animate('150ms cubic-bezier(0.25, 0.8, 0.25, 1)')
                        ]),
                        transition('* => void', [
                            animate('250ms 100ms linear', style({ opacity: 0 }))
                        ])
                    ]),
                    trigger('tagAnimation', [
                        state('*', style({ opacity: 1, transform: 'scale(1)' })),
                        transition('void => *', [
                            style({ opacity: 0, transform: 'scale(0)' }),
                            animate('150ms linear')
                        ]),
                        state('void', style({ opacity: 0, transform: 'scale(0)' })),
                        transition('* => void', [
                            style({ opacity: 1, transform: 'scale(1)' }),
                            animate('150ms linear')
                        ])
                    ])
                ]
            },] },
];
/** @nocollapse */
TreeSelectComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
TreeSelectComponent.propDecorators = {
    "searchInputElementRef": [{ type: ViewChild, args: ['searchInput',] },],
    "trigger": [{ type: ViewChild, args: ['trigger',] },],
    "dropdownUl": [{ type: ViewChild, args: ['dropdownUl',] },],
    "uiSearchChange": [{ type: Output },],
    "uiOpenChange": [{ type: Output },],
    "uiScrollToBottom": [{ type: Output },],
    "uiFilter": [{ type: Input },],
    "uiMaxMultiple": [{ type: Input },],
    "value": [{ type: Input },],
    "uiMaxTags": [{ type: Input },],
    "uiAllowClear": [{ type: Input },],
    "uiKeepUnListOptions": [{ type: Input },],
    "uiMode": [{ type: Input },],
    "uiMultiple": [{ type: Input },],
    "uiPlaceHolder": [{ type: Input },],
    "uiNotFoundContent": [{ type: Input },],
    "uiSize": [{ type: Input },],
    "uiShowSearch": [{ type: Input },],
    "uiTags": [{ type: Input },],
    "uiDisabled": [{ type: Input },],
    "uiOpen": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
    "onKeyDown": [{ type: HostListener, args: ['keydown', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeDragDropService {
    constructor() {
        this.dragStartSource = new Subject();
        this.dragStopSource = new Subject();
        this.dragStart$ = this.dragStartSource.asObservable();
        this.dragStop$ = this.dragStopSource.asObservable();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    startDrag(event) {
        this.dragStartSource.next(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    stopDrag(event) {
        this.dragStopSource.next(event);
    }
}
TreeDragDropService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
TreeDragDropService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeComponent {
    /**
     * @param {?} el
     * @param {?} dragDropService
     */
    constructor(el, dragDropService) {
        this.el = el;
        this.dragDropService = dragDropService;
        this.selectionChange = new EventEmitter();
        this.onNodeSelect = new EventEmitter();
        this.onNodeUnselect = new EventEmitter();
        this.onNodeExpand = new EventEmitter();
        this.onNodeCollapse = new EventEmitter();
        this.onNodeContextMenuSelect = new EventEmitter();
        this.onNodeDrop = new EventEmitter();
        this.layout = 'vertical';
        this.metaKeySelection = true;
        this.propagateSelectionUp = true;
        this.propagateSelectionDown = true;
        this.loadingIcon = 'fa-circle-o-notch';
        this.emptyMessage = 'No records found';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.droppableNodes) {
            this.dragStartSubscription = this.dragDropService.dragStart$.subscribe(event => {
                this.dragNodeTree = event.tree;
                this.dragNode = event.node;
                this.dragNodeSubNodes = event.subNodes;
                this.dragNodeIndex = event.index;
                this.dragNodeScope = event.scope;
            });
            this.dragStopSubscription = this.dragDropService.dragStop$.subscribe(event => {
                this.dragNodeTree = null;
                this.dragNode = null;
                this.dragNodeSubNodes = null;
                this.dragNodeIndex = null;
                this.dragNodeScope = null;
                this.dragHover = false;
            });
        }
    }
    /**
     * @return {?}
     */
    get horizontal() {
        return this.layout === 'horizontal';
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        if (this.templates.length) {
            this.templateMap = {};
        }
        this.templates.forEach((item) => {
            this.templateMap[item.name] = item.template;
        });
    }
    /**
     * @param {?} event
     * @param {?} node
     * @return {?}
     */
    onNodeClick(event, node) {
        const /** @type {?} */ eventTarget = (/** @type {?} */ (event.target));
        if (eventTarget.className && eventTarget.className.indexOf('grf-ui-tree-toggler') === 0) {
            return;
        }
        else if (this.selectionMode) {
            if (node.selectable === false) {
                return;
            }
            const /** @type {?} */ index = this.findIndexInSelection(node);
            const /** @type {?} */ selected = (index >= 0);
            if (this.isCheckboxSelectionMode()) {
                if (selected) {
                    if (this.propagateSelectionDown) {
                        this.propagateDown(node, false);
                    }
                    else {
                        this.selection = this.selection.filter((val, i) => i !== index);
                    }
                    if (this.propagateSelectionUp && node.parent) {
                        this.propagateUp(node.parent, false);
                    }
                    this.selectionChange.emit(this.selection);
                    this.onNodeUnselect.emit({ originalEvent: event, node: node });
                }
                else {
                    if (this.propagateSelectionDown) {
                        this.propagateDown(node, true);
                    }
                    else {
                        this.selection = [...this.selection || [], node];
                    }
                    if (this.propagateSelectionUp && node.parent) {
                        this.propagateUp(node.parent, true);
                    }
                    this.selectionChange.emit(this.selection);
                    this.onNodeSelect.emit({ originalEvent: event, node: node });
                }
            }
            else {
                const /** @type {?} */ metaSelection = this.nodeTouched ? false : this.metaKeySelection;
                if (metaSelection) {
                    const /** @type {?} */ metaKey = (event.metaKey || event.ctrlKey);
                    if (selected && metaKey) {
                        if (this.isSingleSelectionMode()) {
                            this.selectionChange.emit(null);
                        }
                        else {
                            this.selection = this.selection.filter((val, i) => i !== index);
                            this.selectionChange.emit(this.selection);
                        }
                        this.onNodeUnselect.emit({ originalEvent: event, node: node });
                    }
                    else {
                        if (this.isSingleSelectionMode()) {
                            this.selectionChange.emit(node);
                        }
                        else if (this.isMultipleSelectionMode()) {
                            this.selection = (!metaKey) ? [] : this.selection || [];
                            this.selection = [...this.selection, node];
                            this.selectionChange.emit(this.selection);
                        }
                        this.onNodeSelect.emit({ originalEvent: event, node: node });
                    }
                }
                else {
                    if (this.isSingleSelectionMode()) {
                        if (selected) {
                            this.selection = null;
                            this.onNodeUnselect.emit({ originalEvent: event, node: node });
                        }
                        else {
                            this.selection = node;
                            this.onNodeSelect.emit({ originalEvent: event, node: node });
                        }
                    }
                    else {
                        if (selected) {
                            this.selection = this.selection.filter((val, i) => i !== index);
                            this.onNodeUnselect.emit({ originalEvent: event, node: node });
                        }
                        else {
                            this.selection = [...this.selection || [], node];
                            this.onNodeSelect.emit({ originalEvent: event, node: node });
                        }
                    }
                    this.selectionChange.emit(this.selection);
                }
            }
        }
        this.nodeTouched = false;
    }
    /**
     * @return {?}
     */
    onNodeTouchEnd() {
        this.nodeTouched = true;
    }
    /**
     * @param {?} event
     * @param {?} node
     * @return {?}
     */
    onNodeRightClick(event, node) {
        if (this.contextMenu) {
            const /** @type {?} */ eventTarget = (/** @type {?} */ (event.target));
            if (eventTarget.className && eventTarget.className.indexOf('grf-ui-tree-toggler') === 0) {
                return;
            }
            else {
                const /** @type {?} */ index = this.findIndexInSelection(node);
                const /** @type {?} */ selected = (index >= 0);
                if (!selected) {
                    if (this.isSingleSelectionMode()) {
                        this.selectionChange.emit(node);
                    }
                    else {
                        this.selectionChange.emit([node]);
                    }
                }
                this.contextMenu.show(event);
                this.onNodeContextMenuSelect.emit({ originalEvent: event, node: node });
            }
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    findIndexInSelection(node) {
        let /** @type {?} */ index = -1;
        if (this.selectionMode && this.selection) {
            if (this.isSingleSelectionMode()) {
                index = (this.selection === node) ? 0 : -1;
            }
            else {
                for (let /** @type {?} */ i = 0; i < this.selection.length; i++) {
                    if (this.selection[i] === node) {
                        index = i;
                        break;
                    }
                }
            }
        }
        return index;
    }
    /**
     * @param {?} node
     * @param {?} select
     * @return {?}
     */
    propagateUp(node, select) {
        if (node.children && node.children.length) {
            let /** @type {?} */ selectedCount = 0;
            let /** @type {?} */ childPartialSelected = false;
            for (const /** @type {?} */ child of node.children) {
                if (this.isSelected(child)) {
                    selectedCount++;
                }
                else if (child.partialSelected) {
                    childPartialSelected = true;
                }
            }
            if (select && selectedCount === node.children.length) {
                this.selection = [...this.selection || [], node];
                node.partialSelected = false;
            }
            else {
                if (!select) {
                    const /** @type {?} */ index = this.findIndexInSelection(node);
                    if (index >= 0) {
                        this.selection = this.selection.filter((val, i) => i !== index);
                    }
                }
                if (childPartialSelected || selectedCount > 0 && selectedCount !== node.children.length) {
                    node.partialSelected = true;
                }
                else {
                    node.partialSelected = false;
                }
            }
        }
        const /** @type {?} */ parent = node.parent;
        if (parent) {
            this.propagateUp(parent, select);
        }
    }
    /**
     * @param {?} node
     * @param {?} select
     * @return {?}
     */
    propagateDown(node, select) {
        const /** @type {?} */ index = this.findIndexInSelection(node);
        if (select && index === -1) {
            this.selection = [...this.selection || [], node];
        }
        else if (!select && index > -1) {
            this.selection = this.selection.filter((val, i) => i !== index);
        }
        node.partialSelected = false;
        if (node.children && node.children.length) {
            for (const /** @type {?} */ child of node.children) {
                this.propagateDown(child, select);
            }
        }
    }
    /**
     * @param {?} node
     * @return {?}
     */
    isSelected(node) {
        return this.findIndexInSelection(node) !== -1;
    }
    /**
     * @return {?}
     */
    isSingleSelectionMode() {
        return this.selectionMode && this.selectionMode === 'single';
    }
    /**
     * @return {?}
     */
    isMultipleSelectionMode() {
        return this.selectionMode && this.selectionMode === 'multiple';
    }
    /**
     * @return {?}
     */
    isCheckboxSelectionMode() {
        return this.selectionMode && this.selectionMode === 'checkbox';
    }
    /**
     * @param {?} node
     * @return {?}
     */
    getTemplateForNode(node) {
        if (this.templateMap) {
            return node.type ? this.templateMap[node.type] : this.templateMap['default'];
        }
        else {
            return null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragOver(event) {
        if (this.droppableNodes && (!this.value || this.value.length === 0)) {
            event.dataTransfer.dropEffect = 'move';
            event.preventDefault();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDrop(event) {
        if (this.droppableNodes && (!this.value || this.value.length === 0)) {
            event.preventDefault();
            const /** @type {?} */ dragNode = this.dragNode;
            if (this.allowDrop(dragNode, null, this.dragNodeScope)) {
                const /** @type {?} */ dragNodeIndex = this.dragNodeIndex;
                this.dragNodeSubNodes.splice(dragNodeIndex, 1);
                this.value = this.value || [];
                this.value.push(dragNode);
                this.dragDropService.stopDrag({
                    node: dragNode
                });
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragEnter(event) {
        if (this.droppableNodes && this.allowDrop(this.dragNode, null, this.dragNodeScope)) {
            this.dragHover = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragLeave(event) {
        if (this.droppableNodes) {
            const /** @type {?} */ rect = event.currentTarget.getBoundingClientRect();
            if (event.x > rect.left + rect.width || event.x < rect.left || event.y > rect.top + rect.height || event.y < rect.top) {
                this.dragHover = false;
            }
        }
    }
    /**
     * @param {?} dragNode
     * @param {?} dropNode
     * @param {?} dragNodeScope
     * @return {?}
     */
    allowDrop(dragNode, dropNode, dragNodeScope) {
        if (!dragNode) {
            return false;
        }
        else if (this.isValidDragScope(dragNodeScope)) {
            let /** @type {?} */ allow = true;
            if (dropNode) {
                if (dragNode === dropNode) {
                    allow = false;
                }
                else {
                    let /** @type {?} */ parent = dropNode.parent;
                    while (parent != null) {
                        if (parent === dragNode) {
                            allow = false;
                            break;
                        }
                        parent = parent.parent;
                    }
                }
            }
            return allow;
        }
        else {
            return false;
        }
    }
    /**
     * @param {?} dragScope
     * @return {?}
     */
    isValidDragScope(dragScope) {
        const /** @type {?} */ dropScope = this.droppableScope;
        if (dropScope) {
            if (typeof dropScope === 'string') {
                if (typeof dragScope === 'string') {
                    return dropScope === dragScope;
                }
                else if (dragScope instanceof Array) {
                    return (/** @type {?} */ (dragScope)).indexOf(dropScope) !== -1;
                }
            }
            else if (dropScope instanceof Array) {
                if (typeof dragScope === 'string') {
                    return (/** @type {?} */ (dropScope)).indexOf(dragScope) !== -1;
                }
                else if (dragScope instanceof Array) {
                    for (const /** @type {?} */ s of dropScope) {
                        for (const /** @type {?} */ ds of dragScope) {
                            if (s === ds) {
                                return true;
                            }
                        }
                    }
                }
            }
            return false;
        }
        else {
            return true;
        }
    }
    /**
     * @return {?}
     */
    getBlockableElement() {
        return this.el.nativeElement.children[0];
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.dragStartSubscription) {
            this.dragStartSubscription.unsubscribe();
        }
        if (this.dragStopSubscription) {
            this.dragStopSubscription.unsubscribe();
        }
    }
}
TreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tree',
                template: `<div [ngClass]="{'grf-ui-tree grf-ui-widget grf-ui-corner-all':true,'grf-ui-tree-selectable':selectionMode,'grf-ui-treenode-dragover':dragHover,'grf-ui-tree-loading': loading}" [ngStyle]="style" [class]="styleClass" *ngIf="!horizontal" (drop)="onDrop($event)"
    (dragover)="onDragOver($event)" (dragenter)="onDragEnter($event)" (dragleave)="onDragLeave($event)">
    <div class="grf-ui-tree-loading-mask grf-ui-widget-overlay" *ngIf="loading"></div>
    <div class="grf-ui-tree-loading-content" *ngIf="loading">
        <i [class]="'fa fa-spin fa-2x ' + loadingIcon"></i>
    </div>
    <ul class="grf-ui-tree-container" *ngIf="value">
        <grf-treenode *ngFor="let node of value;let firstChild=first;let lastChild=last; let index=index" [node]="node" [firstChild]="firstChild" [lastChild]="lastChild" [index]="index"></grf-treenode>
    </ul>
    <div class="grf-ui-tree-empty-message" *ngIf="!loading && !value">{{emptyMessage}}</div>
</div>
<div [ngClass]="{'grf-ui-tree grf-ui-tree-horizontal grf-ui-widget grf-ui-corner-all':true,'grf-ui-tree-selectable':selectionMode}" [ngStyle]="style" [class]="styleClass" *ngIf="horizontal">
    <div class="grf-ui-tree-loading grf-ui-widget-overlay" *ngIf="loading"></div>
    <div class="grf-ui-tree-loading-content" *ngIf="loading">
        <i [class]="'fa fa-spin fa-2x ' + loadingIcon"></i>
    </div>
    <table *ngIf="value&&value[0]">
        <grf-treenode [node]="value[0]" [root]="true"></grf-treenode>
    </table>
    <div class="grf-ui-tree-empty-message" *ngIf="!loading && !value">{{emptyMessage}}</div>
</div>`,
                styles: [`@charset "UTF-8";@-webkit-keyframes treenodeActive{0%{background-color:#fff;color:#0275d8}100%{background-color:#f9f9f9c7;color:#0275d8}}@keyframes treenodeActive{0%{background-color:#fff;color:#0275d8}100%{background-color:#f9f9f9c7;color:#0275d8}}.grf-ui-tree{width:100%}.grf-ui-tree .grf-ui-tree-container{height:100%;margin:0;overflow:auto;padding:.25em;white-space:nowrap;overflow-y:hidden}.grf-ui-treenode-content{cursor:pointer}.grf-ui-treenode-label:not(.grf-ui-state-highlight):hover{background-color:#fff;color:#0275d8}.grf-ui-treenode-label.grf-ui-state-highlight{background-color:#f9f9f9c7;color:#0275d8;-webkit-animation:.5s treenodeActive;animation:.5s treenodeActive}.grf-ui-treenode .grf-ui-tree-toggler{color:#525252;font-size:14px}.grf-ui-treenode-label{font-size:14px;vertical-align:middle;display:inline-block;color:#525252;padding:2px 12px 2px 2px;margin:0 2px;border-radius:4px}.grf-ui-treenode-icon{vertical-align:middle;display:inline-block;font-size:14px;color:#525252}.grf-ui-treenode ul{list-style:none;padding-left:25px}.grf-ui-treenode .grf-ui-chkbox{display:inline-block;cursor:pointer;vertical-align:bottom;margin-right:.25em}.grf-ui-treenode-droppoint{height:0}`]
            },] },
];
/** @nocollapse */
TreeComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: TreeDragDropService, decorators: [{ type: Optional },] },
];
TreeComponent.propDecorators = {
    "value": [{ type: Input },],
    "selectionMode": [{ type: Input },],
    "selection": [{ type: Input },],
    "selectionChange": [{ type: Output },],
    "onNodeSelect": [{ type: Output },],
    "onNodeUnselect": [{ type: Output },],
    "onNodeExpand": [{ type: Output },],
    "onNodeCollapse": [{ type: Output },],
    "onNodeContextMenuSelect": [{ type: Output },],
    "onNodeDrop": [{ type: Output },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "contextMenu": [{ type: Input },],
    "layout": [{ type: Input },],
    "draggableScope": [{ type: Input },],
    "droppableScope": [{ type: Input },],
    "draggableNodes": [{ type: Input },],
    "droppableNodes": [{ type: Input },],
    "metaKeySelection": [{ type: Input },],
    "propagateSelectionUp": [{ type: Input },],
    "propagateSelectionDown": [{ type: Input },],
    "loading": [{ type: Input },],
    "loadingIcon": [{ type: Input },],
    "emptyMessage": [{ type: Input },],
    "templates": [{ type: ContentChildren, args: [TemplateDirective,] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeNodeComponent {
    /**
     * @param {?} tree
     */
    constructor(tree) {
        this.tree = tree;
        this.checkBoxNode = false;
        this.notAll = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.node.parent = this.parentNode;
    }
    /**
     * @return {?}
     */
    getIcon() {
        let /** @type {?} */ icon;
        if (this.node.icon) {
            icon = this.node.icon;
        }
        else {
            icon = this.node.expanded && this.node.children && this.node.children.length ? this.node.expandedIcon : this.node.collapsedIcon;
        }
        return TreeNodeComponent.ICON_CLASS + ' ' + icon;
    }
    /**
     * @return {?}
     */
    isLeaf() {
        return this.node.leaf === false ? false : !(this.node.children && this.node.children.length);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.node.expanded) {
            this.tree.onNodeCollapse.emit({ originalEvent: event, node: this.node });
        }
        else {
            this.tree.onNodeExpand.emit({ originalEvent: event, node: this.node });
        }
        this.node.expanded = !this.node.expanded;
    }
    /**
     * @param {?} event
     * @param {?} node
     * @return {?}
     */
    onNodeClick(event, node) {
        this.tree.onNodeClick(event, this.node);
    }
    /**
     * @return {?}
     */
    onNodeTouchEnd() {
        this.tree.onNodeTouchEnd();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onNodeRightClick(event) {
        this.tree.onNodeRightClick(event, this.node);
    }
    /**
     * @return {?}
     */
    isSelected() {
        if (this.node.partialSelected && !this.tree.isSelected(this.node)) {
            this.checkBoxNode = false;
            this.notAll = true;
            return this.tree.isSelected(this.node);
        }
        if (!this.tree.isSelected(this.node)) {
            this.checkBoxNode = false;
            this.notAll = false;
            return this.tree.isSelected(this.node);
        }
        else if (this.tree.isSelected(this.node)) {
            this.checkBoxNode = true;
            this.notAll = false;
            return this.tree.isSelected(this.node);
        }
    }
    /**
     * @param {?} event
     * @param {?} position
     * @return {?}
     */
    onDropPoint(event, position) {
        event.preventDefault();
        const /** @type {?} */ dragNode = this.tree.dragNode;
        const /** @type {?} */ dragNodeIndex = this.tree.dragNodeIndex;
        const /** @type {?} */ dragNodeScope = this.tree.dragNodeScope;
        const /** @type {?} */ isValidDropPointIndex = this.tree.dragNodeTree === this.tree ? (position === 1 || dragNodeIndex !== this.index - 1) : true;
        if (this.tree.allowDrop(dragNode, this.node, dragNodeScope) && isValidDropPointIndex) {
            const /** @type {?} */ newNodeList = this.node.parent ? this.node.parent.children : this.tree.value;
            this.tree.dragNodeSubNodes.splice(dragNodeIndex, 1);
            let /** @type {?} */ dropIndex = this.index;
            if (position < 0) {
                dropIndex = (this.tree.dragNodeSubNodes === newNodeList) ? ((this.tree.dragNodeIndex > this.index) ? this.index : this.index - 1) : this.index;
                newNodeList.splice(dropIndex, 0, dragNode);
            }
            else {
                dropIndex = newNodeList.length;
                newNodeList.push(dragNode);
            }
            this.tree.dragDropService.stopDrag({
                node: dragNode,
                subNodes: this.node.parent ? this.node.parent.children : this.tree.value,
                index: dragNodeIndex
            });
            this.tree.onNodeDrop.emit({
                originalEvent: event,
                dragNode: dragNode,
                dropNode: this.node,
                dropIndex: dropIndex
            });
        }
        this.draghoverPrev = false;
        this.draghoverNext = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropPointDragOver(event) {
        event.dataTransfer.dropEffect = 'move';
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @param {?} position
     * @return {?}
     */
    onDropPointDragEnter(event, position) {
        if (this.tree.allowDrop(this.tree.dragNode, this.node, this.tree.dragNodeScope)) {
            if (position < 0) {
                this.draghoverPrev = true;
            }
            else {
                this.draghoverNext = true;
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropPointDragLeave(event) {
        this.draghoverPrev = false;
        this.draghoverNext = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStart(event) {
        if (this.tree.draggableNodes && this.node.draggable !== false) {
            event.dataTransfer.setData('text', 'data');
            this.tree.dragDropService.startDrag({
                tree: this,
                node: this.node,
                subNodes: this.node.parent ? this.node.parent.children : this.tree.value,
                index: this.index,
                scope: this.tree.draggableScope
            });
        }
        else {
            event.preventDefault();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStop(event) {
        this.tree.dragDropService.stopDrag({
            node: this.node,
            subNodes: this.node.parent ? this.node.parent.children : this.tree.value,
            index: this.index
        });
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropNodeDragOver(event) {
        event.dataTransfer.dropEffect = 'move';
        if (this.tree.droppableNodes) {
            event.preventDefault();
            event.stopPropagation();
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropNode(event) {
        if (this.tree.droppableNodes && this.node.droppable !== false) {
            event.preventDefault();
            event.stopPropagation();
            const /** @type {?} */ dragNode = this.tree.dragNode;
            if (this.tree.allowDrop(dragNode, this.node, this.tree.dragNodeScope)) {
                const /** @type {?} */ dragNodeIndex = this.tree.dragNodeIndex;
                this.tree.dragNodeSubNodes.splice(dragNodeIndex, 1);
                if (this.node.children) {
                    this.node.children.push(dragNode);
                }
                else {
                    this.node.children = [dragNode];
                }
                this.tree.dragDropService.stopDrag({
                    node: dragNode,
                    subNodes: this.node.parent ? this.node.parent.children : this.tree.value,
                    index: this.tree.dragNodeIndex
                });
                this.tree.onNodeDrop.emit({
                    originalEvent: event,
                    dragNode: dragNode,
                    dropNode: this.node,
                    index: this.index
                });
            }
        }
        this.draghoverNode = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropNodeDragEnter(event) {
        if (this.tree.droppableNodes && this.node.droppable !== false && this.tree.allowDrop(this.tree.dragNode, this.node, this.tree.dragNodeScope)) {
            this.draghoverNode = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDropNodeDragLeave(event) {
        if (this.tree.droppableNodes) {
            const /** @type {?} */ rect = event.currentTarget.getBoundingClientRect();
            if (event.x > rect.left + rect.width || event.x < rect.left || event.y >= Math.floor(rect.top + rect.height) || event.y < rect.top) {
                this.draghoverNode = false;
            }
        }
    }
}
TreeNodeComponent.ICON_CLASS = 'grf-ui-treenode-icon fa fa-fw';
TreeNodeComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-treenode',
                template: `<ng-template [ngIf]="node">
    <li *ngIf="tree.droppableNodes" class="grf-ui-treenode-droppoint" [ngClass]="{'grf-ui-treenode-droppoint-active grf-ui-state-highlight':draghoverPrev}" (drop)="onDropPoint($event,-1)" (dragover)="onDropPointDragOver($event)" (dragenter)="onDropPointDragEnter($event,-1)"
        (dragleave)="onDropPointDragLeave($event)"></li>
    <li *ngIf="!tree.horizontal" [ngClass]="['grf-ui-treenode',node.styleClass||'', isLeaf() ? 'grf-ui-treenode-leaf': '']">
        <div class="grf-ui-treenode-content" (click)="onNodeClick($event, node)" (contextmenu)="onNodeRightClick($event)" (touchend)="onNodeTouchEnd()" (drop)="onDropNode($event)" (dragover)="onDropNodeDragOver($event)" (dragenter)="onDropNodeDragEnter($event)"
            (dragleave)="onDropNodeDragLeave($event)" [ngClass]="{'grf-ui-treenode-selectable':tree.selectionMode && node.selectable !== false,'grf-ui-treenode-dragover':draghoverNode, 'grf-ui-treenode-content-selected':isSelected()}" [draggable]="tree.draggableNodes"
            (dragstart)="onDragStart($event)" (dragend)="onDragStop($event)">
            <span *ngIf="node.children" class="grf-ui-tree-toggler  fa fa-fw" [ngClass]="{'fa-caret-right':!node.expanded,'fa-caret-down':node.expanded}" (click)="toggle($event)"></span
            ><div class="grf-ui-chkbox" *ngIf="tree.selectionMode == 'checkbox'"><div class="grf-ui-chkbox-box grf-ui-widget grf-ui-corner-all grf-ui-state-default">
            <label grf-checkbox [(ngModel)]="checkBoxNode" [uiIndeterminate]="notAll">
            </label>
        </div>
        </div><span [class]="getIcon()" *ngIf="node.icon||node.expandedIcon||node.collapsedIcon"></span
            ><span class="grf-ui-treenode-label grf-ui-corner-all"
                [ngClass]="{'grf-ui-state-highlight':isSelected()}">
                    <span *ngIf="!tree.getTemplateForNode(node)">{{node.label}}</span>
            <span *ngIf="tree.getTemplateForNode(node)">
                        <ng-container *ngTemplateOutlet="tree.getTemplateForNode(node); context: {$implicit: node}"></ng-container>
                    </span>
            </span>
        </div>
        <ul class="grf-ui-treenode-children" [@treenode]="node.children && node.expanded ? 'visible' : 'hidden'">
            <grf-treenode *ngFor="let childNode of node.children;let firstChild=first;let lastChild=last; let index=index" [node]="childNode" [parentNode]="node" [firstChild]="firstChild" [lastChild]="lastChild" [index]="index"></grf-treenode>
        </ul>
    </li>
    <li *ngIf="tree.droppableNodes&&lastChild" class="grf-ui-treenode-droppoint" [ngClass]="{'grf-ui-treenode-droppoint-active grf-ui-state-highlight':draghoverNext}" (drop)="onDropPoint($event,1)" (dragover)="onDropPointDragOver($event)" (dragenter)="onDropPointDragEnter($event,1)"
        (dragleave)="onDropPointDragLeave($event)"></li>
    <table *ngIf="tree.horizontal" [class]="node.styleClass">
        <tbody>
            <tr>
                <td class="grf-ui-treenode-connector" *ngIf="!root">
                    <table class="grf-ui-treenode-connector-table">
                        <tbody>
                            <tr>
                                <td [ngClass]="{'grf-ui-treenode-connector-line':!firstChild}"></td>
                            </tr>
                            <tr>
                                <td [ngClass]="{'grf-ui-treenode-connector-line':!lastChild}"></td>
                            </tr>
                        </tbody>
                    </table>
                </td>
                <td class="grf-ui-treenode" [ngClass]="{'grf-ui-treenode-collapsed':!node.expanded}">
                    <div class="grf-ui-treenode-content grf-ui-state-default grf-ui-corner-all" [ngClass]="{'grf-ui-treenode-selectable':tree.selectionMode,'grf-ui-state-highlight':isSelected()}" (click)="onNodeClick($event ,node)" (contextmenu)="onNodeRightClick($event)"
                        (touchend)="onNodeTouchEnd()">
                        <span class="grf-ui-tree-toggler fa fa-fw" [ngClass]="{'fa-plus':!node.expanded,'fa-minus':node.expanded}" *ngIf="!isLeaf()" (click)="toggle($event)"></span
                        ><span [class]="getIcon()" *ngIf="node.icon||node.expandedIcon||node.collapsedIcon"></span
                        ><span class="grf-ui-treenode-label grf-ui-corner-all">
                                <span *ngIf="!tree.getTemplateForNode(node)">{{node.label}}</span>
                        <span *ngIf="tree.getTemplateForNode(node)">
                                <ng-container *ngTemplateOutlet="tree.getTemplateForNode(node); context: {$implicit: node}"></ng-container>
                                </span>
                        </span>
                    </div>
                </td>
                <td class="grf-ui-treenode-children-container" *ngIf="node.children && node.expanded" [style.display]="node.expanded ? 'table-cell' : 'none'">
                    <div class="grf-ui-treenode-children">
                        <grf-treenode *ngFor="let childNode of node.children;let firstChild=first;let lastChild=last;" [node]="childNode" [firstChild]="firstChild" [lastChild]="lastChild"></grf-treenode>
                    </div>
                </td>
            </tr>
        </tbody>
    </table>
</ng-template>`,
                styles: [`@charset "UTF-8";@-webkit-keyframes treenodeActive{0%{background-color:#fff;color:#0275d8}100%{background-color:#f9f9f9c7;color:#0275d8}}@keyframes treenodeActive{0%{background-color:#fff;color:#0275d8}100%{background-color:#f9f9f9c7;color:#0275d8}}.grf-ui-tree{width:100%}.grf-ui-tree .grf-ui-tree-container{height:100%;margin:0;overflow:auto;padding:.25em;white-space:nowrap;overflow-y:hidden}.grf-ui-treenode-content{cursor:pointer}.grf-ui-treenode-label:not(.grf-ui-state-highlight):hover{background-color:#fff;color:#0275d8}.grf-ui-treenode-label.grf-ui-state-highlight{background-color:#f9f9f9c7;color:#0275d8;-webkit-animation:.5s treenodeActive;animation:.5s treenodeActive}.grf-ui-treenode .grf-ui-tree-toggler{color:#525252;font-size:14px}.grf-ui-treenode-label{font-size:14px;vertical-align:middle;display:inline-block;color:#525252;padding:2px 12px 2px 2px;margin:0 2px;border-radius:4px}.grf-ui-treenode-icon{vertical-align:middle;display:inline-block;font-size:14px;color:#525252}.grf-ui-treenode ul{list-style:none;padding-left:25px}.grf-ui-treenode .grf-ui-chkbox{display:inline-block;cursor:pointer;vertical-align:bottom;margin-right:.25em}.grf-ui-treenode-droppoint{height:0}`],
                animations: [
                    trigger('treenode', [
                        state('hidden', style({
                            height: '0px',
                            display: 'none',
                            opacity: '0'
                        })),
                        state('visible', style({
                            height: '*',
                            opacity: '1'
                        })),
                        transition('visible => hidden', animate('300ms cubic-bezier(0.86, 0, 0.07, 1)')),
                        transition('hidden => visible', animate('300ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
TreeNodeComponent.ctorParameters = () => [
    { type: TreeComponent, decorators: [{ type: Inject, args: [forwardRef(() => TreeComponent),] },] },
];
TreeNodeComponent.propDecorators = {
    "node": [{ type: Input },],
    "parentNode": [{ type: Input },],
    "root": [{ type: Input },],
    "index": [{ type: Input },],
    "firstChild": [{ type: Input },],
    "lastChild": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CheckboxComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _render
     */
    constructor(_elementRef, _render) {
        this._elementRef = _elementRef;
        this._render = _render;
        this._disabled = false;
        this._indeterminate = false;
        this._prefixCls = 'grf-ui-checkbox';
        this._innerPrefixCls = `${this._prefixCls}-inner`;
        this._inputPrefixCls = `${this._prefixCls}-input`;
        this._checked = false;
        this._focused = false;
        // ngModel Access
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
        this._classMap = {
            [this._prefixCls]: true,
            [`${this._prefixCls}-checked`]: this._checked && !this.uiIndeterminate,
            [`${this._prefixCls}-focused`]: this._focused,
            [`${this._prefixCls}-disabled`]: this.uiDisabled,
            [`${this._prefixCls}-indeterminate`]: this.uiIndeterminate
        };
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiIndeterminate(value) {
        this._indeterminate = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiIndeterminate() {
        return this._indeterminate;
    }
    /**
     * @return {?}
     */
    get uiChecked() {
        return this._checked;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        e.preventDefault();
        if (!this.uiDisabled) {
            this.updateValue(!this._checked);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        if (value === this._checked) {
            return;
        }
        this.onChange(value);
        this._checked = value;
        this.updateClassMap();
    }
    /**
     * @return {?}
     */
    uiFocus() {
        this._focused = true;
    }
    /**
     * @return {?}
     */
    uiBlur() {
        this._focused = false;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._checked = value;
        this.updateClassMap();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.uiDisabled = isDisabled;
    }
    /**
     * @return {?}
     */
    updateClassMap() {
        this._classMap = {
            [this._prefixCls]: true,
            [`${this._prefixCls}-checked`]: this._checked && !this.uiIndeterminate,
            [`${this._prefixCls}-focused`]: this._focused,
            [`${this._prefixCls}-disabled`]: this.uiDisabled,
            [`${this._prefixCls}-indeterminate`]: this.uiIndeterminate
        };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._render.addClass(this._el, `${this._prefixCls}-wrapper`);
        this.updateClassMap();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.updateClassMap();
    }
}
CheckboxComponent.decorators = [
    { type: Component, args: [{
                selector: '[grf-checkbox]',
                encapsulation: ViewEncapsulation.None,
                template: `
      <span [ngClass]='_classMap'>
        <span [ngClass]='_innerPrefixCls'></span>
        <input type='checkbox'
          [ngClass]='_inputPrefixCls'
          [ngModel]='uiChecked'
          (focus)='uiFocus()'
          (blur)='onTouched();uiBlur()'>
      </span>
      <ng-content></ng-content>
    `,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => CheckboxComponent),
                        multi: true
                    }
                ],
                styles: [`@charset "UTF-8";@-webkit-keyframes checkboxEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}@keyframes checkboxEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}.grf-ui-checkbox{white-space:nowrap;cursor:pointer;outline:0;display:inline-block;line-height:1;position:relative;vertical-align:text-bottom}.grf-ui-checkbox-input:focus+.grf-ui-checkbox-inner,.grf-ui-checkbox-wrapper:hover .grf-ui-checkbox-inner,.grf-ui-checkbox:hover .grf-ui-checkbox-inner{border-color:#0275d8}.grf-ui-checkbox-checked:after{position:absolute;top:0;left:0;width:100%;height:100%;border-radius:2px;border:1px solid #0275d8;content:"";-webkit-animation:.36s ease-in-out both checkboxEffect;animation:.36s ease-in-out both checkboxEffect;visibility:hidden}.grf-ui-checkbox-wrapper:hover .grf-ui-checkbox:after,.grf-ui-checkbox:hover:after{visibility:visible}.grf-ui-checkbox-inner{position:relative;top:0;left:0;display:block;width:14px;height:14px;border:1px solid #ced4da;border-radius:2px;background-color:#fff;-webkit-transition:all .3s;transition:all .3s}.grf-ui-checkbox-inner:after{-webkit-transform:rotate(45deg) scale(0);transform:rotate(45deg) scale(0);position:absolute;left:4px;top:1px;display:table;width:3px;height:6px;border:2px solid #fff;border-top:0;border-left:0;content:" ";-webkit-transition:all .1s cubic-bezier(.71,-.46,.88,.6);transition:all .1s cubic-bezier(.71,-.46,.88,.6)}.grf-ui-checkbox-input{position:absolute;left:0;z-index:1;cursor:pointer;opacity:0;top:0;bottom:0;right:0;width:100%;height:100%}.grf-ui-checkbox-indeterminate .grf-ui-checkbox-inner:after{content:" ";-webkit-transform:scale(1);transform:scale(1);position:absolute;left:2px;top:5px;width:6px;height:0}.grf-ui-checkbox-indeterminate.grf-ui-checkbox-disabled .grf-ui-checkbox-inner:after{border-color:#989da1}.grf-ui-checkbox-checked .grf-ui-checkbox-inner:after{-webkit-transform:rotate(45deg) scale(1);transform:rotate(45deg) scale(1);position:absolute;display:table;border:2px solid #fff;border-top:0;border-left:0;content:" ";-webkit-transition:all .2s cubic-bezier(.12,.4,.29,1.46) .1s;transition:all .2s cubic-bezier(.12,.4,.29,1.46) .1s}.grf-ui-checkbox-checked .grf-ui-checkbox-inner,.grf-ui-checkbox-indeterminate .grf-ui-checkbox-inner{background-color:#0275d8;border-color:#0275d8}.grf-ui-checkbox-disabled{cursor:not-allowed}.grf-ui-checkbox-disabled.grf-ui-checkbox-checked .grf-ui-checkbox-inner:after{-webkit-animation-name:none;animation-name:none;border-color:#989da1}.grf-ui-checkbox-disabled .grf-ui-checkbox-input{cursor:not-allowed}.grf-ui-checkbox-disabled .grf-ui-checkbox-inner{border-color:#ced4da!important;background-color:#f7f7f7}.grf-ui-checkbox-disabled .grf-ui-checkbox-inner:after{-webkit-animation-name:none;animation-name:none;border-color:#f7f7f7}.grf-ui-checkbox-disabled+span{color:#989da1;cursor:not-allowed}.grf-ui-checkbox-wrapper{cursor:pointer;font-size:14px;display:inline-block;padding:5px 0}.grf-ui-checkbox+span,.grf-ui-checkbox-wrapper+span{padding-left:8px;padding-right:8px}.grf-ui-checkbox-group{font-size:14px;display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-checkbox-group-item{display:inline-block;margin-right:8px}.grf-ui-checkbox-group-item:last-child{margin-right:0}.grf-ui-checkbox-group-item+.grf-ui-checkbox-group-item{margin-left:0}.grf-ui-checkbox-group.grf-ui-checkbox-group-vertical{-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}`]
            },] },
];
/** @nocollapse */
CheckboxComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
CheckboxComponent.propDecorators = {
    "uiDisabled": [{ type: Input },],
    "uiIndeterminate": [{ type: Input },],
    "uiChecked": [{ type: Input },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CheckboxGroupComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _render
     */
    constructor(_elementRef, _render) {
        this._elementRef = _elementRef;
        this._render = _render;
        this._disabled = false;
        this._prefixCls = 'grf-ui-checkbox-group';
        // ngModel Access
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
        this._el = this._elementRef.nativeElement;
        this._render.setElementClass(this._el, `${this._prefixCls}`, true);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        this._render.setElementClass(this._el, `${this._prefixCls}` + '-' + value, true);
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = this.toBoolean(value);
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @return {?}
     */
    _optionChange() {
        this.onChange(this._options);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._options = value;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.uiDisabled = isDisabled;
    }
}
CheckboxGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-checkbox-group',
                encapsulation: ViewEncapsulation.None,
                template: `
      <label
        grf-checkbox
        *ngFor='let option of _options'
        [uiDisabled]='option.disabled||uiDisabled'
        [(ngModel)]='option.checked'
        (ngModelChange)='_optionChange()'>
        <span>{{ option.label }}</span>
      </label>`,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => CheckboxGroupComponent),
                        multi: true
                    }
                ],
                styles: [`@charset "UTF-8";@-webkit-keyframes checkboxEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}@keyframes checkboxEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}.grf-ui-checkbox{white-space:nowrap;cursor:pointer;outline:0;display:inline-block;line-height:1;position:relative;vertical-align:text-bottom}.grf-ui-checkbox-input:focus+.grf-ui-checkbox-inner,.grf-ui-checkbox-wrapper:hover .grf-ui-checkbox-inner,.grf-ui-checkbox:hover .grf-ui-checkbox-inner{border-color:#0275d8}.grf-ui-checkbox-checked:after{position:absolute;top:0;left:0;width:100%;height:100%;border-radius:2px;border:1px solid #0275d8;content:"";-webkit-animation:.36s ease-in-out both checkboxEffect;animation:.36s ease-in-out both checkboxEffect;visibility:hidden}.grf-ui-checkbox-wrapper:hover .grf-ui-checkbox:after,.grf-ui-checkbox:hover:after{visibility:visible}.grf-ui-checkbox-inner{position:relative;top:0;left:0;display:block;width:14px;height:14px;border:1px solid #ced4da;border-radius:2px;background-color:#fff;-webkit-transition:all .3s;transition:all .3s}.grf-ui-checkbox-inner:after{-webkit-transform:rotate(45deg) scale(0);transform:rotate(45deg) scale(0);position:absolute;left:4px;top:1px;display:table;width:3px;height:6px;border:2px solid #fff;border-top:0;border-left:0;content:" ";-webkit-transition:all .1s cubic-bezier(.71,-.46,.88,.6);transition:all .1s cubic-bezier(.71,-.46,.88,.6)}.grf-ui-checkbox-input{position:absolute;left:0;z-index:1;cursor:pointer;opacity:0;top:0;bottom:0;right:0;width:100%;height:100%}.grf-ui-checkbox-indeterminate .grf-ui-checkbox-inner:after{content:" ";-webkit-transform:scale(1);transform:scale(1);position:absolute;left:2px;top:5px;width:6px;height:0}.grf-ui-checkbox-indeterminate.grf-ui-checkbox-disabled .grf-ui-checkbox-inner:after{border-color:#989da1}.grf-ui-checkbox-checked .grf-ui-checkbox-inner:after{-webkit-transform:rotate(45deg) scale(1);transform:rotate(45deg) scale(1);position:absolute;display:table;border:2px solid #fff;border-top:0;border-left:0;content:" ";-webkit-transition:all .2s cubic-bezier(.12,.4,.29,1.46) .1s;transition:all .2s cubic-bezier(.12,.4,.29,1.46) .1s}.grf-ui-checkbox-checked .grf-ui-checkbox-inner,.grf-ui-checkbox-indeterminate .grf-ui-checkbox-inner{background-color:#0275d8;border-color:#0275d8}.grf-ui-checkbox-disabled{cursor:not-allowed}.grf-ui-checkbox-disabled.grf-ui-checkbox-checked .grf-ui-checkbox-inner:after{-webkit-animation-name:none;animation-name:none;border-color:#989da1}.grf-ui-checkbox-disabled .grf-ui-checkbox-input{cursor:not-allowed}.grf-ui-checkbox-disabled .grf-ui-checkbox-inner{border-color:#ced4da!important;background-color:#f7f7f7}.grf-ui-checkbox-disabled .grf-ui-checkbox-inner:after{-webkit-animation-name:none;animation-name:none;border-color:#f7f7f7}.grf-ui-checkbox-disabled+span{color:#989da1;cursor:not-allowed}.grf-ui-checkbox-wrapper{cursor:pointer;font-size:14px;display:inline-block;padding:5px 0}.grf-ui-checkbox+span,.grf-ui-checkbox-wrapper+span{padding-left:8px;padding-right:8px}.grf-ui-checkbox-group{font-size:14px;display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-checkbox-group-item{display:inline-block;margin-right:8px}.grf-ui-checkbox-group-item:last-child{margin-right:0}.grf-ui-checkbox-group-item+.grf-ui-checkbox-group-item{margin-left:0}.grf-ui-checkbox-group.grf-ui-checkbox-group-vertical{-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}`]
            },] },
];
/** @nocollapse */
CheckboxGroupComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer, },
];
CheckboxGroupComponent.propDecorators = {
    "uiType": [{ type: Input },],
    "uiDisabled": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CheckboxModule {
}
CheckboxModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [CheckboxComponent, CheckboxGroupComponent],
                exports: [CheckboxComponent, CheckboxGroupComponent]
            },] },
];
/** @nocollapse */
CheckboxModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeModule {
}
TreeModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    CheckboxModule,
                    FormsModule
                ],
                declarations: [TreeComponent,
                    TreeNodeComponent],
                exports: [TreeComponent,
                    TreeNodeComponent]
            },] },
];
/** @nocollapse */
TreeModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TreeSelectModule {
}
TreeSelectModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule, FormsModule, OverlayModule, TreeModule
                ],
                declarations: [TreeSelectComponent],
                exports: [TreeSelectComponent]
            },] },
];
/** @nocollapse */
TreeSelectModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RadioGroupComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _render
     */
    constructor(_elementRef, _render) {
        this._elementRef = _elementRef;
        this._render = _render;
        this._prefixCls = 'grf-ui-radio-group';
        this.radios = [];
        this.onChange = () => null;
        this.onTouched = () => null;
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this._size = value;
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this._size;
    }
    /**
     * @return {?}
     */
    get isLarge() {
        return this._size === 'lg';
    }
    /**
     * @return {?}
     */
    get isSmall() {
        return this._size === 'sm';
    }
    /**
     * @param {?} radio
     * @return {?}
     */
    addRadio(radio) {
        this.radios.push(radio);
        radio.uiChecked = radio.uiValue === this._value;
    }
    /**
     * @return {?}
     */
    checkRadios() {
        this.radios.forEach(item => {
            item.uiChecked = item.uiValue === this._value;
        });
    }
    /**
     * @param {?} radio
     * @return {?}
     */
    selectRadio(radio) {
        this.updateValue(radio.uiValue);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        if (this._value === value) {
            return;
        }
        this._value = value;
        this.onChange(value);
        this.checkRadios();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        /** trim text node between radio button */
        Array.from(this._el.childNodes).forEach(node => {
            if (node.nodeType === 3) {
                this._el.removeChild(node);
            }
        });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._value = value;
        this.checkRadios();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.radios.forEach(radio => {
            radio.uiDisabled = isDisabled;
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this._render.addClass(this._el, `${this._prefixCls}`);
    }
}
RadioGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-radio-group',
                encapsulation: ViewEncapsulation.None,
                template: `
      <ng-content></ng-content>`,
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => RadioGroupComponent),
                        multi: true
                    }
                ],
                styles: [`@charset "UTF-8";@-webkit-keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}@keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}span.grf-ui-radio+*{padding-left:8px;padding-right:8px}.grf-ui-radio-wrapper{font-size:14px;display:inline-block;position:relative;white-space:nowrap;margin-right:8px;cursor:pointer}.grf-ui-radio{white-space:nowrap;outline:0;display:inline-block;position:relative;line-height:1;vertical-align:text-bottom;cursor:pointer}.grf-ui-radio .grf-ui-radio-wrapper:hover .grf-ui-radio-inner,.grf-ui-radio-focused .grf-ui-radio-inner,.grf-ui-radio:hover .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked:after{position:absolute;top:0;left:0;width:100%;height:100%;border-radius:50%;border:1px solid #0275d8;content:"";-webkit-animation:.36s ease-in-out both radioEffect;animation:.36s ease-in-out both radioEffect;visibility:hidden}.grf-ui-radio-wrapper:hover .grf-ui-radio:after,.grf-ui-radio:hover:after{visibility:visible}.grf-ui-radio-inner{position:relative;top:0;left:0;display:block;width:14px;height:14px;border-radius:14px;background-color:#fff;-webkit-transition:all .3s;transition:all .3s;border:1px solid #ced4da}.grf-ui-radio-inner:after{position:absolute;width:6px;height:6px;left:3px;top:3px;border-radius:4px;display:table;border-top:0;border-left:0;content:" ";background-color:#0275d8;opacity:0;-webkit-transform:scale(0);transform:scale(0);-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-input{position:absolute;left:0;z-index:1;cursor:pointer;opacity:0;top:0;bottom:0;right:0}.grf-ui-radio-group{display:inline-block;font-size:14px}.grf-ui-radio-checked .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked .grf-ui-radio-inner:after{-webkit-transform:scale(1);transform:scale(1);opacity:1;-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-disabled .grf-ui-radio-inner{border-color:#ced4da!important;background-color:#f7f7f7}.grf-ui-radio-disabled .grf-ui-radio-inner:after{background-color:#ccc}.grf-ui-radio-disabled .grf-ui-radio-input{cursor:not-allowed}.grf-ui-radio-disabled+span{color:#989da1;cursor:not-allowed}.grf-ui-radio-button-wrapper{margin:0;height:34px;line-height:32px;color:rgba(0,0,0,.65);display:inline-block;-webkit-transition:all .3s ease;transition:all .3s ease;cursor:pointer;border:1px solid #ced4da;border-left:0;background:#fff;padding:0 16px;position:relative}.grf-ui-radio-button-wrapper a{color:rgba(0,0,0,.65)}.grf-ui-radio-button-wrapper>.grf-ui-radio-button{margin-left:0;display:block;width:0;height:0}.grf-ui-radio-group-large .grf-ui-radio-button-wrapper{height:40px;line-height:38px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper{height:30px;line-height:28px;padding:0 12px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:first-child{border-radius:2px 0 0 2px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:last-child{border-radius:0 2px 2px 0}.grf-ui-radio-button-wrapper:not(:first-child)::before{content:"";display:block;top:0;left:-1px;width:1px;height:100%;position:absolute;background-color:#ced4da}.grf-ui-radio-button-wrapper:first-child{border-radius:4px 0 0 4px;border-left:1px solid #ced4da}.grf-ui-radio-button-wrapper:last-child{border-radius:0 4px 4px 0}.grf-ui-radio-button-wrapper:first-child:last-child{border-radius:4px}.grf-ui-radio-button-wrapper-focused,.grf-ui-radio-button-wrapper:hover{color:#0275d8;position:relative}.grf-ui-radio-button-wrapper .grf-ui-radio-inner,.grf-ui-radio-button-wrapper input[type=checkbox],.grf-ui-radio-button-wrapper input[type=radio]{opacity:0;width:0;height:0}.grf-ui-radio-button-wrapper-checked{background:#fff;border-color:#0275d8;color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;z-index:1}.grf-ui-radio-button-wrapper-checked::before{background-color:#0275d8!important;opacity:.1}.grf-ui-radio-button-wrapper-checked:first-child{border-color:#0275d8;-webkit-box-shadow:none!important;box-shadow:none!important}.grf-ui-radio-button-wrapper-checked:active,.grf-ui-radio-button-wrapper-checked:hover{border-color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;color:#0275d8}.grf-ui-radio-button-wrapper-disabled{border-color:#ced4da;background-color:#f7f7f7;cursor:not-allowed;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child,.grf-ui-radio-button-wrapper-disabled:hover{border-color:#ced4da;background-color:#f7f7f7;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child{border-left-color:#ced4da}.grf-ui-radio-button-wrapper-disabled.grf-ui-radio-button-wrapper-checked{color:#fff;background-color:#e6e6e6;border-color:#ced4da;-webkit-box-shadow:none;box-shadow:none}`]
            },] },
];
/** @nocollapse */
RadioGroupComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
RadioGroupComponent.propDecorators = {
    "uiSize": [{ type: Input },],
    "isLarge": [{ type: HostBinding, args: ['class.grf-ui-radio-group-large',] },],
    "isSmall": [{ type: HostBinding, args: ['class.grf-ui-radio-group-small',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RadioComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     * @param {?} _radioGroup
     */
    constructor(_elementRef, _renderer, _radioGroup) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._radioGroup = _radioGroup;
        this._focused = false;
        this._checked = false;
        this._disabled = false;
        this._prefixCls = 'grf-ui-radio';
        this._innerPrefixCls = `${this._prefixCls}-inner`;
        this._inputPrefixCls = `${this._prefixCls}-input`;
        // ngModel Access
        this.onChange = () => null;
        this.onTouched = () => null;
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiChecked(value) {
        this._checked = this.toBoolean(value);
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiChecked() {
        return this._checked;
    }
    /**
     * @return {?}
     */
    get uiValue() {
        return this._value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiValue(value) {
        if (this._value === value) {
            return;
        }
        this._value = value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = this.toBoolean(value);
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onClick(e) {
        e.preventDefault();
        if (!this._disabled) {
            if (this._radioGroup) {
                this._checked = true;
                this.setClassMap();
                this._radioGroup.selectRadio(this);
            }
            else {
                this.updateValue(!this._checked);
            }
        }
    }
    /**
     * @return {?}
     */
    uiFocus() {
        this._focused = true;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    uiBlur() {
        this._focused = false;
        this.setClassMap();
        if (this._radioGroup) {
            this._radioGroup.onTouched();
        }
    }
    /**
     * @return {?}
     */
    setClassMap() {
        this._classMap = {
            [this._prefixCls]: true,
            [`${this._prefixCls}-checked`]: this._checked,
            [`${this._prefixCls}-focused`]: this._focused,
            [`${this._prefixCls}-disabled`]: this._disabled
        };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._radioGroup) {
            this._radioGroup.addRadio(this);
        }
        this._renderer.addClass(this._el, `${this._prefixCls}-wrapper`);
        this.setClassMap();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        if (value === this._checked) {
            return;
        }
        this.onChange(value);
        this._focused = false;
        this._checked = value;
        this.setClassMap();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._checked = value;
        this.setClassMap();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
}
RadioComponent.decorators = [
    { type: Component, args: [{
                selector: '[grf-radio]',
                encapsulation: ViewEncapsulation.None,
                template: `
      <span [ngClass]='_classMap'>
        <span [ngClass]='_innerPrefixCls'></span>
        <input type='radio' [ngClass]='_inputPrefixCls' [(ngModel)]='uiChecked' (focus)='uiFocus()' (blur)='uiBlur()'>
      </span>
      <ng-content></ng-content>
    `,
                styles: [`@charset "UTF-8";@-webkit-keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}@keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}span.grf-ui-radio+*{padding-left:8px;padding-right:8px}.grf-ui-radio-wrapper{font-size:14px;display:inline-block;position:relative;white-space:nowrap;margin-right:8px;cursor:pointer}.grf-ui-radio{white-space:nowrap;outline:0;display:inline-block;position:relative;line-height:1;vertical-align:text-bottom;cursor:pointer}.grf-ui-radio .grf-ui-radio-wrapper:hover .grf-ui-radio-inner,.grf-ui-radio-focused .grf-ui-radio-inner,.grf-ui-radio:hover .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked:after{position:absolute;top:0;left:0;width:100%;height:100%;border-radius:50%;border:1px solid #0275d8;content:"";-webkit-animation:.36s ease-in-out both radioEffect;animation:.36s ease-in-out both radioEffect;visibility:hidden}.grf-ui-radio-wrapper:hover .grf-ui-radio:after,.grf-ui-radio:hover:after{visibility:visible}.grf-ui-radio-inner{position:relative;top:0;left:0;display:block;width:14px;height:14px;border-radius:14px;background-color:#fff;-webkit-transition:all .3s;transition:all .3s;border:1px solid #ced4da}.grf-ui-radio-inner:after{position:absolute;width:6px;height:6px;left:3px;top:3px;border-radius:4px;display:table;border-top:0;border-left:0;content:" ";background-color:#0275d8;opacity:0;-webkit-transform:scale(0);transform:scale(0);-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-input{position:absolute;left:0;z-index:1;cursor:pointer;opacity:0;top:0;bottom:0;right:0}.grf-ui-radio-group{display:inline-block;font-size:14px}.grf-ui-radio-checked .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked .grf-ui-radio-inner:after{-webkit-transform:scale(1);transform:scale(1);opacity:1;-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-disabled .grf-ui-radio-inner{border-color:#ced4da!important;background-color:#f7f7f7}.grf-ui-radio-disabled .grf-ui-radio-inner:after{background-color:#ccc}.grf-ui-radio-disabled .grf-ui-radio-input{cursor:not-allowed}.grf-ui-radio-disabled+span{color:#989da1;cursor:not-allowed}.grf-ui-radio-button-wrapper{margin:0;height:34px;line-height:32px;color:rgba(0,0,0,.65);display:inline-block;-webkit-transition:all .3s ease;transition:all .3s ease;cursor:pointer;border:1px solid #ced4da;border-left:0;background:#fff;padding:0 16px;position:relative}.grf-ui-radio-button-wrapper a{color:rgba(0,0,0,.65)}.grf-ui-radio-button-wrapper>.grf-ui-radio-button{margin-left:0;display:block;width:0;height:0}.grf-ui-radio-group-large .grf-ui-radio-button-wrapper{height:40px;line-height:38px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper{height:30px;line-height:28px;padding:0 12px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:first-child{border-radius:2px 0 0 2px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:last-child{border-radius:0 2px 2px 0}.grf-ui-radio-button-wrapper:not(:first-child)::before{content:"";display:block;top:0;left:-1px;width:1px;height:100%;position:absolute;background-color:#ced4da}.grf-ui-radio-button-wrapper:first-child{border-radius:4px 0 0 4px;border-left:1px solid #ced4da}.grf-ui-radio-button-wrapper:last-child{border-radius:0 4px 4px 0}.grf-ui-radio-button-wrapper:first-child:last-child{border-radius:4px}.grf-ui-radio-button-wrapper-focused,.grf-ui-radio-button-wrapper:hover{color:#0275d8;position:relative}.grf-ui-radio-button-wrapper .grf-ui-radio-inner,.grf-ui-radio-button-wrapper input[type=checkbox],.grf-ui-radio-button-wrapper input[type=radio]{opacity:0;width:0;height:0}.grf-ui-radio-button-wrapper-checked{background:#fff;border-color:#0275d8;color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;z-index:1}.grf-ui-radio-button-wrapper-checked::before{background-color:#0275d8!important;opacity:.1}.grf-ui-radio-button-wrapper-checked:first-child{border-color:#0275d8;-webkit-box-shadow:none!important;box-shadow:none!important}.grf-ui-radio-button-wrapper-checked:active,.grf-ui-radio-button-wrapper-checked:hover{border-color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;color:#0275d8}.grf-ui-radio-button-wrapper-disabled{border-color:#ced4da;background-color:#f7f7f7;cursor:not-allowed;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child,.grf-ui-radio-button-wrapper-disabled:hover{border-color:#ced4da;background-color:#f7f7f7;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child{border-left-color:#ced4da}.grf-ui-radio-button-wrapper-disabled.grf-ui-radio-button-wrapper-checked{color:#fff;background-color:#e6e6e6;border-color:#ced4da;-webkit-box-shadow:none;box-shadow:none}`],
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => RadioComponent),
                        multi: true
                    }
                ]
            },] },
];
// endregion
/** @nocollapse */
RadioComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: RadioGroupComponent, decorators: [{ type: Optional },] },
];
RadioComponent.propDecorators = {
    "uiChecked": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-radio-wrapper-checked',] },],
    "uiValue": [{ type: Input },],
    "uiDisabled": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-radio-wrapper-disabled',] },],
    "onClick": [{ type: HostListener, args: ['click', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RadioButtonComponent extends RadioComponent {
    constructor() {
        super(...arguments);
        this._checked = false;
        this._disabled = false;
        this._prefixCls = 'grf-ui-radio-button';
        this._innerPrefixCls = `${this._prefixCls}-inner`;
        this._inputPrefixCls = `${this._prefixCls}-input`;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    toBoolean(value) {
        return value === '' || (value && value !== 'false');
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        this._disabled = this.toBoolean(value);
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiChecked(value) {
        this._checked = this.toBoolean(value);
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiChecked() {
        return this._checked;
    }
}
RadioButtonComponent.decorators = [
    { type: Component, args: [{
                selector: '[grf-radio-button]',
                encapsulation: ViewEncapsulation.None,
                template: `
      <span [ngClass]='_classMap'>
        <span [ngClass]='_innerPrefixCls'></span>
        <input type='radio' [ngClass]='_inputPrefixCls' [(ngModel)]='uiChecked' (focus)='uiFocus()' (blur)='uiBlur()'>
      </span>
      <ng-content></ng-content>
    `,
                styles: [`@charset "UTF-8";@-webkit-keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}@keyframes radioEffect{0%{-webkit-transform:scale(1);transform:scale(1);opacity:.5}100%{-webkit-transform:scale(1.6);transform:scale(1.6);opacity:0}}span.grf-ui-radio+*{padding-left:8px;padding-right:8px}.grf-ui-radio-wrapper{font-size:14px;display:inline-block;position:relative;white-space:nowrap;margin-right:8px;cursor:pointer}.grf-ui-radio{white-space:nowrap;outline:0;display:inline-block;position:relative;line-height:1;vertical-align:text-bottom;cursor:pointer}.grf-ui-radio .grf-ui-radio-wrapper:hover .grf-ui-radio-inner,.grf-ui-radio-focused .grf-ui-radio-inner,.grf-ui-radio:hover .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked:after{position:absolute;top:0;left:0;width:100%;height:100%;border-radius:50%;border:1px solid #0275d8;content:"";-webkit-animation:.36s ease-in-out both radioEffect;animation:.36s ease-in-out both radioEffect;visibility:hidden}.grf-ui-radio-wrapper:hover .grf-ui-radio:after,.grf-ui-radio:hover:after{visibility:visible}.grf-ui-radio-inner{position:relative;top:0;left:0;display:block;width:14px;height:14px;border-radius:14px;background-color:#fff;-webkit-transition:all .3s;transition:all .3s;border:1px solid #ced4da}.grf-ui-radio-inner:after{position:absolute;width:6px;height:6px;left:3px;top:3px;border-radius:4px;display:table;border-top:0;border-left:0;content:" ";background-color:#0275d8;opacity:0;-webkit-transform:scale(0);transform:scale(0);-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-input{position:absolute;left:0;z-index:1;cursor:pointer;opacity:0;top:0;bottom:0;right:0}.grf-ui-radio-group{display:inline-block;font-size:14px}.grf-ui-radio-checked .grf-ui-radio-inner{border-color:#0275d8}.grf-ui-radio-checked .grf-ui-radio-inner:after{-webkit-transform:scale(1);transform:scale(1);opacity:1;-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86)}.grf-ui-radio-disabled .grf-ui-radio-inner{border-color:#ced4da!important;background-color:#f7f7f7}.grf-ui-radio-disabled .grf-ui-radio-inner:after{background-color:#ccc}.grf-ui-radio-disabled .grf-ui-radio-input{cursor:not-allowed}.grf-ui-radio-disabled+span{color:#989da1;cursor:not-allowed}.grf-ui-radio-button-wrapper{margin:0;height:34px;line-height:32px;color:rgba(0,0,0,.65);display:inline-block;-webkit-transition:all .3s ease;transition:all .3s ease;cursor:pointer;border:1px solid #ced4da;border-left:0;background:#fff;padding:0 16px;position:relative}.grf-ui-radio-button-wrapper a{color:rgba(0,0,0,.65)}.grf-ui-radio-button-wrapper>.grf-ui-radio-button{margin-left:0;display:block;width:0;height:0}.grf-ui-radio-group-large .grf-ui-radio-button-wrapper{height:40px;line-height:38px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper{height:30px;line-height:28px;padding:0 12px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:first-child{border-radius:2px 0 0 2px}.grf-ui-radio-group-small .grf-ui-radio-button-wrapper:last-child{border-radius:0 2px 2px 0}.grf-ui-radio-button-wrapper:not(:first-child)::before{content:"";display:block;top:0;left:-1px;width:1px;height:100%;position:absolute;background-color:#ced4da}.grf-ui-radio-button-wrapper:first-child{border-radius:4px 0 0 4px;border-left:1px solid #ced4da}.grf-ui-radio-button-wrapper:last-child{border-radius:0 4px 4px 0}.grf-ui-radio-button-wrapper:first-child:last-child{border-radius:4px}.grf-ui-radio-button-wrapper-focused,.grf-ui-radio-button-wrapper:hover{color:#0275d8;position:relative}.grf-ui-radio-button-wrapper .grf-ui-radio-inner,.grf-ui-radio-button-wrapper input[type=checkbox],.grf-ui-radio-button-wrapper input[type=radio]{opacity:0;width:0;height:0}.grf-ui-radio-button-wrapper-checked{background:#fff;border-color:#0275d8;color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;z-index:1}.grf-ui-radio-button-wrapper-checked::before{background-color:#0275d8!important;opacity:.1}.grf-ui-radio-button-wrapper-checked:first-child{border-color:#0275d8;-webkit-box-shadow:none!important;box-shadow:none!important}.grf-ui-radio-button-wrapper-checked:active,.grf-ui-radio-button-wrapper-checked:hover{border-color:#0275d8;-webkit-box-shadow:-1px 0 0 0 #0275d8;box-shadow:-1px 0 0 0 #0275d8;color:#0275d8}.grf-ui-radio-button-wrapper-disabled{border-color:#ced4da;background-color:#f7f7f7;cursor:not-allowed;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child,.grf-ui-radio-button-wrapper-disabled:hover{border-color:#ced4da;background-color:#f7f7f7;color:#989da1}.grf-ui-radio-button-wrapper-disabled:first-child{border-left-color:#ced4da}.grf-ui-radio-button-wrapper-disabled.grf-ui-radio-button-wrapper-checked{color:#fff;background-color:#e6e6e6;border-color:#ced4da;-webkit-box-shadow:none;box-shadow:none}`]
            },] },
];
/** @nocollapse */
RadioButtonComponent.ctorParameters = () => [];
RadioButtonComponent.propDecorators = {
    "uiDisabled": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-radio-button-wrapper-disabled',] },],
    "uiChecked": [{ type: Input }, { type: HostBinding, args: ['class.grf-ui-radio-button-wrapper-checked',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RadioModule {
}
RadioModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [RadioComponent, RadioButtonComponent, RadioGroupComponent],
                exports: [RadioComponent, RadioButtonComponent, RadioGroupComponent]
            },] },
];
/** @nocollapse */
RadioModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const POSITION_MAP = {
    'top': {
        originX: 'center',
        originY: 'top',
        overlayX: 'center',
        overlayY: 'bottom'
    },
    'topCenter': {
        originX: 'center',
        originY: 'top',
        overlayX: 'center',
        overlayY: 'bottom'
    },
    'topLeft': {
        originX: 'start',
        originY: 'top',
        overlayX: 'start',
        overlayY: 'bottom'
    },
    'topRight': {
        originX: 'end',
        originY: 'top',
        overlayX: 'end',
        overlayY: 'bottom'
    },
    'right': {
        originX: 'end',
        originY: 'center',
        overlayX: 'start',
        overlayY: 'center',
    },
    'rightTop': {
        originX: 'end',
        originY: 'top',
        overlayX: 'start',
        overlayY: 'top',
    },
    'rightBottom': {
        originX: 'end',
        originY: 'bottom',
        overlayX: 'start',
        overlayY: 'bottom',
    },
    'bottom': {
        originX: 'center',
        originY: 'bottom',
        overlayX: 'center',
        overlayY: 'top',
    },
    'bottomCenter': {
        originX: 'center',
        originY: 'bottom',
        overlayX: 'center',
        overlayY: 'top',
    },
    'bottomLeft': {
        originX: 'start',
        originY: 'bottom',
        overlayX: 'start',
        overlayY: 'top',
    },
    'bottomRight': {
        originX: 'end',
        originY: 'bottom',
        overlayX: 'end',
        overlayY: 'top',
    },
    'left': {
        originX: 'start',
        originY: 'center',
        overlayX: 'end',
        overlayY: 'center',
    },
    'leftTop': {
        originX: 'start',
        originY: 'top',
        overlayX: 'end',
        overlayY: 'top',
    },
    'leftBottom': {
        originX: 'start',
        originY: 'bottom',
        overlayX: 'end',
        overlayY: 'bottom',
    },
};
const DEFAULT_4_POSITIONS = _objectValues([POSITION_MAP['top'], POSITION_MAP['right'], POSITION_MAP['bottom'], POSITION_MAP['left']]);
const DEFAULT_DROPDOWN_POSITIONS = _objectValues([POSITION_MAP['bottomLeft'], POSITION_MAP['topLeft']]);

/**
 * @param {?} array
 * @param {?} iteratee
 * @return {?}
 */
function arrayMap(array, iteratee) {
    let /** @type {?} */ index = -1;
    const /** @type {?} */ length = array == null ? 0 : array.length, /** @type {?} */
    result = Array(length);
    while (++index < length) {
        result[index] = iteratee(array[index], index, array);
    }
    return result;
}
/**
 * @param {?} object
 * @param {?} props
 * @return {?}
 */
function baseValues(object, props) {
    return arrayMap(props, function (key) {
        return object[key];
    });
}
/**
 * @param {?} object
 * @return {?}
 */
function _objectValues(object) {
    return object == null ? [] : baseValues(object, Object.keys(object));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TooltipComponent {
    /**
     * @param {?} _cdr
     */
    constructor(_cdr) {
        this._cdr = _cdr;
        this.uiOverlayClassName = '';
        this.uiOverlayStyle = {};
        this.uiVisibleChange = new EventEmitter();
        this.visibleSource = new BehaviorSubject(false);
        this.visible$ = this.visibleSource.asObservable();
        this._prefix = 'grf-ui-tooltip-placement';
        this._positions = [...DEFAULT_4_POSITIONS];
        this._classMap = {};
        this._placement = 'top';
        this._trigger = 'hover';
        this._hasBackdrop = false;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiVisible(value) {
        if (this.visibleSource.value !== value) {
            this.visibleSource.next(value);
            this.uiVisibleChange.emit(value);
        }
    }
    /**
     * @return {?}
     */
    get uiVisible() {
        return this.visibleSource.value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiTrigger(value) {
        this._trigger = value;
        this._hasBackdrop = this._trigger === 'click';
    }
    /**
     * @return {?}
     */
    get uiTrigger() {
        return this._trigger;
    }
    /**
     * @return {?}
     */
    get uiPlacement() {
        return this._placement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiPlacement(value) {
        if (value !== this._placement) {
            this._placement = value;
            this._positions.unshift(/** @type {?} */ (POSITION_MAP[this.uiPlacement]));
        }
    }
    /**
     * @return {?}
     */
    updatePosition() {
        if (this.overlay && this.overlay.overlayRef) {
            this.overlay.overlayRef.updatePosition();
        }
    }
    /**
     * @param {?} $event
     * @return {?}
     */
    onPositionChange($event) {
        for (const /** @type {?} */ key in POSITION_MAP) {
            if (JSON.stringify($event.connectionPair) === JSON.stringify(POSITION_MAP[key])) {
                this.uiPlacement = key;
                break;
            }
        }
        this.setClassMap();
        /** TODO may cause performance problem */
        this._cdr.detectChanges();
    }
    /**
     * @return {?}
     */
    show() {
        this.uiVisible = true;
    }
    /**
     * @return {?}
     */
    hide() {
        this.uiVisible = false;
    }
    /**
     * @return {?}
     */
    tooltipLeave() {
        return false;
    }
    /**
     * @return {?}
     */
    tooltipHover() {
        return false;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    _afterVisibilityAnimation(e) {
        if (e.toState === 'false' && !this.uiVisible) {
            this.uiVisibleChange.emit(false);
        }
        if (e.toState === 'true' && this.uiVisible) {
            this.uiVisibleChange.emit(true);
        }
    }
    /**
     * @return {?}
     */
    setClassMap() {
        this._classMap = {
            [this.uiOverlayClassName]: true,
            [`${this._prefix}-${this._placement}`]: true
        };
    }
    /**
     * @param {?} origin
     * @return {?}
     */
    setOverlayOrigin(origin) {
        this.overlayOrigin = origin;
    }
}
TooltipComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-tooltip',
                template: `<ng-content></ng-content>
<ng-template #overlay="cdkConnectedOverlay" cdkConnectedOverlay [cdkConnectedOverlayOrigin]="overlayOrigin" [cdkConnectedOverlayHasBackdrop]="_hasBackdrop"
    (backdropClick)="hide()" (detach)="hide()" (positionChange)="onPositionChange($event)" [cdkConnectedOverlayPositions]="_positions"
    [cdkConnectedOverlayOpen]="visible$ | async">
    <div class="grf-ui-tooltip" (mouseenter)="tooltipHover()" (mouseleave)="tooltipLeave()" [ngClass]="_classMap" [ngStyle]="uiOverlayStyle" [@fadeAnimation]="''+(visible$ | async)" (@fadeAnimation.done)="_afterVisibilityAnimation($event)">
        <div class="grf-ui-tooltip-content">
            <div class="grf-ui-tooltip-arrow"></div>
            <div class="grf-ui-tooltip-inner">
                <span *ngIf="!uiTemplate">{{uiTitle}}</span>
                <ng-template *ngIf="uiTemplate" [ngTemplateOutlet]="uiTemplate">
                </ng-template>
            </div>
        </div>
    </div>
</ng-template>
`,
                styles: [`@charset "UTF-8";.grf-ui-tooltip{position:relative;z-index:1060;display:block;visibility:visible;font-size:12px;line-height:1.5;pointer-events:none}.grf-ui-tooltip-hidden{display:none}.grf-ui-tooltip-placement-top,.grf-ui-tooltip-placement-topLeft,.grf-ui-tooltip-placement-topRight{padding-bottom:8px}.grf-ui-tooltip-placement-right,.grf-ui-tooltip-placement-rightBottom,.grf-ui-tooltip-placement-rightTop{padding-left:8px}.grf-ui-tooltip-placement-bottom,.grf-ui-tooltip-placement-bottomLeft,.grf-ui-tooltip-placement-bottomRight{padding-top:8px}.grf-ui-tooltip-placement-left,.grf-ui-tooltip-placement-leftBottom,.grf-ui-tooltip-placement-leftTop{padding-right:8px}.grf-ui-tooltip-inner{max-width:250px;padding:8px 10px;color:#fff;text-align:left;text-decoration:none;background-color:rgba(64,64,64,.85);border-radius:4px;-webkit-box-shadow:0 1px 6px rgba(0,0,0,.2);box-shadow:0 1px 6px rgba(0,0,0,.2);min-height:34px}.grf-ui-tooltip-arrow{position:absolute;width:0;height:0;border-color:transparent;border-style:solid}.grf-ui-tooltip-placement-top .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-topLeft .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-topRight .grf-ui-tooltip-arrow{bottom:3px;border-width:5px 5px 0;border-top-color:rgba(64,64,64,.85)}.grf-ui-tooltip-placement-top .grf-ui-tooltip-arrow{left:50%;margin-left:-5px}.grf-ui-tooltip-placement-topLeft .grf-ui-tooltip-arrow{left:16px}.grf-ui-tooltip-placement-topRight .grf-ui-tooltip-arrow{right:16px}.grf-ui-tooltip-placement-right .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-rightBottom .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-rightTop .grf-ui-tooltip-arrow{left:3px;border-width:5px 5px 5px 0;border-right-color:rgba(64,64,64,.85)}.grf-ui-tooltip-placement-right .grf-ui-tooltip-arrow{top:50%;margin-top:-5px}.grf-ui-tooltip-placement-rightTop .grf-ui-tooltip-arrow{top:8px}.grf-ui-tooltip-placement-rightBottom .grf-ui-tooltip-arrow{bottom:8px}.grf-ui-tooltip-placement-left .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-leftBottom .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-leftTop .grf-ui-tooltip-arrow{right:3px;border-width:5px 0 5px 5px;border-left-color:rgba(64,64,64,.85)}.grf-ui-tooltip-placement-left .grf-ui-tooltip-arrow{top:50%;margin-top:-5px}.grf-ui-tooltip-placement-leftTop .grf-ui-tooltip-arrow{top:8px}.grf-ui-tooltip-placement-leftBottom .grf-ui-tooltip-arrow{bottom:8px}.grf-ui-tooltip-placement-bottom .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-bottomLeft .grf-ui-tooltip-arrow,.grf-ui-tooltip-placement-bottomRight .grf-ui-tooltip-arrow{top:3px;border-width:0 5px 5px;border-bottom-color:rgba(64,64,64,.85)}.grf-ui-tooltip-placement-bottom .grf-ui-tooltip-arrow{left:50%;margin-left:-5px}.grf-ui-tooltip-placement-bottomLeft .grf-ui-tooltip-arrow{left:16px}.grf-ui-tooltip-placement-bottomRight .grf-ui-tooltip-arrow{right:16px}`],
                animations: [
                    trigger('fadeAnimation', [
                        state('void', style({ opacity: 0 })),
                        state('true', style({ opacity: 1 })),
                        state('false', style({ opacity: 0 })),
                        transition('* => true', animate('0ms cubic-bezier(0.0, 0.0, 0.2, 1)')),
                        transition('* => void', animate('0ms cubic-bezier(0.4, 0.0, 1, 1)')),
                    ])
                ]
            },] },
];
/** @nocollapse */
TooltipComponent.ctorParameters = () => [
    { type: ChangeDetectorRef, },
];
TooltipComponent.propDecorators = {
    "uiTitle": [{ type: Input },],
    "uiOverlayClassName": [{ type: Input },],
    "uiOverlayStyle": [{ type: Input },],
    "uiVisibleChange": [{ type: Output },],
    "uiTemplate": [{ type: ContentChild, args: ['uiTemplate',] },],
    "overlay": [{ type: ViewChild, args: ['overlay',] },],
    "uiVisible": [{ type: Input },],
    "uiTrigger": [{ type: Input },],
    "uiPlacement": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TooltipDirective {
    /**
     * @param {?} elementRef
     * @param {?} hostView
     * @param {?} resolver
     * @param {?} renderer
     * @param {?} tooltip
     */
    constructor(elementRef, hostView, resolver, renderer, tooltip) {
        this.elementRef = elementRef;
        this.hostView = hostView;
        this.resolver = resolver;
        this.renderer = renderer;
        this.isTooltipOpen = true;
        this.isDynamicTooltip = false;
        // Support faster tooltip mode: <a ui-tooltip="xxx"></a>. [NOTE] Used only under uiTooltipDirective currently.
        if (!tooltip) {
            const /** @type {?} */ factory = this.resolver.resolveComponentFactory(TooltipComponent);
            tooltip = this.hostView.createComponent(factory).instance;
            this.isDynamicTooltip = true;
        }
        tooltip.setOverlayOrigin(this);
        this.tooltip = tooltip;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        if (this.tooltip.uiTrigger === 'hover') {
            this.renderer.listen(this.elementRef.nativeElement, 'mouseenter', () => this.show());
            this.renderer.listen(this.elementRef.nativeElement, 'mouseleave', () => this.hide());
        }
        else if (this.tooltip.uiTrigger === 'focus') {
            this.renderer.listen(this.elementRef.nativeElement, 'focus', () => this.show());
            this.renderer.listen(this.elementRef.nativeElement, 'blur', () => this.hide());
        }
        else if (this.tooltip.uiTrigger === 'click') {
            this.renderer.listen(this.elementRef.nativeElement, 'click', (e) => {
                e.preventDefault();
                this.show();
            });
        }
    }
    /**
     * @return {?}
     */
    show() {
        this.tooltip.show();
        this.isTooltipOpen = true;
    }
    /**
     * @return {?}
     */
    hide() {
        this.tooltip.hide();
        this.isTooltipOpen = false;
    }
}
TooltipDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grfTooltip]'
            },] },
];
/** @nocollapse */
TooltipDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: ViewContainerRef, },
    { type: ComponentFactoryResolver, },
    { type: Renderer2, },
    { type: TooltipComponent, decorators: [{ type: Optional },] },
];
TooltipDirective.propDecorators = {
    "isTooltipOpen": [{ type: HostBinding, args: ['class.grf-ui-tooltip-open',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class TooltipModule {
}
TooltipModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    OverlayModule
                ],
                declarations: [TooltipComponent,
                    TooltipDirective],
                exports: [TooltipComponent,
                    TooltipDirective
                ]
            },] },
];
/** @nocollapse */
TooltipModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BreadCrumbComponent {
    constructor() {
        this.uiSeparator = '/';
        this._uiBreadcrumb = true;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
BreadCrumbComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-breadcrumb',
                template: `<ng-content></ng-content>`,
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";.grf-ui-breadcrumb{color:rgba(0,0,0,.65);font-size:12px;display:block}.grf-ui-breadcrumb a{color:rgba(0,0,0,.65);-webkit-transition:color .3s;transition:color .3s;text-decoration:none}.grf-ui-breadcrumb a:hover{color:#0275d8}.grf-ui-breadcrumb a i{padding-right:5px}.grf-ui-breadcrumb>span:last-child{font-weight:700;color:rgba(0,0,0,.65)}.grf-ui-breadcrumb>span:last-child .grf-ui-breadcrumb-separator{display:none}.grf-ui-breadcrumb-separator{margin:0 8px;color:rgba(0,0,0,.3)}.grf-ui-breadcrumb>.grf-ui-breadcrumb-link>.fa+span{margin-left:4px}.grf-ui-breadcrumb>grf-breadcrumb-item:last-child{font-weight:700;color:rgba(0,0,0,.65)}.grf-ui-breadcrumb>grf-breadcrumb-item:last-child .grf-ui-breadcrumb-separator{display:none}`]
            },] },
];
/** @nocollapse */
BreadCrumbComponent.ctorParameters = () => [];
BreadCrumbComponent.propDecorators = {
    "uiSeparator": [{ type: Input },],
    "_uiBreadcrumb": [{ type: HostBinding, args: ['class.grf-ui-breadcrumb',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BreadCrumbItemComponent {
    /**
     * @param {?} _bread
     */
    constructor(_bread) {
        this._bread = _bread;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
BreadCrumbItemComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-breadcrumb-item',
                template: `<span class="grf-ui-breadcrumb-link">
    <ng-content></ng-content>
</span>
<span class="grf-ui-breadcrumb-separator">{{_bread?.uiSeparator}}</span>
`,
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";.grf-ui-breadcrumb{color:rgba(0,0,0,.65);font-size:12px;display:block}.grf-ui-breadcrumb a{color:rgba(0,0,0,.65);-webkit-transition:color .3s;transition:color .3s;text-decoration:none}.grf-ui-breadcrumb a:hover{color:#0275d8}.grf-ui-breadcrumb a i{padding-right:5px}.grf-ui-breadcrumb>span:last-child{font-weight:700;color:rgba(0,0,0,.65)}.grf-ui-breadcrumb>span:last-child .grf-ui-breadcrumb-separator{display:none}.grf-ui-breadcrumb-separator{margin:0 8px;color:rgba(0,0,0,.3)}.grf-ui-breadcrumb>.grf-ui-breadcrumb-link>.fa+span{margin-left:4px}.grf-ui-breadcrumb>grf-breadcrumb-item:last-child{font-weight:700;color:rgba(0,0,0,.65)}.grf-ui-breadcrumb>grf-breadcrumb-item:last-child .grf-ui-breadcrumb-separator{display:none}`]
            },] },
];
/** @nocollapse */
BreadCrumbItemComponent.ctorParameters = () => [
    { type: BreadCrumbComponent, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BreadCrumbModule {
}
BreadCrumbModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [BreadCrumbComponent,
                    BreadCrumbItemComponent
                ],
                exports: [
                    BreadCrumbComponent,
                    BreadCrumbItemComponent
                ]
            },] },
];
/** @nocollapse */
BreadCrumbModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class AlertComponent {
    constructor() {
        this._display = true;
        this._prefixCls = 'grf-ui-alert';
        this.uiType = 'info';
        this.uiBanner = false;
        this.uiCloseable = false;
        this.uiShowIcon = false;
        this.uiOnClose = new EventEmitter();
        this._classMap = {
            [`${this._prefixCls}`]: true,
            [`${this._prefixCls}-${this.uiType}`]: true,
            [`${this._prefixCls}-no-icon`]: !this.uiShowIcon,
            [`${this._prefixCls}-banner`]: this.uiBanner,
            [`${this._prefixCls}-with-description`]: !!this.uiDescription
        };
    }
    /**
     * @return {?}
     */
    closeAlert() {
        this._display = false;
        this.uiOnClose.emit(true);
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this._classMap = {
            [`${this._prefixCls}`]: true,
            [`${this._prefixCls}-${this.uiType}`]: true,
            [`${this._prefixCls}-no-icon`]: !this.uiShowIcon,
            [`${this._prefixCls}-banner`]: this.uiBanner,
            [`${this._prefixCls}-with-description`]: !!this.uiDescription
        };
    }
}
AlertComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-alert',
                encapsulation: ViewEncapsulation.None,
                animations: [
                    trigger('fadeAnimation', [
                        state('void', style({ opacity: 0 })),
                        state('true', style({ opacity: 1 })),
                        state('false', style({ opacity: 0 })),
                        transition('* => true', animate('150ms cubic-bezier(0.0, 0.0, 0.2, 1)')),
                        transition('* => void', animate('150ms cubic-bezier(0.4, 0.0, 1, 1)')),
                    ])
                ],
                template: `<div [ngClass]="_classMap" *ngIf="_display" [@fadeAnimation]>
    <i class="grf-ui-alert-icon fa" [class.fa-times-circle]="uiType==='error'" [class.fa-check-circle]="uiType==='success'" [class.fa-exclamation-circle]="uiType==='info'" [class.fa-info-circle]="uiType==='warning'" *ngIf="uiShowIcon&&uiDescription"></i>
    <i class="grf-ui-alert-icon fa" [class.fa-times-circle]="uiType==='error'" [class.fa-check-circle]="uiType==='success'" [class.fa-exclamation-circle]="uiType==='info'" [class.fa-info-circle]="uiType==='warning'" *ngIf="uiShowIcon&&!uiDescription"></i>
    <ng-template [ngIf]="uiMessage">
        <span class="grf-ui-alert-message">{{uiMessage}}</span>
    </ng-template>
    <ng-template [ngIf]="!uiMessage">
        <ng-content select="[alert-body]"></ng-content>
    </ng-template>
    <ng-template [ngIf]="uiDescription">
        <span class="grf-ui-alert-description">{{uiDescription}}</span>
    </ng-template>
    <ng-template [ngIf]="!uiDescription">
        <ng-content select="alert-description"></ng-content>
    </ng-template>
    <ng-template [ngIf]="uiCloseable || uiCloseText">
        <a *ngIf="uiCloseable" (click)="closeAlert($event)" class="grf-ui-alert-close-icon">
            <i class="anticon anticon-cross"></i>
        </a>
        <a *ngIf="uiCloseText" class="grf-ui-alert-close-icon" (click)="closeAlert()">{{uiCloseText}}</a>
    </ng-template>
</div>
`,
                styles: [`@charset "UTF-8";@-webkit-keyframes antAlertSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(0);transform:scaleY(0)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@keyframes antAlertSlideUpIn{0%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(0);transform:scaleY(0)}100%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}}@-webkit-keyframes antAlertSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(0);transform:scaleY(0)}}@keyframes antAlertSlideUpOut{0%{opacity:1;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(1);transform:scaleY(1)}100%{opacity:0;-webkit-transform-origin:0 0;transform-origin:0 0;-webkit-transform:scaleY(0);transform:scaleY(0)}}.grf-ui-alert{position:relative;padding:8px 48px 8px 38px;border-radius:4px;color:rgba(0,0,0,.65);font-size:12px;line-height:1.5;display:block}.grf-ui-alert-no-icon{padding:8px 48px 8px 16px}.grf-ui-alert-icon{font-size:14px;top:10px;left:16px;position:absolute}.grf-ui-alert-description{font-size:12px;line-height:21px;display:none}.grf-ui-alert-success{border:1px solid #cfefdf;background-color:#ebf8f2}.grf-ui-alert-success .grf-ui-alert-icon{color:#00a854}.grf-ui-alert-info{border:1px solid #d2eafb;background-color:#ecf6fd}.grf-ui-alert-info .grf-ui-alert-icon{color:#108ee9}.grf-ui-alert-warning{border:1px solid #fff3cf;background-color:#fffaeb}.grf-ui-alert-warning .grf-ui-alert-icon{color:#ffbf00}.grf-ui-alert-error{border:1px solid #fcdbd9;background-color:#fef0ef}.grf-ui-alert-error .grf-ui-alert-icon{color:#f04134}.grf-ui-alert-close-icon{font-size:12px;position:absolute;right:16px;top:10px;height:12px;line-height:12px;overflow:hidden;cursor:pointer}.grf-ui-alert-close-icon .grf-ui-cross{color:rgba(0,0,0,.43);-webkit-transition:color .3s ease;transition:color .3s ease}.grf-ui-alert-close-icon .grf-ui-cross:hover{color:#404040}.grf-ui-alert-close-text{position:absolute;right:16px}.grf-ui-alert-with-description{padding:16px 16px 16px 60px;position:relative;border-radius:4px;color:rgba(0,0,0,.65);line-height:1.5}.grf-ui-alert-with-description .grf-ui-alert-no-icon{padding:16px}.grf-ui-alert-with-description .grf-ui-alert-icon{position:absolute;top:16px;left:20px;font-size:24px}.grf-ui-alert-with-description .grf-ui-alert-close-icon{position:absolute;top:16px;right:16px;cursor:pointer;font-size:12px}.grf-ui-alert-with-description .grf-ui-alert-message{font-size:14px;color:rgba(0,0,0,.85);display:block;margin-bottom:4px}.grf-ui-alert-with-description .grf-ui-alert-description{display:block}.grf-ui-alert-close{height:0!important;margin:0;padding-top:0;padding-bottom:0;-webkit-transition:all .3s cubic-bezier(.78,.14,.15,.86);transition:all .3s cubic-bezier(.78,.14,.15,.86);-webkit-transform-origin:50% 0;transform-origin:50% 0}.grf-ui-alert-slide-up-leave{-webkit-animation:.3s cubic-bezier(.78,.14,.15,.86) antAlertSlideUpOut;animation:.3s cubic-bezier(.78,.14,.15,.86) antAlertSlideUpOut;-webkit-animation-fill-mode:both;animation-fill-mode:both}.grf-ui-alert-banner{border-radius:0;border:0;margin-bottom:0}`]
            },] },
];
/** @nocollapse */
AlertComponent.ctorParameters = () => [];
AlertComponent.propDecorators = {
    "uiType": [{ type: Input },],
    "uiBanner": [{ type: Input },],
    "uiCloseable": [{ type: Input },],
    "uiDescription": [{ type: Input },],
    "uiShowIcon": [{ type: Input },],
    "uiCloseText": [{ type: Input },],
    "uiMessage": [{ type: Input },],
    "uiOnClose": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class AlertModule {
}
AlertModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [AlertComponent],
                exports: [AlertComponent]
            },] },
];
/** @nocollapse */
AlertModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const HIDDEN_TEXTAREA_STYLE = `
min-height:0 !important;
max-height:none !important;
height:0 !important;
visibility:hidden !important;
overflow:hidden !important;
position:absolute !important;
z-index:-1000 !important;
top:0 !important;
right:0 !important
`;
const SIZING_STYLE = [
    'letter-spacing',
    'line-height',
    'padding-top',
    'padding-bottom',
    'font-family',
    'font-weight',
    'font-size',
    'text-rendering',
    'text-transform',
    'width',
    'text-indent',
    'padding-left',
    'padding-right',
    'border-width',
    'box-sizing',
];
let hiddenTextarea;
/**
 * @param {?} node
 * @param {?=} useCache
 * @return {?}
 */
function calculateNodeStyling(node, useCache = false) {
    const /** @type {?} */ nodeRef = (node.getAttribute('id') ||
        node.getAttribute('data-reactid') ||
        node.getAttribute('name'));
    if (useCache && this.computedStyleCache[nodeRef]) {
        return this.computedStyleCache[nodeRef];
    }
    const /** @type {?} */ style$$1 = window.getComputedStyle(node);
    const /** @type {?} */ boxSizing = (style$$1.getPropertyValue('box-sizing') ||
        style$$1.getPropertyValue('-moz-box-sizing') ||
        style$$1.getPropertyValue('-webkit-box-sizing'));
    const /** @type {?} */ paddingSize = (parseFloat(style$$1.getPropertyValue('padding-bottom')) +
        parseFloat(style$$1.getPropertyValue('padding-top')));
    const /** @type {?} */ borderSize = (parseFloat(style$$1.getPropertyValue('border-bottom-width')) +
        parseFloat(style$$1.getPropertyValue('border-top-width')));
    const /** @type {?} */ sizingStyle = SIZING_STYLE
        .map(name => `${name}:${style$$1.getPropertyValue(name)}`)
        .join(';');
    const /** @type {?} */ nodeInfo = {
        sizingStyle,
        paddingSize,
        borderSize,
        boxSizing,
    };
    if (useCache && nodeRef) {
        this.computedStyleCache[nodeRef] = nodeInfo;
    }
    return nodeInfo;
}
/**
 * @param {?} uiTextNode
 * @param {?=} useCache
 * @param {?=} minRows
 * @param {?=} maxRows
 * @return {?}
 */
function calculateNodeHeight(uiTextNode, useCache = false, minRows = null, maxRows = null) {
    if (!hiddenTextarea) {
        hiddenTextarea = document.createElement('textarea');
        document.body.appendChild(hiddenTextarea);
    }
    // Fix wrap="off" issue
    // https://github.com/ant-design/ant-design/issues/6577
    if (uiTextNode.getAttribute('wrap')) {
        hiddenTextarea.setAttribute('wrap', uiTextNode.getAttribute('wrap'));
    }
    else {
        hiddenTextarea.removeAttribute('wrap');
    }
    // Copy all CSS properties that have an impact on the height of the content in
    // the textbox
    const { paddingSize, borderSize, boxSizing, sizingStyle, } = calculateNodeStyling(uiTextNode, useCache);
    // Need to have the overflow attribute to hide the scrollbar otherwise
    // text-lines will not calculated properly as the shadow will technically be
    // narrower for content
    hiddenTextarea.setAttribute('style', `${sizingStyle};${HIDDEN_TEXTAREA_STYLE}`);
    hiddenTextarea.value = uiTextNode.value || uiTextNode.placeholder || '';
    let /** @type {?} */ minHeight = -Infinity;
    let /** @type {?} */ maxHeight = Infinity;
    let /** @type {?} */ height = hiddenTextarea.scrollHeight;
    let /** @type {?} */ overflowY;
    if (boxSizing === 'border-box') {
        // border-box: add border, since height = content + padding + border
        height = height + borderSize / 2 + paddingSize / 2;
    }
    else if (boxSizing === 'content-box') {
        // remove padding, since height = content
        height = height - paddingSize;
    }
    if (minRows !== null || maxRows !== null) {
        // measure height of a textarea with a single row
        hiddenTextarea.value = '';
        const /** @type {?} */ singleRowHeight = hiddenTextarea.scrollHeight - paddingSize;
        if (minRows !== null) {
            minHeight = singleRowHeight * minRows;
            if (boxSizing === 'border-box') {
                minHeight = minHeight + paddingSize + borderSize;
            }
            height = Math.max(minHeight, height);
        }
        if (maxRows !== null) {
            maxHeight = singleRowHeight * maxRows;
            if (boxSizing === 'border-box') {
                maxHeight = maxHeight + paddingSize + borderSize;
            }
            overflowY = height > maxHeight ? '' : 'hidden';
            height = Math.min(maxHeight, height);
        }
    }
    // Remove scroll bar flash when autosize without maxRows
    if (!maxRows) {
        overflowY = 'hidden';
    }
    return { height, minHeight, maxHeight, overflowY };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

class InputDirectiveComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _render
     */
    constructor(_elementRef, _render) {
        this._elementRef = _elementRef;
        this._render = _render;
        this.size = 'default';
        this._classList = [];
        this._readonly = false;
        this._prefixCls = 'grf-ui-input';
        this._autosize = false;
        this._disabled = false;
        this._uiInput = true;
        this.nativeElement = this._elementRef.nativeElement;
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this.size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this.size = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiDisabled() {
        return this._disabled;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiDisabled(value) {
        if (value) {
            this._render.setAttribute(this.nativeElement, 'disabled', '');
        }
        else {
            this._render.removeAttribute(this.nativeElement, 'disabled');
        }
        this._disabled = value;
    }
    /**
     * @return {?}
     */
    get uiReadonly() {
        return this._readonly;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiReadonly(value) {
        if (value) {
            this._render.setAttribute(this.nativeElement, 'readonly', '');
        }
        else {
            this._render.removeAttribute(this.nativeElement, 'readonly');
        }
        this._readonly = value;
    }
    /**
     * @return {?}
     */
    get uiAutosize() {
        return this._autosize;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAutosize(value) {
        if (typeof value === 'string') {
            this._autosize = true;
        }
        else {
            this._autosize = /** @type {?} */ (value);
        }
        if (this._autosize) {
            this._render.setAttribute(this.nativeElement, 'autosize', '');
        }
        else {
            this._render.removeAttribute(this.nativeElement, 'autosize');
        }
    }
    /**
     * @return {?}
     */
    resizeTextarea() {
        const /** @type {?} */ textAreaRef = this._elementRef.nativeElement;
        // eliminate jitter
        this.nativeElement.style.height = 'auto';
        const /** @type {?} */ maxRows = this.uiAutosize ? (/** @type {?} */ (this.uiAutosize)).maxRows || null : null;
        const /** @type {?} */ minRows = this.uiAutosize ? (/** @type {?} */ (this.uiAutosize)).minRows || null : null;
        const /** @type {?} */ textareaStyles = calculateNodeHeight(this.nativeElement, false, minRows, maxRows);
        this.nativeElement.style.height = `${textareaStyles.height}px`;
        this.nativeElement.style.overflowY = textareaStyles.overflowY;
    }
    /**
     * @return {?}
     */
    _setClassMap() {
        this._classList.forEach(_className => {
            this._render.removeClass(this.nativeElement, _className);
        });
        this._classList = [
            this.uiSize && `${this._prefixCls}-${this.uiSize}`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._render.addClass(this.nativeElement, _className);
        });
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onFocus(e) {
        if (this._elementRef.nativeElement.type === 'textarea' && this.uiAutosize) {
            this.resizeTextarea();
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onBlur(e) {
        if (this._elementRef.nativeElement.type === 'textarea' && this.uiAutosize) {
            this.resizeTextarea();
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onKeyup(e) {
        if (this._elementRef.nativeElement.type === 'textarea' && this.uiAutosize) {
            this.resizeTextarea();
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._elementRef.nativeElement.type === 'textarea' && this.uiAutosize) {
            setTimeout(() => this.resizeTextarea(), 0);
        }
        else if (this._elementRef.nativeElement.type === 'textarea') {
            this.nativeElement.style.height = 'auto';
        }
    }
}
InputDirectiveComponent.decorators = [
    { type: Component, args: [{
                selector: '[grf-input]',
                encapsulation: ViewEncapsulation.None,
                template: ``,
                styles: [`@charset "UTF-8";.grf-ui-input-group-line-danger i{color:#f04134}.grf-ui-input-group-line-danger .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#f04134}.grf-ui-input-group-line-danger .grf-ui-input-line,.grf-ui-input-group-line-danger .grf-ui-input:hover+.grf-ui-input-line{background-color:#f04134}.grf-ui-input-group-line-warning i{color:#ffbf00}.grf-ui-input-group-line-warning .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#ffbf00}.grf-ui-input-group-line-warning .grf-ui-input-line,.grf-ui-input-group-line-warning .grf-ui-input:hover+.grf-ui-input-line{background-color:#ffbf00}.grf-ui-input-group-line-primary i{color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input:hover+.grf-ui-input-line{background-color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input-line{background-color:#ced4da}.grf-ui-input-group-line-success i{color:#00a854}.grf-ui-input-group-line-success .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#00a854}.grf-ui-input-group-line-success .grf-ui-input-line,.grf-ui-input-group-line-success .grf-ui-input:hover+.grf-ui-input-line{background-color:#00a854}.lineAnimate{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate}@-webkit-keyframes inputLineAnimate{0%{width:0}100%{width:100%}}@keyframes inputLineAnimate{0%{width:0}100%{width:100%}}.grf-ui-input{-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;display:inline-block;padding:4px 8px;width:100%;height:30px;font-family:sans-serif;cursor:text;font-size:14px;line-height:1.5;color:#495057;background-color:#fff;background-image:none;border:1px solid #ced4da;border-radius:4px;-webkit-transition:all .3s;transition:all .3s}.grf-ui-input::-moz-placeholder{color:#868e96;opacity:1}.grf-ui-input:-ms-input-placeholder{color:#868e96}.grf-ui-input::-webkit-input-placeholder{color:#868e96}.grf-ui-input:hover{border-color:#0275d8}.grf-ui-input:focus{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-input:disabled{background-color:#f7f7f7;opacity:1;pointer-events:none;color:#495057}.grf-ui-input:-moz-read-only{background-color:#f7f7f7;opacity:1;color:#495057;cursor:auto}.grf-ui-input:read-only{background-color:#f7f7f7;opacity:1;color:#495057;cursor:auto}.grf-ui-input textarea{max-width:100%;height:auto;vertical-align:bottom}.grf-ui-input-lg{padding:6px 8px;height:34px}.grf-ui-input-sm{padding:1px 8px;height:24px}.grf-ui-input-group{position:relative;display:table;border-collapse:separate;border-spacing:0;margin:5px 0;width:100%}.grf-ui-input-group>[class*=col-]{padding-right:8px}.grf-ui-input-group .grf-ui-input,.grf-ui-input-group .grf-ui-input-group-addon,.grf-ui-input-group .grf-ui-input-group-wrap{display:table-cell}.grf-ui-input-group .grf-ui-input-group-addon:not(:first-child):not(:last-child),.grf-ui-input-group .grf-ui-input-group-wrap:not(:first-child):not(:last-child),.grf-ui-input-group .grf-ui-input:not(:first-child):not(:last-child){border-radius:0}.grf-ui-input-group .grf-ui-input-group-addon,.grf-ui-input-group .grf-ui-input-group-wrap{width:1px;white-space:nowrap;vertical-align:middle}.grf-ui-input-group .grf-ui-input-group-wrap{display:block!important}.grf-ui-input-group .grf-ui-input{float:left;width:100%;margin-bottom:0}.grf-ui-input-group .grf-ui-input:focus{z-index:1}.grf-ui-input-group .grf-ui-input-group-addon{padding:4px 8px;font-size:14px;font-weight:400;line-height:1;color:#495057;text-align:center;background-color:#eee;border:1px solid #ced4da;border-radius:4px;position:relative;-webkit-transition:all .3s;transition:all .3s}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select{margin:-5px -8px}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select .grf-ui-select-selection{background-color:inherit;margin:-1px;border:1px solid transparent;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select-focused .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select-open .grf-ui-select-selection{color:#0275d8}.grf-ui-input-group .grf-ui-input-group-addon>i:only-child:after{position:absolute;content:'';top:0;left:0;right:0;bottom:0}.grf-ui-input-group .grf-ui-input-group-addon:first-child,.grf-ui-input-group .grf-ui-input-group-addon:first-child .grf-ui-select .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input:not(:last-child),.grf-ui-input-group .grf-ui-input:not(:last-child) .grf-ui-select .grf-ui-select-selection{border-bottom-right-radius:0;border-top-right-radius:0}.grf-ui-input-group .grf-ui-input-affix-wrapper:not(:first-child).grf-ui-input{border-bottom-left-radius:0;border-top-left-radius:0}.grf-ui-input-group .grf-ui-input-affix-wrapper:not(:last-child).grf-ui-input{border-bottom-right-radius:0;border-top-right-radius:0}.grf-ui-input-group .grf-ui-input-group-addon:first-child{border-right:0}.grf-ui-input-group .grf-ui-input-group-addon:last-child{border-left:0}.grf-ui-input-group .grf-ui-input-group-addon:last-child,.grf-ui-input-group .grf-ui-input-group-addon:last-child .grf-ui-select .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input:not(:first-child),.grf-ui-input-group .grf-ui-input:not(:first-child) .grf-ui-select .grf-ui-select-selection{border-bottom-left-radius:0;border-top-left-radius:0}.grf-ui-input-group .grf-ui-input-group-lg .grf-ui-select-selection--single{height:34px}.grf-ui-input-group .grf-ui-input-group-sm .grf-ui-select-selection--single{height:24px}.grf-ui-input-group .grf-ui-input-affix-wrapper{display:table-cell;float:left}.grf-ui-input-group.grf-ui-input-group-compact{border-radius:0;border-right-width:0;vertical-align:middle;float:none}.grf-ui-input-group.grf-ui-input-group-compact>[class*=col-]{padding-right:0}.grf-ui-input-group.grf-ui-input-group-compact .grf-ui-input:not(:last-child){border-right:none}.grf-ui-input-group.grf-ui-input-group-compact .grf-ui-input{float:left;z-index:auto}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker .grf-ui-time-picker-input{border-radius:0;border-right-width:0}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper:first-child .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select:first-child>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker:first-child .grf-ui-time-picker-input,.grf-ui-input-group.grf-ui-input-group-compact>:first-child{border-top-left-radius:4px;border-bottom-left-radius:4px}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper:last-child .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select:last-child>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker:last-child .grf-ui-time-picker-input,.grf-ui-input-group.grf-ui-input-group-compact>:last-child{border-top-right-radius:4px;border-bottom-right-radius:4px;border-right-width:1px}.grf-ui-input-group-lg .grf-ui-input,.grf-ui-input-group-lg>.grf-ui-input-group-addon{padding:6px 8px;height:34px}.grf-ui-input-group-sm .grf-ui-input,.grf-ui-input-group-sm>.grf-ui-input-group-addon{padding:1px 8px;height:24px}.grf-ui-input-group-line .grf-ui-input{border:none;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group-line .grf-ui-input:focus{outline:0;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group-line .grf-ui-input:disabled{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:-moz-read-only{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:read-only{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%}.grf-ui-input-group-line .grf-ui-input-line{position:absolute;top:100%;width:100%;height:1px}.grf-ui-input-group-line .grf-ui-input-line .grf-ui-input-lineChild{height:200%;width:100%}.grf-ui-input-affix-wrapper{position:relative;display:inline-block;width:100%;margin:5px 0}.grf-ui-input-affix-wrapper .grf-ui-input{z-index:1;min-height:100%}.grf-ui-input-affix-wrapper:hover .grf-ui-input:not(.grf-ui-input-disabled){border-color:#0275d8}.grf-ui-input-affix-wrapper .grf-ui-input-prefix,.grf-ui-input-affix-wrapper .grf-ui-input-suffix{position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);z-index:2;line-height:0;color:#495057}.grf-ui-input-affix-wrapper .grf-ui-input-prefix{left:8px}.grf-ui-input-affix-wrapper .grf-ui-input-suffix{right:8px}.grf-ui-input-affix-wrapper .grf-ui-input-suffix .grf-ui-input-suffix-clear{font-size:14px;color:#ced4da}.grf-ui-input-affix-wrapper .grf-ui-input-suffix .grf-ui-input-suffix-clear:hover{color:#868e96}.grf-ui-input-affix-wrapper .grf-ui-input:not(:first-child){padding-left:30px}.grf-ui-input-affix-wrapper .grf-ui-input:not(:last-child){padding-right:30px}label{font-size:14px;color:#868e96;padding:0 8px}.grf-ui-label-prefix{left:22px}.grf-ui-label-focus-danger{color:#f04134}.grf-ui-label-focus-warning{color:#ffbf00}.grf-ui-label-focus-success{color:#00a854}.grf-ui-label-focus-primary{color:#0275d8}.grf-ui-label-danger{color:#f04134;font-size:12px}.grf-ui-label-warning{color:#ffbf00;font-size:12px}.grf-ui-label-success{color:#00a854;font-size:12px}.grf-ui-label-primary{color:#0275d8;font-size:12px}.grf-ui-label-fill-danger{position:absolute;height:100%;z-index:5;top:12px;color:#f04134;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-fill-warning{position:absolute;height:100%;z-index:5;top:12px;color:#ffbf00;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-fill-primary,.grf-ui-label-fill-success{position:absolute;height:100%;z-index:5;top:12px;color:#868e96;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-prefix-fill-danger{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#f04134;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-prefix-fill-warning{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#ffbf00;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-prefix-fill-primary,.grf-ui-label-prefix-fill-success{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#868e96;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-flex-danger,.grf-ui-label-flex-primary,.grf-ui-label-flex-success,.grf-ui-label-flex-warning{position:absolute;height:100%;z-index:5;top:-19px;pointer-events:none;font-size:12px;-webkit-animation:.3s labelFlexAnimate;animation:.3s labelFlexAnimate}.grf-ui-label-prefix-flex-danger,.grf-ui-label-prefix-flex-primary,.grf-ui-label-prefix-flex-success,.grf-ui-label-prefix-flex-warning{position:absolute;height:100%;z-index:5;top:-19px;left:0;pointer-events:none;font-size:12px;-webkit-animation:.3s labelPrefixFlexAnimate;animation:.3s labelPrefixFlexAnimate}@-webkit-keyframes labelFlexAnimate{0%{top:12px}100%{top:-19px}}@keyframes labelFlexAnimate{0%{top:12px}100%{top:-19px}}@-webkit-keyframes fillFillAnimate{0%{top:-19px}100%{top:12px}}@keyframes fillFillAnimate{0%{top:-19px}100%{top:12px}}@-webkit-keyframes labelPrefixFlexAnimate{0%{top:12px;left:22px}100%{top:-19px;left:0}}@keyframes labelPrefixFlexAnimate{0%{top:12px;left:22px}100%{top:-19px;left:0}}@-webkit-keyframes fillPrefixFillAnimate{0%{top:-19px;left:0}100%{top:12px;left:22px}}@keyframes fillPrefixFillAnimate{0%{top:-19px;left:0}100%{top:12px;left:22px}}.grf-ui-input.ng-invalid:not(form),.grf-ui-input.ng-valid.required,.grf-ui-input.ng-valid[required]{border:none}`]
            },] },
];
/** @nocollapse */
InputDirectiveComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
InputDirectiveComponent.propDecorators = {
    "_disabled": [{ type: HostBinding, args: [`class.grf-ui-input-disabled`,] },],
    "_uiInput": [{ type: HostBinding, args: [`class.grf-ui-input`,] },],
    "uiSize": [{ type: Input },],
    "uiDisabled": [{ type: Input },],
    "uiReadonly": [{ type: Input },],
    "uiAutosize": [{ type: Input },],
    "onFocus": [{ type: HostListener, args: ['focus', ['$event'],] },],
    "onBlur": [{ type: HostListener, args: ['blur', ['$event'],] },],
    "onKeyup": [{ type: HostListener, args: ['keyup', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class InputGroupComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _render
     */
    constructor(_elementRef, _render) {
        this._elementRef = _elementRef;
        this._render = _render;
        this._prefixCls = 'grf-ui-input';
        this._prefix = 'grf-ui';
        this._type = 'primary';
        this._classList = [];
        this.clearActive = false;
        this.clearMouseOver = false;
        this.clearEvent = new EventEmitter();
        this.uiLine = false;
        this.uiClear = false;
        this.uiCompact = false;
        this._el = this._elementRef.nativeElement;
        this._render.addClass(this._el, `${this._prefixCls}-group`);
        this.parent = this._render.parentNode(this._el);
    }
    /**
     * @return {?}
     */
    get _isLarge() {
        return this.uiSize === 'lg';
    }
    /**
     * @return {?}
     */
    get _isSmall() {
        return this.uiSize === 'sm';
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        this._type = value;
        this._setClassMap();
        this.contextIconSet();
        if (this.parent.querySelector('#grf-ui-add-label')) {
            if (this.uiLabelType === 'flex') {
                if (!this._prefixContent) {
                    if (this.uiType !== 'primary') {
                        this._render.setAttribute(this.parent.querySelector('#grf-ui-add-label'), 'class', `${this._prefix}-label-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType} `);
                    }
                    else {
                        this._render.setAttribute(this.parent.querySelector('#grf-ui-add-label'), 'class', `${this._prefix}-label-flex-${this.uiType}  `);
                    }
                }
                else if (this._prefixContent) {
                    if (this.uiType !== 'primary') {
                        this._render.setAttribute(this.parent.querySelector('#grf-ui-add-label'), 'class', `${this._prefix}-label-prefix-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType}`);
                    }
                    else {
                        this._render.setAttribute(this.parent.querySelector('#grf-ui-add-label'), 'class', `${this._prefix}-label-flex-${this.uiType}  `);
                    }
                }
            }
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        const /** @type {?} */ inputElement = this._el.getElementsByTagName('input');
        if (this.uiClear) {
            this.elementListen(inputElement);
        }
        if (this._prefixContent || this._suffixContent || this.uiClear || this.contextIconEl) {
            this._render.setAttribute(this._el, 'class', `${this._prefixCls}-affix-wrapper`);
        }
        if ((this._addOnContentBefore || this._addOnContentAfter || this.uiCompact)) {
            this._render.setAttribute(this._el, 'class', `${this._prefixCls}-group`);
        }
        if (this.uiCompact) {
            this._render.addClass(this._el, `${this._prefixCls}-group-compact`);
        }
        if (this.uiLine) {
            this._render.addClass(this._el, `${this._prefixCls}-group-line`);
        }
        if (this.uiLabel && this.uiLabelType === 'row') {
            const /** @type {?} */ labelElement = this._render.createElement('label');
            const /** @type {?} */ text = this._render.createText(this.uiLabel);
            this._render.setAttribute(labelElement, 'id', 'grf-ui-add-label');
            this._render.appendChild(labelElement, text);
            this._render.insertBefore(this._el.parentNode, labelElement, this._el);
            this.focusListen(inputElement, labelElement);
            this.blurListen(inputElement, labelElement);
        }
        if (this.uiLabel && this.uiLabelType === 'col') {
            const /** @type {?} */ labelElement = this._render.createElement('label');
            const /** @type {?} */ text = this._render.createText(this.uiLabel);
            this._render.setAttribute(labelElement, 'id', 'grf-ui-add-label');
            this._render.appendChild(labelElement, text);
            this._render.insertBefore(this._el.parentNode, labelElement, this._el);
            const /** @type {?} */ parent = this._el.parentElement;
            this._render.setStyle(labelElement, 'paddingRight', 20 + 'px');
            this._render.setStyle(labelElement, 'lineHeight', 10 + this._el.offsetHeight + 'px');
            this._render.setStyle(parent, 'display', 'flex');
            this.focusListen(inputElement, labelElement);
            this.blurListen(inputElement, labelElement);
        }
        if (this.uiLabel && this.uiLabelType === 'flex') {
            const /** @type {?} */ labelElement = this._render.createElement('label');
            const /** @type {?} */ text = this._render.createText(this.uiLabel);
            this._render.setAttribute(labelElement, 'id', 'grf-ui-add-label');
            this._render.appendChild(labelElement, text);
            this._render.insertBefore(this._el.parentNode, labelElement, this._el);
            this._render.setAttribute(inputElement[0], 'placeholder', '');
            this.focusListen(inputElement, labelElement, true);
            this.blurListen(inputElement, labelElement, true);
            if (this.uiType !== 'primary') {
                this._render.addClass(labelElement, `${this._prefix}-label-focus-${this.uiType}`);
            }
            if (!this._prefixContent) {
                if (inputElement[0].value) {
                    this._render.addClass(labelElement, `${this._prefix}-label-flex-${this.uiType}`);
                }
                else if (!inputElement[0].value) {
                    this._render.addClass(labelElement, `${this._prefix}-label-fill-${this.uiType}`);
                }
            }
            if (this._prefixContent) {
                if (inputElement[0].value) {
                    this._render.addClass(labelElement, `${this._prefix}-label-prefix-flex-${this.uiType}`);
                }
                else if (!inputElement[0].value) {
                    this._render.addClass(labelElement, `${this._prefix}-label-prefix-fill-${this.uiType}`);
                }
            }
        }
        this.contextIconSet();
        this._setClassMap();
    }
    /**
     * @param {?} inputElement
     * @return {?}
     */
    elementListen(inputElement) {
        Observable.fromEvent(inputElement, 'focus')
            .subscribe((event) => {
            this.clearActive = true;
        });
        Observable.fromEvent(inputElement, 'blur')
            .subscribe((event) => {
            if (this.clearMouseOver === false) {
                this.clearActive = false;
            }
            if (this.clearMouseOver === true) {
                this.clearActive = true;
            }
        });
    }
    /**
     * @param {?} inputElement
     * @param {?=} labelElement
     * @param {?=} flex
     * @return {?}
     */
    focusListen(inputElement, labelElement, flex) {
        Observable.fromEvent(inputElement, 'focus')
            .subscribe((event) => {
            if (labelElement) {
                if (flex) {
                    if (!this._prefixContent) {
                        this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType} `);
                    }
                    else if (this._prefixContent) {
                        this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-prefix-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType}`);
                    }
                }
            }
        });
    }
    /**
     * @param {?} inputElement
     * @param {?=} labelElement
     * @param {?=} flex
     * @return {?}
     */
    blurListen(inputElement, labelElement, flex) {
        Observable.fromEvent(inputElement, 'blur')
            .subscribe((event) => {
            this.contextIconSet();
            if (labelElement) {
                if (flex) {
                    if (!inputElement[0].value) {
                        if (!this._prefixContent) {
                            this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-fill-${this.uiType} ${this._prefix}-label-focus-${this.uiType}`);
                        }
                        else if (this._prefixContent) {
                            this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-prefix-fill-${this.uiType} ${this._prefix}-label-focus-${this.uiType}`);
                        }
                    }
                    else {
                        if (!this._prefixContent) {
                            this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType} `);
                        }
                        else if (this._prefixContent) {
                            this._render.setAttribute(labelElement, 'class', `${this._prefix}-label-prefix-flex-${this.uiType} ${this._prefix}-label-focus-${this.uiType}`);
                        }
                    }
                }
            }
            if (this.uiType === 'primary') {
                this._render.removeClass(labelElement, `${this._prefix}-label-focus-${this.uiType}`);
            }
        });
    }
    /**
     * @return {?}
     */
    clearContent() {
        const /** @type {?} */ inputElement = this._el.getElementsByTagName('input');
        inputElement[0].value = '';
        inputElement[0].focus();
        this.clearEvent.emit();
    }
    /**
     * @return {?}
     */
    clearOver() {
        this.clearMouseOver = true;
    }
    /**
     * @return {?}
     */
    clearOut() {
        this.clearMouseOver = false;
    }
    /**
     * @return {?}
     */
    _setClassMap() {
        this._classList.forEach(_className => {
            this._render.removeClass(this._el, _className);
        });
        this._classList = [
            this.uiLine && this.uiType && `${this._prefixCls}-group-line-${this.uiType}`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._render.addClass(this._el, _className);
        });
    }
    /**
     * @return {?}
     */
    contextIconSet() {
        if (this.contextIconEl) {
            switch (this.uiType) {
                case 'success':
                    this._render.setAttribute(this.contextIconEl.nativeElement, 'class', 'fa fa-check-circle grf-ui-label-success');
                    break;
                case 'warning':
                    this._render.setAttribute(this.contextIconEl.nativeElement, 'class', 'fa fa-exclamation-circle grf-ui-label-warning');
                    break;
                case 'danger':
                    this._render.setAttribute(this.contextIconEl.nativeElement, 'class', 'fa fa-times-circle grf-ui-label-danger');
                    break;
                default:
                    this._render.setAttribute(this.contextIconEl.nativeElement, 'class', '');
                    break;
            }
        }
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
InputGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-input-group',
                template: `<span class="grf-ui-input-group-addon" *ngIf="_addOnContentBefore">
    <ng-template [ngTemplateOutlet]="_addOnContentBefore">
    </ng-template>
</span>
<span class="grf-ui-input-prefix" *ngIf="_prefixContent">
    <ng-template [ngTemplateOutlet]="_prefixContent">
    </ng-template>
</span>
<ng-content></ng-content>
<div class="grf-ui-input-line" *ngIf="uiLine">
    <div class="grf-ui-input-lineChild"></div>
</div>
<span class="grf-ui-input-group-addon" *ngIf="_addOnContentAfter">
    <ng-template [ngTemplateOutlet]="_addOnContentAfter">
    </ng-template>
</span>
<span class="grf-ui-input-suffix" *ngIf="_suffixContent">
    <ng-template [ngTemplateOutlet]="_suffixContent">
    </ng-template>
</span>
<span class="grf-ui-input-suffix" *ngIf="!_suffixContent&&uiClear&&clearActive" (click)="clearContent()" (mouseenter)="clearOver()"
    (mouseleave)="clearOut()">
    <a class="fa fa-times-circle grf-ui-input-suffix-clear"></a>
</span>
<span class="grf-ui-input-suffix" *ngIf="!_suffixContent&&!clearActive&&uiType&&!uiCompact" >
    <a  #contextIcon></a>
</span>
`,
                encapsulation: ViewEncapsulation.None,
                styles: [`@charset "UTF-8";.grf-ui-input-group-line-danger i{color:#f04134}.grf-ui-input-group-line-danger .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#f04134}.grf-ui-input-group-line-danger .grf-ui-input-line,.grf-ui-input-group-line-danger .grf-ui-input:hover+.grf-ui-input-line{background-color:#f04134}.grf-ui-input-group-line-warning i{color:#ffbf00}.grf-ui-input-group-line-warning .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#ffbf00}.grf-ui-input-group-line-warning .grf-ui-input-line,.grf-ui-input-group-line-warning .grf-ui-input:hover+.grf-ui-input-line{background-color:#ffbf00}.grf-ui-input-group-line-primary i{color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input:hover+.grf-ui-input-line{background-color:#0275d8}.grf-ui-input-group-line-primary .grf-ui-input-line{background-color:#ced4da}.grf-ui-input-group-line-success i{color:#00a854}.grf-ui-input-group-line-success .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%;background-color:#00a854}.grf-ui-input-group-line-success .grf-ui-input-line,.grf-ui-input-group-line-success .grf-ui-input:hover+.grf-ui-input-line{background-color:#00a854}.lineAnimate{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate}@-webkit-keyframes inputLineAnimate{0%{width:0}100%{width:100%}}@keyframes inputLineAnimate{0%{width:0}100%{width:100%}}.grf-ui-input{-webkit-box-sizing:border-box;box-sizing:border-box;position:relative;display:inline-block;padding:4px 8px;width:100%;height:30px;font-family:sans-serif;cursor:text;font-size:14px;line-height:1.5;color:#495057;background-color:#fff;background-image:none;border:1px solid #ced4da;border-radius:4px;-webkit-transition:all .3s;transition:all .3s}.grf-ui-input::-moz-placeholder{color:#868e96;opacity:1}.grf-ui-input:-ms-input-placeholder{color:#868e96}.grf-ui-input::-webkit-input-placeholder{color:#868e96}.grf-ui-input:hover{border-color:#0275d8}.grf-ui-input:focus{border-color:#0275d8;outline:0;-webkit-box-shadow:0 0 0 2px #daeeff;box-shadow:0 0 0 2px #daeeff}.grf-ui-input:disabled{background-color:#f7f7f7;opacity:1;pointer-events:none;color:#495057}.grf-ui-input:-moz-read-only{background-color:#f7f7f7;opacity:1;color:#495057;cursor:auto}.grf-ui-input:read-only{background-color:#f7f7f7;opacity:1;color:#495057;cursor:auto}.grf-ui-input textarea{max-width:100%;height:auto;vertical-align:bottom}.grf-ui-input-lg{padding:6px 8px;height:34px}.grf-ui-input-sm{padding:1px 8px;height:24px}.grf-ui-input-group{position:relative;display:table;border-collapse:separate;border-spacing:0;margin:5px 0;width:100%}.grf-ui-input-group>[class*=col-]{padding-right:8px}.grf-ui-input-group .grf-ui-input,.grf-ui-input-group .grf-ui-input-group-addon,.grf-ui-input-group .grf-ui-input-group-wrap{display:table-cell}.grf-ui-input-group .grf-ui-input-group-addon:not(:first-child):not(:last-child),.grf-ui-input-group .grf-ui-input-group-wrap:not(:first-child):not(:last-child),.grf-ui-input-group .grf-ui-input:not(:first-child):not(:last-child){border-radius:0}.grf-ui-input-group .grf-ui-input-group-addon,.grf-ui-input-group .grf-ui-input-group-wrap{width:1px;white-space:nowrap;vertical-align:middle}.grf-ui-input-group .grf-ui-input-group-wrap{display:block!important}.grf-ui-input-group .grf-ui-input{float:left;width:100%;margin-bottom:0}.grf-ui-input-group .grf-ui-input:focus{z-index:1}.grf-ui-input-group .grf-ui-input-group-addon{padding:4px 8px;font-size:14px;font-weight:400;line-height:1;color:#495057;text-align:center;background-color:#eee;border:1px solid #ced4da;border-radius:4px;position:relative;-webkit-transition:all .3s;transition:all .3s}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select{margin:-5px -8px}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select .grf-ui-select-selection{background-color:inherit;margin:-1px;border:1px solid transparent;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select-focused .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input-group-addon .grf-ui-select-open .grf-ui-select-selection{color:#0275d8}.grf-ui-input-group .grf-ui-input-group-addon>i:only-child:after{position:absolute;content:'';top:0;left:0;right:0;bottom:0}.grf-ui-input-group .grf-ui-input-group-addon:first-child,.grf-ui-input-group .grf-ui-input-group-addon:first-child .grf-ui-select .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input:not(:last-child),.grf-ui-input-group .grf-ui-input:not(:last-child) .grf-ui-select .grf-ui-select-selection{border-bottom-right-radius:0;border-top-right-radius:0}.grf-ui-input-group .grf-ui-input-affix-wrapper:not(:first-child).grf-ui-input{border-bottom-left-radius:0;border-top-left-radius:0}.grf-ui-input-group .grf-ui-input-affix-wrapper:not(:last-child).grf-ui-input{border-bottom-right-radius:0;border-top-right-radius:0}.grf-ui-input-group .grf-ui-input-group-addon:first-child{border-right:0}.grf-ui-input-group .grf-ui-input-group-addon:last-child{border-left:0}.grf-ui-input-group .grf-ui-input-group-addon:last-child,.grf-ui-input-group .grf-ui-input-group-addon:last-child .grf-ui-select .grf-ui-select-selection,.grf-ui-input-group .grf-ui-input:not(:first-child),.grf-ui-input-group .grf-ui-input:not(:first-child) .grf-ui-select .grf-ui-select-selection{border-bottom-left-radius:0;border-top-left-radius:0}.grf-ui-input-group .grf-ui-input-group-lg .grf-ui-select-selection--single{height:34px}.grf-ui-input-group .grf-ui-input-group-sm .grf-ui-select-selection--single{height:24px}.grf-ui-input-group .grf-ui-input-affix-wrapper{display:table-cell;float:left}.grf-ui-input-group.grf-ui-input-group-compact{border-radius:0;border-right-width:0;vertical-align:middle;float:none}.grf-ui-input-group.grf-ui-input-group-compact>[class*=col-]{padding-right:0}.grf-ui-input-group.grf-ui-input-group-compact .grf-ui-input:not(:last-child){border-right:none}.grf-ui-input-group.grf-ui-input-group-compact .grf-ui-input{float:left;z-index:auto}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker .grf-ui-time-picker-input{border-radius:0;border-right-width:0}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper:first-child .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete:first-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select:first-child>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker:first-child .grf-ui-time-picker-input,.grf-ui-input-group.grf-ui-input-group-compact>:first-child{border-top-left-radius:4px;border-bottom-left-radius:4px}.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-calendar-picker:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-cascader-picker:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-mention-wrapper:last-child .grf-ui-mention-editor,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select-auto-complete:last-child .grf-ui-input,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-select:last-child>.grf-ui-select-selection,.grf-ui-input-group.grf-ui-input-group-compact>.grf-ui-time-picker:last-child .grf-ui-time-picker-input,.grf-ui-input-group.grf-ui-input-group-compact>:last-child{border-top-right-radius:4px;border-bottom-right-radius:4px;border-right-width:1px}.grf-ui-input-group-lg .grf-ui-input,.grf-ui-input-group-lg>.grf-ui-input-group-addon{padding:6px 8px;height:34px}.grf-ui-input-group-sm .grf-ui-input,.grf-ui-input-group-sm>.grf-ui-input-group-addon{padding:1px 8px;height:24px}.grf-ui-input-group-line .grf-ui-input{border:none;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group-line .grf-ui-input:focus{outline:0;-webkit-box-shadow:none;box-shadow:none}.grf-ui-input-group-line .grf-ui-input:disabled{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:-moz-read-only{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:read-only{background-color:#fff}.grf-ui-input-group-line .grf-ui-input:focus+.grf-ui-input-line .grf-ui-input-lineChild{-webkit-animation:.5s inputLineAnimate;animation:.5s inputLineAnimate;width:100%}.grf-ui-input-group-line .grf-ui-input-line{position:absolute;top:100%;width:100%;height:1px}.grf-ui-input-group-line .grf-ui-input-line .grf-ui-input-lineChild{height:200%;width:100%}.grf-ui-input-affix-wrapper{position:relative;display:inline-block;width:100%;margin:5px 0}.grf-ui-input-affix-wrapper .grf-ui-input{z-index:1;min-height:100%}.grf-ui-input-affix-wrapper:hover .grf-ui-input:not(.grf-ui-input-disabled){border-color:#0275d8}.grf-ui-input-affix-wrapper .grf-ui-input-prefix,.grf-ui-input-affix-wrapper .grf-ui-input-suffix{position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);z-index:2;line-height:0;color:#495057}.grf-ui-input-affix-wrapper .grf-ui-input-prefix{left:8px}.grf-ui-input-affix-wrapper .grf-ui-input-suffix{right:8px}.grf-ui-input-affix-wrapper .grf-ui-input-suffix .grf-ui-input-suffix-clear{font-size:14px;color:#ced4da}.grf-ui-input-affix-wrapper .grf-ui-input-suffix .grf-ui-input-suffix-clear:hover{color:#868e96}.grf-ui-input-affix-wrapper .grf-ui-input:not(:first-child){padding-left:30px}.grf-ui-input-affix-wrapper .grf-ui-input:not(:last-child){padding-right:30px}label{font-size:14px;color:#868e96;padding:0 8px}.grf-ui-label-prefix{left:22px}.grf-ui-label-focus-danger{color:#f04134}.grf-ui-label-focus-warning{color:#ffbf00}.grf-ui-label-focus-success{color:#00a854}.grf-ui-label-focus-primary{color:#0275d8}.grf-ui-label-danger{color:#f04134;font-size:12px}.grf-ui-label-warning{color:#ffbf00;font-size:12px}.grf-ui-label-success{color:#00a854;font-size:12px}.grf-ui-label-primary{color:#0275d8;font-size:12px}.grf-ui-label-fill-danger{position:absolute;height:100%;z-index:5;top:12px;color:#f04134;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-fill-warning{position:absolute;height:100%;z-index:5;top:12px;color:#ffbf00;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-fill-primary,.grf-ui-label-fill-success{position:absolute;height:100%;z-index:5;top:12px;color:#868e96;pointer-events:none;font-size:14px;-webkit-animation:.3s fillFillAnimate;animation:.3s fillFillAnimate}.grf-ui-label-prefix-fill-danger{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#f04134;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-prefix-fill-warning{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#ffbf00;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-prefix-fill-primary,.grf-ui-label-prefix-fill-success{position:absolute;height:100%;z-index:5;top:12px;left:22px;color:#868e96;pointer-events:none;font-size:14px;-webkit-animation:.3s fillPrefixFillAnimate;animation:.3s fillPrefixFillAnimate}.grf-ui-label-flex-danger,.grf-ui-label-flex-primary,.grf-ui-label-flex-success,.grf-ui-label-flex-warning{position:absolute;height:100%;z-index:5;top:-19px;pointer-events:none;font-size:12px;-webkit-animation:.3s labelFlexAnimate;animation:.3s labelFlexAnimate}.grf-ui-label-prefix-flex-danger,.grf-ui-label-prefix-flex-primary,.grf-ui-label-prefix-flex-success,.grf-ui-label-prefix-flex-warning{position:absolute;height:100%;z-index:5;top:-19px;left:0;pointer-events:none;font-size:12px;-webkit-animation:.3s labelPrefixFlexAnimate;animation:.3s labelPrefixFlexAnimate}@-webkit-keyframes labelFlexAnimate{0%{top:12px}100%{top:-19px}}@keyframes labelFlexAnimate{0%{top:12px}100%{top:-19px}}@-webkit-keyframes fillFillAnimate{0%{top:-19px}100%{top:12px}}@keyframes fillFillAnimate{0%{top:-19px}100%{top:12px}}@-webkit-keyframes labelPrefixFlexAnimate{0%{top:12px;left:22px}100%{top:-19px;left:0}}@keyframes labelPrefixFlexAnimate{0%{top:12px;left:22px}100%{top:-19px;left:0}}@-webkit-keyframes fillPrefixFillAnimate{0%{top:-19px;left:0}100%{top:12px;left:22px}}@keyframes fillPrefixFillAnimate{0%{top:-19px;left:0}100%{top:12px;left:22px}}.grf-ui-input.ng-invalid:not(form),.grf-ui-input.ng-valid.required,.grf-ui-input.ng-valid[required]{border:none}`]
            },] },
];
/** @nocollapse */
InputGroupComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
InputGroupComponent.propDecorators = {
    "_addOnContentBefore": [{ type: ContentChild, args: ['addOnBefore',] },],
    "_addOnContentAfter": [{ type: ContentChild, args: ['addOnAfter',] },],
    "_prefixContent": [{ type: ContentChild, args: ['prefix',] },],
    "_suffixContent": [{ type: ContentChild, args: ['suffix',] },],
    "contextIconEl": [{ type: ViewChild, args: ['contextIcon',] },],
    "_isLarge": [{ type: HostBinding, args: [`class.grf-ui-input-group-lg`,] },],
    "_isSmall": [{ type: HostBinding, args: [`class.grf-ui-input-group-sm`,] },],
    "clearEvent": [{ type: Output },],
    "uiLine": [{ type: Input },],
    "uiClear": [{ type: Input },],
    "uiLabel": [{ type: Input },],
    "uiLabelType": [{ type: Input },],
    "uiType": [{ type: Input },],
    "uiSize": [{ type: Input },],
    "uiCompact": [{ type: Input }, { type: HostBinding, args: [`class.grf-ui-input-group-compact`,] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class InputModule {
}
InputModule.decorators = [
    { type: NgModule, args: [{
                declarations: [InputDirectiveComponent, InputGroupComponent],
                exports: [InputDirectiveComponent, InputGroupComponent],
                imports: [CommonModule, FormsModule]
            },] },
];
/** @nocollapse */
InputModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ButtonComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._classList = [];
        this._iconOnly = false;
        this._loading = false;
        this._clicked = false;
        this._prefixCls = 'grf-ui-btn';
        this.nativeElement = this._elementRef.nativeElement;
        this._renderer.addClass(this.nativeElement, this._prefixCls);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiOutlineType(value) {
        this._ghost = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiOutlineType() {
        return this._ghost;
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        this._type = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiShape() {
        return this._shape;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiShape(value) {
        this._shape = value;
        this._setClassMap();
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this._size = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this._size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiLoading(value) {
        this._loading = value;
        this._setClassMap();
        this._setIconDisplay(value);
    }
    /**
     * @return {?}
     */
    get uiLoading() {
        return this._loading;
    }
    /**
     * toggle button clicked animation
     * @return {?}
     */
    _onClick() {
        this._clicked = true;
        this._setClassMap();
        setTimeout(() => {
            this._clicked = false;
            this._setClassMap();
        }, 300);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    _setIconDisplay(value) {
        const /** @type {?} */ innerI = this._iconElement;
        if (innerI) {
            this._renderer.setStyle(innerI, 'display', value ? 'none' : 'inline-block');
        }
    }
    /**
     * temp solution since no method add classMap to host https://github.com/angular/angular/issues/7289
     * @return {?}
     */
    _setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.removeClass(this.nativeElement, _className);
        });
        this._classList = [
            this.uiType && `${this._prefixCls}-${this.uiType}`,
            this.uiShape && `${this._prefixCls}-${this.uiShape}`,
            this.uiSize && `${this._prefixCls}-${this.uiSize}`,
            this.uiLoading && `${this._prefixCls}-loading`,
            this._clicked && `${this._prefixCls}-clicked`,
            this._iconOnly && `${this._prefixCls}-icon-only`,
            this.uiOutlineType && `${this._prefixCls}-outline-${this.uiOutlineType}`,
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this.nativeElement, _className);
        });
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._iconElement = this._innerIElement;
        /** check if host children only has i element */
        if (this._iconElement && this.nativeElement.children.length === 1 && (this._iconElement.isEqualNode(this.nativeElement.children[0]))) {
            this._iconOnly = true;
            this._setClassMap();
        }
        this._setIconDisplay(this.uiLoading);
    }
    /**
     * @return {?}
     */
    get _innerIElement() {
        return this.nativeElement.querySelector('i');
    }
}
ButtonComponent.decorators = [
    { type: Component, args: [{
                selector: '[grf-button]',
                encapsulation: ViewEncapsulation.None,
                template: `
      <i class="fa fa-spinner fa-spin" *ngIf="uiLoading"></i>
      <ng-content></ng-content>
    `,
                styles: [`@charset "UTF-8";.grf-ui-btn{display:inline-block;font-weight:400;text-align:center;white-space:nowrap;vertical-align:middle;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;cursor:pointer;border:1px solid transparent;padding:4px 16px;font-size:14px;line-height:1.5;border-radius:.25rem;-webkit-transition:background-color .15s ease-in-out,border-color .15s ease-in-out,-webkit-box-shadow .15s ease-in-out;transition:background-color .15s ease-in-out,border-color .15s ease-in-out,-webkit-box-shadow .15s ease-in-out;transition:background-color .15s ease-in-out,border-color .15s ease-in-out,box-shadow .15s ease-in-out;transition:background-color .15s ease-in-out,border-color .15s ease-in-out,box-shadow .15s ease-in-out,-webkit-box-shadow .15s ease-in-out}.grf-ui-btn:focus,.grf-ui-btn:hover{text-decoration:none}.grf-ui-btn.focus,.grf-ui-btn:focus{outline:0}.grf-ui-btn.disabled,.grf-ui-btn:disabled{opacity:.5;cursor:default}.grf-ui-btn:not([disabled]):not(.disabled).active,.grf-ui-btn:not([disabled]):not(.disabled):active{background-image:none}a.grf-ui-btn.disabled,fieldset[disabled] a.grf-ui-btn{pointer-events:none}.grf-ui-btn-primary{color:#fff;background-color:#108ee9;border-color:#108ee9}.grf-ui-btn-primary:hover{color:#fff;background-color:#0e78c5;border-color:#0d71b9}.grf-ui-btn-primary.disabled,.grf-ui-btn-primary:disabled{background-color:#108ee9;border-color:#108ee9}.grf-ui-btn-primary:not([disabled]):not(.disabled).active,.grf-ui-btn-primary:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-primary.dropdown-toggle{color:#fff;background-color:#0d71b9;border-color:#0c6aad}.grf-ui-btn-secondary{color:#fff;background-color:#868e96;border-color:#868e96}.grf-ui-btn-secondary:hover{color:#fff;background-color:#727b84;border-color:#6c757d}.grf-ui-btn-secondary.disabled,.grf-ui-btn-secondary:disabled{background-color:#868e96;border-color:#868e96}.grf-ui-btn-secondary:not([disabled]):not(.disabled).active,.grf-ui-btn-secondary:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-secondary.dropdown-toggle{color:#fff;background-color:#6c757d;border-color:#666e76}.grf-ui-btn-success{color:#fff;background-color:#00a854;border-color:#00a854}.grf-ui-btn-success:hover{color:#fff;background-color:#008241;border-color:#00753b}.grf-ui-btn-success.disabled,.grf-ui-btn-success:disabled{background-color:#00a854;border-color:#00a854}.grf-ui-btn-success:not([disabled]):not(.disabled).active,.grf-ui-btn-success:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-success.dropdown-toggle{color:#fff;background-color:#00753b;border-color:#006834}.grf-ui-btn-info{color:#fff;background-color:#00a2ae;border-color:#00a2ae}.grf-ui-btn-info:hover{color:#fff;background-color:#007e88;border-color:#00737b}.grf-ui-btn-info.disabled,.grf-ui-btn-info:disabled{background-color:#00a2ae;border-color:#00a2ae}.grf-ui-btn-info:not([disabled]):not(.disabled).active,.grf-ui-btn-info:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-info.dropdown-toggle{color:#fff;background-color:#00737b;border-color:#00676e}.grf-ui-btn-warning{color:#fff;background-color:#ffbf00;border-color:#ffbf00}.grf-ui-btn-warning:hover{color:#fff;background-color:#d9a200;border-color:#c90}.grf-ui-btn-warning.disabled,.grf-ui-btn-warning:disabled{background-color:#ffbf00;border-color:#ffbf00}.grf-ui-btn-warning:not([disabled]):not(.disabled).active,.grf-ui-btn-warning:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-warning.dropdown-toggle{color:#fff;background-color:#c90;border-color:#bf8f00}.grf-ui-btn-danger{color:#fff;background-color:#f04134;border-color:#f04134}.grf-ui-btn-danger:hover{color:#fff;background-color:#ec2111;border-color:#e01f11}.grf-ui-btn-danger.disabled,.grf-ui-btn-danger:disabled{background-color:#f04134;border-color:#f04134}.grf-ui-btn-danger:not([disabled]):not(.disabled).active,.grf-ui-btn-danger:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-danger.dropdown-toggle{color:#fff;background-color:#e01f11;border-color:#d51d10}.grf-ui-btn-light{color:#111;background-color:#f8f9fa;border-color:#f8f9fa}.grf-ui-btn-light:hover{color:#fff;background-color:#e2e6ea;border-color:#dae0e5}.grf-ui-btn-light.disabled,.grf-ui-btn-light:disabled{background-color:#f8f9fa;border-color:#f8f9fa}.grf-ui-btn-light:not([disabled]):not(.disabled).active,.grf-ui-btn-light:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-light.dropdown-toggle{color:#fff;background-color:#dae0e5;border-color:#d3d9df}.grf-ui-btn-dark{color:#fff;background-color:#495057;border-color:#495057}.grf-ui-btn-dark:hover{color:#fff;background-color:#383d42;border-color:#32373b}.grf-ui-btn-dark.disabled,.grf-ui-btn-dark:disabled{background-color:#495057;border-color:#495057}.grf-ui-btn-dark:not([disabled]):not(.disabled).active,.grf-ui-btn-dark:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-dark.dropdown-toggle{color:#fff;background-color:#32373b;border-color:#2c3034}.grf-ui-btn-outline-primary{color:#108ee9;background-color:transparent;background-image:none;border-color:#108ee9}.grf-ui-btn-outline-primary:hover{color:#fff;background-color:#108ee9;border-color:#108ee9}.grf-ui-btn-outline-primary.disabled,.grf-ui-btn-outline-primary:disabled{color:#108ee9;background-color:transparent}.grf-ui-btn-outline-primary:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-primary:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-primary.dropdown-toggle{color:#fff;background-color:#108ee9;border-color:#108ee9}.grf-ui-btn-outline-secondary{color:#868e96;background-color:transparent;background-image:none;border-color:#868e96}.grf-ui-btn-outline-secondary:hover{color:#fff;background-color:#868e96;border-color:#868e96}.grf-ui-btn-outline-secondary.disabled,.grf-ui-btn-outline-secondary:disabled{color:#868e96;background-color:transparent}.grf-ui-btn-outline-secondary:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-secondary:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-secondary.dropdown-toggle{color:#fff;background-color:#868e96;border-color:#868e96}.grf-ui-btn-outline-success{color:#00a854;background-color:transparent;background-image:none;border-color:#00a854}.grf-ui-btn-outline-success:hover{color:#fff;background-color:#00a854;border-color:#00a854}.grf-ui-btn-outline-success.disabled,.grf-ui-btn-outline-success:disabled{color:#00a854;background-color:transparent}.grf-ui-btn-outline-success:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-success:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-success.dropdown-toggle{color:#fff;background-color:#00a854;border-color:#00a854}.grf-ui-btn-outline-info{color:#00a2ae;background-color:transparent;background-image:none;border-color:#00a2ae}.grf-ui-btn-outline-info:hover{color:#fff;background-color:#00a2ae;border-color:#00a2ae}.grf-ui-btn-outline-info.disabled,.grf-ui-btn-outline-info:disabled{color:#00a2ae;background-color:transparent}.grf-ui-btn-outline-info:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-info:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-info.dropdown-toggle{color:#fff;background-color:#00a2ae;border-color:#00a2ae}.grf-ui-btn-outline-warning{color:#ffbf00;background-color:transparent;background-image:none;border-color:#ffbf00}.grf-ui-btn-outline-warning:hover{color:#fff;background-color:#ffbf00;border-color:#ffbf00}.grf-ui-btn-outline-warning.disabled,.grf-ui-btn-outline-warning:disabled{color:#ffbf00;background-color:transparent}.grf-ui-btn-outline-warning:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-warning:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-warning.dropdown-toggle{color:#fff;background-color:#ffbf00;border-color:#ffbf00}.grf-ui-btn-outline-danger{color:#f04134;background-color:transparent;background-image:none;border-color:#f04134}.grf-ui-btn-outline-danger:hover{color:#fff;background-color:#f04134;border-color:#f04134}.grf-ui-btn-outline-danger.disabled,.grf-ui-btn-outline-danger:disabled{color:#f04134;background-color:transparent}.grf-ui-btn-outline-danger:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-danger:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-danger.dropdown-toggle{color:#fff;background-color:#f04134;border-color:#f04134}.grf-ui-btn-outline-light{color:#f8f9fa;background-color:transparent;background-image:none;border-color:#f8f9fa}.grf-ui-btn-outline-light:hover{color:#212529;background-color:#f8f9fa;border-color:#f8f9fa}.grf-ui-btn-outline-light.disabled,.grf-ui-btn-outline-light:disabled{color:#f8f9fa;background-color:transparent}.grf-ui-btn-outline-light:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-light:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-light.dropdown-toggle{color:#212529;background-color:#f8f9fa;border-color:#f8f9fa}.grf-ui-btn-outline-dark{color:#495057;background-color:transparent;background-image:none;border-color:#495057}.grf-ui-btn-outline-dark:hover{color:#fff;background-color:#495057;border-color:#495057}.grf-ui-btn-outline-dark.disabled,.grf-ui-btn-outline-dark:disabled{color:#495057;background-color:transparent}.grf-ui-btn-outline-dark:not([disabled]):not(.disabled).active,.grf-ui-btn-outline-dark:not([disabled]):not(.disabled):active,.show>.grf-ui-btn-outline-dark.dropdown-toggle{color:#fff;background-color:#495057;border-color:#495057}.grf-ui-btn-link{font-weight:400;color:#108ee9;background-color:transparent}.grf-ui-btn-link:hover{color:#0b62a1;text-decoration:underline;background-color:transparent;border-color:transparent}.grf-ui-btn-link.focus,.grf-ui-btn-link:focus{border-color:transparent;-webkit-box-shadow:none;box-shadow:none}.grf-ui-btn-link.disabled,.grf-ui-btn-link:disabled{color:#868e96}.grf-ui-btn-group-lg>.grf-ui-btn,.grf-ui-btn-lg{padding:4px 16px;font-size:17.5px;line-height:1.5;border-radius:.3rem}.grf-ui-btn-group-sm>.grf-ui-btn,.grf-ui-btn-sm{padding:4px 16px;font-size:12.25px;line-height:1.5;border-radius:.2rem}.grf-ui-btn-block{display:block;width:100%}.grf-ui-btn-block+.grf-ui-btn-block{margin-top:.5rem}input[type=button].grf-ui-btn-block,input[type=reset].grf-ui-btn-block,input[type=submit].grf-ui-btn-block{width:100%}.grf-ui-btn-circle{width:30px;padding:0;font-size:16px;border-radius:50%;height:30px}.grf-ui-btn-circle.grf-ui-btn-lg,.grf-ui-btn-group-lg>.grf-ui-btn-circle.grf-ui-btn{width:34px;padding:0;font-size:19.5px;border-radius:50%;height:34px}.grf-ui-btn-circle.grf-ui-btn-sm,.grf-ui-btn-group-sm>.grf-ui-btn-circle.grf-ui-btn{width:24px;padding:0;font-size:14.25px;border-radius:50%;height:24px}.grf-ui-btn-loading:before{display:block}.grf-ui-btn-loading{pointer-events:none}.grf-ui-btn-group,.grf-ui-btn-group-vertical{position:relative;display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex;vertical-align:middle}.grf-ui-btn-group-vertical>.grf-ui-btn,.grf-ui-btn-group>.grf-ui-btn{position:relative;-webkit-box-flex:0;-ms-flex:0 1 auto;flex:0 1 auto}.grf-ui-btn-group-vertical>.grf-ui-btn.active,.grf-ui-btn-group-vertical>.grf-ui-btn:active,.grf-ui-btn-group-vertical>.grf-ui-btn:focus,.grf-ui-btn-group-vertical>.grf-ui-btn:hover,.grf-ui-btn-group>.grf-ui-btn.active,.grf-ui-btn-group>.grf-ui-btn:active,.grf-ui-btn-group>.grf-ui-btn:focus,.grf-ui-btn-group>.grf-ui-btn:hover{z-index:2}.grf-ui-btn-group .grf-ui-btn+.grf-ui-btn,.grf-ui-btn-group .grf-ui-btn+.grf-ui-btn-group,.grf-ui-btn-group .grf-ui-btn-group+.grf-ui-btn,.grf-ui-btn-group .grf-ui-btn-group+.grf-ui-btn-group,.grf-ui-btn-group-vertical .grf-ui-btn+.grf-ui-btn,.grf-ui-btn-group-vertical .grf-ui-btn+.grf-ui-btn-group,.grf-ui-btn-group-vertical .grf-ui-btn-group+.grf-ui-btn,.grf-ui-btn-group-vertical .grf-ui-btn-group+.grf-ui-btn-group{margin-left:-1px}.grf-ui-btn-toolbar{display:-webkit-box;display:-ms-flexbox;display:flex;-ms-flex-wrap:wrap;flex-wrap:wrap;-webkit-box-pack:start;-ms-flex-pack:start;justify-content:flex-start}.grf-ui-btn-toolbar .input-group{width:auto}.grf-ui-btn-group .grf-ui-btn:not(:first-child):not(:last-child):not(.dropdown-toggle){border-radius:0}.grf-ui-btn-group>.grf-ui-btn:first-child{margin-left:0}.grf-ui-btn-group>.grf-ui-btn:first-child:not(:last-child):not(.dropdown-toggle){border-top-right-radius:0;border-bottom-right-radius:0}.grf-ui-btn-group>.dropdown-toggle:not(:first-child),.grf-ui-btn-group>.grf-ui-btn:last-child:not(:first-child){border-top-left-radius:0;border-bottom-left-radius:0}.grf-ui-btn-group>.grf-ui-btn-group{float:left}.grf-ui-btn-group .grf-ui-btn-group:not(:first-child):not(:last-child) .grf-ui-btn{border-radius:0}.grf-ui-btn-group>.grf-ui-btn-group:first-child:not(:last-child)>.dropdown-toggle,.grf-ui-btn-group>.grf-ui-btn-group:first-child:not(:last-child)>.grf-ui-btn:last-child{border-top-right-radius:0;border-bottom-right-radius:0}.grf-ui-btn-group>.grf-ui-btn-group:last-child:not(:first-child)>.grf-ui-btn:first-child{border-top-left-radius:0;border-bottom-left-radius:0}.grf-ui-btn+.dropdown-toggle-split{padding-right:12px;padding-left:12px}.grf-ui-btn+.dropdown-toggle-split::after{margin-left:0}.grf-ui-btn-group-lg>.grf-ui-btn+.dropdown-toggle-split,.grf-ui-btn-group-sm>.grf-ui-btn+.dropdown-toggle-split,.grf-ui-btn-lg+.dropdown-toggle-split,.grf-ui-btn-sm+.dropdown-toggle-split{padding-right:12px;padding-left:12px}.grf-ui-btn-group-vertical{-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center}.grf-ui-btn-group-vertical .grf-ui-btn,.grf-ui-btn-group-vertical .grf-ui-btn-group{width:100%}.grf-ui-btn-group-vertical>.grf-ui-btn+.grf-ui-btn,.grf-ui-btn-group-vertical>.grf-ui-btn+.grf-ui-btn-group,.grf-ui-btn-group-vertical>.grf-ui-btn-group+.grf-ui-btn,.grf-ui-btn-group-vertical>.grf-ui-btn-group+.grf-ui-btn-group{margin-top:-1px;margin-left:0}.grf-ui-btn-group-vertical .grf-ui-btn:not(:first-child):not(:last-child){border-radius:0}.grf-ui-btn-group-vertical .grf-ui-btn:first-child:not(:last-child){border-bottom-right-radius:0;border-bottom-left-radius:0}.grf-ui-btn-group-vertical .grf-ui-btn:last-child:not(:first-child){border-top-left-radius:0;border-top-right-radius:0}.grf-ui-btn-group-vertical .grf-ui-btn-group:not(:first-child):not(:last-child) .grf-ui-btn{border-radius:0}.grf-ui-btn-group-vertical>.grf-ui-btn-group:first-child:not(:last-child)>.dropdown-toggle,.grf-ui-btn-group-vertical>.grf-ui-btn-group:first-child:not(:last-child)>.grf-ui-btn:last-child{border-bottom-right-radius:0;border-bottom-left-radius:0}.grf-ui-btn-group-vertical>.grf-ui-btn-group:last-child:not(:first-child)>.grf-ui-btn:first-child{border-top-left-radius:0;border-top-right-radius:0}[data-toggle=buttons]>.grf-ui-btn input[type=checkbox],[data-toggle=buttons]>.grf-ui-btn input[type=radio],[data-toggle=buttons]>.grf-ui-btn-group>.grf-ui-btn input[type=checkbox],[data-toggle=buttons]>.grf-ui-btn-group>.grf-ui-btn input[type=radio]{position:absolute;clip:rect(0,0,0,0);pointer-events:none}`]
            },] },
];
/** @nocollapse */
ButtonComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
ButtonComponent.propDecorators = {
    "uiOutlineType": [{ type: Input },],
    "uiType": [{ type: Input },],
    "uiShape": [{ type: Input },],
    "uiSize": [{ type: Input },],
    "uiLoading": [{ type: Input },],
    "_onClick": [{ type: HostListener, args: ['click',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ButtonGroupComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._vertical = false;
        this._prefixCls = 'grf-ui-btn-group';
        this._classList = [];
    }
    /**
     * @return {?}
     */
    get uiSize() {
        return this._size;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiSize(value) {
        this._size = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    get uiVertical() {
        return this._vertical;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiVertical(value) {
        this._vertical = value;
        this._setClassMap();
    }
    /**
     * @return {?}
     */
    _setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.addClass(this._groupWrapper, _className);
        });
        this._classList = [
            this.uiSize && `${this._prefixCls}-${this.uiSize}`,
            this.uiVertical && `${this._prefixCls}-vertical`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this._groupWrapper.nativeElement, _className);
        });
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this._renderer.addClass(this._groupWrapper.nativeElement, this._prefixCls);
    }
}
ButtonGroupComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-button-group',
                encapsulation: ViewEncapsulation.None,
                template: `
    <div #groupWrapper>
      <ng-content></ng-content>
    </div>
  `
            },] },
];
/** @nocollapse */
ButtonGroupComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
ButtonGroupComponent.propDecorators = {
    "_groupWrapper": [{ type: ViewChild, args: ['groupWrapper',] },],
    "uiSize": [{ type: Input },],
    "uiVertical": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ButtonModule {
}
ButtonModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [ButtonComponent, ButtonGroupComponent],
                declarations: [ButtonComponent, ButtonGroupComponent]
            },] },
];
/** @nocollapse */
ButtonModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CardComponent {
    /**
     * @param {?} el
     * @param {?} _render
     */
    constructor(el, _render) {
        this.el = el;
        this._render = _render;
        this.collapsed = false;
        this.expandIcon = 'fa-chevron-down';
        this.collapseIcon = 'fa-chevron-up';
        this.uiHover = true;
        this.uiLoading = false;
        this.uiMax = false;
        this.collapsedChange = new EventEmitter();
        this.onBeforeToggle = new EventEmitter();
        this.onAfterToggle = new EventEmitter();
        this.cardHover = false;
        this._max = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.uiMax) {
            window.addEventListener('scroll', () => {
                if (this._max) {
                    this.maxCard();
                    console.log('22');
                }
            });
            window.addEventListener('resize', () => {
                if (this._max) {
                    this._render.setStyle(this._card.nativeElement, 'width', window.innerWidth - 8 + 'px');
                }
            });
        }
    }
    /**
     * @return {?}
     */
    mouseHover() {
        if (this.uiHover) {
            this.cardHover = true;
        }
    }
    /**
     * @return {?}
     */
    mouseOut() {
        if (this.uiHover) {
            this.cardHover = false;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.animating) {
            return false;
        }
        this.animating = true;
        this.onBeforeToggle.emit({ originalEvent: event, collapsed: this.collapsed });
        if (this.uiMax && this.toggleable) {
            if (this._max) {
                if (this.collapsed) {
                    this.expand(event);
                    this.maxCard();
                }
                else {
                    this.collapse(event);
                    this.maxCard();
                }
            }
            else {
                if (this.collapsed) {
                    this.expand(event);
                }
                else {
                    this.collapse(event);
                }
            }
            return;
        }
        if (this.toggleable) {
            if (this.collapsed) {
                this.expand(event);
            }
            else {
                this.collapse(event);
            }
        }
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    expand(event) {
        this.collapsed = false;
        this.collapsedChange.emit(this.collapsed);
    }
    /**
     * @param {?} even
     * @return {?}
     */
    collapse(even) {
        this.collapsed = true;
        this.collapsedChange.emit(this.collapsed);
    }
    /**
     * @return {?}
     */
    maxCard() {
        if (this.uiMax) {
            if (this.toggleable) {
                if (this.collapsed && !this._max) {
                    this.collapsed = false;
                    this.collapsedChange.emit(this.collapsed);
                    this.fullScreen();
                }
                else {
                    this.fullScreen();
                }
                return;
            }
            this.fullScreen();
        }
    }
    /**
     * @return {?}
     */
    fullScreen() {
        const /** @type {?} */ rect = this._card.nativeElement.getBoundingClientRect();
        if (!this._max) {
            this._max = true;
            this._render.setStyle(this._card.nativeElement, 'height', window.innerHeight + 'px');
            this._render.setStyle(this._card.nativeElement, 'width', window.innerWidth - 8 + 'px');
            this._render.setStyle(this._card.nativeElement, 'left', -rect.left + 'px');
            this._render.setStyle(this._card.nativeElement, 'position', 'relative');
            this._render.setStyle(this._card.nativeElement, 'z-index', '1000');
            this._render.setStyle(this._card.nativeElement, 'top', -rect.top + 'px');
            return;
        }
        else {
            this._max = false;
            this._render.removeStyle(this._card.nativeElement, 'height');
            this._render.removeStyle(this._card.nativeElement, 'width', 100);
            this._render.removeStyle(this._card.nativeElement, 'left');
            this._render.removeStyle(this._card.nativeElement, 'position');
            this._render.removeStyle(this._card.nativeElement, 'z-index');
            this._render.removeStyle(this._card.nativeElement, 'top');
        }
    }
    /**
     * @return {?}
     */
    getBlockableElement() {
        return this.el.nativeElement.children[0];
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onToggleDone(event) {
        this.animating = false;
        this.onAfterToggle.emit({ originalEvent: event, collapsed: this.collapsed });
    }
}
CardComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-card',
                template: `<div #card [ngClass]="{'grf-ui-card-hover':cardHover}" class="grf-ui-card grf-ui-widget grf-ui-widget-content grf-ui-corner-all"
    [ngStyle]="style" [class]="styleClass" (mouseover)="mouseHover()" (mouseout)="mouseOut()">
    <div *ngIf="title" class="grf-ui-card-titlebar grf-ui-widget-header grf-ui-helper-clearfix grf-ui-corner-all">
        <span class="grf-ui-card-title">
            <ng-template [ngTemplateOutlet]="title">
            </ng-template>
        </span>
        <span class="grf-ui-card-operate">
            <ng-template [ngTemplateOutlet]="operate">
            </ng-template>
            <a *ngIf="toggleable && !_max" class="grf-ui-card-titlebar-icon grf-ui-card-titlebar-toggler grf-ui-corner-all grf-ui-state-default"
               (click)="toggle($event)">
                <span [class]="collapsed ? 'fa fa-fw ' + expandIcon : 'fa fa-fw ' + collapseIcon"></span>
            </a>
            <a *ngIf="uiMax" class="grf-ui-card-titlebar-icon grf-ui-card-titlebar-toggler grf-ui-corner-all grf-ui-state-default"
                 (click)="maxCard()">
                 <span [class]="_max ? 'fa fa-compress '  : 'fa fa-arrows-alt '"></span>
            </a>
        </span>

    </div>
    <div class="grf-ui-card-content-wrapper" [@cardContent]="collapsed ? 'hidden' : 'visible'" (@cardContent.done)="onToggleDone($event)"
        [ngClass]="{'grf-ui-card-content-wrapper-overflown': collapsed||animating}">
        <div class="grf-ui-card-content grf-ui-widget-content">
            <div *ngIf="uiLoading" class="grf-ui-card-content-loading">
                <span class="fa fa-spinner fa-spin grf-ui-card-loding-icon"></span>
            </div>
            <ng-content *ngIf="!uiLoading"></ng-content>
        </div>
    </div>
</div>
`,
                styles: [`.grf-ui-card{border-color:rgba(0,0,0,.125)}.grf-ui-card .grf-ui-card-title{color:#495057}.grf-ui-card .grf-ui-card-titlebar{border:0;padding:.75em 1.25em;background-color:#fff;border-bottom:1px solid rgba(0,0,0,.125);border-radius:.25em .25em 0 0}.grf-ui-card .grf-ui-card-titlebar .grf-ui-card-operate{float:right}.grf-ui-card .grf-ui-card-titlebar .grf-ui-card-titlebar-icon{position:relative;color:#767b7f;-webkit-transition:color .3s;transition:color .3s;font-size:14px}.grf-ui-card .grf-ui-card-titlebar .grf-ui-card-titlebar-icon:hover{color:#2d353c}.grf-ui-card .grf-ui-card-titlebar-icon{cursor:pointer;margin-left:.5em;margin-top:-.1em}.grf-ui-card .grf-ui-card-content{border:0;background:0 0;padding:1.25em;height:100%;-webkit-box-sizing:border-box;box-sizing:border-box}.grf-ui-card .grf-ui-card-content .grf-ui-card-content-loading{width:100%;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;padding:50px 0;display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-card .grf-ui-card-content .grf-ui-card-content-loading .grf-ui-card-loding-icon{font-size:48px;color:#000;opacity:.6}.grf-ui-card .grf-ui-card-footer{padding:.375em .75em;text-align:left;border:0;border-top:1px solid #c7c7c7;margin:0}.grf-ui-card-hover{-webkit-box-shadow:0 0 10px rgba(0,0,0,.18);box-shadow:0 0 10px rgba(0,0,0,.18);-webkit-animation:.25s cardHover;animation:.25s cardHover}@-webkit-keyframes cardHover{from{-webkit-box-shadow:none;box-shadow:none}to{-webkit-box-shadow:0 0 10px rgba(0,0,0,.18);box-shadow:0 0 10px rgba(0,0,0,.18)}}@keyframes cardHover{from{-webkit-box-shadow:none;box-shadow:none}to{-webkit-box-shadow:0 0 10px rgba(0,0,0,.18);box-shadow:0 0 10px rgba(0,0,0,.18)}}.grf-ui-card-content-wrapper-overflown{overflow:hidden}`],
                animations: [
                    trigger('cardContent', [
                        state('hidden', style({
                            height: '0'
                        })),
                        state('visible', style({
                            height: '*'
                        })),
                        transition('visible <=> hidden', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
CardComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
CardComponent.propDecorators = {
    "toggleable": [{ type: Input },],
    "collapsed": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "expandIcon": [{ type: Input },],
    "collapseIcon": [{ type: Input },],
    "uiHover": [{ type: Input },],
    "uiLoading": [{ type: Input },],
    "uiMax": [{ type: Input },],
    "collapsedChange": [{ type: Output },],
    "onBeforeToggle": [{ type: Output },],
    "onAfterToggle": [{ type: Output },],
    "title": [{ type: ContentChild, args: ['title',] },],
    "operate": [{ type: ContentChild, args: ['operate',] },],
    "_card": [{ type: ViewChild, args: ['card',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CardModule {
}
CardModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                exports: [CardComponent],
                declarations: [CardComponent
                ]
            },] },
];
/** @nocollapse */
CardModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class RowComponent$1 {
    /**
     * @param {?} _elementRef
     * @param {?} _renderer
     */
    constructor(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
        this._classList = [];
        this._prefixCls = 'grf-ui-row';
        this._align = 'top';
        this._justify = 'start';
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiType(value) {
        this._type = value;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiType() {
        return this._type;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiAlign(value) {
        this._align = value;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiAlign() {
        return this._align;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiJustify(value) {
        this._justify = value;
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    get uiJustify() {
        return this._justify;
    }
    /**
     * @return {?}
     */
    get uiGutter() {
        return this._gutter;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set uiGutter(value) {
        this._gutter = value;
        // this.setStyle();
    }
    /**
     * @return {?}
     */
    setStyle() {
        this._renderer.setStyle(this._el, 'margin-left', `-${this._gutter / 2}px`);
        this._renderer.setStyle(this._el, 'margin-right', `-${this._gutter / 2}px`);
    }
    /**
     * temp solution since no method add classMap to host https://github.com/angular/angular/issues/7289
     * @return {?}
     */
    setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.removeClass(this._el, _className);
        });
        this._classList = [
            (!this.uiType) && this._prefixCls,
            this.uiType && `${this._prefixCls}-${this.uiType}`,
            this.uiType && this.uiAlign && `${this._prefixCls}-${this.uiType}-${this.uiAlign}`,
            this.uiType && this.uiJustify && `${this._prefixCls}-${this.uiType}-${this.uiJustify}`
        ].filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this._el, _className);
        });
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.setClassMap();
    }
}
RowComponent$1.decorators = [
    { type: Component, args: [{
                selector: '[grf-row],grf-row',
                encapsulation: ViewEncapsulation.None,
                template: `<ng-content></ng-content>`,
                styles: [`.grf-ui-row{position:relative;margin-left:0;margin-right:0;height:auto;zoom:1}.grf-ui-row:after,.grf-ui-row:before{content:" ";display:table}.grf-ui-row:after{clear:both;visibility:hidden;font-size:0;height:0}.grf-ui-row-flex{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-flow:row wrap;flex-flow:row wrap}.grf-ui-row-flex:after,.grf-ui-row-flex:before{display:-webkit-box;display:-ms-flexbox;display:flex}.grf-ui-row-flex-start{-webkit-box-pack:start;-ms-flex-pack:start;justify-content:flex-start}.grf-ui-row-flex-center{-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center}.grf-ui-row-flex-end{-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.grf-ui-row-flex-space-between{-webkit-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.grf-ui-row-flex-space-around{-ms-flex-pack:distribute;justify-content:space-around}.grf-ui-row-flex-top{-webkit-box-align:start;-ms-flex-align:start;align-items:flex-start}.grf-ui-row-flex-middle{-webkit-box-align:center;-ms-flex-align:center;align-items:center}.grf-ui-row-flex-bottom{-webkit-box-align:end;-ms-flex-align:end;align-items:flex-end}.grf-ui-col{position:relative;display:block}.grf-ui-col-1,.grf-ui-col-10,.grf-ui-col-11,.grf-ui-col-12,.grf-ui-col-13,.grf-ui-col-14,.grf-ui-col-15,.grf-ui-col-16,.grf-ui-col-17,.grf-ui-col-18,.grf-ui-col-19,.grf-ui-col-2,.grf-ui-col-20,.grf-ui-col-21,.grf-ui-col-22,.grf-ui-col-23,.grf-ui-col-24,.grf-ui-col-3,.grf-ui-col-4,.grf-ui-col-5,.grf-ui-col-6,.grf-ui-col-7,.grf-ui-col-8,.grf-ui-col-9,.grf-ui-col-lg-1,.grf-ui-col-lg-10,.grf-ui-col-lg-11,.grf-ui-col-lg-12,.grf-ui-col-lg-13,.grf-ui-col-lg-14,.grf-ui-col-lg-15,.grf-ui-col-lg-16,.grf-ui-col-lg-17,.grf-ui-col-lg-18,.grf-ui-col-lg-19,.grf-ui-col-lg-2,.grf-ui-col-lg-20,.grf-ui-col-lg-21,.grf-ui-col-lg-22,.grf-ui-col-lg-23,.grf-ui-col-lg-24,.grf-ui-col-lg-3,.grf-ui-col-lg-4,.grf-ui-col-lg-5,.grf-ui-col-lg-6,.grf-ui-col-lg-7,.grf-ui-col-lg-8,.grf-ui-col-lg-9,.grf-ui-col-md-1,.grf-ui-col-md-10,.grf-ui-col-md-11,.grf-ui-col-md-12,.grf-ui-col-md-13,.grf-ui-col-md-14,.grf-ui-col-md-15,.grf-ui-col-md-16,.grf-ui-col-md-17,.grf-ui-col-md-18,.grf-ui-col-md-19,.grf-ui-col-md-2,.grf-ui-col-md-20,.grf-ui-col-md-21,.grf-ui-col-md-22,.grf-ui-col-md-23,.grf-ui-col-md-24,.grf-ui-col-md-3,.grf-ui-col-md-4,.grf-ui-col-md-5,.grf-ui-col-md-6,.grf-ui-col-md-7,.grf-ui-col-md-8,.grf-ui-col-md-9,.grf-ui-col-sm-1,.grf-ui-col-sm-10,.grf-ui-col-sm-11,.grf-ui-col-sm-12,.grf-ui-col-sm-13,.grf-ui-col-sm-14,.grf-ui-col-sm-15,.grf-ui-col-sm-16,.grf-ui-col-sm-17,.grf-ui-col-sm-18,.grf-ui-col-sm-19,.grf-ui-col-sm-2,.grf-ui-col-sm-20,.grf-ui-col-sm-21,.grf-ui-col-sm-22,.grf-ui-col-sm-23,.grf-ui-col-sm-24,.grf-ui-col-sm-3,.grf-ui-col-sm-4,.grf-ui-col-sm-5,.grf-ui-col-sm-6,.grf-ui-col-sm-7,.grf-ui-col-sm-8,.grf-ui-col-sm-9,.grf-ui-col-xs-1,.grf-ui-col-xs-10,.grf-ui-col-xs-11,.grf-ui-col-xs-12,.grf-ui-col-xs-13,.grf-ui-col-xs-14,.grf-ui-col-xs-15,.grf-ui-col-xs-16,.grf-ui-col-xs-17,.grf-ui-col-xs-18,.grf-ui-col-xs-19,.grf-ui-col-xs-2,.grf-ui-col-xs-20,.grf-ui-col-xs-21,.grf-ui-col-xs-22,.grf-ui-col-xs-23,.grf-ui-col-xs-24,.grf-ui-col-xs-3,.grf-ui-col-xs-4,.grf-ui-col-xs-5,.grf-ui-col-xs-6,.grf-ui-col-xs-7,.grf-ui-col-xs-8,.grf-ui-col-xs-9{position:relative;min-height:1px;padding-left:0;padding-right:0}.grf-ui-col-1,.grf-ui-col-10,.grf-ui-col-11,.grf-ui-col-12,.grf-ui-col-13,.grf-ui-col-14,.grf-ui-col-15,.grf-ui-col-16,.grf-ui-col-17,.grf-ui-col-18,.grf-ui-col-19,.grf-ui-col-2,.grf-ui-col-20,.grf-ui-col-21,.grf-ui-col-22,.grf-ui-col-23,.grf-ui-col-24,.grf-ui-col-3,.grf-ui-col-4,.grf-ui-col-5,.grf-ui-col-6,.grf-ui-col-7,.grf-ui-col-8,.grf-ui-col-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-24{display:block;width:100%}.grf-ui-col-push-24{left:100%}.grf-ui-col-pull-24{right:100%}.grf-ui-col-offset-24{margin-left:100%}.grf-ui-col-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-23{display:block;width:95.83333333%}.grf-ui-col-push-23{left:95.83333333%}.grf-ui-col-pull-23{right:95.83333333%}.grf-ui-col-offset-23{margin-left:95.83333333%}.grf-ui-col-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-22{display:block;width:91.66666667%}.grf-ui-col-push-22{left:91.66666667%}.grf-ui-col-pull-22{right:91.66666667%}.grf-ui-col-offset-22{margin-left:91.66666667%}.grf-ui-col-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-21{display:block;width:87.5%}.grf-ui-col-push-21{left:87.5%}.grf-ui-col-pull-21{right:87.5%}.grf-ui-col-offset-21{margin-left:87.5%}.grf-ui-col-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-20{display:block;width:83.33333333%}.grf-ui-col-push-20{left:83.33333333%}.grf-ui-col-pull-20{right:83.33333333%}.grf-ui-col-offset-20{margin-left:83.33333333%}.grf-ui-col-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-19{display:block;width:79.16666667%}.grf-ui-col-push-19{left:79.16666667%}.grf-ui-col-pull-19{right:79.16666667%}.grf-ui-col-offset-19{margin-left:79.16666667%}.grf-ui-col-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-18{display:block;width:75%}.grf-ui-col-push-18{left:75%}.grf-ui-col-pull-18{right:75%}.grf-ui-col-offset-18{margin-left:75%}.grf-ui-col-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-17{display:block;width:70.83333333%}.grf-ui-col-push-17{left:70.83333333%}.grf-ui-col-pull-17{right:70.83333333%}.grf-ui-col-offset-17{margin-left:70.83333333%}.grf-ui-col-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-16{display:block;width:66.66666667%}.grf-ui-col-push-16{left:66.66666667%}.grf-ui-col-pull-16{right:66.66666667%}.grf-ui-col-offset-16{margin-left:66.66666667%}.grf-ui-col-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-15{display:block;width:62.5%}.grf-ui-col-push-15{left:62.5%}.grf-ui-col-pull-15{right:62.5%}.grf-ui-col-offset-15{margin-left:62.5%}.grf-ui-col-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-14{display:block;width:58.33333333%}.grf-ui-col-push-14{left:58.33333333%}.grf-ui-col-pull-14{right:58.33333333%}.grf-ui-col-offset-14{margin-left:58.33333333%}.grf-ui-col-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-13{display:block;width:54.16666667%}.grf-ui-col-push-13{left:54.16666667%}.grf-ui-col-pull-13{right:54.16666667%}.grf-ui-col-offset-13{margin-left:54.16666667%}.grf-ui-col-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-12{display:block;width:50%}.grf-ui-col-push-12{left:50%}.grf-ui-col-pull-12{right:50%}.grf-ui-col-offset-12{margin-left:50%}.grf-ui-col-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-11{display:block;width:45.83333333%}.grf-ui-col-push-11{left:45.83333333%}.grf-ui-col-pull-11{right:45.83333333%}.grf-ui-col-offset-11{margin-left:45.83333333%}.grf-ui-col-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-10{display:block;width:41.66666667%}.grf-ui-col-push-10{left:41.66666667%}.grf-ui-col-pull-10{right:41.66666667%}.grf-ui-col-offset-10{margin-left:41.66666667%}.grf-ui-col-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-9{display:block;width:37.5%}.grf-ui-col-push-9{left:37.5%}.grf-ui-col-pull-9{right:37.5%}.grf-ui-col-offset-9{margin-left:37.5%}.grf-ui-col-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-8{display:block;width:33.33333333%}.grf-ui-col-push-8{left:33.33333333%}.grf-ui-col-pull-8{right:33.33333333%}.grf-ui-col-offset-8{margin-left:33.33333333%}.grf-ui-col-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-7{display:block;width:29.16666667%}.grf-ui-col-push-7{left:29.16666667%}.grf-ui-col-pull-7{right:29.16666667%}.grf-ui-col-offset-7{margin-left:29.16666667%}.grf-ui-col-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-6{display:block;width:25%}.grf-ui-col-push-6{left:25%}.grf-ui-col-pull-6{right:25%}.grf-ui-col-offset-6{margin-left:25%}.grf-ui-col-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-5{display:block;width:20.83333333%}.grf-ui-col-push-5{left:20.83333333%}.grf-ui-col-pull-5{right:20.83333333%}.grf-ui-col-offset-5{margin-left:20.83333333%}.grf-ui-col-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-4{display:block;width:16.66666667%}.grf-ui-col-push-4{left:16.66666667%}.grf-ui-col-pull-4{right:16.66666667%}.grf-ui-col-offset-4{margin-left:16.66666667%}.grf-ui-col-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-3{display:block;width:12.5%}.grf-ui-col-push-3{left:12.5%}.grf-ui-col-pull-3{right:12.5%}.grf-ui-col-offset-3{margin-left:12.5%}.grf-ui-col-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-2{display:block;width:8.33333333%}.grf-ui-col-push-2{left:8.33333333%}.grf-ui-col-pull-2{right:8.33333333%}.grf-ui-col-offset-2{margin-left:8.33333333%}.grf-ui-col-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-1{display:block;width:4.16666667%}.grf-ui-col-push-1{left:4.16666667%}.grf-ui-col-pull-1{right:4.16666667%}.grf-ui-col-offset-1{margin-left:4.16666667%}.grf-ui-col-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-0{display:none}.grf-ui-col-offset-0{margin-left:0}.grf-ui-col-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}.grf-ui-col-xs-1,.grf-ui-col-xs-10,.grf-ui-col-xs-11,.grf-ui-col-xs-12,.grf-ui-col-xs-13,.grf-ui-col-xs-14,.grf-ui-col-xs-15,.grf-ui-col-xs-16,.grf-ui-col-xs-17,.grf-ui-col-xs-18,.grf-ui-col-xs-19,.grf-ui-col-xs-2,.grf-ui-col-xs-20,.grf-ui-col-xs-21,.grf-ui-col-xs-22,.grf-ui-col-xs-23,.grf-ui-col-xs-24,.grf-ui-col-xs-3,.grf-ui-col-xs-4,.grf-ui-col-xs-5,.grf-ui-col-xs-6,.grf-ui-col-xs-7,.grf-ui-col-xs-8,.grf-ui-col-xs-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-xs-24{display:block;width:100%}.grf-ui-col-xs-push-24{left:100%}.grf-ui-col-xs-pull-24{right:100%}.grf-ui-col-xs-offset-24{margin-left:100%}.grf-ui-col-xs-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-xs-23{display:block;width:95.83333333%}.grf-ui-col-xs-push-23{left:95.83333333%}.grf-ui-col-xs-pull-23{right:95.83333333%}.grf-ui-col-xs-offset-23{margin-left:95.83333333%}.grf-ui-col-xs-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-xs-22{display:block;width:91.66666667%}.grf-ui-col-xs-push-22{left:91.66666667%}.grf-ui-col-xs-pull-22{right:91.66666667%}.grf-ui-col-xs-offset-22{margin-left:91.66666667%}.grf-ui-col-xs-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-xs-21{display:block;width:87.5%}.grf-ui-col-xs-push-21{left:87.5%}.grf-ui-col-xs-pull-21{right:87.5%}.grf-ui-col-xs-offset-21{margin-left:87.5%}.grf-ui-col-xs-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-xs-20{display:block;width:83.33333333%}.grf-ui-col-xs-push-20{left:83.33333333%}.grf-ui-col-xs-pull-20{right:83.33333333%}.grf-ui-col-xs-offset-20{margin-left:83.33333333%}.grf-ui-col-xs-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-xs-19{display:block;width:79.16666667%}.grf-ui-col-xs-push-19{left:79.16666667%}.grf-ui-col-xs-pull-19{right:79.16666667%}.grf-ui-col-xs-offset-19{margin-left:79.16666667%}.grf-ui-col-xs-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-xs-18{display:block;width:75%}.grf-ui-col-xs-push-18{left:75%}.grf-ui-col-xs-pull-18{right:75%}.grf-ui-col-xs-offset-18{margin-left:75%}.grf-ui-col-xs-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-xs-17{display:block;width:70.83333333%}.grf-ui-col-xs-push-17{left:70.83333333%}.grf-ui-col-xs-pull-17{right:70.83333333%}.grf-ui-col-xs-offset-17{margin-left:70.83333333%}.grf-ui-col-xs-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-xs-16{display:block;width:66.66666667%}.grf-ui-col-xs-push-16{left:66.66666667%}.grf-ui-col-xs-pull-16{right:66.66666667%}.grf-ui-col-xs-offset-16{margin-left:66.66666667%}.grf-ui-col-xs-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-xs-15{display:block;width:62.5%}.grf-ui-col-xs-push-15{left:62.5%}.grf-ui-col-xs-pull-15{right:62.5%}.grf-ui-col-xs-offset-15{margin-left:62.5%}.grf-ui-col-xs-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-xs-14{display:block;width:58.33333333%}.grf-ui-col-xs-push-14{left:58.33333333%}.grf-ui-col-xs-pull-14{right:58.33333333%}.grf-ui-col-xs-offset-14{margin-left:58.33333333%}.grf-ui-col-xs-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-xs-13{display:block;width:54.16666667%}.grf-ui-col-xs-push-13{left:54.16666667%}.grf-ui-col-xs-pull-13{right:54.16666667%}.grf-ui-col-xs-offset-13{margin-left:54.16666667%}.grf-ui-col-xs-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-xs-12{display:block;width:50%}.grf-ui-col-xs-push-12{left:50%}.grf-ui-col-xs-pull-12{right:50%}.grf-ui-col-xs-offset-12{margin-left:50%}.grf-ui-col-xs-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-xs-11{display:block;width:45.83333333%}.grf-ui-col-xs-push-11{left:45.83333333%}.grf-ui-col-xs-pull-11{right:45.83333333%}.grf-ui-col-xs-offset-11{margin-left:45.83333333%}.grf-ui-col-xs-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-xs-10{display:block;width:41.66666667%}.grf-ui-col-xs-push-10{left:41.66666667%}.grf-ui-col-xs-pull-10{right:41.66666667%}.grf-ui-col-xs-offset-10{margin-left:41.66666667%}.grf-ui-col-xs-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-xs-9{display:block;width:37.5%}.grf-ui-col-xs-push-9{left:37.5%}.grf-ui-col-xs-pull-9{right:37.5%}.grf-ui-col-xs-offset-9{margin-left:37.5%}.grf-ui-col-xs-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-xs-8{display:block;width:33.33333333%}.grf-ui-col-xs-push-8{left:33.33333333%}.grf-ui-col-xs-pull-8{right:33.33333333%}.grf-ui-col-xs-offset-8{margin-left:33.33333333%}.grf-ui-col-xs-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-xs-7{display:block;width:29.16666667%}.grf-ui-col-xs-push-7{left:29.16666667%}.grf-ui-col-xs-pull-7{right:29.16666667%}.grf-ui-col-xs-offset-7{margin-left:29.16666667%}.grf-ui-col-xs-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-xs-6{display:block;width:25%}.grf-ui-col-xs-push-6{left:25%}.grf-ui-col-xs-pull-6{right:25%}.grf-ui-col-xs-offset-6{margin-left:25%}.grf-ui-col-xs-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-xs-5{display:block;width:20.83333333%}.grf-ui-col-xs-push-5{left:20.83333333%}.grf-ui-col-xs-pull-5{right:20.83333333%}.grf-ui-col-xs-offset-5{margin-left:20.83333333%}.grf-ui-col-xs-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-xs-4{display:block;width:16.66666667%}.grf-ui-col-xs-push-4{left:16.66666667%}.grf-ui-col-xs-pull-4{right:16.66666667%}.grf-ui-col-xs-offset-4{margin-left:16.66666667%}.grf-ui-col-xs-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-xs-3{display:block;width:12.5%}.grf-ui-col-xs-push-3{left:12.5%}.grf-ui-col-xs-pull-3{right:12.5%}.grf-ui-col-xs-offset-3{margin-left:12.5%}.grf-ui-col-xs-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-xs-2{display:block;width:8.33333333%}.grf-ui-col-xs-push-2{left:8.33333333%}.grf-ui-col-xs-pull-2{right:8.33333333%}.grf-ui-col-xs-offset-2{margin-left:8.33333333%}.grf-ui-col-xs-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-xs-1{display:block;width:4.16666667%}.grf-ui-col-xs-push-1{left:4.16666667%}.grf-ui-col-xs-pull-1{right:4.16666667%}.grf-ui-col-xs-offset-1{margin-left:4.16666667%}.grf-ui-col-xs-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-xs-0{display:none}.grf-ui-col-push-0{left:auto}.grf-ui-col-pull-0{right:auto}.grf-ui-col-xs-push-0{left:auto}.grf-ui-col-xs-pull-0{right:auto}.grf-ui-col-xs-offset-0{margin-left:0}.grf-ui-col-xs-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}@media (min-width:480px){.grf-ui-xs-visible{display:block}.grf-ui-sm-visible,.grf-ui-xs-hidden{display:none}.grf-ui-sm-hidden{display:block}.grf-ui-md-visible{display:none}.grf-ui-md-hidden{display:block}.grf-ui-lg-visible{display:none}.grf-ui-lg-hidden{display:block}.grf-ui-xl-visible{display:none}.grf-ui-xl-hidden{display:block}}@media (min-width:768px){.grf-ui-col-sm-1,.grf-ui-col-sm-10,.grf-ui-col-sm-11,.grf-ui-col-sm-12,.grf-ui-col-sm-13,.grf-ui-col-sm-14,.grf-ui-col-sm-15,.grf-ui-col-sm-16,.grf-ui-col-sm-17,.grf-ui-col-sm-18,.grf-ui-col-sm-19,.grf-ui-col-sm-2,.grf-ui-col-sm-20,.grf-ui-col-sm-21,.grf-ui-col-sm-22,.grf-ui-col-sm-23,.grf-ui-col-sm-24,.grf-ui-col-sm-3,.grf-ui-col-sm-4,.grf-ui-col-sm-5,.grf-ui-col-sm-6,.grf-ui-col-sm-7,.grf-ui-col-sm-8,.grf-ui-col-sm-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-sm-24{display:block;width:100%}.grf-ui-col-sm-push-24{left:100%}.grf-ui-col-sm-pull-24{right:100%}.grf-ui-col-sm-offset-24{margin-left:100%}.grf-ui-col-sm-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-sm-23{display:block;width:95.83333333%}.grf-ui-col-sm-push-23{left:95.83333333%}.grf-ui-col-sm-pull-23{right:95.83333333%}.grf-ui-col-sm-offset-23{margin-left:95.83333333%}.grf-ui-col-sm-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-sm-22{display:block;width:91.66666667%}.grf-ui-col-sm-push-22{left:91.66666667%}.grf-ui-col-sm-pull-22{right:91.66666667%}.grf-ui-col-sm-offset-22{margin-left:91.66666667%}.grf-ui-col-sm-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-sm-21{display:block;width:87.5%}.grf-ui-col-sm-push-21{left:87.5%}.grf-ui-col-sm-pull-21{right:87.5%}.grf-ui-col-sm-offset-21{margin-left:87.5%}.grf-ui-col-sm-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-sm-20{display:block;width:83.33333333%}.grf-ui-col-sm-push-20{left:83.33333333%}.grf-ui-col-sm-pull-20{right:83.33333333%}.grf-ui-col-sm-offset-20{margin-left:83.33333333%}.grf-ui-col-sm-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-sm-19{display:block;width:79.16666667%}.grf-ui-col-sm-push-19{left:79.16666667%}.grf-ui-col-sm-pull-19{right:79.16666667%}.grf-ui-col-sm-offset-19{margin-left:79.16666667%}.grf-ui-col-sm-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-sm-18{display:block;width:75%}.grf-ui-col-sm-push-18{left:75%}.grf-ui-col-sm-pull-18{right:75%}.grf-ui-col-sm-offset-18{margin-left:75%}.grf-ui-col-sm-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-sm-17{display:block;width:70.83333333%}.grf-ui-col-sm-push-17{left:70.83333333%}.grf-ui-col-sm-pull-17{right:70.83333333%}.grf-ui-col-sm-offset-17{margin-left:70.83333333%}.grf-ui-col-sm-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-sm-16{display:block;width:66.66666667%}.grf-ui-col-sm-push-16{left:66.66666667%}.grf-ui-col-sm-pull-16{right:66.66666667%}.grf-ui-col-sm-offset-16{margin-left:66.66666667%}.grf-ui-col-sm-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-sm-15{display:block;width:62.5%}.grf-ui-col-sm-push-15{left:62.5%}.grf-ui-col-sm-pull-15{right:62.5%}.grf-ui-col-sm-offset-15{margin-left:62.5%}.grf-ui-col-sm-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-sm-14{display:block;width:58.33333333%}.grf-ui-col-sm-push-14{left:58.33333333%}.grf-ui-col-sm-pull-14{right:58.33333333%}.grf-ui-col-sm-offset-14{margin-left:58.33333333%}.grf-ui-col-sm-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-sm-13{display:block;width:54.16666667%}.grf-ui-col-sm-push-13{left:54.16666667%}.grf-ui-col-sm-pull-13{right:54.16666667%}.grf-ui-col-sm-offset-13{margin-left:54.16666667%}.grf-ui-col-sm-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-sm-12{display:block;width:50%}.grf-ui-col-sm-push-12{left:50%}.grf-ui-col-sm-pull-12{right:50%}.grf-ui-col-sm-offset-12{margin-left:50%}.grf-ui-col-sm-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-sm-11{display:block;width:45.83333333%}.grf-ui-col-sm-push-11{left:45.83333333%}.grf-ui-col-sm-pull-11{right:45.83333333%}.grf-ui-col-sm-offset-11{margin-left:45.83333333%}.grf-ui-col-sm-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-sm-10{display:block;width:41.66666667%}.grf-ui-col-sm-push-10{left:41.66666667%}.grf-ui-col-sm-pull-10{right:41.66666667%}.grf-ui-col-sm-offset-10{margin-left:41.66666667%}.grf-ui-col-sm-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-sm-9{display:block;width:37.5%}.grf-ui-col-sm-push-9{left:37.5%}.grf-ui-col-sm-pull-9{right:37.5%}.grf-ui-col-sm-offset-9{margin-left:37.5%}.grf-ui-col-sm-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-sm-8{display:block;width:33.33333333%}.grf-ui-col-sm-push-8{left:33.33333333%}.grf-ui-col-sm-pull-8{right:33.33333333%}.grf-ui-col-sm-offset-8{margin-left:33.33333333%}.grf-ui-col-sm-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-sm-7{display:block;width:29.16666667%}.grf-ui-col-sm-push-7{left:29.16666667%}.grf-ui-col-sm-pull-7{right:29.16666667%}.grf-ui-col-sm-offset-7{margin-left:29.16666667%}.grf-ui-col-sm-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-sm-6{display:block;width:25%}.grf-ui-col-sm-push-6{left:25%}.grf-ui-col-sm-pull-6{right:25%}.grf-ui-col-sm-offset-6{margin-left:25%}.grf-ui-col-sm-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-sm-5{display:block;width:20.83333333%}.grf-ui-col-sm-push-5{left:20.83333333%}.grf-ui-col-sm-pull-5{right:20.83333333%}.grf-ui-col-sm-offset-5{margin-left:20.83333333%}.grf-ui-col-sm-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-sm-4{display:block;width:16.66666667%}.grf-ui-col-sm-push-4{left:16.66666667%}.grf-ui-col-sm-pull-4{right:16.66666667%}.grf-ui-col-sm-offset-4{margin-left:16.66666667%}.grf-ui-col-sm-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-sm-3{display:block;width:12.5%}.grf-ui-col-sm-push-3{left:12.5%}.grf-ui-col-sm-pull-3{right:12.5%}.grf-ui-col-sm-offset-3{margin-left:12.5%}.grf-ui-col-sm-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-sm-2{display:block;width:8.33333333%}.grf-ui-col-sm-push-2{left:8.33333333%}.grf-ui-col-sm-pull-2{right:8.33333333%}.grf-ui-col-sm-offset-2{margin-left:8.33333333%}.grf-ui-col-sm-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-sm-1{display:block;width:4.16666667%}.grf-ui-col-sm-push-1{left:4.16666667%}.grf-ui-col-sm-pull-1{right:4.16666667%}.grf-ui-col-sm-offset-1{margin-left:4.16666667%}.grf-ui-col-sm-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-sm-0{display:none}.grf-ui-col-push-0{left:auto}.grf-ui-col-pull-0{right:auto}.grf-ui-col-sm-push-0{left:auto}.grf-ui-col-sm-pull-0{right:auto}.grf-ui-col-sm-offset-0{margin-left:0}.grf-ui-col-sm-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}.grf-ui-sm-visible{display:block}.grf-ui-sm-hidden,.grf-ui-xs-visible{display:none}.grf-ui-xs-hidden{display:block}.grf-ui-md-visible{display:none}.grf-ui-md-hidden{display:block}.grf-ui-lg-visible{display:none}.grf-ui-lg-hidden{display:block}.grf-ui-xl-visible{display:none}.grf-ui-xl-hidden{display:block}}@media (min-width:992px){.grf-ui-col-md-1,.grf-ui-col-md-10,.grf-ui-col-md-11,.grf-ui-col-md-12,.grf-ui-col-md-13,.grf-ui-col-md-14,.grf-ui-col-md-15,.grf-ui-col-md-16,.grf-ui-col-md-17,.grf-ui-col-md-18,.grf-ui-col-md-19,.grf-ui-col-md-2,.grf-ui-col-md-20,.grf-ui-col-md-21,.grf-ui-col-md-22,.grf-ui-col-md-23,.grf-ui-col-md-24,.grf-ui-col-md-3,.grf-ui-col-md-4,.grf-ui-col-md-5,.grf-ui-col-md-6,.grf-ui-col-md-7,.grf-ui-col-md-8,.grf-ui-col-md-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-md-24{display:block;width:100%}.grf-ui-col-md-push-24{left:100%}.grf-ui-col-md-pull-24{right:100%}.grf-ui-col-md-offset-24{margin-left:100%}.grf-ui-col-md-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-md-23{display:block;width:95.83333333%}.grf-ui-col-md-push-23{left:95.83333333%}.grf-ui-col-md-pull-23{right:95.83333333%}.grf-ui-col-md-offset-23{margin-left:95.83333333%}.grf-ui-col-md-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-md-22{display:block;width:91.66666667%}.grf-ui-col-md-push-22{left:91.66666667%}.grf-ui-col-md-pull-22{right:91.66666667%}.grf-ui-col-md-offset-22{margin-left:91.66666667%}.grf-ui-col-md-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-md-21{display:block;width:87.5%}.grf-ui-col-md-push-21{left:87.5%}.grf-ui-col-md-pull-21{right:87.5%}.grf-ui-col-md-offset-21{margin-left:87.5%}.grf-ui-col-md-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-md-20{display:block;width:83.33333333%}.grf-ui-col-md-push-20{left:83.33333333%}.grf-ui-col-md-pull-20{right:83.33333333%}.grf-ui-col-md-offset-20{margin-left:83.33333333%}.grf-ui-col-md-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-md-19{display:block;width:79.16666667%}.grf-ui-col-md-push-19{left:79.16666667%}.grf-ui-col-md-pull-19{right:79.16666667%}.grf-ui-col-md-offset-19{margin-left:79.16666667%}.grf-ui-col-md-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-md-18{display:block;width:75%}.grf-ui-col-md-push-18{left:75%}.grf-ui-col-md-pull-18{right:75%}.grf-ui-col-md-offset-18{margin-left:75%}.grf-ui-col-md-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-md-17{display:block;width:70.83333333%}.grf-ui-col-md-push-17{left:70.83333333%}.grf-ui-col-md-pull-17{right:70.83333333%}.grf-ui-col-md-offset-17{margin-left:70.83333333%}.grf-ui-col-md-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-md-16{display:block;width:66.66666667%}.grf-ui-col-md-push-16{left:66.66666667%}.grf-ui-col-md-pull-16{right:66.66666667%}.grf-ui-col-md-offset-16{margin-left:66.66666667%}.grf-ui-col-md-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-md-15{display:block;width:62.5%}.grf-ui-col-md-push-15{left:62.5%}.grf-ui-col-md-pull-15{right:62.5%}.grf-ui-col-md-offset-15{margin-left:62.5%}.grf-ui-col-md-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-md-14{display:block;width:58.33333333%}.grf-ui-col-md-push-14{left:58.33333333%}.grf-ui-col-md-pull-14{right:58.33333333%}.grf-ui-col-md-offset-14{margin-left:58.33333333%}.grf-ui-col-md-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-md-13{display:block;width:54.16666667%}.grf-ui-col-md-push-13{left:54.16666667%}.grf-ui-col-md-pull-13{right:54.16666667%}.grf-ui-col-md-offset-13{margin-left:54.16666667%}.grf-ui-col-md-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-md-12{display:block;width:50%}.grf-ui-col-md-push-12{left:50%}.grf-ui-col-md-pull-12{right:50%}.grf-ui-col-md-offset-12{margin-left:50%}.grf-ui-col-md-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-md-11{display:block;width:45.83333333%}.grf-ui-col-md-push-11{left:45.83333333%}.grf-ui-col-md-pull-11{right:45.83333333%}.grf-ui-col-md-offset-11{margin-left:45.83333333%}.grf-ui-col-md-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-md-10{display:block;width:41.66666667%}.grf-ui-col-md-push-10{left:41.66666667%}.grf-ui-col-md-pull-10{right:41.66666667%}.grf-ui-col-md-offset-10{margin-left:41.66666667%}.grf-ui-col-md-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-md-9{display:block;width:37.5%}.grf-ui-col-md-push-9{left:37.5%}.grf-ui-col-md-pull-9{right:37.5%}.grf-ui-col-md-offset-9{margin-left:37.5%}.grf-ui-col-md-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-md-8{display:block;width:33.33333333%}.grf-ui-col-md-push-8{left:33.33333333%}.grf-ui-col-md-pull-8{right:33.33333333%}.grf-ui-col-md-offset-8{margin-left:33.33333333%}.grf-ui-col-md-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-md-7{display:block;width:29.16666667%}.grf-ui-col-md-push-7{left:29.16666667%}.grf-ui-col-md-pull-7{right:29.16666667%}.grf-ui-col-md-offset-7{margin-left:29.16666667%}.grf-ui-col-md-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-md-6{display:block;width:25%}.grf-ui-col-md-push-6{left:25%}.grf-ui-col-md-pull-6{right:25%}.grf-ui-col-md-offset-6{margin-left:25%}.grf-ui-col-md-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-md-5{display:block;width:20.83333333%}.grf-ui-col-md-push-5{left:20.83333333%}.grf-ui-col-md-pull-5{right:20.83333333%}.grf-ui-col-md-offset-5{margin-left:20.83333333%}.grf-ui-col-md-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-md-4{display:block;width:16.66666667%}.grf-ui-col-md-push-4{left:16.66666667%}.grf-ui-col-md-pull-4{right:16.66666667%}.grf-ui-col-md-offset-4{margin-left:16.66666667%}.grf-ui-col-md-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-md-3{display:block;width:12.5%}.grf-ui-col-md-push-3{left:12.5%}.grf-ui-col-md-pull-3{right:12.5%}.grf-ui-col-md-offset-3{margin-left:12.5%}.grf-ui-col-md-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-md-2{display:block;width:8.33333333%}.grf-ui-col-md-push-2{left:8.33333333%}.grf-ui-col-md-pull-2{right:8.33333333%}.grf-ui-col-md-offset-2{margin-left:8.33333333%}.grf-ui-col-md-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-md-1{display:block;width:4.16666667%}.grf-ui-col-md-push-1{left:4.16666667%}.grf-ui-col-md-pull-1{right:4.16666667%}.grf-ui-col-md-offset-1{margin-left:4.16666667%}.grf-ui-col-md-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-md-0{display:none}.grf-ui-col-push-0{left:auto}.grf-ui-col-pull-0{right:auto}.grf-ui-col-md-push-0{left:auto}.grf-ui-col-md-pull-0{right:auto}.grf-ui-col-md-offset-0{margin-left:0}.grf-ui-col-md-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}.grf-ui-md-visible{display:block}.grf-ui-md-hidden,.grf-ui-xs-visible{display:none}.grf-ui-xs-hidden{display:block}.grf-ui-sm-visible{display:none}.grf-ui-sm-hidden{display:block}.grf-ui-lg-visible{display:none}.grf-ui-lg-hidden{display:block}.grf-ui-xl-visible{display:none}.grf-ui-xl-hidden{display:block}}@media (min-width:1200px){.grf-ui-col-lg-1,.grf-ui-col-lg-10,.grf-ui-col-lg-11,.grf-ui-col-lg-12,.grf-ui-col-lg-13,.grf-ui-col-lg-14,.grf-ui-col-lg-15,.grf-ui-col-lg-16,.grf-ui-col-lg-17,.grf-ui-col-lg-18,.grf-ui-col-lg-19,.grf-ui-col-lg-2,.grf-ui-col-lg-20,.grf-ui-col-lg-21,.grf-ui-col-lg-22,.grf-ui-col-lg-23,.grf-ui-col-lg-24,.grf-ui-col-lg-3,.grf-ui-col-lg-4,.grf-ui-col-lg-5,.grf-ui-col-lg-6,.grf-ui-col-lg-7,.grf-ui-col-lg-8,.grf-ui-col-lg-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-lg-24{display:block;width:100%}.grf-ui-col-lg-push-24{left:100%}.grf-ui-col-lg-pull-24{right:100%}.grf-ui-col-lg-offset-24{margin-left:100%}.grf-ui-col-lg-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-lg-23{display:block;width:95.83333333%}.grf-ui-col-lg-push-23{left:95.83333333%}.grf-ui-col-lg-pull-23{right:95.83333333%}.grf-ui-col-lg-offset-23{margin-left:95.83333333%}.grf-ui-col-lg-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-lg-22{display:block;width:91.66666667%}.grf-ui-col-lg-push-22{left:91.66666667%}.grf-ui-col-lg-pull-22{right:91.66666667%}.grf-ui-col-lg-offset-22{margin-left:91.66666667%}.grf-ui-col-lg-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-lg-21{display:block;width:87.5%}.grf-ui-col-lg-push-21{left:87.5%}.grf-ui-col-lg-pull-21{right:87.5%}.grf-ui-col-lg-offset-21{margin-left:87.5%}.grf-ui-col-lg-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-lg-20{display:block;width:83.33333333%}.grf-ui-col-lg-push-20{left:83.33333333%}.grf-ui-col-lg-pull-20{right:83.33333333%}.grf-ui-col-lg-offset-20{margin-left:83.33333333%}.grf-ui-col-lg-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-lg-19{display:block;width:79.16666667%}.grf-ui-col-lg-push-19{left:79.16666667%}.grf-ui-col-lg-pull-19{right:79.16666667%}.grf-ui-col-lg-offset-19{margin-left:79.16666667%}.grf-ui-col-lg-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-lg-18{display:block;width:75%}.grf-ui-col-lg-push-18{left:75%}.grf-ui-col-lg-pull-18{right:75%}.grf-ui-col-lg-offset-18{margin-left:75%}.grf-ui-col-lg-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-lg-17{display:block;width:70.83333333%}.grf-ui-col-lg-push-17{left:70.83333333%}.grf-ui-col-lg-pull-17{right:70.83333333%}.grf-ui-col-lg-offset-17{margin-left:70.83333333%}.grf-ui-col-lg-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-lg-16{display:block;width:66.66666667%}.grf-ui-col-lg-push-16{left:66.66666667%}.grf-ui-col-lg-pull-16{right:66.66666667%}.grf-ui-col-lg-offset-16{margin-left:66.66666667%}.grf-ui-col-lg-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-lg-15{display:block;width:62.5%}.grf-ui-col-lg-push-15{left:62.5%}.grf-ui-col-lg-pull-15{right:62.5%}.grf-ui-col-lg-offset-15{margin-left:62.5%}.grf-ui-col-lg-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-lg-14{display:block;width:58.33333333%}.grf-ui-col-lg-push-14{left:58.33333333%}.grf-ui-col-lg-pull-14{right:58.33333333%}.grf-ui-col-lg-offset-14{margin-left:58.33333333%}.grf-ui-col-lg-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-lg-13{display:block;width:54.16666667%}.grf-ui-col-lg-push-13{left:54.16666667%}.grf-ui-col-lg-pull-13{right:54.16666667%}.grf-ui-col-lg-offset-13{margin-left:54.16666667%}.grf-ui-col-lg-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-lg-12{display:block;width:50%}.grf-ui-col-lg-push-12{left:50%}.grf-ui-col-lg-pull-12{right:50%}.grf-ui-col-lg-offset-12{margin-left:50%}.grf-ui-col-lg-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-lg-11{display:block;width:45.83333333%}.grf-ui-col-lg-push-11{left:45.83333333%}.grf-ui-col-lg-pull-11{right:45.83333333%}.grf-ui-col-lg-offset-11{margin-left:45.83333333%}.grf-ui-col-lg-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-lg-10{display:block;width:41.66666667%}.grf-ui-col-lg-push-10{left:41.66666667%}.grf-ui-col-lg-pull-10{right:41.66666667%}.grf-ui-col-lg-offset-10{margin-left:41.66666667%}.grf-ui-col-lg-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-lg-9{display:block;width:37.5%}.grf-ui-col-lg-push-9{left:37.5%}.grf-ui-col-lg-pull-9{right:37.5%}.grf-ui-col-lg-offset-9{margin-left:37.5%}.grf-ui-col-lg-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-lg-8{display:block;width:33.33333333%}.grf-ui-col-lg-push-8{left:33.33333333%}.grf-ui-col-lg-pull-8{right:33.33333333%}.grf-ui-col-lg-offset-8{margin-left:33.33333333%}.grf-ui-col-lg-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-lg-7{display:block;width:29.16666667%}.grf-ui-col-lg-push-7{left:29.16666667%}.grf-ui-col-lg-pull-7{right:29.16666667%}.grf-ui-col-lg-offset-7{margin-left:29.16666667%}.grf-ui-col-lg-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-lg-6{display:block;width:25%}.grf-ui-col-lg-push-6{left:25%}.grf-ui-col-lg-pull-6{right:25%}.grf-ui-col-lg-offset-6{margin-left:25%}.grf-ui-col-lg-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-lg-5{display:block;width:20.83333333%}.grf-ui-col-lg-push-5{left:20.83333333%}.grf-ui-col-lg-pull-5{right:20.83333333%}.grf-ui-col-lg-offset-5{margin-left:20.83333333%}.grf-ui-col-lg-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-lg-4{display:block;width:16.66666667%}.grf-ui-col-lg-push-4{left:16.66666667%}.grf-ui-col-lg-pull-4{right:16.66666667%}.grf-ui-col-lg-offset-4{margin-left:16.66666667%}.grf-ui-col-lg-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-lg-3{display:block;width:12.5%}.grf-ui-col-lg-push-3{left:12.5%}.grf-ui-col-lg-pull-3{right:12.5%}.grf-ui-col-lg-offset-3{margin-left:12.5%}.grf-ui-col-lg-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-lg-2{display:block;width:8.33333333%}.grf-ui-col-lg-push-2{left:8.33333333%}.grf-ui-col-lg-pull-2{right:8.33333333%}.grf-ui-col-lg-offset-2{margin-left:8.33333333%}.grf-ui-col-lg-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-lg-1{display:block;width:4.16666667%}.grf-ui-col-lg-push-1{left:4.16666667%}.grf-ui-col-lg-pull-1{right:4.16666667%}.grf-ui-col-lg-offset-1{margin-left:4.16666667%}.grf-ui-col-lg-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-lg-0{display:none}.grf-ui-col-push-0{left:auto}.grf-ui-col-pull-0{right:auto}.grf-ui-col-lg-push-0{left:auto}.grf-ui-col-lg-pull-0{right:auto}.grf-ui-col-lg-offset-0{margin-left:0}.grf-ui-col-lg-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}.grf-ui-lg-visible{display:block}.grf-ui-lg-hidden,.grf-ui-xs-visible{display:none}.grf-ui-xs-hidden{display:block}.grf-ui-sm-visible{display:none}.grf-ui-sm-hidden{display:block}.grf-ui-md-visible{display:none}.grf-ui-md-hidden{display:block}.grf-ui-xl-visible{display:none}.grf-ui-xl-hidden{display:block}}@media (min-width:1600px){.grf-ui-col-xl-1,.grf-ui-col-xl-10,.grf-ui-col-xl-11,.grf-ui-col-xl-12,.grf-ui-col-xl-13,.grf-ui-col-xl-14,.grf-ui-col-xl-15,.grf-ui-col-xl-16,.grf-ui-col-xl-17,.grf-ui-col-xl-18,.grf-ui-col-xl-19,.grf-ui-col-xl-2,.grf-ui-col-xl-20,.grf-ui-col-xl-21,.grf-ui-col-xl-22,.grf-ui-col-xl-23,.grf-ui-col-xl-24,.grf-ui-col-xl-3,.grf-ui-col-xl-4,.grf-ui-col-xl-5,.grf-ui-col-xl-6,.grf-ui-col-xl-7,.grf-ui-col-xl-8,.grf-ui-col-xl-9{float:left;-webkit-box-flex:0;-ms-flex:0 0 auto;flex:0 0 auto}.grf-ui-col-xl-24{display:block;width:100%}.grf-ui-col-xl-push-24{left:100%}.grf-ui-col-xl-pull-24{right:100%}.grf-ui-col-xl-offset-24{margin-left:100%}.grf-ui-col-xl-order-24{-webkit-box-ordinal-group:25;-ms-flex-order:24;order:24}.grf-ui-col-xl-23{display:block;width:95.83333333%}.grf-ui-col-xl-push-23{left:95.83333333%}.grf-ui-col-xl-pull-23{right:95.83333333%}.grf-ui-col-xl-offset-23{margin-left:95.83333333%}.grf-ui-col-xl-order-23{-webkit-box-ordinal-group:24;-ms-flex-order:23;order:23}.grf-ui-col-xl-22{display:block;width:91.66666667%}.grf-ui-col-xl-push-22{left:91.66666667%}.grf-ui-col-xl-pull-22{right:91.66666667%}.grf-ui-col-xl-offset-22{margin-left:91.66666667%}.grf-ui-col-xl-order-22{-webkit-box-ordinal-group:23;-ms-flex-order:22;order:22}.grf-ui-col-xl-21{display:block;width:87.5%}.grf-ui-col-xl-push-21{left:87.5%}.grf-ui-col-xl-pull-21{right:87.5%}.grf-ui-col-xl-offset-21{margin-left:87.5%}.grf-ui-col-xl-order-21{-webkit-box-ordinal-group:22;-ms-flex-order:21;order:21}.grf-ui-col-xl-20{display:block;width:83.33333333%}.grf-ui-col-xl-push-20{left:83.33333333%}.grf-ui-col-xl-pull-20{right:83.33333333%}.grf-ui-col-xl-offset-20{margin-left:83.33333333%}.grf-ui-col-xl-order-20{-webkit-box-ordinal-group:21;-ms-flex-order:20;order:20}.grf-ui-col-xl-19{display:block;width:79.16666667%}.grf-ui-col-xl-push-19{left:79.16666667%}.grf-ui-col-xl-pull-19{right:79.16666667%}.grf-ui-col-xl-offset-19{margin-left:79.16666667%}.grf-ui-col-xl-order-19{-webkit-box-ordinal-group:20;-ms-flex-order:19;order:19}.grf-ui-col-xl-18{display:block;width:75%}.grf-ui-col-xl-push-18{left:75%}.grf-ui-col-xl-pull-18{right:75%}.grf-ui-col-xl-offset-18{margin-left:75%}.grf-ui-col-xl-order-18{-webkit-box-ordinal-group:19;-ms-flex-order:18;order:18}.grf-ui-col-xl-17{display:block;width:70.83333333%}.grf-ui-col-xl-push-17{left:70.83333333%}.grf-ui-col-xl-pull-17{right:70.83333333%}.grf-ui-col-xl-offset-17{margin-left:70.83333333%}.grf-ui-col-xl-order-17{-webkit-box-ordinal-group:18;-ms-flex-order:17;order:17}.grf-ui-col-xl-16{display:block;width:66.66666667%}.grf-ui-col-xl-push-16{left:66.66666667%}.grf-ui-col-xl-pull-16{right:66.66666667%}.grf-ui-col-xl-offset-16{margin-left:66.66666667%}.grf-ui-col-xl-order-16{-webkit-box-ordinal-group:17;-ms-flex-order:16;order:16}.grf-ui-col-xl-15{display:block;width:62.5%}.grf-ui-col-xl-push-15{left:62.5%}.grf-ui-col-xl-pull-15{right:62.5%}.grf-ui-col-xl-offset-15{margin-left:62.5%}.grf-ui-col-xl-order-15{-webkit-box-ordinal-group:16;-ms-flex-order:15;order:15}.grf-ui-col-xl-14{display:block;width:58.33333333%}.grf-ui-col-xl-push-14{left:58.33333333%}.grf-ui-col-xl-pull-14{right:58.33333333%}.grf-ui-col-xl-offset-14{margin-left:58.33333333%}.grf-ui-col-xl-order-14{-webkit-box-ordinal-group:15;-ms-flex-order:14;order:14}.grf-ui-col-xl-13{display:block;width:54.16666667%}.grf-ui-col-xl-push-13{left:54.16666667%}.grf-ui-col-xl-pull-13{right:54.16666667%}.grf-ui-col-xl-offset-13{margin-left:54.16666667%}.grf-ui-col-xl-order-13{-webkit-box-ordinal-group:14;-ms-flex-order:13;order:13}.grf-ui-col-xl-12{display:block;width:50%}.grf-ui-col-xl-push-12{left:50%}.grf-ui-col-xl-pull-12{right:50%}.grf-ui-col-xl-offset-12{margin-left:50%}.grf-ui-col-xl-order-12{-webkit-box-ordinal-group:13;-ms-flex-order:12;order:12}.grf-ui-col-xl-11{display:block;width:45.83333333%}.grf-ui-col-xl-push-11{left:45.83333333%}.grf-ui-col-xl-pull-11{right:45.83333333%}.grf-ui-col-xl-offset-11{margin-left:45.83333333%}.grf-ui-col-xl-order-11{-webkit-box-ordinal-group:12;-ms-flex-order:11;order:11}.grf-ui-col-xl-10{display:block;width:41.66666667%}.grf-ui-col-xl-push-10{left:41.66666667%}.grf-ui-col-xl-pull-10{right:41.66666667%}.grf-ui-col-xl-offset-10{margin-left:41.66666667%}.grf-ui-col-xl-order-10{-webkit-box-ordinal-group:11;-ms-flex-order:10;order:10}.grf-ui-col-xl-9{display:block;width:37.5%}.grf-ui-col-xl-push-9{left:37.5%}.grf-ui-col-xl-pull-9{right:37.5%}.grf-ui-col-xl-offset-9{margin-left:37.5%}.grf-ui-col-xl-order-9{-webkit-box-ordinal-group:10;-ms-flex-order:9;order:9}.grf-ui-col-xl-8{display:block;width:33.33333333%}.grf-ui-col-xl-push-8{left:33.33333333%}.grf-ui-col-xl-pull-8{right:33.33333333%}.grf-ui-col-xl-offset-8{margin-left:33.33333333%}.grf-ui-col-xl-order-8{-webkit-box-ordinal-group:9;-ms-flex-order:8;order:8}.grf-ui-col-xl-7{display:block;width:29.16666667%}.grf-ui-col-xl-push-7{left:29.16666667%}.grf-ui-col-xl-pull-7{right:29.16666667%}.grf-ui-col-xl-offset-7{margin-left:29.16666667%}.grf-ui-col-xl-order-7{-webkit-box-ordinal-group:8;-ms-flex-order:7;order:7}.grf-ui-col-xl-6{display:block;width:25%}.grf-ui-col-xl-push-6{left:25%}.grf-ui-col-xl-pull-6{right:25%}.grf-ui-col-xl-offset-6{margin-left:25%}.grf-ui-col-xl-order-6{-webkit-box-ordinal-group:7;-ms-flex-order:6;order:6}.grf-ui-col-xl-5{display:block;width:20.83333333%}.grf-ui-col-xl-push-5{left:20.83333333%}.grf-ui-col-xl-pull-5{right:20.83333333%}.grf-ui-col-xl-offset-5{margin-left:20.83333333%}.grf-ui-col-xl-order-5{-webkit-box-ordinal-group:6;-ms-flex-order:5;order:5}.grf-ui-col-xl-4{display:block;width:16.66666667%}.grf-ui-col-xl-push-4{left:16.66666667%}.grf-ui-col-xl-pull-4{right:16.66666667%}.grf-ui-col-xl-offset-4{margin-left:16.66666667%}.grf-ui-col-xl-order-4{-webkit-box-ordinal-group:5;-ms-flex-order:4;order:4}.grf-ui-col-xl-3{display:block;width:12.5%}.grf-ui-col-xl-push-3{left:12.5%}.grf-ui-col-xl-pull-3{right:12.5%}.grf-ui-col-xl-offset-3{margin-left:12.5%}.grf-ui-col-xl-order-3{-webkit-box-ordinal-group:4;-ms-flex-order:3;order:3}.grf-ui-col-xl-2{display:block;width:8.33333333%}.grf-ui-col-xl-push-2{left:8.33333333%}.grf-ui-col-xl-pull-2{right:8.33333333%}.grf-ui-col-xl-offset-2{margin-left:8.33333333%}.grf-ui-col-xl-order-2{-webkit-box-ordinal-group:3;-ms-flex-order:2;order:2}.grf-ui-col-xl-1{display:block;width:4.16666667%}.grf-ui-col-xl-push-1{left:4.16666667%}.grf-ui-col-xl-pull-1{right:4.16666667%}.grf-ui-col-xl-offset-1{margin-left:4.16666667%}.grf-ui-col-xl-order-1{-webkit-box-ordinal-group:2;-ms-flex-order:1;order:1}.grf-ui-col-xl-0{display:none}.grf-ui-col-push-0{left:auto}.grf-ui-col-pull-0{right:auto}.grf-ui-col-xl-push-0{left:auto}.grf-ui-col-xl-pull-0{right:auto}.grf-ui-col-xl-offset-0{margin-left:0}.grf-ui-col-xl-order-0{-webkit-box-ordinal-group:1;-ms-flex-order:0;order:0}.grf-ui-xl-visible{display:block}.grf-ui-sm-visible,.grf-ui-xl-hidden{display:none}.grf-ui-sm-hidden{display:block}.grf-ui-xs-visible{display:none}.grf-ui-xs-hidden{display:block}.grf-ui-md-visible{display:none}.grf-ui-md-hidden{display:block}.grf-ui-lg-visible{display:none}.grf-ui-lg-hidden{display:block}}`]
            },] },
];
/** @nocollapse */
RowComponent$1.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
RowComponent$1.propDecorators = {
    "uiType": [{ type: Input },],
    "uiAlign": [{ type: Input },],
    "uiJustify": [{ type: Input },],
    "uiGutter": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @abstract
 */

class ColComponent {
    /**
     * @param {?} _elementRef
     * @param {?} _uiRow
     * @param {?} _renderer
     */
    constructor(_elementRef, _uiRow, _renderer) {
        this._elementRef = _elementRef;
        this._uiRow = _uiRow;
        this._renderer = _renderer;
        this._classList = [];
        this._prefixCls = 'grf-ui-col';
        this._prefix = 'grf-ui';
        this._el = this._elementRef.nativeElement;
    }
    /**
     * @return {?}
     */
    get paddingLeft() {
        return this._uiRow && this._uiRow._gutter / 2;
    }
    /**
     * @return {?}
     */
    get paddingRight() {
        return this._uiRow && this._uiRow._gutter / 2;
    }
    /**
     * temp solution since no method add classMap to host https://github.com/angular/angular/issues/7289
     * @return {?}
     */
    setClassMap() {
        this._classList.forEach(_className => {
            this._renderer.removeClass(this._el, _className);
        });
        this._classList = [
            this.uiSpan && `${this._prefixCls}-${this.uiSpan}`,
            this.uiOrder && `${this._prefixCls}-order-${this.uiOrder}`,
            this.uiOffset && `${this._prefixCls}-offset-${this.uiOffset}`,
            this.uiPull && `${this._prefixCls}-pull-${this.uiPull}`,
            this.uiPush && `${this._prefixCls}-push-${this.uiPush}`,
            ...this.generateClass()
        ];
        this._classList = this._classList.filter((item) => {
            return !!item;
        });
        this._classList.forEach(_className => {
            this._renderer.addClass(this._el, _className);
        });
    }
    /**
     * @return {?}
     */
    generateClass() {
        const /** @type {?} */ listOfSizeInputName = ['uiXs', 'uiSm', 'uiMd', 'uiLg', 'uiXl'];
        const /** @type {?} */ listOfClassName = new Array();
        listOfSizeInputName.forEach(name => {
            const /** @type {?} */ sizeName = name.replace('ui', '').toLowerCase();
            if ((typeof (this[name]) === 'number') || (typeof (this[name]) === 'string')) {
                listOfClassName.push(this[name] && `${this._prefixCls}-${sizeName}-${this[name]}`);
            }
            else {
                listOfClassName.push(this[name] && this[name]['span'] && `${this._prefixCls}-${sizeName}-${this[name]['span']}`);
                listOfClassName.push(this[name] && this[name]['pull'] && `${this._prefixCls}-${sizeName}-pull-${this[name]['pull']}`);
                listOfClassName.push(this[name] && this[name]['push'] && `${this._prefixCls}-${sizeName}-push-${this[name]['push']}`);
                listOfClassName.push(this[name] && this[name]['offset'] && `${this._prefixCls}-${sizeName}-offset-${this[name]['offset']}`);
                listOfClassName.push(this[name] && this[name]['order'] && `${this._prefixCls}-${sizeName}-order-${this[name]['order']}`);
                listOfClassName.push(this[name] && this[name]['show'] && `${this._prefix}-${sizeName}-${this[name]['show']}`);
            }
        });
        return listOfClassName;
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        this.setClassMap();
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.setClassMap();
    }
}
ColComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-col',
                template: `
      <ng-content></ng-content>
    `,
                styles: []
            },] },
];
/** @nocollapse */
ColComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: RowComponent$1, decorators: [{ type: Optional }, { type: Host },] },
    { type: Renderer2, },
];
ColComponent.propDecorators = {
    "paddingLeft": [{ type: HostBinding, args: ['style.padding-left.px',] },],
    "paddingRight": [{ type: HostBinding, args: ['style.padding-right.px',] },],
    "uiSpan": [{ type: Input },],
    "uiOrder": [{ type: Input },],
    "uiOffset": [{ type: Input },],
    "uiPush": [{ type: Input },],
    "uiPull": [{ type: Input },],
    "uiXs": [{ type: Input },],
    "uiSm": [{ type: Input },],
    "uiMd": [{ type: Input },],
    "uiLg": [{ type: Input },],
    "uiXl": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ColDirective extends ColComponent {
}
ColDirective.decorators = [
    { type: Directive, args: [{
                selector: '[grf-col]'
            },] },
];
/** @nocollapse */
ColDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GridModule {
}
GridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [RowComponent$1, ColDirective, ColComponent],
                exports: [RowComponent$1, ColDirective, ColComponent],
                imports: [CommonModule]
            },] },
];
/** @nocollapse */
GridModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FlexibleMenuService {
    constructor() {
        this.flagchanges = new Subject$1();
    }
    /**
     * @param {?} ifClick
     * @return {?}
     */
    ifChildClick(ifClick) {
        this.childClick = ifClick;
        this.flagchanges.next(this.childClick);
    }
}
FlexibleMenuService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
FlexibleMenuService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FlexibleMenuComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     * @param {?} _flexMenu
     */
    constructor(el, domHandler, renderer, _flexMenu) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this._flexMenu = _flexMenu;
        this.itemClickEvent = new EventEmitter();
        this.flexiMenuShow = false;
        this._flexFixed = false;
        this.dropList = [
            { label: '', icon: 'fa-list-alt', items: [] }
        ];
    }
    /**
     * @param {?} item
     * @return {?}
     */
    set model(item) {
        this._model = item;
        this.menuItemInit();
    }
    /**
     * @return {?}
     */
    get model() {
        return this._model;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    set activeItem(item) {
        this._initActiveItem = item;
        this.activeItemInit();
    }
    /**
     * @return {?}
     */
    get activeItem() {
        return this._initActiveItem;
    }
    /**
     * @param {?} fixed
     * @return {?}
     */
    set flexFixed(fixed) {
        this._flexFixed = fixed;
        this.menuItemInit();
    }
    /**
     * @return {?}
     */
    get flexFixed() {
        return this._flexFixed;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    set menuParent(item) {
        this._menuParent = item;
        if (this._model && this._model.length) {
            if (this.menuParent && this.menuParent.label && this.menuParent.icon) {
                this.dropList[0].label = this.menuParent.label;
                this.dropList[0].icon = this.menuParent.icon;
            }
        }
        this.menuItemInit();
    }
    /**
     * @return {?}
     */
    get menuParent() {
        return this._menuParent;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    mouseHover(item) {
        this.hoverItem = item;
    }
    /**
     * @return {?}
     */
    menuItemInit() {
        if (this._model && this._model.length) {
            if (!this.flexFixed) {
                this.indexSwitch();
                window.onresize = () => {
                    if (this._model && this._model.length) {
                        this.indexSwitch();
                    }
                };
            }
            else if (this.flexFixed) {
                this.listCal(0);
            }
        }
    }
    /**
     * @return {?}
     */
    activeItemInit() {
        if (this.model && this.model.length && this.tabList && this.dropList) {
            // 设置首次进入时激活的元素是在展开的菜单还是下拉的菜单中
            if (JSON.stringify(this.tabList).indexOf(JSON.stringify(this._initActiveItem)) !== -1) {
                this.tabActiveItem = this._initActiveItem;
            }
            else if (JSON.stringify(this.dropList).indexOf(JSON.stringify(this._initActiveItem)) !== -1) {
                this.childActiveItem = this._initActiveItem;
            }
        }
    }
    /**
     * @return {?}
     */
    tabListInit() {
        // copy一份model
        if (this.model) {
            this.origiList = new Array(this.model.length);
            for (let /** @type {?} */ i = 0; i < this.model.length; i++) {
                this.origiList[i] = this.model[i];
            }
            this.tabList = this.origiList;
        }
    }
    /**
     * @param {?} index
     * @return {?}
     */
    listCal(index) {
        // 根据传入的index,把一部分数据传入tablist中，视图上显示在平铺的blockmenu上
        // 另一部分数据则传入dropItems中,收缩至下拉菜单中
        if (this.model.length > index) {
            this.flexiMenuShow = true;
            this.tabList = new Array(index);
            this.dropItems = new Array(this.model.length - index);
            for (let /** @type {?} */ i = 0; i < this.model.length; i++) {
                if (i < index) {
                    this.tabList[i] = this.model[i];
                }
            }
            for (let /** @type {?} */ i = 0, /** @type {?} */ y = 0; i < this.model.length; i++) {
                if (i >= index) {
                    this.dropItems[y] = this.model[i];
                    y++;
                }
            }
            this.dropList[0].items = this.dropItems;
        }
        else {
            this.flexiMenuShow = false;
            this.tabList = new Array(this.model.length);
            for (let /** @type {?} */ i = 0; i < this.model.length; i++) {
                this.tabList[i] = this.model[i];
            }
        }
    }
    /**
     * @return {?}
     */
    indexSwitch() {
        // 根据响应式变化的宽度来定放多少项在平铺的blockmenu中
        const /** @type {?} */ boxWidth = this.box.nativeElement.offsetWidth;
        if (boxWidth > 900) {
            this.listCal(6);
        }
        else if (boxWidth < 800 && boxWidth > 700) {
            this.listCal(5);
        }
        else if (boxWidth < 700 && boxWidth > 600) {
            this.listCal(3);
        }
        else if (boxWidth < 600 && boxWidth > 500) {
            this.listCal(4);
        }
        else if (boxWidth < 500 && boxWidth > 400) {
            this.listCal(3);
        }
        else if (boxWidth < 400 && boxWidth > 300) {
            this.listCal(2);
        }
        else if (boxWidth < 300 && boxWidth > 200) {
            this.listCal(1);
        }
        else if (boxWidth < 200 && boxWidth > 100) {
            this.listCal(0);
        }
    }
    /**
     * @param {?} list
     * @return {?}
     */
    indexCal(list) {
        // 打算使用更加智能的算法 未使用 优化中
        const /** @type {?} */ listLength = list.children.length;
        let /** @type {?} */ lengthSum = 0;
        for (let /** @type {?} */ i = 0; i < listLength; i++) {
            lengthSum = lengthSum + list.children[i].clientWidth;
            if (lengthSum >= this.box.nativeElement.offsetWidth) {
                this.listCal(i - 1);
                return i;
            }
        }
    }
    /**
     * @return {?}
     */
    _getHostElement() {
        return this.el.nativeElement;
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.tabActiveItem = item;
        this.itemClickEvent.emit(item);
        this._flexMenu.ifChildClick(true);
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
    /**
     * @return {?}
     */
    menuBarHover() {
        this.hoverItem = null;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
        this.tabActiveItem = null;
    }
}
FlexibleMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-flexiblemenu',
                template: `<!-- <div class="{{theme}}">
    <div [ngClass]="{'grf-ui-menu grf-ui-corner-all grf-ui-flexiblemenubar grf-ui-helper-clearfix':true}" [class]="styleClass" [ngStyle]="style">
        <grf-flexiblemenusub [item]="model" root="root" (itemClickEvent)="componentClick($event)">
            <ng-content></ng-content>
        </grf-flexiblemenusub>
    </div>
</div> -->

<div #box class="grf-ui-flexibelmenu-box">
    <div [ngClass]="'grf-ui-corner-all grf-ui-blockmenu'" [ngStyle]="style" [class]="styleClass">
        <ul #list class="grf-ui-helper-reset grf-ui-helper-clearfix grf-ui-corner-all grf-ui-blockmenu-nav" role="tablist">
            <li #listItem *ngFor="let item of tabList" [ngClass]="{'grf-ui-blockmenuitem grf-ui-state-default grf-ui-corner-top':true,'grf-ui-state-disabled':item.disabled,
                    'grf-ui-blockmenuitem-hasicon':item.icon}">
                <a *ngIf="!item.routerLink" [href]="item.url||'#'" class="grf-ui-menuitem-link grf-ui-corner-all" (click)="itemClick($event,item)" (mouseover)="mouseHover(item)" style="position:relative" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)"
         [ngClass]="{'grf-ui-blockmenuitem-state-active':item==tabActiveItem }" [attr.target]="item.target" [attr.title]="item.title">
                    <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon"></span>
                    <span class="grf-ui-menuitem-text">{{item.label}}</span>
                </a>
                <a *ngIf="item.routerLink" [routerLink]="item.routerLink" [routerLinkActive]="'grf-ui-blockmenuitem-state-active'" (mouseover)="mouseHover(item)" style="position:relative" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)"
                    [routerLinkActiveOptions]="item.routerLinkActiveOptions||{exact:false}" class="grf-ui-menuitem-link grf-ui-corner-all" (click)="itemClick($event,item)" [attr.target]="item.target" [attr.title]="item.title">
                    <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon"></span>
                    <span class="grf-ui-menuitem-text">{{item.label}}</span>
                </a>
            </li>

        </ul>
    </div>
    <div *ngIf="flexiMenuShow" [ngClass]="{'grf-ui-menu grf-ui-corner-all grf-ui-flexiblemenubar grf-ui-helper-clearfix':true}" [class]="styleClass" [ngStyle]="style" (mouseover)="menuBarHover()">
        <grf-flexiblemenusub [item]="dropList" root="root" (itemClickEvent)="componentClick($event)" [menuParent]="dropList[0]" [childClickItem]="childActiveItem">
            <ng-content></ng-content>
        </grf-flexiblemenusub>
    </div>
</div>
`,
                styles: [`@charset "UTF-8";.grf-ui-blockmenu{border:0;margin-top:0;margin-bottom:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav{padding:0;margin:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav a{padding:.5em 1em}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem{margin:0;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;list-style:none;float:left;position:relative;padding:0;white-space:nowrap;display:block;border-bottom:0;height:70px;color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem a{float:left;padding:.5em 1em;text-decoration:none}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-blockmenuitem-state-active{color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link{color:#2d353c;height:70px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon{padding:5px}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon,.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-text{vertical-align:middle;display:inline-block;color:#fff;font-size:14px;opacity:.9}.grf-ui-flexiblemenubaritem{height:70px}.flexiblemenubar-item{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:14px;opacity:.9}.flexiblemenubar-allitem{font-size:14px}.grf-ui-flexiblemenubar li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}.grf-ui-flexibelmenu-box{display:-webkit-box;display:-ms-flexbox;display:flex;min-width:150px}.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child{padding:0}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-flexiblemenubar-rootitem{height:70px}.grf-ui-menuitem-active>.grf-ui-menuitem-link{color:#fff}.grf-ui-menuitem-link{padding:.5em .75em;width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-menuitem-link:hover{color:#fff}.grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-flexiblemenubaritem-spans{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:14px;opacity:.9}.grf-ui-flexiblemenubaritem-text{font-size:14px}.grf-ui-flexiblemenubar-state-active{color:#fff;border-radius:.25em}.grf-ui-menuitem-show ul{display:block}.grf-ui-flexiblemenubar-root-list li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}.grf-ui-flexiblemenubar-item:hover{border-radius:4px}.grf-ui-flexiblemenusub-head{width:350px;height:65px;position:absolute;left:0;top:0;display:none}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans{display:none;margin-left:120px;position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans .grf-ui-flexiblemenusub-head-icon{font-size:20px;color:#0275d8}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans .grf-ui-flexiblemenusub-head-text{font-size:20px;color:#0275d8;padding-left:.25em}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem:not(.grf-ui-state-active):hover,.grf-ui-blockmenuitem-state-active,.grf-ui-flexiblemenubar-state-active,.grf-ui-menuitem-active>.grf-ui-menuitem-link,.grf-ui-menuitem-link:hover{background-color:#1890ff}`],
                providers: [DomHandler, FlexibleMenuService]
            },] },
];
/** @nocollapse */
FlexibleMenuComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
    { type: FlexibleMenuService, },
];
FlexibleMenuComponent.propDecorators = {
    "model": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "activeItem": [{ type: Input },],
    "flexFixed": [{ type: Input },],
    "menuParent": [{ type: Input },],
    "_menuParent": [{ type: Input },],
    "list": [{ type: ViewChild, args: ['list',] },],
    "box": [{ type: ViewChild, args: ['box',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FlexibleMenuSubComponent {
    /**
     * @param {?} domHandler
     * @param {?} _flexMenu
     */
    constructor(domHandler, _flexMenu) {
        this.domHandler = domHandler;
        this._flexMenu = _flexMenu;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} menuitem
     * @return {?}
     */
    onItemMouseEnter(event, item, menuitem) {
        if (menuitem.disabled) {
            return;
        }
        this.activeItem = item;
        const /** @type {?} */ nextElement = /** @type {?} */ (item.children[0].nextElementSibling);
        if (nextElement) {
            const /** @type {?} */ sublist = /** @type {?} */ (nextElement.children[0]);
            sublist.style.zIndex = String(++DomHandler.zindex);
            if (this.root) {
                if (document.body.clientWidth > 768) {
                    sublist.style.top = this.domHandler.getOuterHeight(item.children[0]) + 'px';
                    sublist.style.left = '-' + this.list.nativeElement.getBoundingClientRect().left + 'px';
                    sublist.style.width = window.innerWidth + 'px';
                    sublist.style.paddingLeft = '350px';
                    sublist.style.paddingRight = window.innerWidth * 0.18 + 'px';
                    sublist.style.paddingTop = '15px';
                    sublist.style.paddingBottom = '15px';
                }
                else {
                    sublist.style.top = this.domHandler.getOuterHeight(item.children[0]) + 'px';
                    sublist.style.left = '-' + this.list.nativeElement.getBoundingClientRect().left + 'px';
                    sublist.style.width = window.innerWidth + 'px';
                    sublist.style.paddingLeft = '100px';
                    sublist.style.paddingRight = window.innerWidth * 0.18 + 'px';
                    sublist.style.paddingTop = '15px';
                    sublist.style.paddingBottom = '15px';
                }
            }
            else {
                sublist.style.top = '0px';
                sublist.style.left = this.domHandler.getOuterWidth(item.children[0]) + 'px';
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onItemMouseLeave(event) {
        this.activeItem = null;
        this.clickItem = null;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.itemClickEvent.emit(item);
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
        this.activeItem = null;
        this.childClickItem = item;
        this._flexMenu.ifChildClick(false);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    onItemClick(event, item) {
        this.clickItem = item;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    listClick(event) {
        this.activeItem = null;
    }
}
FlexibleMenuSubComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-flexiblemenusub',
                template: `<ul #list [ngClass]="{'grf-ui-flexiblemenubar-root-list grf-ui-helper-clearfix':root, 'grf-ui-widget-content grf-ui-corner-all grf-ui-helper-clearfix grf-ui-menu-child grf-ui-shadow':!root}"
    class="grf-ui-menu-list" (click)="listClick($event)">
    <div [ngClass]="{'grf-ui-flexiblemenusub-head':!root}">
        <div class="grf-ui-flexiblemenusub-head-spans">
            <span *ngIf="!root" class="grf-ui-flexiblemenusub-head-icon fa fa-fw " [ngClass]="menuParent.icon"></span>
            <span *ngIf="!root" class="grf-ui-flexiblemenusub-head-text">{{menuParent.label}}</span>
        </div>
    </div>
    <ng-template ngFor let-child [ngForOf]="(root ? item : item.items)">

        <li *ngIf="child.separator" class="grf-ui-menu-separator grf-ui-widget-content">
            <li *ngIf="!child.separator" #listItem (click)="onItemClick($event,child)" [ngClass]="{'grf-ui-menuitem grf-ui-widget grf-ui-corner-all':true,'grf-ui-menu-parent':child.items,'grf-ui-menuitem-active':root &&(listItem==activeItem||child==clickItem)}"
                (mouseenter)="onItemMouseEnter($event,listItem,child)" (mouseleave)="onItemMouseLeave($event)">
                <a *ngIf="!child.routerLink && !_flexMenu.childClick" [href]="child.url||'#'" [attr.target]="child.target" [attr.title]="child.title"
                    (click)="itemClick($event, child)" [ngClass]="{'grf-ui-menuitem-link grf-ui-corner-all grf-ui-flexiblemenubar-item':true,'grf-ui-state-disabled':child.disabled,
                'grf-ui-flexiblemenubar-rootitem':root,'grf-ui-flexiblemenubar-state-active':child==childClickItem && !root}"
                    [ngStyle]="child.style" [class]="child.styleClass" class="grf-flexiblemenu-link">
                    <div [ngClass]="{'grf-ui-flexiblemenubaritem-spans':root}" class="grf-ui-flexiblemenubaritem-text">
                        <span class="grf-ui-menuitem-icon fa fa-fw" *ngIf="child.icon" [ngClass]="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-submenu-icon fa fa-fw" *ngIf="child.items" [ngClass]="{'fa-angle-down':root}"></span>
                    </div>
                </a>
                <a *ngIf="!child.routerLink && _flexMenu.childClick" [href]="child.url||'#'" [attr.target]="child.target" [attr.title]="child.title"
                    (click)="itemClick($event, child)" [ngClass]="{'grf-ui-menuitem-link grf-ui-corner-all grf-ui-flexiblemenubar-item':true,'grf-ui-state-disabled':child.disabled,
                'grf-ui-flexiblemenubar-rootitem':root}" [ngStyle]="child.style" [class]="child.styleClass" class="grf-flexiblemenu-link">
                    <div [ngClass]="{'grf-ui-flexiblemenubaritem-spans':root}" class="grf-ui-flexiblemenubaritem-text">
                        <span class="grf-ui-menuitem-icon fa fa-fw" *ngIf="child.icon" [ngClass]="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-submenu-icon fa fa-fw" *ngIf="child.items" [ngClass]="{'fa-angle-down':root}"></span>
                    </div>
                </a>
                <a *ngIf="child.routerLink" [routerLink]="child.routerLink" [routerLinkActive]="'grf-ui-flexiblemenubar-state-active'" [routerLinkActiveOptions]="child.routerLinkActiveOptions||{exact:false}"
                    [attr.target]="child.target" [attr.title]="child.title" (click)="itemClick($event, child)" [ngClass]="{'grf-ui-menuitem-link grf-ui-corner-all grf-ui-flexiblemenubar-item':true,'grf-ui-state-disabled':child.disabled,'grf-ui-flexiblemenubar-rootitem':root}"
                    [ngStyle]="child.style" [class]="child.styleClass" class="grf-flexiblemenu-link">
                    <div [ngClass]="{'grf-ui-flexiblemenubaritem-spans':root}" class="grf-ui-flexiblemenubaritem-text">
                        <span class="grf-ui-menuitem-icon fa fa-fw" *ngIf="child.icon" [ngClass]="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-submenu-icon fa fa-fw" *ngIf="child.items" [ngClass]="{'fa-angle-down':root}"></span>
                    </div>
                </a>
                <grf-flexiblemenusub class="grf-ui-submenu" (itemClickEvent)="componentClick($event)" [item]="child" *ngIf="child.items&&root"
                    [menuParent]="menuParent" [childClickItem]="childClickItem"></grf-flexiblemenusub>
            </li>
    </ng-template>
    <li class="grf-ui-menuitem grf-ui-menuitem-custom grf-ui-widget grf-ui-corner-all">
        <ng-content></ng-content>
    </li>

</ul>
`,
                styles: [`@charset "UTF-8";.grf-ui-blockmenu{border:0;margin-top:0;margin-bottom:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav{padding:0;margin:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav a{padding:.5em 1em}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem{margin:0;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;list-style:none;float:left;position:relative;padding:0;white-space:nowrap;display:block;border-bottom:0;height:70px;color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem a{float:left;padding:.5em 1em;text-decoration:none}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-blockmenuitem-state-active{color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link{color:#2d353c;height:70px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon{padding:5px}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon,.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-text{vertical-align:middle;display:inline-block;color:#fff;font-size:14px;opacity:.9}.grf-ui-flexiblemenubaritem{height:70px}.flexiblemenubar-item{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:14px;opacity:.9}.flexiblemenubar-allitem{font-size:14px}.grf-ui-flexiblemenubar li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}.grf-ui-flexibelmenu-box{display:-webkit-box;display:-ms-flexbox;display:flex;min-width:150px}.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child{padding:0}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-flexiblemenubar-rootitem{height:70px}.grf-ui-menuitem-active>.grf-ui-menuitem-link{color:#fff}.grf-ui-menuitem-link{padding:.5em .75em;width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-menuitem-link:hover{color:#fff}.grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-flexiblemenubaritem-spans{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:14px;opacity:.9}.grf-ui-flexiblemenubaritem-text{font-size:14px}.grf-ui-flexiblemenubar-state-active{color:#fff;border-radius:.25em}.grf-ui-menuitem-show ul{display:block}.grf-ui-flexiblemenubar-root-list li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}.grf-ui-flexiblemenubar-item:hover{border-radius:4px}.grf-ui-flexiblemenusub-head{width:350px;height:65px;position:absolute;left:0;top:0;display:none}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans{display:none;margin-left:120px;position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans .grf-ui-flexiblemenusub-head-icon{font-size:20px;color:#0275d8}.grf-ui-flexiblemenusub-head .grf-ui-flexiblemenusub-head-spans .grf-ui-flexiblemenusub-head-text{font-size:20px;color:#0275d8;padding-left:.25em}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem:not(.grf-ui-state-active):hover,.grf-ui-blockmenuitem-state-active,.grf-ui-flexiblemenubar-state-active,.grf-ui-menuitem-active>.grf-ui-menuitem-link,.grf-ui-menuitem-link:hover{background-color:#1890ff}`],
                providers: [DomHandler],
            },] },
];
/** @nocollapse */
FlexibleMenuSubComponent.ctorParameters = () => [
    { type: DomHandler, },
    { type: FlexibleMenuService, },
];
FlexibleMenuSubComponent.propDecorators = {
    "item": [{ type: Input },],
    "root": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "childClickItem": [{ type: Input },],
    "list": [{ type: ViewChild, args: ['list',] },],
    "menuParent": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class FlexibleMenuModule {
}
FlexibleMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, MatRippleModule],
                exports: [FlexibleMenuComponent, FlexibleMenuSubComponent],
                declarations: [FlexibleMenuComponent, FlexibleMenuSubComponent]
            },] },
];
/** @nocollapse */
FlexibleMenuModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class IconMenuComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     */
    constructor(el, domHandler, renderer) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this.itemClickEvent = new EventEmitter();
        this.itemHoverEvent = new EventEmitter();
        this.itemHoverLeaveEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.container = this.el.nativeElement.children[0];
        if (this.popup) {
            if (this.appendTo) {
                if (this.appendTo === 'body') {
                    document.body.appendChild(this.container);
                }
                else {
                    this.domHandler.appendChild(this.container, this.appendTo);
                }
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.container.offsetParent) {
            this.hide();
        }
        else {
            this.show(event);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    show(event) {
        this.preventDocumentDefault = true;
        this.container.style.display = 'block';
        this.domHandler.absolutePosition(this.container, event.currentTarget);
        this.domHandler.fadeIn(this.container, 250);
        this.bindDocumentClickListener();
    }
    /**
     * @return {?}
     */
    hide() {
        this.container.style.display = 'none';
        this.unbindDocumentClickListener();
    }
    /**
     * @return {?}
     */
    unbindDocumentClickListener() {
        if (this.documentClickListener) {
            this.documentClickListener();
            this.documentClickListener = null;
        }
    }
    /**
     * @return {?}
     */
    bindDocumentClickListener() {
        if (!this.documentClickListener) {
            this.documentClickListener = this.renderer.listen('document', 'click', () => {
                if (!this.preventDocumentDefault) {
                    this.hide();
                }
                this.preventDocumentDefault = false;
            });
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentHover(item) {
        this.itemHoverEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentHoverLeave(item) {
        this.itemHoverLeaveEvent.emit(item);
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.popup && this.documentClickListener) {
            this.unbindDocumentClickListener();
            if (this.appendTo) {
                this.el.nativeElement.appendChild(this.container);
            }
        }
    }
}
IconMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-iconmenu',
                template: `<div class="{{theme}}">
    <div [ngClass]="{'grf-ui-iconmenu grf-ui-menu grf-ui-widget  grf-ui-corner-all grf-ui-helper-clearfix':true,'grf-ui-menu-dynamic grf-ui-shadow':popup}" [class]="styleClass" [ngStyle]="style">
        <grf-iconmenusub [item]="model" root="root" (itemClickEvent)="componentClick($event)" (itemHoverEvent)="componentHover($event)" (itemHoverLeaveEvent)="componentHoverLeave($event)"></grf-iconmenusub>
    </div>
</div>
`,
                styles: [`@charset "UTF-8";.grf-ui-iconmenu{width:70px;background-color:none}.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-menu-parent .grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem-active>.grf-ui-menuitem-link,.grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-menu.grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-menuitem-link{width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-childitem-link{padding:.5em .75em}.grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-iconmenurooticon{font-size:20px;position:relative;left:50%;padding:15px 0;-webkit-transform:translateX(-50%);transform:translateX(-50%)}.grf-ui-iconmenufont{font-size:14px;padding:2px 0}.grf-ui-menu-icon li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.default .grf-ui-iconmenurooticon,.default .grf-ui-menuitem-active>.grf-ui-menuitem-link,.default .grf-ui-menuitem-link:hover{color:#0275d8}`],
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
IconMenuComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
];
IconMenuComponent.propDecorators = {
    "model": [{ type: Input },],
    "popup": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "appendTo": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "itemHoverEvent": [{ type: Output },],
    "itemHoverLeaveEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class IconMenuSubComponent {
    /**
     * @param {?} domHandler
     */
    constructor(domHandler) {
        this.domHandler = domHandler;
        this.itemClickEvent = new EventEmitter();
        this.itemHoverEvent = new EventEmitter();
        this.itemHoverLeaveEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} menuitem
     * @return {?}
     */
    onItemMouseEnter(event, item, menuitem) {
        if (menuitem.disabled) {
            return;
        }
        this.activeItem = item;
        const /** @type {?} */ nextElement = /** @type {?} */ (item.children[0].nextElementSibling);
        if (nextElement) {
            const /** @type {?} */ sublist = /** @type {?} */ (nextElement.children[0]);
            sublist.style.zIndex = String(++DomHandler.zindex);
            sublist.style.top = '0px';
            sublist.style.left = this.domHandler.getOuterWidth(item.children[0]) + 'px';
        }
        this.itemHoverEvent.emit(menuitem);
    }
    /**
     * @param {?} menuitem
     * @return {?}
     */
    onItemMouseLeave(menuitem) {
        this.activeItem = null;
        this.itemHoverLeaveEvent.emit(menuitem);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.itemClickEvent.emit(item);
        if (item.disabled) {
            event.preventDefault();
            return true;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentHover(item) {
        this.itemHoverEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentHoverLeave(item) {
        this.itemHoverLeaveEvent.emit(item);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    listClick(event) {
        this.activeItem = null;
    }
}
IconMenuSubComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-iconmenusub',
                template: `<ul [ngClass]="{'grf-ui-helper-reset':root, 'grf-ui-widget-content grf-ui-corner-all grf-ui-helper-clearfix grf-ui-menu-child grf-ui-shadow':!root, 'grf-ui-iconmenurootitem':root}" class="grf-ui-menu-list grf-ui-menu-icon" (click)="listClick($event)">
    <div class="{{theme}}">
        <ng-template ngFor let-child [ngForOf]="(root ? item : item.items)">
            <li *ngIf="child.separator" class="grf-ui-menu-separator grf-ui-widget-content">

                <li *ngIf="!child.separator" #listItem [ngClass]="{'grf-ui-menuitem grf-ui-widget grf-ui-corner-all':true,'grf-ui-menu-parent':child.items,'grf-ui-menuitem-active':listItem==activeItem}" (mouseenter)="onItemMouseEnter($event, listItem, child)" (mouseleave)="onItemMouseLeave(child)">

                    <a *ngIf="!child.routerLink" [href]="child.url||'#'" class="grf-ui-menuitem-link grf-ui-corner-all grf-ui-iconmenu-link" [attr.target]="child.target" [attr.title]="child.title" [ngClass]="{'grf-ui-state-disabled':child.disabled,'grf-ui-iconmenuitem-link':root,'grf-ui-childitem-link':!root}"
                        (click)="itemClick($event, child)">
                        <span class="grf-ui-submenu-icon fa fa-fw fa-caret-right grf-ui-iconmenufont" *ngIf="child.items&&!root"></span>
                        <span class="grf-ui-menuitem-icon fa fa-fw grf-ui-iconmenufont" *ngIf="child.icon&&!root" [ngClass]="child.icon"></span>

                        <grf-tooltip *ngIf="child.icon&&root" [uiTitle]="child.label" [uiPlacement]="'bottom'">
                            <span grfTooltip class="grf-ui-menuitem-icon grf-ui-iconmenurooticon fa fa-fw" [ngClass]="child.icon"></span>
                        </grf-tooltip>

                        <span *ngIf="!root" class="grf-ui-menuitem-text grf-ui-iconmenufont">{{child.label}}</span>
                    </a>
                    <a *ngIf="child.routerLink" [routerLink]="child.routerLink" [routerLinkActive]="'grf-ui-iconmenu-state-active'" [routerLinkActiveOptions]="child.routerLinkActiveOptions||{exact:false}" [href]="child.url||'#'" class="grf-ui-menuitem-link grf-ui-corner-all grf-ui-iconmenu-link"
                        [attr.target]="child.target" [attr.title]="child.title" [ngClass]="{'grf-ui-state-disabled':child.disabled,'grf-ui-iconmenuitem-link':root,'grf-ui-childitem-link':!root}" (click)="itemClick($event, child)">
                        <span class="grf-ui-submenu-icon fa fa-fw fa-caret-right grf-ui-iconmenufont" *ngIf="child.items&&!root"></span>
                        <span class="grf-ui-menuitem-icon fa fa-fw grf-ui-iconmenufont" *ngIf="child.icon&&!root" [ngClass]="child.icon"></span>
                        <grf-tooltip *ngIf="child.icon&&root" [uiTitle]="child.label" [uiPlacement]="'bottom'">
                            <span grfTooltip class="grf-ui-menuitem-icon grf-ui-iconmenurooticon  fa fa-fw" [ngClass]="child.icon"></span>
                        </grf-tooltip>

                        <span *ngIf="!root" class="grf-ui-menuitem-text grf-ui-iconmenufont">{{child.label}}</span>
                    </a>

                    <grf-iconmenusub class="grf-ui-submenu" [item]="child" *ngIf="child.items" (itemClickEvent)="componentClick($event)" (itemHoverEvent)="componentHover($event)" (itemHoverLeaveEvent)="componentHoverLeave($event)"></grf-iconmenusub>
                </li>

        </ng-template>
    </div>
</ul>`,
                styles: [`@charset "UTF-8";.grf-ui-iconmenu{width:70px;background-color:none}.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-menu-parent .grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem-active>.grf-ui-menuitem-link,.grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-menu.grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-menuitem-link{width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-childitem-link{padding:.5em .75em}.grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-iconmenurooticon{font-size:20px;position:relative;left:50%;padding:15px 0;-webkit-transform:translateX(-50%);transform:translateX(-50%)}.grf-ui-iconmenufont{font-size:14px;padding:2px 0}.grf-ui-menu-icon li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.default .grf-ui-iconmenurooticon,.default .grf-ui-menuitem-active>.grf-ui-menuitem-link,.default .grf-ui-menuitem-link:hover{color:#0275d8}`],
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
IconMenuSubComponent.ctorParameters = () => [
    { type: DomHandler, },
];
IconMenuSubComponent.propDecorators = {
    "item": [{ type: Input },],
    "root": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "itemHoverEvent": [{ type: Output },],
    "itemHoverLeaveEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class IconMenuModule {
}
IconMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, TooltipModule],
                exports: [IconMenuComponent],
                declarations: [IconMenuComponent, IconMenuSubComponent]
            },] },
];
/** @nocollapse */
IconMenuModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class VerticalTabMenuComponent {
    /**
     * @param {?} _element
     */
    constructor(_element) {
        this._element = _element;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.itemClickEvent.emit(item);
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
        this.activeItem = item;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    mouseHover(item) {
        this.hoverItem = item;
    }
    /**
     * @return {?}
     */
    _getHostElement() {
        return this._element.nativeElement;
    }
}
VerticalTabMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-verticaltabmenu',
                template: `<div class="{{theme}}">
    <div [ngClass]="'grf-ui-verticaltabmenu grf-ui-widget'" [ngStyle]="style" [class]="styleClass">
        <ul class="grf-ui-verticaltabmenu-nav grf-ui-helper-reset grf-ui-helper-clearfix  grf-ui-verticaltabmenu-nav" role="tablist">
            <li *ngFor="let item of model" [ngClass]="{'grf-ui-verticaltabmenuitem grf-ui-state-default grf-ui-corner-top':true,'grf-ui-state-disabled':item.disabled,
                'grf-ui-verticaltabmenuitem-hasicon':item.icon}" (mouseover)="mouseHover(item)">

                <a *ngIf="!item.routerLink" [href]="item.url||'#'" class="grf-ui-menuitem-link grf-ui-verticaltabmenu" (click)="itemClick($event,item)" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)" [attr.target]="item.target"
                    [attr.title]="item.title">
                    <div class="grf-ui-verticaltabmenu-item">
                        <div>
                            <span class="grf-ui-menuitem-icon menu-text-color fa" [ngClass]="item.icon"></span>
                        </div>
                        <div>
                            <span class="grf-ui-menuitem-text menu-text-color">{{item.label}}</span>
                        </div>
                    </div>
                </a>
                <a *ngIf="item.routerLink" [routerLink]="item.routerLink" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)" [routerLinkActiveOptions]="item.routerLinkActiveOptions||{exact:false}" class="grf-ui-menuitem-link grf-ui-verticaltabmenu"
                    (click)="itemClick($event,item)" [attr.target]="item.target" [attr.title]="item.label" [routerLinkActive]="'grf-ui-verticaltabmenu-link'">
                    <div class="grf-ui-verticaltabmenu-item">
                        <div>
                            <span class="grf-ui-menuitem-icon menu-text-color fa" [ngClass]="item.icon"></span>
                        </div>
                        <div>
                            <span class="grf-ui-menuitem-text menu-text-color">{{item.label}}</span>
                        </div>
                    </div>
                </a>
            </li>
        </ul>
    </div>
</div>
`,
                styles: [`@charset "UTF-8";.grf-ui-verticaltabmenu{border:0;height:100px;position:relative}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenu-nav{padding:0 .25em;margin:0}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem{margin:0 .2em 1px 0;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;list-style:none;float:left;position:relative;padding:0;white-space:nowrap;display:block;border-bottom:0;top:1px}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-icon{float:left}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-state-disabled a{cursor:default}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .menu-text-color{color:#474747;opacity:.8}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon,.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-text{vertical-align:middle;display:inline-block}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-text{font-size:16px}.grf-ui-verticaltabmenu .grf-ui-verticaltabmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon{font-size:32px}.grf-ui-verticaltabmenuitem:not(.grf-ui-state-active):hover{background-color:#e6e6e6;border-radius:0}.grf-ui-verticaltabmenu-item{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);text-align:center}.grf-ui-verticaltabmenuitem a{float:left;padding:0 1em;text-decoration:none}.grf-ui-verticaltabmenu-nav a{padding:.5em 1em}.default .grf-ui-verticaltabmenu-link .grf-ui-verticaltabmenu-item .grf-ui-menuitem-icon,.default .grf-ui-verticaltabmenu-link .grf-ui-verticaltabmenu-item .menu-text-color{color:#0275d8}`],
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
VerticalTabMenuComponent.ctorParameters = () => [
    { type: ElementRef, },
];
VerticalTabMenuComponent.propDecorators = {
    "model": [{ type: Input },],
    "activeItem": [{ type: Input },],
    "popup": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class VerticalTabMenuModule {
}
VerticalTabMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, MatRippleModule, RouterModule],
                exports: [VerticalTabMenuComponent, RouterModule],
                declarations: [VerticalTabMenuComponent]
            },] },
];
/** @nocollapse */
VerticalTabMenuModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SearchService {
    constructor() {
        this.result_changes = new Subject$1();
    }
    /**
     * @param {?} result
     * @return {?}
     */
    searchResult(result) {
        this.search_result = result;
        this.result_changes.next(this.search_result);
    }
    /**
     * @param {?} data
     * @return {?}
     */
    parentSet(data) {
        for (const /** @type {?} */ i in data) {
            if (data) {
                data[i].parents = [];
                data[i].parents.push(data[i]);
                if (data[i].items) {
                    this.parentItemSet(data[i], data[i].parents);
                }
            }
        }
    }
    /**
     * @param {?} data
     * @param {?} parent
     * @return {?}
     */
    parentItemSet(data, parent) {
        for (const /** @type {?} */ i in data) {
            if (i === 'items') {
                const /** @type {?} */ item = data[i];
                for (const /** @type {?} */ item_i in item) {
                    if (item) {
                        const /** @type {?} */ parentArr = this.parentArr(parent, item[item_i]);
                        item[item_i].parents = parentArr;
                        if (item[item_i].items) {
                            this.parentItemSet(item[item_i], item[item_i].parents);
                        }
                    }
                }
            }
        }
    }
    /**
     * @param {?} parent
     * @param {?} item
     * @return {?}
     */
    parentArr(parent, item) {
        const /** @type {?} */ parentArr = [];
        for (let /** @type {?} */ parent_i = 0; parent_i < parent.length; parent_i++) {
            parentArr.push(parent[parent_i]);
        }
        parentArr.push(item);
        return parentArr;
    }
}
SearchService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
SearchService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SearchComponent {
    /**
     * @param {?} _search
     * @param {?} e
     */
    constructor(_search, e) {
        this._search = _search;
        this.e = e;
        this.notFound = true;
        this.inputFocus = false;
        this.itemClickEvent = new EventEmitter();
        this.searchEvent = new EventEmitter();
        this.search_result = [];
        this.theme = 'default';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        const /** @type {?} */ end = this.getArray(this.model, this.search_keyword);
        this.model = end;
    }
    /**
     * @param {?} keyword
     * @return {?}
     */
    keyInput(keyword) {
        this.search(keyword);
    }
    /**
     * @return {?}
     */
    keyInputBlur() {
        this.inputFocus = false;
    }
    /**
     * @param {?} keyword
     * @return {?}
     */
    keyInputFocus(keyword) {
        this.inputFocus = true;
        this.search(keyword);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    handleClick(event, item) {
        // 把搜索后点击的对应的结果暴露出去
        this.itemClickEvent.emit(item);
        console.log(item);
        this.notFound = true;
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
    /**
     * @param {?} data
     * @param {?} name
     * @param {?=} parent
     * @return {?}
     */
    getArray(data, name, parent) {
        for (const /** @type {?} */ i in data) {
            if (data) {
                const /** @type {?} */ patt = new RegExp(name, 'i');
                if (patt.test(data[i].label)) {
                    if (!parent) {
                        // 如果没有父级了，也就是说这个就是根级，那可以直接输出到结果
                        this.search_result.push(data[i]);
                    }
                    else if (parent) {
                        // 如果它有父级, 它的父级会被展开
                        this.search_result.push(data[i]);
                    }
                }
                if (data[i].items) {
                    // 如果包含子级的话
                    if (!parent) {
                        // 根级，parents中push自己进去即可
                        this.getArray(data[i].items, name, data[i]);
                    }
                    else if (parent) {
                        // 非根级，继承父级的parents属性并push自己
                        this.getArray(data[i].items, name, data[i]);
                    }
                }
            }
        }
        return this.search_result;
    }
    /**
     * @param {?} key
     * @return {?}
     */
    search(key) {
        // 执行搜索， 搜索出结果之后把结果放到服务里存起来，并且把点击事件和结果发送暴露出去
        if (key === '' || key === undefined) {
            // 如果关键字是空字符，则为未找到
            this.notFound = true;
            return;
        }
        this.search_result = [];
        const /** @type {?} */ end = this.getArray(this.searchModel, key);
        if (end.length === 0) {
            // 如果搜索结果长度为0 ，则为未找到
            this.notFound = true;
            return;
        }
        this._search.searchResult(end);
        this.model = end;
        this.searchEvent.emit(end);
        this.notFound = false;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    MouseHover(item) {
    }
    /**
     * @return {?}
     */
    onToggleDone() {
        this.animating = false;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
}
SearchComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-search',
                template: `<div class="{{theme}}">
    <div class="grf-ui-search">
        <span class="fa fa-search grf-ui-search-icon" [ngClass]="{'grf-ui-search-icon-active':inputFocus}"></span>
        <input type="text" (keyup)="keyInput(search_keyword)" (blur)="keyInputBlur()" placeholder="请输入关键字进行搜索" (focus)="keyInputFocus(search_keyword)" #searchInput class="form-control" class="grf-ui-search-keyword-input" required [(ngModel)]="search_keyword"
            name="search_keyword" #name="ngModel">
        <div class="grf-ui-search-line">
            <div class="grf-ui-search-lineChild" [ngClass]="{'lineAnimate':inputFocus}" *ngIf="inputFocus"></div>
        </div>
        <div *ngIf="!notFound" [class]="styleClass" [ngStyle]="style" [ngClass]="'grf-ui-searchmenu grf-ui-widget'" class="grf-ui-search-panel">
            <div *ngFor="let item of model;let f=first;let l=last;">
                <div [ngClass]="{'grf-ui-widget grf-ui-searchmenu-header grf-ui-search-header grf-ui-state-default':true,'grf-ui-corner-top':f,
                'grf-ui-corner-bottom':l&&!item.expanded,'grf-ui-state-disabled':item.disabled}" [class]="item.styleClass" [ngStyle]="item.style">
                    <a *ngIf="!item.routerLink" [href]="item.url||'#'" [ngClass]="{'grf-ui-searchmenu-headerlink-hasicon':item.icon}" (click)="handleClick($event,item)" [attr.target]="item.target" [attr.title]="item.title">
                        <span class="grf-ui-search-expand-icon fa" [ngClass]="{'fa-file-text-o':!item.items,'fa-folder-open-o':item.items}"></span>
                        <span class="grf-ui-menuitem-text" [innerHTML]="item.label|keyword:search_keyword">{{item.label}}</span>
                    </a>
                    <a *ngIf="item.routerLink" [routerLink]="item.routerLink" [ngClass]="{'grf-ui-searchmenu-headerlink-hasicon':item.icon}" (click)="handleClick($event,item)" [attr.target]="item.target" [attr.title]="item.title">
                        <span class="grf-ui-search-expand-icon fa" [ngClass]="{'fa-file-text-o':!item.items,'fa-folder-open-o':item.items}"></span>
                        <span class="grf-ui-menuitem-text" [innerHTML]="item.label|keyword:search_keyword">{{item.label}}</span>
                    </a>
                </div>
                <div *ngIf="item.items" class="grf-ui-searchmenu-content-wrapper" [@rootItem]="item.expanded ? 'visible' : 'hidden'" (@rootItem.done)="onToggleDone($event)" [ngClass]="{'grf-ui-searchmenu-content-wrapper-overflown': !item.expanded||animating}">
                    <div class="grf-ui-searchmenu-content grf-ui-widget-content grf-ui-accordionmenu-content" style="border:none">
                    </div>
                </div>
            </div>
        </div>
    </div>
</div>`,
                styles: [`@charset "UTF-8";.grf-ui-searchmenu{width:auto}.grf-ui-searchmenu .grf-ui-searchmenu-panel{padding:0;margin:0}.grf-ui-searchmenu .grf-ui-searchmenu-header{cursor:pointer;position:relative;margin:-1px 0 0;zoom:1}.grf-ui-searchmenu .grf-ui-searchmenu-header a{display:block;text-decoration:none;padding:.5em .75em;color:#2d353c}.grf-ui-searchmenu .fa{width:1em;text-align:center;vertical-align:middle;margin-right:.25em}.grf-ui-searchmenu .grf-ui-menuitem-text{margin-left:.125em}.grf-ui-searchmenu-icon{float:right}.grf-ui-searchmenu span{vertical-align:middle}.grf-ui-searchmenu .grf-ui-searchmenu-content{padding:.2em 0;border-top:0;overflow:auto;zoom:1;outline:0;margin-bottom:1px}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menu-parent{overflow:hidden}.grf-ui-searchmenu .grf-ui-searchmenu-content-wrapper{-webkit-box-sizing:border-box;box-sizing:border-box}.grf-ui-searchmenu .grf-ui-searchmenu-content-wrapper-overflown{overflow:hidden}.grf-ui-searchmenu .grf-ui-searchmenu-header.grf-ui-state-disabled,.grf-ui-searchmenu .grf-ui-searchmenu-header.grf-ui-state-disabled a{cursor:default}.grf-ui-searchmenu .grf-ui-menu-list{position:static}.grf-ui-searchmenu .grf-ui-menuitem{margin:1px 0;padding:0}.grf-ui-searchmenu .grf-ui-menu-separator{width:95%;margin:0 auto}.grf-ui-searchmenu .grf-ui-menuitem-link{display:block;text-decoration:none;font-weight:400;margin-top:1px;margin-bottom:1px;line-height:1em;cursor:pointer;position:relative;padding:.25em .5em}.grf-ui-searchmenu .grf-ui-menu-parent .grf-ui-menu-list{margin-left:1.5em}.grf-ui-search-panel{-webkit-box-shadow:1px 1px 6px 2px rgba(0,0,0,.2);box-shadow:1px 1px 6px 2px rgba(0,0,0,.2);margin-left:15px;margin-right:15px;margin-top:2px}.grf-ui-search{width:100%}.grf-ui-searchmenu .grf-ui-search-header:not(.grf-ui-state-active):hover{background-color:#e6e6e6;border-right:none}.grf-ui-searchmenu .fa{position:static}.grf-ui-searchmenu .grf-ui-searchmenu-header{background-color:#fff;padding:0;font-size:14px;color:#2d353c;-webkit-transition:background-color .3s;transition:background-color .3s}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link{padding:.375em .75em;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link span.fa{display:inline-block;vertical-align:middle;left:auto}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link .grf-ui-menuitem-text{display:inline-block;vertical-align:middle;font-size:14px}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-icon{font-size:14px}.grf-ui-search-expand-icon{display:inline-block;vertical-align:middle}.grf-ui-search-icon{padding-left:10px;font-size:14px;color:#525252}.grf-ui-search-keyword-input{border:none;width:calc(100% - 30px);height:30px;font-size:14px;text-indent:5px;color:#525252}.grf-ui-search-line{width:100%;height:1px;background-color:#525252}.grf-ui-search-line .grf-ui-search-lineChild{height:200%;width:100%}.grf-ui-search-keyword-input:focus{outline:0}.lineAnimate{-webkit-animation:.5s borderAnimate;animation:.5s borderAnimate}.default .grf-ui-search-icon-active,.default .grf-ui-searchmenu .grf-ui-search-header:not(.grf-ui-state-active):hover{color:#0275d8}.default .grf-ui-search-line .grf-ui-search-lineChild{background-color:#0275d8}.default .lineAnimate{-webkit-animation:.5s borderAnimate;animation:.5s borderAnimate}@-webkit-keyframes borderAnimate{0%{width:0;background-color:#0275d8}100%{width:100%;background-color:#0275d8}}@keyframes borderAnimate{0%{width:0;background-color:#0275d8}100%{width:100%;background-color:#0275d8}}`],
                animations: [
                    trigger('rootItem', [
                        state('hidden', style({
                            height: '0px'
                        })),
                        state('visible', style({
                            height: '*'
                        })),
                        transition('visible => hidden', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
                        transition('hidden => visible', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
SearchComponent.ctorParameters = () => [
    { type: SearchService, },
    { type: ElementRef, },
];
SearchComponent.propDecorators = {
    "searchModel": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "searchEvent": [{ type: Output },],
    "searchInput": [{ type: ViewChild, args: ['searchInput',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class KeywordPipe {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @param {?} val
     * @param {?} keyword
     * @return {?}
     */
    transform(val, keyword) {
        const /** @type {?} */ Reg = new RegExp(keyword, 'i');
        if (val) {
            const /** @type {?} */ res = val.replace(Reg, `<span style="background-color: #f8ce5b;">${keyword}</span>`);
            return this.sanitizer.bypassSecurityTrustHtml(res);
        }
    }
}
KeywordPipe.decorators = [
    { type: Pipe, args: [{
                name: 'keyword'
            },] },
    { type: Injectable },
];
/** @nocollapse */
KeywordPipe.ctorParameters = () => [
    { type: DomSanitizer, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SearchSubComponent {
    constructor() {
        this.itemClickEvent = new EventEmitter();
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} haschlid
     * @return {?}
     */
    chlidClick(event, item, haschlid) {
        this.itemClickEvent.emit(item);
        this.activeItem = null;
        
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    handleClick(event, item) {
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        item.expanded = !item.expanded;
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
}
SearchSubComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-searchsub',
                template: `<ul class="grf-ui-helper-reset grf-ui-accordion-menu-list" [@submenu]="expanded ? 'visible' : 'hidden'">
    <ng-template ngFor let-child [ngForOf]="item.items">
        <li *ngIf="child.separator" class="grf-ui-menu-separator grf-ui-widget-content grf-ui-accordion-menuitem">
            <li *ngIf="!child.separator" class="grf-ui-corner-all grf-ui-accordion-menuitem" [ngClass]="{'grf-ui-menu-parent':child.items}" [class]="child.styleClass" [ngStyle]="child.style">
                <a *ngIf="!child.routerLink" [href]="child.url||'#'" class="grf-ui-menuitem-link grf-ui-corner-all grf-ui-accordionmenuitem-link" [attr.tabindex]="item.expanded ? null : '-1'" [ngClass]="{'grf-ui-menuitem-link-hasicon':child.icon&&child.items,'grf-ui-state-disabled':child.disabled, 'grf-ui-state-active':child==activeItem}"
                    (click)="handleClick($event,child); chlidClick($event,child,child.items)" [attr.target]="child.target" [attr.title]="child.title">
                    <span class="grf-ui-menuitem-icon fa fa-fw" [ngClass]="child.icon" *ngIf="child.icon"></span
                ><span class="grf-ui-menuitem-text">{{child.label}}</span>
                    <span class="grf-ui-searchmenu-icon fa fa-fw" [ngClass]="{'fa-angle-down':!child.expanded,'fa-angle-up':child.expanded}" *ngIf="child.items"></span
                >
            </a>
            <a *ngIf="child.routerLink" [routerLink]="child.routerLink"  [routerLinkActiveOptions]="child.routerLinkActiveOptions||{exact:false}"
                [ngClass]="{'grf-ui-menuitem-link-hasicon':child.icon&&child.items,'grf-ui-state-disabled':child.disabled}" [attr.tabindex]="item.expanded ? null : '-1'"
                (click)="handleClick($event,child); chlidClick($event,child,child.items)" class="grf-ui-menuitem-link grf-ui-corner-all"
                 [routerLinkActive]="'grf-ui-state-accordionmenu-active'" [attr.target]="child.target" [attr.title]="child.title">
               <span class="grf-ui-menuitem-icon fa fa-fw" [ngClass]="child.icon" *ngIf="child.icon"></span
                ><span class="grf-ui-menuitem-text">{{child.label}}</span>
                    <span class="grf-ui-searchmenu-icon fa fa-fw" [ngClass]="{'fa-angle-down':!child.expanded,'fa-angle-up':child.expanded}" *ngIf="child.items"></span
                >
            </a>
            <grf-searchsub [item]="child" [expanded]="child.expanded" *ngIf="child.items" (itemClickEvent)="componentClick($event)" ></grf-searchsub>
        </li>
    </ng-template>
</ul>`,
                styles: [`@charset "UTF-8";.grf-ui-searchmenu{width:auto}.grf-ui-searchmenu .grf-ui-searchmenu-panel{padding:0;margin:0}.grf-ui-searchmenu .grf-ui-searchmenu-header{cursor:pointer;position:relative;margin:-1px 0 0;zoom:1}.grf-ui-searchmenu .grf-ui-searchmenu-header a{display:block;text-decoration:none;padding:.5em .75em;color:#2d353c}.grf-ui-searchmenu .fa{width:1em;text-align:center;vertical-align:middle;margin-right:.25em}.grf-ui-searchmenu .grf-ui-menuitem-text{margin-left:.125em}.grf-ui-searchmenu-icon{float:right}.grf-ui-searchmenu span{vertical-align:middle}.grf-ui-searchmenu .grf-ui-searchmenu-content{padding:.2em 0;border-top:0;overflow:auto;zoom:1;outline:0;margin-bottom:1px}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menu-parent{overflow:hidden}.grf-ui-searchmenu .grf-ui-searchmenu-content-wrapper{-webkit-box-sizing:border-box;box-sizing:border-box}.grf-ui-searchmenu .grf-ui-searchmenu-content-wrapper-overflown{overflow:hidden}.grf-ui-searchmenu .grf-ui-searchmenu-header.grf-ui-state-disabled,.grf-ui-searchmenu .grf-ui-searchmenu-header.grf-ui-state-disabled a{cursor:default}.grf-ui-searchmenu .grf-ui-menu-list{position:static}.grf-ui-searchmenu .grf-ui-menuitem{margin:1px 0;padding:0}.grf-ui-searchmenu .grf-ui-menu-separator{width:95%;margin:0 auto}.grf-ui-searchmenu .grf-ui-menuitem-link{display:block;text-decoration:none;font-weight:400;margin-top:1px;margin-bottom:1px;line-height:1em;cursor:pointer;position:relative;padding:.25em .5em}.grf-ui-searchmenu .grf-ui-menu-parent .grf-ui-menu-list{margin-left:1.5em}.grf-ui-search-panel{-webkit-box-shadow:1px 1px 6px 2px rgba(0,0,0,.2);box-shadow:1px 1px 6px 2px rgba(0,0,0,.2);margin-left:15px;margin-right:15px;margin-top:2px}.grf-ui-search{width:100%}.grf-ui-searchmenu .grf-ui-search-header:not(.grf-ui-state-active):hover{background-color:#e6e6e6;border-right:none}.grf-ui-searchmenu .fa{position:static}.grf-ui-searchmenu .grf-ui-searchmenu-header{background-color:#fff;padding:0;font-size:14px;color:#2d353c;-webkit-transition:background-color .3s;transition:background-color .3s}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link{padding:.375em .75em;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link span.fa{display:inline-block;vertical-align:middle;left:auto}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-link .grf-ui-menuitem-text{display:inline-block;vertical-align:middle;font-size:14px}.grf-ui-searchmenu .grf-ui-searchmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-icon{font-size:14px}.grf-ui-search-expand-icon{display:inline-block;vertical-align:middle}.grf-ui-search-icon{padding-left:10px;font-size:14px;color:#525252}.grf-ui-search-keyword-input{border:none;width:calc(100% - 30px);height:30px;font-size:14px;text-indent:5px;color:#525252}.grf-ui-search-line{width:100%;height:1px;background-color:#525252}.grf-ui-search-line .grf-ui-search-lineChild{height:200%;width:100%}.grf-ui-search-keyword-input:focus{outline:0}.lineAnimate{-webkit-animation:.5s borderAnimate;animation:.5s borderAnimate}.default .grf-ui-search-icon-active,.default .grf-ui-searchmenu .grf-ui-search-header:not(.grf-ui-state-active):hover{color:#0275d8}.default .grf-ui-search-line .grf-ui-search-lineChild{background-color:#0275d8}.default .lineAnimate{-webkit-animation:.5s borderAnimate;animation:.5s borderAnimate}@-webkit-keyframes borderAnimate{0%{width:0;background-color:#0275d8}100%{width:100%;background-color:#0275d8}}@keyframes borderAnimate{0%{width:0;background-color:#0275d8}100%{width:100%;background-color:#0275d8}}`],
                animations: [
                    trigger('submenu', [
                        state('hidden', style({
                            height: '0px'
                        })),
                        state('visible', style({
                            height: '*'
                        })),
                        transition('visible => hidden', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
                        transition('hidden => visible', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
SearchSubComponent.ctorParameters = () => [];
SearchSubComponent.propDecorators = {
    "item": [{ type: Input },],
    "expanded": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class SearchModule {
}
SearchModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, FormsModule],
                exports: [SearchComponent, KeywordPipe],
                declarations: [SearchComponent, SearchSubComponent, KeywordPipe]
            },] },
];
/** @nocollapse */
SearchModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DropdownComponent {
    /**
     * @param {?} renderer2
     */
    constructor(renderer2) {
        this.renderer2 = renderer2;
        this.direction = 'bottom-right';
        this.caret = true;
        this.itemClickEvent = new EventEmitter();
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    clickDisabled(event, item) {
        if (!item.url) {
            event.preventDefault();
            return false;
        }
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.addClass(this.dropdownMenu.nativeElement, `grf-ui-dropdown-${this.direction}`);
    }
    /**
     * @param {?} elem
     * @param {?} className
     * @return {?}
     */
    addClass(elem, className) {
        const /** @type {?} */ classes = className.split(/\s+/);
        for (const /** @type {?} */ cName of classes) {
            this.renderer2.addClass(elem, cName);
        }
    }
    /**
     * @param {?} menu
     * @param {?=} event
     * @return {?}
     */
    open(menu, event) {
        this.clickitem = menu;
        if (!this.hover) {
            this.selfClick = true;
            if (!this.isOpen) {
                this.isOpen = true;
            }
            else {
                this.close();
                this.isOpen = false;
            }
            this.onDocumentClickListener();
        }
    }
    /**
     * @return {?}
     */
    onMouseover() {
        if (this.hover) {
            this.isOpen = true;
        }
    }
    /**
     * @return {?}
     */
    onMouseout() {
        if (this.hover) {
            this.isOpen = false;
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    onItemClick(item) {
        this.itemClickEvent.emit(item);
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
        if (this.hover) {
            this.isOpen = false;
        }
        else {
            this.close();
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    onParentClick(item) {
        this.itemClickEvent.emit(item);
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
    /**
     * @return {?}
     */
    onDocumentClickListener() {
        if (!this.documentClickListener) {
            this.documentClickListener = this.renderer2.listen('body', 'click', () => {
                if (!this.selfClick && !this.itemClick) {
                    this.close();
                }
                this.selfClick = false;
                this.itemClick = false;
            });
        }
    }
    /**
     * @return {?}
     */
    close() {
        this.isOpen = false;
        this.offDocumentClickListener();
    }
    /**
     * @return {?}
     */
    offDocumentClickListener() {
        if (this.documentClickListener) {
            this.documentClickListener();
            this.documentClickListener = null;
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.offDocumentClickListener();
    }
}
DropdownComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-dropdown',
                template: `<div class="grf-dropdown">

    <a *ngFor="let menu of menus" class="grf-ui-dropdown" #container (mouseover)="onMouseover()" (mouseout)="onMouseout()" (click)="onParentClick(menu)">
        <grf-tooltip [uiTitle]="menu.label" [uiPlacement]="'bottom'">
            <a grfTooltip *ngIf="menu.routerLink" [routerLink]="menu.routerLink" class="grf-ui-dropdown-header" [class.active]="isOpen" (click)="open(menu,$event)">
                <span class="grf-ui-dropdownmenu-icon fa fa-fw" [ngClass]="menu.icon" *ngIf="menu.icon"></span>
            </a>
            <a grfTooltip *ngIf="menu.url" [href]="menu.url" class="grf-ui-dropdown-header" [class.active]="isOpen" (click)="open(menu,$event)">
                <span class="grf-ui-dropdownmenu-icon fa fa-fw" [ngClass]="menu.icon" *ngIf="menu.icon"></span>
            </a>
            <a grfTooltip *ngIf="!menu.url&&!menu.routerLink" class="grf-ui-dropdown-header" [class.active]="isOpen" (click)="open(menu,$event)">
                <span class="grf-ui-dropdownmenu-icon fa fa-fw" [ngClass]="menu.icon" *ngIf="menu.icon"></span>
            </a>
        </grf-tooltip>

        <div #dropdownMenu class="grf-ui-dropdown-menu grf-ui-dropdown-bottom-right" [@dropdownState]="isOpen&&clickitem==menu ? 'active' : 'inactive'">
            <ul *ngIf="menu.items">
                <li *ngFor="let menuitem of menu.items" (click)="onItemClick(menuitem)">
                    <a *ngIf="menuitem.routerLink" [routerLink]="menuitem.routerLink" [attr.target]="menuitem.target">
                        <i *ngIf="menuitem.icon" class="fa {{ menuitem.icon}}"></i> {{menuitem.label}}
                    </a>
                    <a *ngIf="!menuitem.routerLink" [href]="menuitem.url || '#'" (click)="clickDisabled($event, menuitem)" [attr.target]="menuitem.target">
                        <i *ngIf="menuitem.icon" class="fa {{ menuitem.icon}}"></i> {{menuitem.label}}
                    </a>
                </li>
            </ul>
            <div *ngIf="!menu" class="grf-ui-dropdown-wrapper" (click)="onItemClick()">
                <ng-content></ng-content>
            </div>
        </div>

    </a>

</div>`,
                styles: [`.grf-ui-dropdown{display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex;padding:10px}.grf-dropdown{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row}.grf-dropdown-menu{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.grf-ui-dropdownmenu-icon{color:#fff;font-size:16px}.grf-ui-dropdown .grf-ui-dropdown-menu ul{padding-left:0;min-width:180px}.grf-ui-dropdown .grf-ui-dropdown-menu ul li{position:relative;-webkit-transition:all .3s;transition:all .3s}.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-left{-webkit-transform-origin:left top 10px;transform-origin:left top 10px}.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-right{-webkit-transform-origin:right top 10px;transform-origin:right top 10px}.grf-ui-dropdown .grf-ui-dropdown-header,.grf-ui-dropdown .grf-ui-dropdown-header f-header{display:-webkit-box;display:-ms-flexbox;display:flex;display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.grf-ui-dropdown{position:relative;display:-webkit-inline-box;display:-ms-inline-flexbox;display:inline-flex}.grf-ui-dropdown .btn{margin:0;width:100%}.grf-ui-dropdown .grf-ui-dropdown-wrapper{padding:.5rem;width:100%}.grf-ui-dropdown .d-caret{padding-left:.2rem}.grf-ui-dropdown .grf-ui-dropdown-menu{position:absolute;opacity:0;min-width:7rem;background:#fff;border-radius:2px;-webkit-transform:scale(0);transform:scale(0);-webkit-box-shadow:0 1px 5px rgba(0,0,0,.2),0 2px 2px rgba(0,0,0,.14),0 3px 1px -2px rgba(0,0,0,.12);box-shadow:0 1px 5px rgba(0,0,0,.2),0 2px 2px rgba(0,0,0,.14),0 3px 1px -2px rgba(0,0,0,.12);z-index:999;-webkit-transition:all .25s cubic-bezier(.23,1,.32,1);transition:all .25s cubic-bezier(.23,1,.32,1)}.grf-ui-dropdown .grf-ui-dropdown-menu ul li{list-style:none;padding:.4rem 2rem .4rem 1rem;cursor:pointer;white-space:nowrap;-webkit-transition:all .3s;transition:all .3s;line-height:20px}.grf-ui-dropdown .grf-ui-dropdown-menu ul li i{padding-right:.5rem}.grf-ui-dropdown .grf-ui-dropdown-menu ul li:hover{background:#e6e6e6}.grf-ui-dropdown .grf-ui-dropdown-menu ul li a{color:#525252;font-size:14px;text-decoration:none}.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-left,.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-right{top:100%}.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-left{left:0;-webkit-transform-origin:left top 0;transform-origin:left top 0}.grf-ui-dropdown .grf-ui-dropdown-menu.grf-ui-dropdown-bottom-right{right:0;-webkit-transform-origin:right top 0;transform-origin:right top 0}.grf-ui-dropdown .grf-ui-dropdown-header{cursor:pointer}.grf-ui-dropdown .grf-ui-dropdown-header header{font-weight:700;font-size:14px}.grf-ui-dropdown .grf-ui-dropdown-header header img{width:1.8rem;height:1.8rem;border-radius:50%;margin-right:.2rem}`],
                animations: [
                    trigger('dropdownState', [
                        state('active', style({
                            transform: 'scale(1)',
                            opacity: 1
                        })),
                        state('inactive', style({
                            transform: 'scale(0)',
                            opacity: 0
                        })),
                        transition('inactive <=> active', animate('300ms ease'))
                    ])
                ]
            },] },
];
/** @nocollapse */
DropdownComponent.ctorParameters = () => [
    { type: Renderer2, },
];
DropdownComponent.propDecorators = {
    "menus": [{ type: Input },],
    "header": [{ type: Input },],
    "direction": [{ type: Input },],
    "theme": [{ type: Input },],
    "caret": [{ type: Input },],
    "hover": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "container": [{ type: ViewChild, args: ['container',] },],
    "dropdownMenu": [{ type: ViewChild, args: ['dropdownMenu',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class DropdownModule {
}
DropdownModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, TooltipModule],
                declarations: [DropdownComponent],
                exports: [DropdownComponent]
            },] },
];
/** @nocollapse */
DropdownModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BlockMenuComponent {
    /**
     * @param {?} _element
     */
    constructor(_element) {
        this._element = _element;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.activeItem && this.model && this.model.length) {
            this.activeItem = this.model[0];
        }
    }
    /**
     * @return {?}
     */
    fun() {
        console.log('你的变量');
    }
    /**
     * @param {?} item
     * @return {?}
     */
    mouseHover(item) {
        this.hoverItem = item;
    }
    /**
     * @return {?}
     */
    _getHostElement() {
        return this._element.nativeElement;
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.itemClickEvent.emit(item);
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
        this.activeItem = item;
    }
}
BlockMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-blockmenu',
                template: `<div class="{{theme}}">
    <div [ngClass]="'grf-ui-corner-all grf-ui-blockmenu'" [ngStyle]="style" [class]="styleClass">
        <ul class="grf-ui-helper-reset grf-ui-helper-clearfix grf-ui-corner-all grf-ui-blockmenu-nav" role="tablist">
            <li *ngFor="let item of model" [ngClass]="{'grf-ui-blockmenuitem grf-ui-state-default grf-ui-corner-top':true,'grf-ui-state-disabled':item.disabled,
                'grf-ui-blockmenuitem-hasicon':item.icon ,'grf-ui-blockmenuitem-state-active':activeItem==item}">
                <a *ngIf="!item.routerLink" [href]="item.url||'#'" class="grf-ui-menuitem-link grf-ui-corner-all" (click)="itemClick($event,item)" (mouseover)="mouseHover(item)" style="position:relative" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)"
                    [attr.target]="item.target" [attr.title]="item.title">
                    <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon"></span>
                    <span class="grf-ui-menuitem-text">{{item.label}}</span>
                </a>
                <a *ngIf="item.routerLink" [routerLink]="item.routerLink" [routerLinkActive]="'grf-ui-blockmenuitem-state-active'" (mouseover)="mouseHover(item)" style="position:relative" mat-ripple [matRippleTrigger]="_getHostElement()" [matRippleDisabled]="!(hoverItem==item)"
                    [routerLinkActiveOptions]="item.routerLinkActiveOptions||{exact:false}" class="grf-ui-menuitem-link grf-ui-corner-all" (click)="itemClick($event,item)" [attr.target]="item.target" [attr.title]="item.title">
                    <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon"></span>
                    <span class="grf-ui-menuitem-text">{{item.label}}</span>
                </a>
            </li>
        </ul>
    </div>
</div>`,
                styles: [`@charset "UTF-8";.grf-ui-blockmenu{border:0;margin-top:0;margin-bottom:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav{padding:0;margin:0;border-radius:0}.grf-ui-blockmenu .grf-ui-blockmenu-nav a{padding:.5em 1em}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem{margin:0;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;list-style:none;float:left;position:relative;padding:0;white-space:nowrap;display:block;border-bottom:0;line-height:56px;color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem a{float:left;padding:.5em 1em;text-decoration:none}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-blockmenuitem-state-active{color:#fff}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link{color:#2d353c;padding:.375em 1em}.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-icon,.grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem .grf-ui-menuitem-link .grf-ui-menuitem-text{vertical-align:middle;display:inline-block;color:#fff;font-size:.9em;opacity:.9}.grf-ui-blockmenubaritem{height:70px}.blockmenubar-item{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:.9em;opacity:.9}.blockmenubar-allitem{font-size:.9em}.grf-ui-blockmenubar li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.default .grf-ui-blockmenu .grf-ui-blockmenu-nav .grf-ui-blockmenuitem:not(.grf-ui-state-active):hover,.default .grf-ui-blockmenuitem-state-active{background-color:#0275d8}`],
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
BlockMenuComponent.ctorParameters = () => [
    { type: ElementRef, },
];
BlockMenuComponent.propDecorators = {
    "model": [{ type: Input },],
    "activeItem": [{ type: Input },],
    "popup": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BlockMenuModule {
}
BlockMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, MatRippleModule],
                exports: [BlockMenuComponent],
                declarations: [BlockMenuComponent]
            },] },
];
/** @nocollapse */
BlockMenuModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BlockMenuBarComponent {
    /**
     * @param {?} el
     * @param {?} domHandler
     * @param {?} renderer
     */
    constructor(el, domHandler, renderer) {
        this.el = el;
        this.domHandler = domHandler;
        this.renderer = renderer;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
}
BlockMenuBarComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-blockmenubar',
                template: `<div class="{{theme}}">
    <div [ngClass]="{'grf-ui-menu grf-ui-corner-all grf-ui-blockmenubar grf-ui-helper-clearfix':true}" [class]="styleClass" [ngStyle]="style">
        <grf-blockmenubarsub [item]="model" root="root" (itemClickEvent)="componentClick($event)">
            <ng-content></ng-content>
        </grf-blockmenubarsub>
    </div>
</div>`,
                styles: [`@charset "UTF-8";.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child{padding:0}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-blockmenubar-rootitem{height:70px}.grf-ui-menuitem-active>.grf-ui-menuitem-link{color:#fff}.grf-ui-menuitem-link{padding:.5em .75em;width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-menuitem-link:hover{color:#fff}.grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-blockmenubaritem-spans{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:.9em;opacity:.9}.grf-ui-blockmenubaritem-text{font-size:.9em}.grf-ui-blockmenubar-state-active{color:#fff}.grf-ui-menuitem-show ul{display:block}.grf-ui-blockmenubar-root-list li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.default .grf-ui-blockmenubar-state-active,.default .grf-ui-menuitem-active>.grf-ui-menuitem-link,.default .grf-ui-menuitem-link:hover{background-color:#0275d8}`],
                providers: [DomHandler]
            },] },
];
/** @nocollapse */
BlockMenuBarComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: DomHandler, },
    { type: Renderer2, },
];
BlockMenuBarComponent.propDecorators = {
    "model": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BlockMenuBarSubComponent {
    /**
     * @param {?} domHandler
     */
    constructor(domHandler) {
        this.domHandler = domHandler;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} menuitem
     * @return {?}
     */
    onItemMouseEnter(event, item, menuitem) {
        if (menuitem.disabled) {
            return;
        }
        this.activeItem = item;
        const /** @type {?} */ nextElement = /** @type {?} */ (item.children[0].nextElementSibling);
        if (nextElement) {
            const /** @type {?} */ sublist = /** @type {?} */ (nextElement.children[0]);
            sublist.style.zIndex = String(++DomHandler.zindex);
            if (this.root) {
                sublist.style.top = this.domHandler.getOuterHeight(item.children[0]) + 'px';
                sublist.style.left = '0px';
            }
            else {
                sublist.style.top = '0px';
                sublist.style.left = this.domHandler.getOuterWidth(item.children[0]) + 'px';
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onItemMouseLeave(event) {
        this.activeItem = null;
        this.clickItem = null;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    itemClick(event, item) {
        this.itemClickEvent.emit(item);
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
        this.activeItem = null;
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    onItemClick(event, item) {
        this.clickItem = item;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    listClick(event) {
        this.activeItem = null;
    }
}
BlockMenuBarSubComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-blockmenubarsub',
                template: `<ul [ngClass]="{'grf-ui-blockmenubar-root-list grf-ui-helper-clearfix':root, 'grf-ui-widget-content grf-ui-corner-all grf-ui-helper-clearfix grf-ui-menu-child grf-ui-shadow':!root}" class="grf-ui-menu-list {{theme}}" (click)="listClick($event)">
    <ng-template ngFor let-child [ngForOf]="(root ? item : item.items)">
        <li *ngIf="child.separator" class="grf-ui-menu-separator grf-ui-widget-content">
            <li *ngIf="!child.separator" #listItem (click)="onItemClick($event,child)" [ngClass]="{'grf-ui-menuitem grf-ui-widget grf-ui-corner-all':true,'grf-ui-menu-parent':child.items,'grf-ui-menuitem-active':listItem==activeItem||child==clickItem}" (mouseenter)="onItemMouseEnter($event,listItem,child)"
                (mouseleave)="onItemMouseLeave($event)">
                <a *ngIf="!child.routerLink" [href]="child.url||'#'" [attr.target]="child.target" [attr.title]="child.title" (click)="itemClick($event, child)" [ngClass]="{'grf-ui-menuitem-link grf-ui-corner-all':true,'grf-ui-state-disabled':child.disabled,'grf-ui-blockmenubar-rootitem':root}"
                    [ngStyle]="child.style" [class]="child.styleClass" class="grf-blockmenubar-link">
                    <div [ngClass]="{'grf-ui-blockmenubaritem-spans':root}" class="grf-ui-blockmenubaritem-text">
                        <span class="grf-ui-menuitem-icon fa fa-fw" *ngIf="child.icon" [ngClass]="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-submenu-icon fa fa-fw" *ngIf="child.items" [ngClass]="{'fa-angle-down':root,'fa-angle-right':!root}"></span>
                    </div>
                </a>
                <a *ngIf="child.routerLink" [routerLink]="child.routerLink" [routerLinkActive]="'grf-ui-blockmenubar-state-active'" [routerLinkActiveOptions]="child.routerLinkActiveOptions||{exact:false}" [attr.target]="child.target" [attr.title]="child.title" (click)="itemClick($event, child)"
                    [ngClass]="{'grf-ui-menuitem-link grf-ui-corner-all':true,'grf-ui-state-disabled':child.disabled,'grf-ui-blockmenubar-rootitem':root}" [ngStyle]="child.style" [class]="child.styleClass" class="grf-blockmenubar-link">
                    <div [ngClass]="{'grf-ui-blockmenubaritem-spans':root}" class="grf-ui-blockmenubaritem-text">
                        <span class="grf-ui-menuitem-icon fa fa-fw" *ngIf="child.icon" [ngClass]="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-submenu-icon fa fa-fw" *ngIf="child.items" [ngClass]="{'fa-angle-down':root,'fa-angle-right':!root}"></span>
                    </div>
                </a>
                <grf-blockmenubarsub class="grf-ui-submenu" (itemClickEvent)="componentClick($event)" [item]="child" *ngIf="child.items"></grf-blockmenubarsub>
            </li>
    </ng-template>
    <li class="grf-ui-menuitem grf-ui-menuitem-custom grf-ui-widget grf-ui-corner-all">
        <ng-content></ng-content>
    </li>

</ul>`,
                styles: [`@charset "UTF-8";.grf-ui-menu{padding:0}.grf-ui-menu .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child{padding:0}.grf-ui-menu .grf-ui-menu-list .grf-ui-menu-child .grf-ui-shadow{-webkit-box-shadow:0 1px 2px 0 rgba(0,0,0,.3);box-shadow:0 1px 2px 0 rgba(0,0,0,.3)}.grf-ui-blockmenubar-rootitem{height:70px}.grf-ui-menuitem-active>.grf-ui-menuitem-link{color:#fff}.grf-ui-menuitem-link{padding:.5em .75em;width:100%;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0;color:#525252}.grf-ui-menuitem-link:hover{color:#fff}.grf-ui-submenu-icon{margin-top:.2em}.grf-ui-menuitem{margin:0;-webkit-box-sizing:border-box;box-sizing:border-box;border-radius:0}.grf-ui-blockmenubaritem-spans{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);color:#fff;font-size:.9em;opacity:.9}.grf-ui-blockmenubaritem-text{font-size:.9em}.grf-ui-blockmenubar-state-active{color:#fff}.grf-ui-menuitem-show ul{display:block}.grf-ui-blockmenubar-root-list li:hover ul{animation:.5s menuopacity;-moz-animation:.5s menuopacity;-webkit-animation:.5s menuopacity;-o-animation:.5s menuopacity}@keyframes menuopacity{from{opacity:0}to{opacity:1}}@-webkit-keyframes menuopacity{from{opacity:0}to{opacity:1}}.default .grf-ui-blockmenubar-state-active,.default .grf-ui-menuitem-active>.grf-ui-menuitem-link,.default .grf-ui-menuitem-link:hover{background-color:#0275d8}`],
                providers: [DomHandler],
            },] },
];
/** @nocollapse */
BlockMenuBarSubComponent.ctorParameters = () => [
    { type: DomHandler, },
];
BlockMenuBarSubComponent.propDecorators = {
    "item": [{ type: Input },],
    "root": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BlockMenuBarModule {
}
BlockMenuBarModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule],
                exports: [BlockMenuBarComponent, BlockMenuBarSubComponent],
                declarations: [BlockMenuBarComponent, BlockMenuBarSubComponent]
            },] },
];
/** @nocollapse */
BlockMenuBarModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class AccordionMenuComponent {
    constructor() {
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    handleClick(event, item) {
        this.itemClickEvent.emit(item);
        this.animating = true;
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        item.expanded = !item.expanded;
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
    /**
     * @param {?} result
     * @return {?}
     */
    getResult(result) {
        console.log(result);
        // this.model[0].expanded = true;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    MouseHover(item) {
    }
    /**
     * @return {?}
     */
    onToggleDone() {
        this.animating = false;
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
}
AccordionMenuComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-accordionmenu',
                template: `<div class="{{theme}}">
    <div [class]="styleClass" [ngStyle]="style" [ngClass]="'grf-ui-accordionmenu grf-ui-widget'">
        <div *ngFor="let item of model;let f=first;let l=last;">
            <div [ngClass]="{'grf-ui-widget grf-ui-accordionmenu-header grf-ui-state-default':true,'grf-ui-corner-top':f,'grf-ui-corner-bottom':l&&!item.expanded,'grf-ui-state-disabled':item.disabled}" [class]="item.styleClass" [ngStyle]="item.style">
                <a *ngIf="!item.routerLink" [href]="item.url||'#'" [ngClass]="{'grf-ui-accordionmenu-headerlink-hasicon':item.icon,'grf-ui-accordionmenu-link':!item.expanded}" (click)="handleClick($event,item)" [attr.target]="item.target"
                    [attr.title]="item.title">
                    <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon" *ngIf="item.icon"></span
              ><span class="grf-ui-menuitem-text">{{item.label}}</span>
                    <span *ngIf="item.items" class="grf-ui-menuitem-expandicon fa" [ngClass]="{'fa-angle-down':!item.expanded,'fa-angle-up':item.expanded}"></span
              >
          </a>
          <a *ngIf="item.routerLink" [routerLink]="item.routerLink" [routerLinkActive]="'grf-ui-accordionmenu-active'" [routerLinkActiveOptions]="item.routerLinkActiveOptions||{exact:false}"
              [ngClass]="{'grf-ui-accordionmenu-headerlink-hasicon':item.icon,'grf-ui-accordionmenu-link':!item.expanded}" (click)="handleClick($event,item)"
              [attr.target]="item.target" [attr.title]="item.title">
              <span class="grf-ui-menuitem-icon fa" [ngClass]="item.icon" *ngIf="item.icon"></span
              ><span class="grf-ui-menuitem-text">{{item.label}}</span>
                    <span *ngIf="item.items" class="grf-ui-menuitem-expandicon fa" [ngClass]="{'fa-angle-down':!item.expanded,'fa-angle-up':item.expanded}"></span
              >
          </a>
      </div>
      <div *ngIf="item.items" class="grf-ui-accordionmenu-content-wrapper" [@rootItem]="item.expanded ? 'visible' : 'hidden'"  (@rootItem.done)="onToggleDone($event)"
          [ngClass]="{'grf-ui-accordionmenu-content-wrapper-overflown': !item.expanded||animating}">
          <div class="grf-ui-accordionmenu-content grf-ui-widget-content grf-ui-accordionmenu-content" style="border:none">
              <grf-accordionmenusub [item]="item" [expanded]="true" (itemClickEvent)="componentClick($event)"></grf-accordionmenusub>
          </div>
      </div>
  </div>
</div>
</div>
`,
                styles: [`@charset "UTF-8";.grf-ui-accordionmenu .fa{position:static}.grf-ui-accordionmenu .grf-ui-accordionmenu-header{background-color:#fff;padding:0;font-size:14px;-webkit-transition:background-color .3s;transition:background-color .3s}.grf-ui-accordionmenu .grf-ui-accordionmenu-header a{padding:8px 12px;color:#525252}.grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover{background-color:#fff}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link{padding:.375em .75em;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;border:none}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-text{display:inline-block;vertical-align:middle;font-size:14px}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-icon{font-size:14px}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-accordionmenu .grf-ui-accordionmenu-content span.fa{display:inline-block;vertical-align:middle;left:auto}.grf-ui-menuitem-expandicon{font-size:14px;float:right;padding-top:.35em}.grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-accordionmenuitem{padding-left:15px;margin:1px 0}.grf-ui-menuitem-link{font-size:14px}.grf-ui-accordionmenu-text-link{color:#525252}.default .grf-ui-accordionmenu .fa{position:static}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header .grf-ui-accordionmenu-active{color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover{border-right:2px solid #0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover a{color:#0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover .grf-ui-accordionmenu-active{color:#0275d8;border:none}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link:hover,.default .grf-ui-menuitem-link:hover{color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-menuitem-link:hover{background-color:#fff;color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-state-accordionmenu-active>.grf-ui-menuitem-icon,.default .grf-ui-state-accordionmenu-active>.grf-ui-menuitem-text{color:#0275d8}.default .grf-ui-state-accordionmenu-active{border-right:2px solid #0275d8;color:#0275d8}.default .grf-ui-accordionmenu-text-active{color:#0275d8}`],
                animations: [
                    trigger('rootItem', [
                        state('hidden', style({
                            height: '0px'
                        })),
                        state('visible', style({
                            height: '*'
                        })),
                        transition('visible => hidden', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
                        transition('hidden => visible', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
AccordionMenuComponent.ctorParameters = () => [];
AccordionMenuComponent.propDecorators = {
    "model": [{ type: Input },],
    "style": [{ type: Input },],
    "styleClass": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "searchResult": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class AccordionMenuSubComponent {
    /**
     * @param {?} _el
     */
    constructor(_el) {
        this._el = _el;
        this.itemClickEvent = new EventEmitter();
        this.theme = 'default';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
    /**
     * @param {?} event
     * @param {?} item
     * @param {?} haschlid
     * @return {?}
     */
    chlidClick(event, item, haschlid) {
        this.itemClickEvent.emit(item);
        this.activeItem = null;
        
    }
    /**
     * @param {?} child
     * @return {?}
     */
    domClick(child) {
        //  console.log(child.parentElement);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    componentClick(item) {
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} event
     * @param {?} item
     * @return {?}
     */
    handleClick(event, item) {
        if (item.disabled) {
            event.preventDefault();
            return;
        }
        item.expanded = !item.expanded;
        if (!item.url) {
            event.preventDefault();
        }
        if (item.command) {
            item.command({
                originalEvent: event,
                item: item
            });
        }
    }
}
AccordionMenuSubComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-accordionmenusub',
                template: `<ul class="grf-ui-helper-reset grf-ui-accordionmenu-list" [@submenu]="expanded ? 'visible' : 'hidden'">
    <div class="{{theme}}">
        <ng-template ngFor let-child [ngForOf]="item.items">

            <li *ngIf="child.separator" class="grf-ui-menu-separator grf-ui-widget-content grf-ui-accordionmenuitem">
                <li *ngIf="!child.separator" #listItem (click)="domClick(listItem)" class=" grf-ui-accordionmenuitem" [ngClass]="{'grf-ui-menu-parent':child.items}" [class]="child.styleClass" [ngStyle]="child.style">
                    <a *ngIf="!child.routerLink" [href]="child.url||'#'" class="grf-ui-menuitem-link " [attr.tabindex]="item.expanded ? null : '-1'" [ngClass]="{'grf-ui-menuitem-link-hasicon':child.icon&&child.items,'grf-ui-state-disabled':child.disabled, 'grf-ui-state-active':child==activeItem,
             'grf-ui-accordionmenu-text-link':!(child.items&&child.expanded), 'grf-ui-accordionmenu-text-active':child.items&&child.expanded}" (click)="handleClick($event,child); chlidClick($event,child,child.items)" [attr.target]="child.target" [attr.title]="child.title">
                        <span class="grf-ui-menuitem-icon fa fa-fw" [ngClass]="child.icon" *ngIf="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-menuitem-expandicon fa fa-fw" [ngClass]="{'fa-angle-down':!child.expanded,'fa-angle-up':child.expanded}" *ngIf="child.items"></span>
                    </a>

                    <a *ngIf="child.routerLink" [routerLink]="child.routerLink" [routerLinkActiveOptions]="child.routerLinkActiveOptions||{exact:false}" class="grf-ui-menuitem-link  " [ngClass]="{'grf-ui-menuitem-link-hasicon':child.icon&&child.items,'grf-ui-state-disabled':child.disabled,
                'grf-ui-accordionmenu-text-link':!(child.items&&child.expanded), 'grf-ui-accordionmenu-text-active':child.items&&child.expanded}" [attr.tabindex]="item.expanded ? null : '-1'" (click)="handleClick($event,child); chlidClick($event,child,child.items)"
                        [routerLinkActive]="'grf-ui-state-accordionmenu-active'" [attr.target]="child.target" [attr.title]="child.title">
                        <span class="grf-ui-menuitem-icon fa fa-fw" [ngClass]="child.icon" *ngIf="child.icon"></span>
                        <span class="grf-ui-menuitem-text">{{child.label}}</span>
                        <span class="grf-ui-menuitem-expandicon fa fa-fw" [ngClass]="{'fa-angle-down':!child.expanded,'fa-angle-up':child.expanded}" *ngIf="child.items"></span>
                    </a>
                    <grf-accordionmenusub [item]="child" [expanded]="child.expanded" *ngIf="child.items" (itemClickEvent)="componentClick($event)"></grf-accordionmenusub>
                </li>
        </ng-template>
    </div>
</ul>`,
                styles: [`@charset "UTF-8";.grf-ui-accordionmenu .fa{position:static}.grf-ui-accordionmenu .grf-ui-accordionmenu-header{background-color:#fff;padding:0;font-size:14px;-webkit-transition:background-color .3s;transition:background-color .3s}.grf-ui-accordionmenu .grf-ui-accordionmenu-header a{padding:8px 12px;color:#525252}.grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover{background-color:#fff}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link{padding:.375em .75em;-webkit-transition:background-color .3s;transition:background-color .3s;border-radius:0;border:none}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-text{display:inline-block;vertical-align:middle;font-size:14px}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link .grf-ui-menuitem-icon{font-size:14px}.grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-accordionmenu .grf-ui-accordionmenu-content span.fa{display:inline-block;vertical-align:middle;left:auto}.grf-ui-menuitem-expandicon{font-size:14px;float:right;padding-top:.35em}.grf-ui-menuitem-link:hover{background-color:#fff}.grf-ui-accordionmenuitem{padding-left:15px;margin:1px 0}.grf-ui-menuitem-link{font-size:14px}.grf-ui-accordionmenu-text-link{color:#525252}.default .grf-ui-accordionmenu .fa{position:static}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header .grf-ui-accordionmenu-active{color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover{border-right:2px solid #0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover a{color:#0275d8}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-header:not(.grf-ui-state-active):hover .grf-ui-accordionmenu-active{color:#0275d8;border:none}.default .grf-ui-accordionmenu .grf-ui-accordionmenu-content .grf-ui-menuitem-link:hover,.default .grf-ui-menuitem-link:hover{color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-menuitem-link:hover{background-color:#fff;color:#0275d8;border-right:2px solid #0275d8}.default .grf-ui-state-accordionmenu-active>.grf-ui-menuitem-icon,.default .grf-ui-state-accordionmenu-active>.grf-ui-menuitem-text{color:#0275d8}.default .grf-ui-state-accordionmenu-active{border-right:2px solid #0275d8;color:#0275d8}.default .grf-ui-accordionmenu-text-active{color:#0275d8}`],
                animations: [
                    trigger('submenu', [
                        state('hidden', style({
                            height: '0px'
                        })),
                        state('visible', style({
                            height: '*'
                        })),
                        transition('visible => hidden', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)')),
                        transition('hidden => visible', animate('400ms cubic-bezier(0.86, 0, 0.07, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
AccordionMenuSubComponent.ctorParameters = () => [
    { type: ElementRef, },
];
AccordionMenuSubComponent.propDecorators = {
    "item": [{ type: Input },],
    "expanded": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class AccordionMenuModule {
}
AccordionMenuModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, RouterModule, MatRippleModule],
                exports: [AccordionMenuComponent, AccordionMenuSubComponent],
                declarations: [AccordionMenuComponent, AccordionMenuSubComponent]
            },] },
];
/** @nocollapse */
AccordionMenuModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ArchitectureComponent {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        this.topCenterMenuClickEvent = new EventEmitter();
        this.topTopMenuClickEvent = new EventEmitter();
        this.topRightMenuClickEvent = new EventEmitter();
        this.leftMenuClickEvent = new EventEmitter();
        this.topTopMenuShow = true;
        this.leftShow = true;
        this.rightShow = false;
        this.topBottomShow = false;
        this.topTopShow = false;
        this.leftSmall = false;
        this.leftExpandDone = false;
        this.top_Center_Menu_HasChild = true;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.topCenterMenuJudge(this.model.top_center_menu_items);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topCenterMenuClick(item) {
        this.topCenterMenuClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topTopMenuClick(item) {
        this.topTopMenuClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topRightMenuClick(item) {
        this.topRightMenuClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    leftMenuClick(item) {
        this.leftMenuClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topCenterMenuJudge(item) {
        for (let /** @type {?} */ i = 0; i < item.length; i++) {
            for (const /** @type {?} */ a in item[i]) {
                if (a === 'items') {
                    this.top_Center_Menu_HasChild = true;
                    return;
                }
                else {
                    this.top_Center_Menu_HasChild = false;
                }
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.animating) {
            return false;
        }
        this.animating = true;
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    expand(event) {
        this.topTopShow = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    collapse(event) {
        this.topTopShow = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftScollDone(event) {
        if (event.toState === 'expand') {
            this.leftExpandDone = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftExpand(event) {
        this.leftSmall = false;
        this.leftExpandDone = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftCollapse(event) {
        this.leftSmall = true;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onToggleDone(event) {
        this.animating = false;
    }
    /**
     * @param {?} classname
     * @return {?}
     */
    ChangeClass(classname) {
        switch (classname) {
            case 'left':
                this.leftShow = false;
                this.el.nativeElement.querySelector('.architecture-body').style.left = '0px';
                this.el.nativeElement.querySelector('.architecture-body').style.width = '100%';
                break;
            case 'right':
                this.rightShow = false;
                break;
            case 'top-bottom':
                this.topBottomShow = false;
                if (this.topTopShow === false) {
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '70px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.height = 'calc(100% - 100px)';
                }
                else if (this.topTopShow === true) {
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '170px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.height = 'calc(100% - 100px)';
                }
                break;
            case 'top-top':
                this.topTopShow = false;
                if (this.topBottomShow === false) {
                    this.el.nativeElement.querySelector('.architecture-top').style.top = '0px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '70px';
                    this.el.nativeElement.querySelector('.architecture-left').style.top = '70px';
                }
                break;
        }
    }
    /**
     * @param {?} classname
     * @return {?}
     */
    AddClass(classname) {
        switch (classname) {
            case 'left':
                this.leftShow = true;
                this.el.nativeElement.querySelector('.architecture-body').style.left = '300px';
                this.el.nativeElement.querySelector('.architecture-body').style.width = 'calc(100% - 300px)';
                break;
            case 'right':
                this.rightShow = true;
                break;
            case 'top-bottom':
                this.topBottomShow = true;
                if (this.topTopShow === false) {
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '110px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.height = 'calc(100% - 140px)';
                }
                else if (this.topTopShow === true) {
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '210px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.height = 'calc(100% - 140px)';
                    console.log(this.el.nativeElement.querySelector('.architecture-top-top'));
                    // this.el.nativeElement.querySelector('.architecture-top-bottom').style.top = '170px';
                }
                break;
            case 'top-top':
                this.topTopShow = true;
                if (this.topBottomShow === false) {
                    this.el.nativeElement.querySelector('.architecture-top').style.top = '100px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '170px';
                    this.el.nativeElement.querySelector('.architecture-left').style.top = '170px';
                }
                else if (this.topBottomShow === true) {
                    this.el.nativeElement.querySelector('.architecture-top').style.top = '100px';
                    this.el.nativeElement.querySelector('.architecture-top-bottom').style.top = '170px';
                    this.el.nativeElement.querySelector('.architecture-boxup').style.top = '210px';
                }
                break;
        }
    }
}
ArchitectureComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-template-architecture',
                template: `<div class="architecture-box">
    <div class="architecture-top-top" [@TopTopAnimation]="topTopShow ? 'visible' : 'hidden'" (@TopTopAnimation.done)="onToggleDone($event)">
        <div class="top-top-menu">
            <grf-verticaltabmenu [model]="model.top_top_menu_items" (itemClickEvent)="topTopMenuClick($event)"></grf-verticaltabmenu>
        </div>
    </div>
    <div class="architecture-top" [@TopAnimation]="topTopShow ? 'down' : 'up'">
        <div class="architecture-top-left">
            <img #img [src]="model.top_left_logo" class="top-left-logo" alt="grf">
            <a class="top-left-scoll-icon" *ngIf="topTopMenuShow">
                <span (click)="expand($event)" [ngClass]="{'fa fa-fw fa-chevron-down': !topTopShow}"></span>
                <span (click)="collapse($event)" [ngClass]="{'fa fa-fw fa-chevron-up': topTopShow}"></span>
            </a>
        </div>
        <div class="architecture-top-center">
            <div class="top-center-menu">
                <grf-blockmenubar [model]="model.top_center_menu_items" (itemClickEvent)="topCenterMenuClick($event)" *ngIf="top_Center_Menu_HasChild"></grf-blockmenubar>
                <grf-blockmenu [model]="model.top_center_menu_items" (itemClickEvent)="topCenterMenuClick($event)" *ngIf="!top_Center_Menu_HasChild"></grf-blockmenu>
            </div>
        </div>
        <div class="architecture-top-right">
            <div class="top-right-menu">
                <grf-dropdown [menus]="model.top_right_menu_items" direction="bottom-right" (itemClickEvent)="topRightMenuClick($event)"></grf-dropdown>
            </div>
        </div>
    </div>
    <div class="architecture-top-bottom" *ngIf="topBottomShow">top-bottom</div>
    <div class="architecture-left" [@LeftAnimation]="topTopShow ? 'down' : 'up'" [@leftSmallAnimation]="leftSmall ? 'collapse' : 'expand'" *ngIf="leftShow" (@leftSmallAnimation.done)="leftScollDone($event)">
        <div class="left-header">
            <div class="left-header-content">
                <span *ngIf="!leftSmall&&leftExpandDone" [ngClass]="model.left_header_icon" class="fa fa-fw left-header-icon fa-2x"></span><span *ngIf="!leftSmall&&leftExpandDone" class="left-header-text">{{model.left_header_name}}</span>
                <span *ngIf="leftSmall" [ngClass]="model.left_header_icon" class="fa fa-fw  fa-2x left-headersamll-icon" (click)="leftExpand($event)"></span>
                <a class="left-header-expandicon">
                    <span (click)="leftCollapse($event)" *ngIf="!leftSmall" class="fa fa-fw fa-list-ul"></span>
                </a>
            </div>
        </div>
        <div class="accordionmenu_box">
            <grf-accordionmenu *ngIf="!leftSmall&&leftExpandDone" [model]="model.left_menu_items" (itemClickEvent)="leftMenuClick($event)"></grf-accordionmenu>
            <grf-iconmenu *ngIf="leftSmall" [model]="model.left_menu_items" (itemClickEvent)="leftMenuClick($event)"></grf-iconmenu>
        </div>
    </div>
    <div class="architecture-boxup" [@BoxUpAnimation]="topTopShow ? 'down' : 'up'">

        <div class="architecture-body" [@BodyAnimation]="leftSmall ? 'expand' : 'collapse'">
            <div class="architecture-center">
                <div class="architecture-center-body">
                    <ng-content></ng-content>
                </div>
                <div class="architecture-bottom"><span class="bottom-text">{{model.bottom_text}}</span></div>
            </div>
            <div class="architecture-right" *ngIf="rightShow">right</div>
        </div>
    </div>


</div>`,
                animations: [
                    trigger('TopTopAnimation', [
                        state('hidden', style({
                            height: '0'
                        })),
                        state('visible', style({
                            height: '100px'
                        })),
                        transition('visible <=> hidden', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('TopAnimation', [
                        state('up', style({
                            top: '0'
                        })),
                        state('down', style({
                            top: '100px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('BoxUpAnimation', [
                        state('up', style({
                            top: '70px'
                        })),
                        state('down', style({
                            top: '170px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('LeftAnimation', [
                        state('up', style({
                            top: '70px'
                        })),
                        state('down', style({
                            top: '170px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('leftSmallAnimation', [
                        state('collapse', style({
                            width: '70px',
                            backgroundColor: '#e6e6e6',
                            overflow: 'visible'
                        })),
                        state('expand', style({
                            width: '300px',
                            backgroundColor: '#fff',
                            overflow: 'auto'
                        })),
                        transition('collapse <=> expand', animate('500ms cubic-bezier(0.175, 0.885, 0.32, 1.275)'))
                    ]),
                    trigger('BodyAnimation', [
                        state('collapse', style({
                            left: '300px',
                            width: 'calc(100% - 300px)',
                        })),
                        state('expand', style({
                            left: '70px',
                            width: 'calc(100% - 70px)',
                        })),
                        transition('collapse <=> expand', animate('500ms cubic-bezier(0.175, 0.885, 0.32, 1.275)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
ArchitectureComponent.ctorParameters = () => [
    { type: ElementRef, },
];
ArchitectureComponent.propDecorators = {
    "model": [{ type: Input },],
    "topCenterMenuClickEvent": [{ type: Output },],
    "topTopMenuClickEvent": [{ type: Output },],
    "topRightMenuClickEvent": [{ type: Output },],
    "leftMenuClickEvent": [{ type: Output },],
    "topTopMenuShow": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ArchitectureModule {
}
ArchitectureModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule,
                    FormsModule,
                    VerticalTabMenuModule,
                    BlockMenuModule,
                    BlockMenuBarModule,
                    DropdownModule,
                    AccordionMenuModule,
                    IconMenuModule
                ],
                exports: [ArchitectureComponent],
                declarations: [ArchitectureComponent]
            },] },
];
/** @nocollapse */
ArchitectureModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LayoutComponent {
    constructor() { }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
LayoutComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-layout',
                template: ``
            },] },
];
/** @nocollapse */
LayoutComponent.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LayoutService {
    constructor() {
        this.flagchanges = new Subject$1();
        this.topTopShow = false;
        this.topTopShowchanges = new Subject$1();
        this._menuHeader = '';
        this.menuHeaderchanges = new Subject$1();
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set menuHeader(name) {
        this._menuHeader = name;
        this.menuHeaderchanges.next(this._menuHeader);
    }
    /**
     * @return {?}
     */
    get menuHeader() {
        return this._menuHeader;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    expand(event) {
        this.topTopShow = true;
        this.topTopShowchanges.next(this.topTopShow);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    collapse(event) {
        this.topTopShow = false;
        this.topTopShowchanges.next(this.topTopShow);
    }
    /**
     * @param {?} only
     * @return {?}
     */
    onlyParentRoute(only) {
        this.parentRoute = only;
        this.flagchanges.next(this.parentRoute);
    }
}
LayoutService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
LayoutService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class HeaderComponent$1 {
    /**
     * @param {?} _layout
     */
    constructor(_layout) {
        this._layout = _layout;
        this.itemClickEvent = new EventEmitter();
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @param {?} event
     * @return {?}
     */
    onToggleDone(event) {
        this.animating = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    toggle(event) {
        if (this.animating) {
            return false;
        }
        this.animating = true;
        event.preventDefault();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    expand(event) {
        this._layout.expand(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    collapse(event) {
        this._layout.collapse(event);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topTopMenuClick(item) {
        this.collapse(event);
        this.itemClickEvent.emit(item);
        if (item) {
            this._layout.onlyParentRoute(true);
        }
    }
}
HeaderComponent$1.decorators = [
    { type: Component, args: [{
                selector: 'grf-header',
                template: `<div class="architecture-top-top" [@TopTopAnimation]="_layout.topTopShow ? 'visible' : 'hidden'" (@TopTopAnimation.done)="onToggleDone($event)">
    <div class="top-top-menu">
        <grf-verticaltabmenu [model]="model" (itemClickEvent)="topTopMenuClick($event)"></grf-verticaltabmenu>
    </div>
</div>
`,
                styles: [`.architecture-top-top{background-color:#fff;height:0;width:100%;position:fixed;top:0;bottom:0;left:0;z-index:11}.architecture-top-top .top-top-menu{position:relative;padding-left:20px;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}`],
                animations: [
                    trigger('TopTopAnimation', [
                        state('hidden', style({
                            display: 'none',
                            height: '0'
                        })),
                        state('visible', style({
                            display: 'block',
                            height: '100px'
                        })),
                        transition('visible <=> hidden', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ])
                ]
            },] },
];
/** @nocollapse */
HeaderComponent$1.ctorParameters = () => [
    { type: LayoutService, },
];
HeaderComponent$1.propDecorators = {
    "model": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class HeadmenubarComponent {
    /**
     * @param {?} _layout
     */
    constructor(_layout) {
        this._layout = _layout;
        this.topBottomShow = false;
        this.top_Center_Menu_HasChild = true;
        this.headerShow = true;
        this.centerMenuClickEvent = new EventEmitter();
        this.rightMenuClickEvent = new EventEmitter();
        this.theme = 'default';
        this.menuFixed = false;
        this.rightMiniShow = true;
        this.rightMiniModel = [
            {
                label: '更多', icon: 'fa-list-ul',
                items: []
            }
        ];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.response();
        this.rightMiniModel[0].items = this.rightModel;
        console.log(this.rightMiniModel);
        window.addEventListener('resize', () => {
            this.response();
        });
    }
    /**
     * @return {?}
     */
    response() {
        if (document.body.clientWidth > 768) {
            this.rightMiniShow = false;
        }
        else {
            this.rightMiniShow = true;
            this.collapse(this.rightMiniShow);
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    expand(event) {
        this._layout.expand(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    collapse(event) {
        this._layout.collapse(event);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topCenterMenuJudge(item) {
        if (item) {
            for (let /** @type {?} */ i = 0; i < item.length; i++) {
                for (const /** @type {?} */ a in item[i]) {
                    if (a === 'items') {
                        this.top_Center_Menu_HasChild = true;
                        return;
                    }
                    else {
                        this.top_Center_Menu_HasChild = false;
                    }
                }
            }
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topCenterMenuClick(item) {
        if (item.label && !item.items) {
            this._layout.menuHeader = item.label;
        }
        this.centerMenuClickEvent.emit(item);
        if (item) {
            this._layout.onlyParentRoute(true);
        }
    }
    /**
     * @param {?} item
     * @return {?}
     */
    topRightMenuClick(item) {
        this.rightMenuClickEvent.emit(item);
    }
}
HeadmenubarComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-headmenubar',
                template: `<div class="{{theme}}">
    <div class="architecture-top" [@TopAnimation]="_layout.topTopShow ? 'down' : 'up'">
        <div class="architecture-top-left">
            <img #img [src]="logoSrc" class="top-left-logo" alt="Logo">
            <a class="top-left-scoll-icon" *ngIf="headerShow&&!rightMiniShow">
                <span (click)="expand($event)" [ngClass]="{'fa fa-fw fa-chevron-down': !_layout.topTopShow}"></span>
                <span (click)="collapse($event)" [ngClass]="{'fa fa-fw fa-chevron-up': _layout.topTopShow}"></span>
            </a>
        </div>
        <div [ngClass]="{'architecture-top-center-mini':rightMiniShow,'architecture-top-center':!rightMiniShow}">
            <div class="top-center-menu">
                <grf-flexiblemenu [activeItem]="activeItem" [model]="centerModel" (itemClickEvent)="topCenterMenuClick($event)" *ngIf="top_Center_Menu_HasChild" [flexFixed]="menuFixed" [menuParent]="menuParent"></grf-flexiblemenu>
            </div>
        </div>
        <div class="architecture-top-right" *ngIf="!rightMiniShow">
            <span class="top-right-after" *ngIf="_rightAfter">
                <ng-template [ngTemplateOutlet]="_rightAfter">
                </ng-template>
            </span>
            <div class="top-right-menu">
                <grf-dropdown [menus]="rightModel" direction="bottom-right" (itemClickEvent)="topRightMenuClick($event)"></grf-dropdown>
            </div>
            <span class="top-right-before" *ngIf="_rightBefore">
                <ng-template [ngTemplateOutlet]="_rightBefore">
                </ng-template>
            </span>

        </div>
        <div class="architecture-top-right-mini" *ngIf="rightMiniShow">
            <div class="top-right-menu">
                <grf-dropdown [menus]="rightMiniModel" direction="bottom-right" (itemClickEvent)="topRightMenuClick($event)"></grf-dropdown>
            </div>
        </div>
    </div>
    <div class="architecture-top-bottom" *ngIf="topBottomShow">top-bottom</div>
</div>
`,
                styles: [`@charset "UTF-8";.architecture-top{height:70px;width:100%;position:fixed;display:-webkit-box;display:-ms-flexbox;display:flex;top:0;bottom:0;left:0;z-index:12;-webkit-box-shadow:0 3px 5px rgba(0,0,0,.18);box-shadow:0 3px 5px rgba(0,0,0,.18)}.architecture-top .architecture-top-left{width:300px;position:absolute;height:100%;left:0}.architecture-top .architecture-top-left .top-left-logo{float:left;position:relative;height:40px;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);margin-left:20px}.architecture-top .architecture-top-left .top-left-scoll-icon{font-size:14px;opacity:.9;right:0;position:relative;padding-left:20px;top:40%;cursor:pointer}.architecture-top .architecture-top-left .top-left-scoll-icon:hover{opacity:.6}.architecture-top .architecture-top-center{width:calc(100% - 600px);position:relative;right:0;height:100%;left:300px}.architecture-top .architecture-top-center-mini{width:calc(100% - 600px);position:relative;right:0;height:100%;left:100px}.architecture-top .architecture-top-right{width:300px;position:absolute;height:100%;right:0;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.architecture-top .architecture-top-right .top-right-after{padding-right:30px}.architecture-top .architecture-top-right .top-right-before,.architecture-top .architecture-top-right .top-right-menu{padding-right:20px}.architecture-top .architecture-top-right-mini{position:absolute;height:100%;right:0;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.architecture-top .architecture-top-right-mini .top-right-menu{padding-right:10px}.architecture-top-bottom{height:40px;width:100%;position:fixed;top:70px;bottom:0;z-index:999999}.default .architecture-top{background:#002344}.default .architecture-top .architecture-top-left .top-left-scoll-icon,.default .architecture-top .architecture-top-left .top-left-scoll-icon:hover{color:#fff}`],
                animations: [
                    trigger('TopAnimation', [
                        state('up', style({
                            top: '0'
                        })),
                        state('down', style({
                            top: '100px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                ],
            },] },
];
/** @nocollapse */
HeadmenubarComponent.ctorParameters = () => [
    { type: LayoutService, },
];
HeadmenubarComponent.propDecorators = {
    "centerModel": [{ type: Input },],
    "rightModel": [{ type: Input },],
    "logoSrc": [{ type: Input },],
    "headerShow": [{ type: Input },],
    "centerMenuClickEvent": [{ type: Output },],
    "rightMenuClickEvent": [{ type: Output },],
    "_rightBefore": [{ type: ContentChild, args: ['rightBefore',] },],
    "_rightAfter": [{ type: ContentChild, args: ['rightAfter',] },],
    "menuParent": [{ type: Input },],
    "menuFixed": [{ type: Input },],
    "activeItem": [{ type: Input },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BreadItem {
    /**
     * @param {?} label
     * @param {?} url
     * @param {?} current
     */
    constructor(label, url, current) {
        this.label = label;
        this.url = url;
        this.current = current;
    }
}
class MainComponent {
    /**
     * @param {?} _layout
     * @param {?} _search
     * @param {?} el
     * @param {?} _router
     */
    constructor(_layout, _search, el, _router) {
        this._layout = _layout;
        this._search = _search;
        this.el = el;
        this._router = _router;
        this.rightShow = false;
        this.leftExpandDone = false;
        this.menuNameSet = false;
        this.theme = 'default';
        this._footerShow = true;
        this._menuShow = true;
        this.menuSearchShow = true;
        this.itemClickEvent = new EventEmitter();
        this.miniMenuHoverEvent = new EventEmitter();
        this.miniMenuHoverLeaveEvent = new EventEmitter();
        this.breadCrumb = true;
    }
    /**
     * @param {?} show
     * @return {?}
     */
    set footerShow(show) {
        this._footerShow = show;
        this.footerShowJudge(this.footerShow);
    }
    /**
     * @return {?}
     */
    get footerShow() {
        return this._footerShow;
    }
    /**
     * @param {?} show
     * @return {?}
     */
    set menuShow(show) {
        this._menuShow = show;
        this.menuShowJudge(this.menuShow);
    }
    /**
     * @return {?}
     */
    get menuShow() {
        return this._menuShow;
    }
    /**
     * @param {?} name
     * @return {?}
     */
    set menuHeaderName(name) {
        this._menuHeaderName = name;
        if (this.menuHeaderName) {
            this.menuNameSet = true;
        }
        else {
            this.menuNameSet = false;
        }
    }
    /**
     * @return {?}
     */
    get menuHeaderName() {
        return this._menuHeaderName;
    }
    /**
     * @param {?} items
     * @return {?}
     */
    set menuModel(items) {
        this._menuModel = items;
    }
    /**
     * @return {?}
     */
    get menuModel() {
        return this._menuModel;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // 如果底部不显示做的处理
        this.footerShowJudge(this.footerShow);
        // 如果菜单不显示做的处理
        this.menuShowJudge(this.menuShow);
        // 对需要有搜索功能的数据的处理
        this._search.parentSet(this.menuModel);
        this.breadUrlSet();
        /*        Observable.fromEvent(window, 'load')
                           .subscribe((event) => {
                               // 左侧菜单响应式变化
                               console.log('222');
                           }); */
        this._router.events.subscribe((event) => {
            this.breadUrlSet();
        });
        // this.response();
        this.menuShowJudge(this.menuShow);
        window.addEventListener('resize', () => {
            setTimeout(() => this.response(), 1000);
        });
    }
    /**
     * @return {?}
     */
    response() {
        if (this.menuShow) {
            if (document.body.clientWidth > 768) {
                this.leftSmall = false;
            }
            else {
                this.leftSmall = true;
            }
        }
    }
    /**
     * @return {?}
     */
    breadUrlSet() {
        const /** @type {?} */ breadUrl = this._router.routerState.snapshot.url;
        const /** @type {?} */ urlHandle = breadUrl.replace(/\//g, '//') + '/';
        const /** @type {?} */ patt = /\/[^/]+\//g;
        const /** @type {?} */ urlHandleArray = urlHandle.match(patt);
        this.breadRouterArray = new Array(urlHandleArray.length);
        for (let /** @type {?} */ i = 0; i < urlHandleArray.length; i++) {
            const /** @type {?} */ label = urlHandleArray[i].replace(/\//g, '');
            switch (i) {
                case 0:
                    this.breadRouterArray[i] = new BreadItem(label, '', false);
                    break;
                case 1:
                    this.breadRouterArray[i] = new BreadItem(label, label, false);
                    break;
                default:
                    this.breadRouterArray[i] = new BreadItem(label, this.breadRouterArray[i - 1].url + '/' + label, false);
            }
            if (i === urlHandleArray.length - 1) {
                switch (i) {
                    case 0:
                        this.breadRouterArray[i] = new BreadItem(label, '', true);
                        break;
                    case 1:
                        this.breadRouterArray[i] = new BreadItem(label, label, true);
                        break;
                    default:
                        this.breadRouterArray[i] = new BreadItem(label, this.breadRouterArray[i - 1].url + '/' + label, true);
                }
            }
        }
    }
    /**
     * @param {?} result
     * @return {?}
     */
    searchMenu(result) {
        if (result.parents) {
            const /** @type {?} */ parent = result.parents;
            for (let /** @type {?} */ i = 0; i < parent.length; i++) {
                parent[i].expanded = true;
            }
        }
    }
    /**
     * @return {?}
     */
    labelSet() {
        this.menuActiveItemLabel = '';
    }
    /**
     * @param {?} item
     * @return {?}
     */
    leftMenuClick(item) {
        if (item.label && item.routerLink) {
            this.menuActiveItemLabel = item.label;
            this._layout.onlyParentRoute(false);
            this.breadUrlSet();
        }
        this.itemClickEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    miniMenuHover(item) {
        this.miniMenuHoverEvent.emit(item);
    }
    /**
     * @param {?} item
     * @return {?}
     */
    miniMenuHoverLeave(item) {
        this.miniMenuHoverLeaveEvent.emit(item);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftScollDone(event) {
        if (event.toState === 'expand') {
            this.leftExpandDone = true;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftExpand(event) {
        this.leftSmall = false;
        this.leftExpandDone = false;
    }
    /**
     * @param {?} event
     * @return {?}
     */
    leftCollapse(event) {
        this.leftSmall = true;
    }
    /**
     * @param {?} menushow
     * @return {?}
     */
    menuShowJudge(menushow) {
        if (menushow === true) {
            this.el.nativeElement.querySelector('.architecture-body').style.width = 'calc(100% - 300px)';
            this.el.nativeElement.querySelector('.architecture-body').style.left = '300px';
        }
        else if (menushow === false) {
            this.el.nativeElement.querySelector('.architecture-body').style.width = '100%';
            this.el.nativeElement.querySelector('.architecture-body').style.left = '0';
        }
    }
    /**
     * @param {?} footshow
     * @return {?}
     */
    footerShowJudge(footshow) {
        if (footshow) {
            this.el.nativeElement.querySelector('.architecture-center-body').style.height = 'calc(100% - 30px)';
        }
        else {
            this.el.nativeElement.querySelector('.architecture-center-body').style.height = 'calc(100%)';
        }
    }
}
MainComponent.decorators = [
    { type: Component, args: [{
                selector: 'grf-main',
                template: `<div class="{{theme}} ">
    <div class="architecture-left" [@LeftAnimation]="_layout.topTopShow ? 'down' : 'up'" [@leftSmallAnimation]="leftSmall ? 'collapse' : 'expand'"
        *ngIf="menuShow" (@leftSmallAnimation.done)="leftScollDone($event)">
        <div class="left-header">
            <div class="left-header-content">
                <span *ngIf="!leftSmall&&leftExpandDone" class="fa fa-fw left-header-icon" (click)="leftCollapse($event)">
                    <span *ngIf="menuHeaderIcon" [ngClass]="menuHeaderIcon" class="left-header-fa-icon"></span>
                    <span *ngIf="!menuHeaderIcon&&menuHeaderIconHtml" [innerHtml]="menuHeaderIconHtml"></span>
                </span>
                <span *ngIf="!menuNameSet&&(!leftSmall&&leftExpandDone)" class="left-header-text">{{_layout.menuHeader}}</span>
                <span *ngIf="menuHeaderName&&(!leftSmall&&leftExpandDone)" class="left-header-text">{{menuHeaderName}}</span>
                <span *ngIf="leftSmall" [ngClass]="menuHeaderIcon" class="fa fa-fw left-headersamll-icon" (click)="leftExpand($event)">
                    <span *ngIf="menuHeaderIcon" [ngClass]="menuHeaderIcon" class="left-header-fa-icon"></span>
                    <span *ngIf="!menuHeaderIcon&&menuHeaderIconHtml" [innerHtml]="menuHeaderIconHtml"></span>
                </span>
                <a class="left-header-expandicon">
                    <span (click)="leftCollapse($event)" *ngIf="!leftSmall" class="fa fa-fw fa-list-ul"></span>
                </a>
            </div>
        </div>
        <div *ngIf="!leftSmall&&leftExpandDone&&menuSearchShow" class="left-search">
            <grf-search [searchModel]="menuModel" (itemClickEvent)="searchMenu($event)"></grf-search>
        </div>
        <div [ngClass]="{'accordionmenu_box':!leftSmall&&leftExpandDone&&menuSearchShow}">
            <grf-accordionmenu *ngIf="!leftSmall&&leftExpandDone" [model]="menuModel" [searchResult]="searchResult" (itemClickEvent)="leftMenuClick($event)"></grf-accordionmenu>
            <grf-iconmenu *ngIf="leftSmall" [model]="menuModel" (itemClickEvent)="leftMenuClick($event)" (itemHoverEvent)="miniMenuHover($event)" (itemHoverLeaveEvent)="miniMenuHoverLeave($event)"></grf-iconmenu>
        </div>
    </div>
    <div class="architecture-boxup" [@BoxUpAnimation]="_layout.topTopShow ? 'down' : 'up'">
        <div class="architecture-body" [@BodyAnimation]="leftSmall ? 'expand' : 'collapse'" [@BodySizeAnimation]="menuShow ? 'right' :'all'">
            <div #center class="architecture-center">
                <div class="architecture-center-body">
                    <div class="architecture-center-content">
                        <div class="center-nav-box">
                            <div class="title-box">
                                <span class="center-title" *ngIf="!_layout.parentRoute">
                                    {{menuActiveItemLabel}}
                                </span>
                            </div>
                            <div class="bread-box" >
                                <ng-template  *ngIf="_breadcrumb&&!breadCrumb" [ngTemplateOutlet]="_breadcrumb"></ng-template>
                                <grf-breadcrumb *ngIf="breadCrumb">
                                    <grf-breadcrumb-item *ngFor="let item of breadRouterArray">
                                        <a *ngIf="!item.current" [routerLink]="item.url" (click)="labelSet()">
                                            {{item.label}}
                                        </a>
                                        <a *ngIf="item.current" class="current-item">
                                            {{item.label}}
                                        </a>
                                    </grf-breadcrumb-item>
                                </grf-breadcrumb>
                            </div>

                        </div>
                        <ng-content></ng-content>
                    </div>
                    <div class="architecture-bottom" *ngIf="footerShow">
                        <span class="bottom-text">{{footerText}}</span>
                    </div>
                </div>
            </div>
            <div class="architecture-right" *ngIf="rightShow">right</div>
        </div>
    </div>
</div>
`,
                styles: [`@charset "UTF-8";.markdown{color:#767676;font-size:14px;line-height:1.8}.markdown img{vertical-align:middle;max-width:100%}.markdown h1{color:#4d4c4c;font-weight:400;line-height:40px;margin-bottom:24px;margin-top:8px;font-family:微软雅黑;font-size:28px}.markdown h1 .subtitle{font-weight:400;font-size:90%;margin-left:12px}.markdown h2,.markdown h3,.markdown h4,.markdown h5,.markdown h6{color:#4d4c4c;font-family:微软雅黑;margin:1.6em 0 .6em;font-weight:500;clear:both}.markdown h2{font-size:22px}.markdown h3{font-size:18px}.markdown h4{font-size:16px}.markdown h5{font-size:14px}.markdown h6{font-size:12px}.markdown hr{height:1px;border:0;background:#e9e9e9;margin:16px 0;clear:both}.markdown p,.markdown pre{margin:1em 0}.markdown ul>li{list-style-type:circle;margin-left:20px;padding-left:4px}.markdown ul>li:empty{display:none}.markdown ol>li{list-style-type:decimal;margin-left:20px;padding-left:4px}.markdown ol>li>p,.markdown ul>li>p{margin:.2em 0}.markdown code{margin:0 1px;background:#f7f7f7;padding:.2em .4em;border-radius:3px;font-size:.9em;border:1px solid #eee}.markdown pre{padding:10px;background:#f7f7f7}.markdown pre code{border:none;background:#f7f7f7;margin:0;padding:.3em;font-size:14px;font-family:'微软雅黑';color:#767676;overflow:auto}.markdown b,.markdown strong{font-weight:600}.markdown>table{border-collapse:collapse;border-spacing:0;empty-cells:show;border:1px solid #e9e9e9;width:100%;margin:8px 0 16px}.markdown>table th{white-space:nowrap;color:#5c6b77;font-weight:600;background:#f7f7f7}.markdown>table td,.markdown>table th{border:1px solid #e9e9e9;padding:8px 16px;text-align:left}.markdown blockquote{font-size:14px;background:#f7f3f7;padding:10px 10px 10px .8em;border-left:4px solid #0275d8;margin:1em 0}.markdown blockquote p{margin:0}.markdown .anchor{opacity:0;-webkit-transition:opacity .3s ease;transition:opacity .3s ease;margin-left:8px}.markdown .waiting{color:#ccc;cursor:not-allowed}.markdown a.edit-button{line-height:12px;display:inline-block;margin-left:10px;height:12px}.markdown a.edit-button i{color:#999}.markdown a.edit-button i:hover{color:#0275d8}.markdown a.edit-button .anticon{display:block;font-size:16px}.markdown h1:hover .anchor,.markdown h2:hover .anchor,.markdown h3:hover .anchor,.markdown h4:hover .anchor,.markdown h5:hover .anchor,.markdown h6:hover .anchor{opacity:1;display:inline-block}.markdown>br,.markdown>p>br{clear:both}.markdown.api-container table{font-size:13px}.markdown.api-container table td:first-child{background:#fcfcfc;font-weight:500;width:20%;font-family:"Lucida Console",Consolas,Menlo,Courier,monospace}.markdown.api-container table td:nth-child(3){width:22%;font-size:12px;font-family:"Lucida Console",Consolas,Menlo,Courier,monospace}.markdown.api-container table td:last-child{width:13%;font-size:12px;font-family:"Lucida Console",Consolas,Menlo,Courier,monospace}.grid-demo .code-box-demo .demo-row,.grid-demo .demo-row,[id^=components-grid-demo-] .code-box-demo .demo-row,[id^=components-grid-demo-] .demo-row{background-image:-webkit-gradient(linear,left top,right top,color-stop(4.16667%,#f5f5f5),color-stop(4.16667%,transparent),color-stop(8.33333%,transparent),color-stop(8.33333%,#f5f5f5),color-stop(12.5%,#f5f5f5),color-stop(12.5%,transparent),color-stop(16.66667%,transparent),color-stop(16.66667%,#f5f5f5),color-stop(20.83333%,#f5f5f5),color-stop(20.83333%,transparent),color-stop(25%,transparent),color-stop(25%,#f5f5f5),color-stop(29.16667%,#f5f5f5),color-stop(29.16667%,transparent),color-stop(33.33333%,transparent),color-stop(33.33333%,#f5f5f5),color-stop(37.5%,#f5f5f5),color-stop(37.5%,transparent),color-stop(41.66667%,transparent),color-stop(41.66667%,#f5f5f5),color-stop(45.83333%,#f5f5f5),color-stop(45.83333%,transparent),color-stop(50%,transparent),color-stop(50%,#f5f5f5),color-stop(54.16667%,#f5f5f5),color-stop(54.16667%,transparent),color-stop(58.33333%,transparent),color-stop(58.33333%,#f5f5f5),color-stop(62.5%,#f5f5f5),color-stop(62.5%,transparent),color-stop(66.66667%,transparent),color-stop(66.66667%,#f5f5f5),color-stop(70.83333%,#f5f5f5),color-stop(70.83333%,transparent),color-stop(75%,transparent),color-stop(75%,#f5f5f5),color-stop(79.16667%,#f5f5f5),color-stop(79.16667%,transparent),color-stop(83.33333%,transparent),color-stop(83.33333%,#f5f5f5),color-stop(87.5%,#f5f5f5),color-stop(87.5%,transparent),color-stop(91.66667%,transparent),color-stop(91.66667%,#f5f5f5),color-stop(95.83333%,#f5f5f5),color-stop(95.83333%,transparent));background-image:linear-gradient(90deg,#f5f5f5 4.16667%,transparent 4.16667%,transparent 8.33333%,#f5f5f5 8.33333%,#f5f5f5 12.5%,transparent 12.5%,transparent 16.66667%,#f5f5f5 16.66667%,#f5f5f5 20.83333%,transparent 20.83333%,transparent 25%,#f5f5f5 25%,#f5f5f5 29.16667%,transparent 29.16667%,transparent 33.33333%,#f5f5f5 33.33333%,#f5f5f5 37.5%,transparent 37.5%,transparent 41.66667%,#f5f5f5 41.66667%,#f5f5f5 45.83333%,transparent 45.83333%,transparent 50%,#f5f5f5 50%,#f5f5f5 54.16667%,transparent 54.16667%,transparent 58.33333%,#f5f5f5 58.33333%,#f5f5f5 62.5%,transparent 62.5%,transparent 66.66667%,#f5f5f5 66.66667%,#f5f5f5 70.83333%,transparent 70.83333%,transparent 75%,#f5f5f5 75%,#f5f5f5 79.16667%,transparent 79.16667%,transparent 83.33333%,#f5f5f5 83.33333%,#f5f5f5 87.5%,transparent 87.5%,transparent 91.66667%,#f5f5f5 91.66667%,#f5f5f5 95.83333%,transparent 95.83333%);overflow:hidden;margin-bottom:8px}.grid-demo .ant-row-flex,.grid-demo .code-box-demo .ant-row-flex,[id^=components-grid-demo-] .ant-row-flex,[id^=components-grid-demo-] .code-box-demo .ant-row-flex{background:#f5f5f5}.grid-demo .ant-row-flex>div,.grid-demo .ant-row>div,.grid-demo .code-box-demo .ant-row-flex>div,.grid-demo .code-box-demo .ant-row>div,[id^=components-grid-demo-] .ant-row-flex>div,[id^=components-grid-demo-] .ant-row>div,[id^=components-grid-demo-] .code-box-demo .ant-row-flex>div,[id^=components-grid-demo-] .code-box-demo .ant-row>div{padding:5px 0;text-align:center;border-radius:0;min-height:30px;margin-top:8px;margin-bottom:8px;color:#fff}.grid-demo .code-box-demo .ant-row-flex>div:not(.gutter-row),.grid-demo .code-box-demo .ant-row>div:not(.gutter-row),[id^=components-grid-demo-] .code-box-demo .ant-row-flex>div:not(.gutter-row),[id^=components-grid-demo-] .code-box-demo .ant-row>div:not(.gutter-row){background:#00a0e9;padding:16px 0}.grid-demo .code-box-demo .ant-row-flex>div:not(.gutter-row):nth-child(2n+1),.grid-demo .code-box-demo .ant-row>div:not(.gutter-row):nth-child(2n+1),[id^=components-grid-demo-] .code-box-demo .ant-row-flex>div:not(.gutter-row):nth-child(2n+1),[id^=components-grid-demo-] .code-box-demo .ant-row>div:not(.gutter-row):nth-child(2n+1){background:fade(#00a0e9,70%)}.grid-demo .ant-row .demo-col,.grid-demo .code-box-demo .ant-row .demo-col,[id^=components-grid-demo-] .ant-row .demo-col,[id^=components-grid-demo-] .code-box-demo .ant-row .demo-col{text-align:center;padding:30px 0;color:#fff;font-size:18px;border:none;margin-top:0;margin-bottom:0}.grid-demo .ant-row .demo-col-1,[id^=components-grid-demo-] .ant-row .demo-col-1{background:fade(#00a0e9,70%)}.grid-demo .ant-row .demo-col-2,.grid-demo .code-box-demo .ant-row .demo-col-2,[id^=components-grid-demo-] .ant-row .demo-col-2,[id^=components-grid-demo-] .code-box-demo .ant-row .demo-col-2{background:fade(#00a0e9,50%)}.grid-demo .ant-row .demo-col-3,.grid-demo .code-box-demo .ant-row .demo-col-3,[id^=components-grid-demo-] .ant-row .demo-col-3,[id^=components-grid-demo-] .code-box-demo .ant-row .demo-col-3{background:rgba(255,255,255,.2);color:#999}.grid-demo .ant-row .demo-col-4,.grid-demo .code-box-demo .ant-row .demo-col-4,[id^=components-grid-demo-] .ant-row .demo-col-4,[id^=components-grid-demo-] .code-box-demo .ant-row .demo-col-4{background:fade(#00a0e9,60%)}.grid-demo .ant-row .demo-col-5,.grid-demo .code-box-demo .ant-row .demo-col-5,[id^=components-grid-demo-] .ant-row .demo-col-5,[id^=components-grid-demo-] .code-box-demo .ant-row .demo-col-5{background:rgba(255,255,255,.5);color:#999}.grid-demo .code-box-demo .height-100,[id^=components-grid-demo-] .code-box-demo .height-100{height:100px;line-height:100px}.grid-demo .code-box-demo .height-50,[id^=components-grid-demo-] .code-box-demo .height-50{height:50px;line-height:50px}.grid-demo .code-box-demo .height-120,[id^=components-grid-demo-] .code-box-demo .height-120{height:120px;line-height:120px}.grid-demo .code-box-demo .height-80,[id^=components-grid-demo-] .code-box-demo .height-80{height:80px;line-height:80px}.markdown ul.ant-timeline li.ant-timeline-item{list-style:none;margin:0;padding:0 0 30px}.markdown ul.ant-timeline li.ant-timeline-item .ant-timeline-item-content{font-size:14px;padding-left:32px;position:relative;top:-14px}.markdown ul.ant-timeline li.ant-timeline-item .ant-timeline-item-content>h2{margin-top:0;padding-top:.5px}.markdown ul.ant-timeline li.ant-timeline-item:first-child{margin-top:40px}.architecture-boxup{width:100%;height:calc(100% - 70px);background-color:#fff;top:70px;bottom:0;position:relative}.architecture-boxup .architecture-body{height:calc(100%);position:relative;bottom:0;top:0;left:300px;right:0;display:-webkit-box;display:-ms-flexbox;display:flex}.architecture-boxup .architecture-body .architecture-right{background-color:#fff;width:350px;position:relative;height:calc(100% - 70px);left:0;right:0}.architecture-boxup .architecture-body .architecture-center{background-color:#fff;width:100%;position:relative;right:0;height:100%;left:0}.architecture-boxup .architecture-body .architecture-center .architecture-center-body{position:relative;height:calc(100% - 30px)}.architecture-boxup .architecture-body .architecture-center .architecture-center-body .architecture-center-content{min-height:100%;padding:15px}.architecture-boxup .architecture-body .architecture-center .architecture-center-body .architecture-bottom{width:100%;height:30px;display:block;z-index:0;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse}.center-nav-box{padding:0 0 15px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:normal;-ms-flex-direction:row;flex-direction:row}.center-nav-box .bread-box{position:relative;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:horizontal;-webkit-box-direction:reverse;-ms-flex-direction:row-reverse;flex-direction:row-reverse;width:90%;right:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.center-nav-box .bread-box .current-item{cursor:default}.center-nav-box .title-box{width:50%;padding-right:15px}.center-nav-box .center-title{color:#4d4c4c;font-weight:500;line-height:40px;margin-bottom:24px;margin-top:8px;font-family:微软雅黑;font-size:24px}.architecture-left::-webkit-scrollbar{display:none}.architecture-left{background-color:#fff;width:300px;height:calc(100% - 70px);position:fixed;display:block;top:70px;z-index:10;-webkit-box-shadow:3px 0 15px rgba(0,0,0,.18);box-shadow:3px 0 15px rgba(0,0,0,.18);overflow:auto}.architecture-left .left-header{height:70px;cursor:pointer}.architecture-left .left-header .left-header-content{position:relative;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%)}.architecture-left .left-header .left-header-content .left-header-icon{margin-left:10px;font-size:28px}.architecture-left .left-header .left-header-content .left-header-fa-icon{color:#000;opacity:.6}.architecture-left .left-header .left-header-content .left-header-text{margin-left:10px;color:#000;opacity:.7;font-size:18px;margin-top:3px}.architecture-left .left-header .left-header-content .left-headersamll-icon{margin-left:14px;font-size:28px}.architecture-left .left-header .left-header-content .left-header-expandicon{float:right;margin-right:10px;color:#000;font-size:14px;opacity:.6;margin-top:8px}.architecture-left .left-header .left-header-content .left-header-expandicon:hover{color:#000;opacity:.3}.architecture-left .left-search{width:90%;z-index:2;position:relative;left:50%;-webkit-transform:translateX(-50%);transform:translateX(-50%)}.architecture-left .accordionmenu_box{z-index:1;position:absolute;top:120px;width:100%;padding-left:15px}.bottom-text{margin-right:10px;color:#fff;opacity:.8;font-size:14px}.default{width:100%;height:100%}.default .architecture-boxup .architecture-body .architecture-center .architecture-bottom{background:#002344}`],
                animations: [
                    trigger('BoxUpAnimation', [
                        state('up', style({
                            top: '70px'
                        })),
                        state('down', style({
                            top: '170px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('LeftAnimation', [
                        state('up', style({
                            top: '70px'
                        })),
                        state('down', style({
                            top: '170px'
                        })),
                        transition('up <=> down', animate('500ms cubic-bezier(0.23, 1, 0.32, 1)'))
                    ]),
                    trigger('leftSmallAnimation', [
                        state('collapse', style({
                            width: '70px',
                            backgroundColor: '#e6e6e6',
                            overflow: 'visible'
                        })),
                        state('expand', style({
                            width: '300px',
                            backgroundColor: '#fff',
                            overflow: 'auto'
                        })),
                        transition('collapse <=> expand', animate('250ms cubic-bezier(0.47, 0, 0.745, 0.715)'))
                    ]),
                    trigger('BodyAnimation', [
                        state('collapse', style({
                            left: '300px',
                            width: 'calc(100% - 300px)',
                        })),
                        state('expand', style({
                            left: '70px',
                            width: 'calc(100% - 70px)',
                        })),
                        transition('collapse <=> expand', animate('250ms cubic-bezier(0.47, 0, 0.745, 0.715)'))
                    ]),
                    trigger('BottomAnimation', [
                        state('collapse', style({
                            left: '300x',
                            width: 'calc(100% - 300px)',
                        })),
                        state('expand', style({
                            width: 'calc(100% - 70px)',
                            left: '70px'
                        }))
                    ]),
                    trigger('BodySizeAnimation', [
                        state('all', style({
                            left: '0px',
                            width: 'calc(100%)',
                        })),
                        state('right', style({
                            left: '300px',
                            width: 'calc(100% - 300px)',
                        })),
                        transition('collapse <=> expand', animate('250ms cubic-bezier(0.47, 0, 0.745, 0.715)'))
                    ])
                ],
                providers: [SearchService]
            },] },
];
/** @nocollapse */
MainComponent.ctorParameters = () => [
    { type: LayoutService, },
    { type: SearchService, },
    { type: ElementRef, },
    { type: Router, },
];
MainComponent.propDecorators = {
    "menuActiveItemLabel": [{ type: Input },],
    "accordionMenu": [{ type: ViewChild, args: [AccordionMenuComponent,] },],
    "footerShow": [{ type: Input },],
    "menuShow": [{ type: Input },],
    "menuHeaderName": [{ type: Input },],
    "menuHeaderIcon": [{ type: Input },],
    "menuSearchShow": [{ type: Input },],
    "footerText": [{ type: Input },],
    "itemClickEvent": [{ type: Output },],
    "miniMenuHoverEvent": [{ type: Output },],
    "miniMenuHoverLeaveEvent": [{ type: Output },],
    "menuModel": [{ type: Input },],
    "_menuModel": [{ type: Input },],
    "menuHeaderIconHtml": [{ type: Input },],
    "breadCrumb": [{ type: Input },],
    "center": [{ type: ViewChild, args: ['center',] },],
    "_breadcrumb": [{ type: ContentChild, args: ['bread',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LayoutModule {
}
LayoutModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    VerticalTabMenuModule,
                    BlockMenuModule,
                    BlockMenuBarModule,
                    DropdownModule,
                    AccordionMenuModule,
                    IconMenuModule,
                    SearchModule,
                    FlexibleMenuModule,
                    BreadCrumbModule,
                    RouterModule
                ],
                declarations: [LayoutComponent,
                    HeaderComponent$1,
                    HeadmenubarComponent,
                    MainComponent
                ],
                exports: [LayoutComponent,
                    HeaderComponent$1,
                    HeadmenubarComponent,
                    MainComponent
                ]
            },] },
];
/** @nocollapse */
LayoutModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { PaginationModule, TableModule, TabModule, TreeSelectModule, TreeModule, TreeDragDropService, RadioModule, CheckboxModule, SelectModule, TooltipModule, BreadCrumbModule, AlertModule, InputModule, ButtonModule, CardComponent, CardModule, GridModule, FlexibleMenuComponent, FlexibleMenuModule, IconMenuModule, IconMenuComponent, VerticalTabMenuComponent, VerticalTabMenuModule, SearchComponent, SearchModule, DropdownComponent, DropdownModule, SearchService, BlockMenuModule, BlockMenuComponent, BlockMenuBarComponent, BlockMenuBarModule, AccordionMenuComponent, AccordionMenuModule, ArchitectureModule, ArchitectureComponent, LayoutModule, HeaderComponent$1 as HeaderComponent, HeadmenubarComponent, MainComponent, LayoutService };
//# sourceMappingURL=grf-ui.js.map
